Package: FORTLS
Title: Automatic Processing of Terrestrial-Based Technologies Point
        Cloud Data for Forestry Purposes
Version: 2.0.1
Date: 2026-4-19
Authors@R: c(
    person("Juan Alberto", "Molina-Valero", role = c("aut", "cph", "cre"),
    email = "jamolinavalero@gmail.com"),
    person("Adela", "Martínez-Calvo", role = c("aut", "com")),
    person("Juan Gabriel", "Álvarez-González", role = c("aut", "ths")),
    person("Fernando", "Montes", role = "aut"),
    person("César", "Pérez-Cruzado", role = c("aut", "ths")))
Author: Juan Alberto Molina-Valero [aut, cph, cre],
  Adela Martínez-Calvo [aut, com],
  Juan Gabriel Álvarez-González [aut, ths],
  Fernando Montes [aut],
  César Pérez-Cruzado [aut, ths]
Maintainer: Juan Alberto Molina-Valero <jamolinavalero@gmail.com>
Description: Process automation of point cloud data derived from terrestrial-based technologies such as Terrestrial Laser Scanner (TLS) or Mobile Laser Scanner. 'FORTLS' enables (i) detection of trees and estimation of tree-level attributes (e.g. diameters and heights), (ii) estimation of stand-level variables (e.g. density, basal area, mean and dominant height), (iii) computation of metrics related to important forest attributes estimated in Forest Inventories at stand-level, and (iv) optimization of plot design for combining TLS data and field measured data. Documentation about 'FORTLS' is described in Molina-Valero et al. (2022, <doi:10.1016/j.envsoft.2022.105337>). 
URL: https://molina-valero.github.io/FORTLS/,
        https://github.com/Molina-Valero/FORTLS
BugReports: https://github.com/Molina-Valero/FORTLS/issues
License: GPL-3
Depends: R (>= 4.1.0)
Imports: data.table, dbscan, Distance, htmlwidgets, lidR, methods,
        moments, parallel, plotly, progress, raster, Rcpp(>= 1.1.1),
        RCSF, reticulate, scales, sf, tidyr, VoxR, vroom
Suggests: devtools, kableExtra, knitr, rmarkdown, systemfonts, testthat
LinkingTo: Rcpp, RcppArmadillo, RcppEigen
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
StagedInstall: yes
NeedsCompilation: yes
Packaged: 2026-04-19 20:50:23 UTC; jamol
Repository: CRAN
Date/Publication: 2026-04-20 23:10:03 UTC

export(normalize)
import(RCSF)
importFrom(Rcpp, evalCpp)
importFrom(data.table, ":=")
importFrom(tidyr, "%>%")
importFrom("stats", "sd", "uniroot", "var")
importFrom("graphics", "legend", "points")
importFrom("stats", "ave", "coef", "lm", "predict")
importFrom("utils", "read.csv")
useDynLib(FORTLS, .registration=TRUE)
exportPattern("^[[:alpha:]]+")

![alt](https://raw.githubusercontent.com/fortran-lang/fortls/master/assets/logo.png)

# fortls - Fortran Language Server

[![Powered by NumFOCUS](https://img.shields.io/badge/powered%20by-NumFOCUS-orange.svg?style=flat-square&colorA=E1523D&colorB=007D8A)](https://numfocus.org)
![PyPI](https://img.shields.io/pypi/v/fortls?style=flat-square)
![PyPI - Python Version](https://img.shields.io/pypi/pyversions/fortls?style=flat-square)
[![PyPI - Downloads](https://img.shields.io/pypi/dm/fortls?style=flat-square&label=PyPi)](https://pepy.tech/project/fortls)
![Conda](https://img.shields.io/conda/dn/conda-forge/fortls?label=Anaconda&style=flat-square)
![GitHub License](https://img.shields.io/github/license/fortran-lang/fortls?style=flat-square)
![GitHub Workflow Status](https://img.shields.io/github/actions/workflow/status/fortran-lang/fortls/main.yml?branch=master&label=CI&style=flat-square)
![GitHub Workflow Status](https://img.shields.io/github/actions/workflow/status/fortran-lang/fortls/docs.yml?branch=master&label=Docs&style=flat-square)
![Codecov](https://img.shields.io/codecov/c/github/fortran-lang/fortls?style=flat-square)
[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg?style=flat-square)](https://github.com/psf/black)
![GitHub Repo stars](https://img.shields.io/github/stars/fortran-lang/fortls?color=yellow&style=flat-square)
[![DOI](https://zenodo.org/badge/412392321.svg?style=flat-square)](https://zenodo.org/badge/latestdoi/412392321)

<!-- [<img alt="https://github.com/sponsors/gnikit" src="https://img.shields.io/static/v1?style=social&label=Sponsor&message=%E2%9D%A4&logo=GitHub&color&link=%3Curl%3E" height="30" />](https://github.com/sponsors/gnikit)
[<img alt="https://paypal.me/inikit" src="https://img.shields.io/static/v1?style=social&label=Donate&message=%E2%9D%A4&logo=Paypal&color&link=%3Curl%3E" height="30" />](https://paypal.me/inikit) -->

![alt](https://raw.githubusercontent.com/fortran-lang/fortls/master/assets/animations/intro-demo.gif)

`fortls` is an implementation of the [Language Server Protocol](https://github.com/Microsoft/language-server-protocol)
(LSP) for Fortran using Python (3.7+).

All code editors that support LSP can integrate with `fortls` see the section
[Editor Integration](https://fortls.fortran-lang.org/editor_integration.html#editor-integration) in the documentation.
Some supported code editors include:
[Visual Studio Code](https://fortls.fortran-lang.org/editor_integration.html#visual-studio-code),
[Atom](https://fortls.fortran-lang.org/editor_integration.html#atom),
[Sublime Text](https://fortls.fortran-lang.org/editor_integration.html#sublime-text),
[(Neo)Vim](https://fortls.fortran-lang.org/editor_integration.html#vim-neovim-gvim),
and [Emacs](https://fortls.fortran-lang.org/editor_integration.html#emacs).

## Features

- Project-wide and Document symbol detection and Renaming
- Hover support, Signature help and Auto-completion
- GoTo/Peek implementation and Find/Peek references
- Preprocessor support
- Documentation parsing ([Doxygen](http://www.doxygen.org/) and
  [FORD](https://github.com/Fortran-FOSS-Programmers/ford) styles)
- Access to multiple intrinsic modules and functions
  - `ISO_FORTRAN_ENV` GCC 11.2.0
  - `ISO_C_BINDING` GCC 11.2.0
  - `IEEE_EXCEPTIONS`, `IEEE_ARITHMETIC`, `IEEE_FEATURES` GCC 11.2.0
  - OpenMP `OMP_LIB`, `OMP_LIB_KINDS` v5.0
  - OpenACC `OPENACC`, `OPENACC_KINDS` v3.1
- Diagnostics
  - Multiple definitions with the same variable name
  - Variable definition masks definition from parent scope
  - Missing subroutine/function arguments
  - Unknown user-defined type used in `TYPE`/`CLASS` definition
    (only if visible in project)
  - Unclosed blocks/scopes
  - Invalid scope nesting
  - Unknown modules in `USE` statement
  - Unimplemented deferred type-bound procedures
  - Use of non-imported variables/objects in interface blocks
  - Statement placement errors (`CONTAINS`, `IMPLICIT`, `IMPORT`)
- Code actions
  - Generate type-bound procedures and implementation templates for
    deferred procedures

### Notes/Limitations

- Signature help and hover does not handle elegantly overloaded functions i.e. interfaces

## Documentation

The full documentation for `fortls` can be found at
[fortls.fortran-lang.org](https://fortls.fortran-lang.org/).

## Installation

### PyPi

```sh
pip install fortls
```

### Anaconda

```sh
conda install -c conda-forge fortls
```

for more information about the Anaconda installation [see](https://github.com/conda-forge/fortls-feedstock#about-fortls)

### Common installation problems

It is **NOT** recommended having `fortls` and `fortran-language-server`
simultaneously installed, since they use the same executable name. If you are having trouble
getting `fortls` to work try uninstalling `fortran-language-server` and reinstalling `fortls`.

With `pip`

```sh
pip uninstall fortran-language-server
pip install fortls --upgrade
```

or with Anaconda

```sh
conda uninstall fortran-language-server
conda install -c conda-forge fortls
```

## Settings

`fortls` can be configured through both the command line e.g.
`fortls --hover_signature` or through a Configuration json file.
The two interfaces are identical and a full list of the available options can
be found in the [Documentation](https://fortls.fortran-lang.org/options.html)
or through `fortls -h`

An example for a Configuration file is given below

```json
{
  "incremental_sync": true,
  "lowercase_intrinsics": true,
  "hover_signature": true,
  "use_signature_help": true,
  "excl_paths": ["tests/**", "tools/**"],
  "excl_suffixes": ["_skip.f90"],
  "include_dirs": ["include/**"],
  "pp_suffixes": [".F90", ".h"],
  "pp_defs": { "HAVE_HDF5": "", "MPI_Comm": "integer" }
}
```

## Implemented server requests

| Request                          | Description                                            |
| -------------------------------- | ------------------------------------------------------ |
| `workspace/symbol`               | Get workspace-wide symbols                             |
| `textDocument/documentSymbol`    | Get document symbols e.g. functions, subroutines, etc. |
| `textDocument/completion`        | Suggested tab-completion when typing                   |
| `textDocument/signatureHelp`     | Get signature information at a given cursor position   |
| `textDocument/definition`        | GoTo definition/Peek definition                        |
| `textDocument/references`        | Find all/Peek references                               |
| `textDocument/documentHighlight` | Same as `textDocument/references`                      |
| `textDocument/hover`             | Show messages and signatures upon hover                |
| `textDocument/implementation`    | GoTo implementation/Peek implementation                |
| `textDocument/rename`            | Rename a symbol across the workspace                   |
| `textDocument/didOpen`           | Document synchronisation upon opening                  |
| `textDocument/didSave`           | Document synchronisation upon saving                   |
| `textDocument/didClose`          | Document synchronisation upon closing                  |
| `textDocument/didChange`         | Document synchronisation upon changes to the document  |
| `textDocument/codeAction`        | **Experimental** Generate code                         |

## Future plans

`fortls` has reached a point where it is feature complete and stable enough to be used in many modern Fortran projects without any issues.
It does however still have fundamental limitations,
namely its ability to understand all Fortran syntax and semantics that has been used throughout the 65+ years of the language. **The good news is that we have a plan to address this issue!**

We are excited to announce that we are working on creating a new Fortran Language Server
based on the actively developed [LFortran](https://lfortran.org/) compiler 🎉.
The new Language Server will be able to understand all Fortran syntax, be faster,
and give more accurate autocompletion, hover and diagnostic information. That means we plan on investing any future funding on creating our new language server and ultimately creating a better user experience for everyone.

<!--  Have a look at our roadmap (link) and consider contributing to our efforts (donations and contribution tasks link).  -->

### What about `fortls`?

Not to worry, `fortls` will continue to be here. We will keep `fortls` in active maintenance mode with bug fixes and new features from volunteer contributors, but otherwise we will be focusing our efforts into making the new language server using LFortran's parser a reality.

## `fortls` vs `fortran-language-server`

This project was originally based on `fortran-language-server` LSP implementation, but the two projects have since diverged.

`fortls` (this project) is now developed independently of the upstream `hansec/fortran-language-server` project and contains numerous new features and bug fixes
the original `fortran-language-server` does not.

For a complete and detailed list of the differences between the two Language Servers
see the Documentation section: [Unique fortls features (not in fortran-language-server)](https://fortls.fortran-lang.org/fortls_changes.html)

The name of executable for this project has been chosen to remain `fortls`
to allow for integration with pre-existing plugins and workflows, but it could
change in the future.

## Acknowledgements

This project would not have been possible without the original work of [@hansec](https://github.com/hansec/)
in [`fortran-language-server`](https://github.com/hansec/fortran-language-server)

## Support

You can support Fortran-lang as a whole by donating at
[Fortran-lang - NumFOCUS](https://numfocus.org/donate-for-fortran-lang).

## Bug reports

When [filing bugs](https://github.com/fortran-lang/fortls/issues/new)
please provide example code to reproduce the observed issue.

## Security Policy

To report a security vulnerability please follow the instructions in our
[Security page](https://github.com/fortran-lang/fortls/security/policy).

## License

This project is made available under the [MIT License](https://github.com/fortran-lang/fortls/blob/master/LICENSE).

![alt](https://raw.githubusercontent.com/gnikit/fortls/master/assets/logo.png)

# fortls - Fortran Language Server

![PyPI](https://img.shields.io/pypi/v/fortls)
![PyPI - Python Version](https://img.shields.io/pypi/pyversions/fortls)
[![Tests](https://github.com/gnikit/fortls/actions/workflows/main.yml/badge.svg)](https://github.com/gnikit/fortls/actions/workflows/main.yml)
[![Documentation](https://github.com/gnikit/fortls/actions/workflows/docs.yml/badge.svg)](https://github.com/gnikit/fortls/actions/workflows/docs.yml)
[![codecov](https://codecov.io/gh/gnikit/fortls/branch/master/graph/badge.svg?token=ZEXUX0R65M)](https://codecov.io/gh/gnikit/fortls)
[![GitHub license](https://img.shields.io/github/license/gnikit/fortls)](https://github.com/gnikit/fortls/blob/dev/LICENSE)
[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)

[![GitHub Sponsor](https://img.shields.io/static/v1?style=social&label=Sponsor&message=%E2%9D%A4&logo=GitHub&color&link=%3Curl%3E)](https://github.com/sponsors/gnikit)
[![PayPal](https://img.shields.io/static/v1?style=social&label=Donate&message=%E2%9D%A4&logo=Paypal&color&link=%3Curl%3E)](https://paypal.me/inikit)

![alt](https://raw.githubusercontent.com/gnikit/fortls/master/assets/animations/intro-demo.gif)

`fortls` is an implementation of the [Language Server Protocol](https://github.com/Microsoft/language-server-protocol)
(LSP) for Fortran using Python (3.7+).

All code editors that support LSP can integrate with `fortls` see the section
[Editor Integration](https://gnikit.github.io/fortls/editor_integration.html#editor-integration) in the documentation.
Some supported code editors include:
[Visual Studio Code](https://gnikit.github.io/fortls/editor_integration.html#visual-studio-code),
[Atom](https://gnikit.github.io/fortls/editor_integration.html#atom),
[Sublime Text](https://gnikit.github.io/fortls/editor_integration.html#sublime-text),
[(Neo)Vim](https://gnikit.github.io/fortls/editor_integration.html#vim-neovim-gvim),
and [Emacs](https://gnikit.github.io/fortls/editor_integration.html#emacs).

## Features

- Project-wide and Document symbol detection and Renaming
- Hover support, Signature help and Auto-completion
- GoTo/Peek implementation and Find/Peek references
- Preprocessor support
- Documentation parsing ([Doxygen](http://www.doxygen.org/) and
  [FORD](https://github.com/Fortran-FOSS-Programmers/ford) styles)
- Access to multiple intrinsic modules and functions
  - `ISO_FORTRAN_ENV` GCC 11.2.0
  - `IOS_C_BINDING` GCC 11.2.0
  - `IEEE_EXCEPTIONS`, `IEEE_ARITHMETIC`, `IEEE_FEATURES` GCC 11.2.0
  - OpenMP `OMP_LIB`, `OMP_LIB_KINDS` v5.0
  - OpenACC `OPENACC`, `OPENACC_KINDS` v3.1
- Diagnostics
  - Multiple definitions with the same variable name
  - Variable definition masks definition from parent scope
  - Missing subroutine/function arguments
  - Unknown user-defined type used in `TYPE`/`CLASS` definition
    (only if visible in project)
  - Unclosed blocks/scopes
  - Invalid scope nesting
  - Unknown modules in `USE` statement
  - Unimplemented deferred type-bound procedures
  - Use of non-imported variables/objects in interface blocks
  - Statement placement errors (`CONTAINS`, `IMPLICIT`, `IMPORT`)
- Code actions
  - Generate type-bound procedures and implementation templates for
    deferred procedures

### Notes/Limitations

- Signature help and hover does not handle elegantly overloaded functions i.e. interfaces

## `fortls` vs `fortran-language-server`

This project was originally based on `fortran-language-server` LSP implementation, but the two projects have since diverged.

`fortls` (this project) is now developed independently of the upstream `hansec/fortran-language-server` project and contains numerous new features and bug fixes
the original `fortran-language-server` does not.

For a complete and detailed list of the differences between the two Language Servers
see the Documentation section: [Unique fortls features (not in fortran-language-server)](https://gnikit.github.io/fortls/fortls_changes.html)

The name of executable for this project has been chosen to remain `fortls`
to allow for integration with pre-existing plugins and workflows, but it could
change in the future.

## Installation

### PyPi

```sh
pip install fortls
```

### Anaconda

```sh
conda install -c conda-forge fortls
```

for more information about the Anaconda installation [see](https://github.com/conda-forge/fortls-feedstock#about-fortls)

### Common installation problems

It is **NOT** recommended having `fortls` and `fortran-language-server`
simultaneously installed, since they use the same binary name. If you are having trouble
getting `fortls` to work try uninstalling `fortran-language-server` and reinstalling `fortls`.

With `pip`

```sh
pip uninstall fortran-language-server
pip install fortls --upgrade
```

or with Anaconda

```sh
conda uninstall fortran-language-server
conda install -c conda-forge fortls
```

## Settings

`fortls` can be configured through both the command line e.g.
`fortls --hover_signature` or through a Configuration json file.
The two interfaces are identical and a full list of the available options can
be found in the [Documentation](https://gnikit.github.io/fortls/options.html)
or through `fortls -h`

An example for a Configuration file is given below

```json
{
  "incremental_sync": true,
  "lowercase_intrinsics": true,
  "hover_signature": true,
  "use_signature_help": true,
  "excl_paths": ["tests/**", "tools/**"],
  "excl_suffixes": ["_skip.f90"],
  "include_dirs": ["include/**"],
  "pp_suffixes": [".F90", ".h"],
  "pp_defs": { "HAVE_HDF5": "", "MPI_Comm": "integer" }
}
```

## Implemented server requests

| Request                       | Description                                            |
| ----------------------------- | ------------------------------------------------------ |
| `workspace/symbol`            | Get workspace-wide symbols                             |
| `textDocument/documentSymbol` | Get document symbols e.g. functions, subroutines, etc. |
| `textDocument/completion`     | Suggested tab-completion when typing                   |
| `textDocument/signatureHelp`  | Get signature information at a given cursor position   |
| `textDocument/definition`     | GoTo definition/Peek definition                        |
| `textDocument/references`     | Find all/Peek references                               |
| `textDocument/hover`          | Show messages and signatures upon hover                |
| `textDocument/implementation` | GoTo implementation/Peek implementation                |
| `textDocument/rename`         | Rename a symbol across the workspace                   |
| `textDocument/didOpen`        | Document synchronisation upon opening                  |
| `textDocument/didSave`        | Document synchronisation upon saving                   |
| `textDocument/didClose`       | Document synchronisation upon closing                  |
| `textDocument/didChange`      | Document synchronisation upon changes to the document  |
| `textDocument/codeAction`     | **Experimental** Generate code                         |

## Acknowledgements

This project would not have been possible without the original work of [@hansec](https://github.com/hansec/)
in [`fortran-language-server`](https://github.com/hansec/fortran-language-server)

<!-- ## Support

If you want to support this project you can do it through

[![Alt](https://www.paypalobjects.com/webstatic/mktg/Logo/pp-logo-150px.png)](https://paypal.me/inikit)
[!["Buy Me A Coffee"](https://www.buymeacoffee.com/assets/img/custom_images/orange_img.png)](https://www.buymeacoffee.com/gnikit) -->

## Bug reports

When [filing bugs](https://github.com/gnikit/fortls/issues/new)
please provide example code to reproduce the observed issue.

## License

This project is made available under the [MIT License](https://github.com/gnikit/fortls/blob/dev/LICENSE).

# Info about Licenses

All the contents of this directory (original URL: <https://github.com/fortran-lang/fortls/tree/master/assets>) and any subsequent directories or links or copies of this directory
are licensed under the
[Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International](https://creativecommons.org/licenses/by-nc-nd/4.0/) license.

The `symbol-class.svg` is licensed by Microsoft under a [Creative Commons Attribution 4.0 International License](https://creativecommons.org/licenses/by/4.0/legalcode),
which allows for derivative works to change the original license.

The owner of this repository reserve all other rights, whether under their respective copyrights, patents, or trademarks, whether by implication, estoppel or otherwise.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----include=FALSE------------------------------------------------------------
dir.data <- system.file("data", package="FORTLS")
setwd(dir.data)
load("Rioja.data.RData")
load("Rioja.simulations.RData")
tree.tls <- Rioja.data$tree.tls
tree.field <- Rioja.data$tree.field
dir.data <- system.file("exdata", package="FORTLS")
library(FORTLS)

## ----fig.height=7, fig.width=7, fig.align = "center", message=FALSE, warning=FALSE,fig.alt = "Line charts output obtained with the estimation.plot.size function"----
estimation.plot.size(tree.tls = tree.tls,
                     plot.parameters = data.frame(radius.max = 25, k.max = 50, BAF.max = 4),
                     dbh.min = 4,
                     average = TRUE, all.plot.designs = FALSE)

## ----eval=FALSE, include=TRUE-------------------------------------------------
# simulations <- simulations(tree.tls = tree.tls, tree.ds = tree.ds, tree.field = tree.field,
#             plot.design = c("fixed.area", "k.tree", "angle.count"),
#             plot.parameters = data.frame(radius.max = 25, k.max = 50, BAF.max = 4),
#             scan.approach = "single", var.metr = list(tls = NULL, field = NULL),
#             dbh.min = 4, h.min = 1.3, max.dist = Inf,
#             dir.data = dir.data, save.result = FALSE, dir.result = NULL)

## ----eval=FALSE, include=TRUE-------------------------------------------------
# head(simulations$fixed.area)

## ----echo=FALSE---------------------------------------------------------------
kableExtra::scroll_box(kable_input = kableExtra::kable(head(Rioja.simulations$fixed.area), 
                                                       format = "html"), width = "100%")

## ----eval=FALSE, include=TRUE-------------------------------------------------
# tail(simulations$fixed.area)

## ----echo=FALSE---------------------------------------------------------------
kableExtra::scroll_box(kable_input = kableExtra::kable(tail(Rioja.simulations$fixed.area), 
                                                       format = "html"), width = "100%")

## ----eval=TRUE----------------------------------------------------------------
bias <- relative.bias(simulations = Rioja.simulations,
              variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
              save.result = FALSE, dir.result = NULL)

## ----eval=FALSE, include=TRUE-------------------------------------------------
# head(bias$fixed.area)

## ----echo=FALSE---------------------------------------------------------------
kableExtra::scroll_box(kable_input = kableExtra::kable(head(bias$fixed.area), 
                                                       format = "html"), width = "100%")

## -----------------------------------------------------------------------------
fixed.area.simulations <- list(fixed.area = Rioja.simulations$fixed.area[Rioja.simulations$fixed.area$radius < 7.5, ])
cor <- correlations(simulations = fixed.area.simulations,
             variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
             method = c("pearson", "spearman"), 
             save.result = FALSE, dir.result = NULL)

## ----eval=FALSE, include=TRUE-------------------------------------------------
# cor$correlations$pearson$fixed.area[20:26,1:15]

## ----echo=FALSE---------------------------------------------------------------
kableExtra::scroll_box(kable_input = kableExtra::kable(
  cor$correlations$pearson$fixed.area[20:26,1:15], format = "html"), width = "100%")

## ----eval=FALSE, include=TRUE-------------------------------------------------
# cor$correlations.pval$pearson$fixed.area[20:26,1:15]

## ----echo=FALSE---------------------------------------------------------------
kableExtra::scroll_box(kable_input = kableExtra::kable(
  cor$correlations.pval$pearson$fixed.area[20:26,1:15], format = "html"), width = "100%")

## ----eval=FALSE, include=TRUE-------------------------------------------------
# cor$opt.correlations$pearson$fixed.area[20:26,]

## ----echo=FALSE---------------------------------------------------------------
kableExtra::scroll_box(kable_input = kableExtra::kable(
  cor$opt.correlations$pearson$fixed.area[20:26,], format = "html"), width = "100%")

## ----eval=FALSE---------------------------------------------------------------
# optimize.plot.design(correlations = cor$opt.correlations,
#                      variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
#                      dir.result = NULL)

module test_rename_intrinsic
    implicit none
    interface size
        module procedure size_comp
    end interface size
contains

    subroutine size_comp(val, ret)
        integer, intent(in) :: val(:)
        integer, intent(out) :: ret
        integer, dimension(5) :: fixed
        ret = maxval([size(val), size(fixed)])
    end subroutine size_comp

end module test_rename_intrinsic

program driver
    use test_rename_intrinsic
    implicit none
    integer, dimension(10) :: val
    integer, dimension(5) :: tmp
    integer :: sz
    call size(val, sz)  ! This is fortran_sub and should be renamed
    print*, size(val)   ! This is an intrinsic, should be skipped in renaming
end program driver

module mod
    implicit none

    type :: t
    contains
        procedure :: foo
    end type t

contains

    subroutine foo(self)
        class(t), intent(in) :: self
        call self%foo()
    end subroutine foo
end module mod

program test_semicolon
    implicit none
    integer :: a = 1; character(len=1) :: v; real, parameter :: p = 0.1E-4; character(len=10), parameter :: str = "a;val;that"
    character(len=100), parameter :: str2 = "a;string;"&
    "that;becomes"//       &
    ";"&
    &"multiline";integer&
    :: b;real           &
    &,&
    parameter&
    ::&
    c&
    =&
    100&
    &0090;real :: d;real::e;real::f
    print*, "one";
    print*, str2
    print*, a; print*, p; ! a; comment; that;contains; semi-colons
end program test_semicolon

subroutine spaced_keywords(arg1,  arg2)
    real, dimension (:, :), intent (in) :: arg1
    real, dimension ( size(arg1, 1), maxval([size(arg1, 2), size(arg1, 1)]) ), intent (out) ::  arg2
end subroutine spaced_keywords

module mod_a
   integer, parameter   :: q_a = 4
end module

module mod_b
   use mod_a
   integer, parameter   :: q_b = 8
end module

program test_use_ordering
   use mod_b,  only: q_b
   use mod_a

   real(q_a) :: r_a
   real(q_b) :: r_b
end program test_use_ordering

program test_lines
    implicit none
    character(len=123) :: val = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nam sodales imperdiet dolor, sit amet venenatis magna dictum id."
    ! Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nam sodales imperdiet dolor, sit amet venenatis magna dictum id.
end program test_lines

MODULE test_free
USE, INTRINSIC :: iso_fortran_env, ONLY: error_unit
IMPLICIT NONE
! ą
TYPE :: scale_type
  REAL(8) :: val = 1.d0
END TYPE scale_type
!
TYPE :: vector
  INTEGER(4) :: n
  REAL(8), POINTER, DIMENSION(:) :: v => NULL()
  PROCEDURE(fort_wrap), NOPASS, POINTER :: bound_nopass => NULL()
CONTAINS
  PROCEDURE :: create => vector_create !< Doc 1
  PROCEDURE :: norm => vector_norm !< Doc 2
  PROCEDURE, PASS(self) :: bound_pass => bound_pass
END TYPE vector
!
TYPE, EXTENDS(vector) :: scaled_vector
  TYPE(scale_type) :: scale
CONTAINS
  PROCEDURE :: set_scale => scaled_vector_set !<
  PROCEDURE :: norm => scaled_vector_norm !< Doc 3
END TYPE scaled_vector
!
INTERFACE
  SUBROUTINE fort_wrap(a,b)
  INTEGER(4), INTENT(in) :: a
  REAL(8), INTENT(out) :: b
  END SUBROUTINE fort_wrap
END INTERFACE
!
LOGICAL :: module_variable
CONTAINS
!> Doc 4
SUBROUTINE vector_create(self, n)
CLASS(vector), INTENT(inout) :: self
INTEGER(4), INTENT(in) :: n !! Doc 5
self%n=n
ALLOCATE(self%v(n))
self%v=0.d0
END SUBROUTINE vector_create
!> Doc 6
FUNCTION vector_norm(self) RESULT(norm)
CLASS(vector), INTENT(in) :: self
REAL(8) :: norm
norm = SQRT(DOT_PRODUCT(self%v,self%v))
END FUNCTION vector_norm
!> Doc 7
SUBROUTINE scaled_vector_set(self, scale)
CLASS(scaled_vector), INTENT(inout) :: self
REAL(8), INTENT(in) :: scale !< Doc 8
self%scale%val = scale
END SUBROUTINE scaled_vector_set
!>
FUNCTION scaled_vector_norm(self) RESULT(norm)
CLASS(scaled_vector), INTENT(in) :: self
REAL(8) :: norm
norm = self%scale%val*SQRT(DOT_PRODUCT(self%v,self%v))
END FUNCTION scaled_vector_norm
!
PURE REAL(8) FUNCTION unscaled_norm(self)
CLASS(scaled_vector), INTENT(in) :: self
! REAL(8) :: unscaled_norm
unscaled_norm = SQRT(DOT_PRODUCT(self%v,self%v))
END FUNCTION unscaled_norm
!
SUBROUTINE test_sig_Sub(arg1,arg2,opt1,opt2,opt3)
INTEGER, INTENT(in) :: arg1,arg2
INTEGER, OPTIONAL, INTENT(in) :: opt1,opt2,opt3
END SUBROUTINE test_sig_Sub
!
SUBROUTINE bound_pass(arg1, self)
INTEGER(4), INTENT(in) :: arg1  !< Doc 9
  !! Doc 10

!> Doc 11
!! Doc 12
CLASS(vector), INTENT(inout) :: self
self%n = arg1
END SUBROUTINE bound_pass
END MODULE test_free

module test_doxygen
    implicit none
    
contains

    !> @brief inserts a value into an ordered array
    !!
    !! An array "list" consisting of n ascending ordered values. The method insert a
    !! "new_entry" into the array.
    !! hint: use cshift and eo-shift
    !!
    !! @param[in,out]   list    a real array, size: max_size
    !! @param[in]       n       current values in the array
    !! @param[in]       max_size    size if the array
    !! @param[in]       new_entry   the value to insert
    subroutine insert(list, n, max_size, new_entry)
        real, dimension (:), intent (inout) :: list
        integer, intent (in) :: n, max_size
        real, intent (in) :: new_entry
    end subroutine insert

    !> @brief calcs the angle between two given vectors
    !!
    !! using the standard formula:
    !!  \f$\cos \theta = \frac{ \vec v \cdot \vec w}{\abs{v}\abs{w}}\f$.
    !!
    !! @param[in]   \f$v,w\f$   real vectors
    !! @return  a real value describing the angle. 0 if \f$\abs v\f$ or \f$\abs w\f$ below a
    !!          threshold.
    pure function calc_angle(v, w) result (theta)
        real, dimension (:), intent (in) :: v, w
        real :: theta
    end function calc_angle

end module test_doxygen

subroutine parse_line_continuations
  call report_test("[adaptivity output]", .false., .false., "Congratulations! &
                   & The output from adaptivity might even be OK if you get this far.")
end subroutine parse_line_continuations

module tree
    type tree_inode
        integer :: value = 0
        type (tree_inode), pointer :: left=>null()
        type (tree_inode), pointer :: right=>null()
        type (tree_inode), pointer :: parent=>null()
    end type tree_inode

contains
    recursive subroutine recursive_assign_descending(node, vector, current_loc)
        type(tree_inode), pointer, intent(in) :: node
        integer, dimension(:), intent(inout)  :: vector
        integer, intent(inout)                :: current_loc

        if (associated(node)) then
            call recursive_assign_descending(node%right, vector, current_loc)
            vector(current_loc) = node%value
            current_loc = current_loc + 1
            call recursive_assign_descending(node%left, vector, current_loc)
        end if
        return
    end subroutine recursive_assign_descending
end module tree

module m
    interface
        module subroutine sub(arg)
            integer :: arg
        end subroutine
    end interface
end module m

submodule (m) n

    use, intrinsic :: iso_fortran_env, only: int8, int16, int32, int64
    implicit none

    integer, parameter :: sp = selected_real_kind(6)
    integer, parameter :: dp = selected_real_kind(15)

contains

    pure recursive module function foo_sp(x) result(fi)
        real(sp), intent(in) :: x
        real(sp) :: fi
    end function foo_sp

    pure recursive module function foo_dp(x) result(fi)
        real(dp), intent(in) :: x
        real(dp) :: fi
    end function foo_dp
end submodule n

module oumods
  use, intrinsic :: iso_c_binding
  implicit integer(c_int) (i-k), integer(c_int) (m,n), &
       & real(c_double) (a-h), real(c_double) (l), real(c_double) (o-z)

       TYPE :: ex_type
       INTEGER :: A = 0
     CONTAINS
       FINAL :: del_ex_type
       PROCEDURE :: sub => ex_sub
     END TYPE ex_type
     
contains
  subroutine zI12(t,c,alpha,beta,r)
    complex(c_double_complex) c,r,    x,y,z
     z = c*t
     y = exp(z)
     x = (2.0_c_double * cosh((z - cmplx(0._c_double,3.14159265358979324_c_double, kind(1._c_double))) &
          & /2._c_double )) / (c / exp((z + cmplx(0._c_double,3.14159265358979324_c_double,kind(1._c_double)))/2._c_double))
     r = beta*r+alpha*((t*y - x)/c)
  end subroutine
end module

module mod
    implicit none
contains

    subroutine fi()
        contains
            subroutine phi()
                integer :: a(5)
                print*, size(a) ! this is an intrinsic
            end subroutine phi
    end subroutine fi
end module mod

! simple function
function fun1(arg)
    integer, intent(in) :: arg
    integer :: fun1
end function fun1

! function with type on definition, implied result
integer function fun2(arg)
    integer, intent(in) :: arg
end function fun2

! function with return
function fun3(arg) result(retval)
    integer, intent(in) :: arg
    integer :: retval
end function fun3

! function with type on definition and return
integer function fun4(arg) result(retval)
    integer, intent(in) :: arg
end function fun4

! function with type on definition, return and keywords
pure integer elemental function fun5(arg) result(retval)
    integer, intent(in) :: arg
end function fun5

! function with type on definition and return
function fun6(arg) result(retval)
    integer, intent(in) :: arg
    integer, dimension(10,10) :: retval
end function fun6

! functions with complex result type
pure function outer_product(x, y)
    real, dimension(:), intent(in) :: x, y
    real, dimension(size(x), size(y)) :: outer_product
    integer :: i, j
    forall (i=1:size(x))
        forall (j=1:size(y))
            outer_product(i, j) = x(i) * y(j)
        end forall
    end forall
end function outer_product

! functions with no result type, common in interfaces
function dlamch(CMACH)
    character :: CMACH
end function dlamch

! intrinsic functions like c_loc display a return type
function fun7() result(val)
    use, intrinsic :: iso_c_binding
    integer, dimension(1), target :: ar
    type(c_ptr) :: val
    val = c_loc(ar)
end function fun7

MODULE test_inherit
USE :: test_free, ONLY: scaled_vector
IMPLICIT NONE
!
TYPE, EXTENDS(scaled_vector) :: myvec
  REAL(8) :: x
END TYPE myvec
CONTAINS
SUBROUTINE inherit_completion(self)
TYPE(myvec), INTENT(INOUT) :: self
self%scale%val
END SUBROUTINE inherit_completion
END MODULE test_inherit

PROGRAM test_program
! Here is a commonly included unicode character "–"
USE test_free, ONLY: vector, scaled_vector, module_variable, test_sig_sub
IMPLICIT NONE
!
CHARACTER(LEN=*) :: test_str1 = "i2.2,':',i2.2", test_str2 = 'i2.2,":",i2.2'
INTEGER(4) :: n,a,b,c,d
REAL(8) :: x,y
COMPLEX(8) :: xc,yc
TYPE(vector) :: loc_vector
TYPE(scaled_vector) :: stretch_vector
!
y = myfun(n,x)
CALL glob_sub(n,xc,yc)
!
CALL loc_vector%create(n)
x = loc_vector%norm()
CALL loc_vector%bound_nopass(a,x)
CALL loc_vector%bound_pass(n)
!
CALL stretch_vector%create(n)
CALL stretch_vector%set_scale(loc_vector%norm(self))
x = stretch_vector%norm()
y = stretch_vector%scale%val
!
CALL test_sig_Sub(a,b,opt2=c,opt3=d)
PRINT*, module_variable
END PROGRAM test_program

module visibility
    private :: name
    private :: generic_interf
    interface name
        module procedure :: name_sp
    end interface name
    interface
        subroutine generic_interf(noop)
            integer, intent(in) :: noop
        end subroutine generic_interf
    end interface
contains
    subroutine name_sp(val)
        real(4), intent(in) :: val
        print *, 'name_sp', val
    end subroutine name_sp
end module visibility

program test_enum
   implicit none
   enum, bind(c)

      enumerator :: red =1, blue, black =5
      enumerator yellow
      enumerator gold, silver, bronze
      enumerator :: purple
      enumerator :: pink, lavender

   endenum
end program test_enum

optimize.plot.design <- function(correlations,
                                 variables = c("N", "G", "d", "dg", "d.0",
                                               "h", "h.0"),
                                 dir.result = NULL) {

  opt.cor <- correlations

  # if("W.user.cor" %in% colnames(opt.cor$pearson$fixed.area) |
  #    "W.user.cor" %in% colnames(opt.cor$spearman$fixed.area))
  #   variables = c(variables, "W.user")


  # Define a character vector containing index name (radius, k or BAF) for each
  # available plot design
  .plot.design <- c(fixed.area = "radius", k.tree = "k", angle.count = "BAF")

  # Define character vectors containing the implemented field variables and TLS
  # metrics
  .field.names <- c(
    # Density (trees/ha), basal area (m2/ha) and volume (m3/ha)
    "N", "G", "V",

    # Volume (m3/ha)
    if("V.user.cor" %in% colnames(opt.cor$pearson$fixed.area) |
       "V.user.cor" %in% colnames(opt.cor$spearman$fixed.area))
    "V.user",

    # Biomass (Mg/ha)
    if("W.user.cor" %in% colnames(opt.cor$pearson$fixed.area) |
       "W.user.cor" %in% colnames(opt.cor$spearman$fixed.area))
    "W.user",

    # Mean diameters (cm), and mean dominant diameters (cm)
    "d", "dg", "dgeom", "dharm",
    paste(c("d", "dg", "dgeom", "dharm"), "0", sep = "."),

    # Mean heights (m), and mean dominant heights (m)
    "h", "hg", "hgeom", "hharm",
    paste(c("h", "hg", "hgeom", "hharm"), "0", sep = "."))

  # Define a character vector containing colors for heatmaps
  .color <- c("#2166ac", "#67a9cf", "#d1e5f0", "#f7f7f7", "#f7f7f7", "#fddbc7",
              "#ef8a62", "#b2182b")

  # Define a character vector containing the available correlation measurements
  .cor.method <- c("pearson", "spearman")

  # Define character matrix containing column names for optimal correlation
  # values and their corresponding TLS metric
  .opt.cor.names <- c("cor", "metric")
  .opt.cor.names <- matrix(outer(.field.names, .opt.cor.names, paste,
                                 sep = "."),
                           nrow = length(.field.names),
                           ncol = length(.opt.cor.names),
                           dimnames = list(.field.names, .opt.cor.names))


  # Check arguments


  # 'correlations' must be a list with all or any of the preset elements, and
  # being at least one of them no NULL
  if (!is.list(correlations)) stop("'correlations' must be a list")
  if (is.null(correlations) || all(!.cor.method %in% names(correlations)))
    stop("'correlations' must have at least one of the following elements:",
         "'pearson' or 'spearman'")
  if (any(!names(correlations) %in% .cor.method)) {

    correlations <- correlations[names(correlations) %in% .cor.method]
    warning("There is any element in 'correlations' which do not correspond ",
            "with preset ones. It was not taken into account during the ",
            "execution")

  }
  correlations <- correlations[!sapply(correlations, is.null)]
  if (length(correlations) == 0)
    stop("'correlations' must have at least one of the following elements ",
         "different from 'NULL': 'pearson' or 'spearman'")
  for (.i in names(correlations)) {

    # Each 'correlations' element must be a list with all or any of the preset
    # elements, and being at least one of them no NULL
    if (!is.list(correlations[[.i]]))
      stop("'correlations$", .i, "' must be a list")
    if (is.null(correlations[[.i]]) ||
        all(!names(.plot.design) %in% names(correlations[[.i]])))
      stop("'correlations$", .i, "' must have at least one of the following ",
           "elements: 'fixed.area', 'k.tree' or 'angle.count'")
    if (any(!names(correlations[[.i]]) %in% names(.plot.design))) {

      correlations[[.i]] <- correlations[[.i]][names(correlations[[.i]]) %in%
                                                 names(.plot.design)]
      warning("There is any element in 'correlations$", .i, "' which do not ",
              "correspond with preset ones. It was not taken into account ",
              "during the execution")

    }
    correlations[[.i]] <- correlations[[.i]][!sapply(correlations[[.i]],
                                                     is.null)]
    if (length(correlations[[.i]]) == 0)
      stop("'correlations$", .i, "' must have at least one of the following ",
           "elements different from 'NULL': 'fixed.area', 'k.tree' ",
           "or 'angle.count'")
    for (.j in names(correlations[[.i]])) {

      # All elements in each 'correlations' element must be data frames with at
      # least one row, certain mandatory columns and all of them numeric
      if (!is.data.frame(correlations[[.i]][[.j]]))
        stop("'correlations$", .i, "$", .j, "' must be a data.frame")
      if (nrow(correlations[[.i]][[.j]]) < 1)
        stop("'correlations$", .i, "$", .j, "' must have at least one row")
      if (!.plot.design[.j] %in% colnames(correlations[[.i]][[.j]]))
        stop("'correlations$", .i, "$", .j, "' must have a column named '",
             .plot.design[.j], "'")
      if (sum(apply(apply(.opt.cor.names, 1:2,
                          function(x) {x %in%
                              colnames(correlations[[.i]][[.j]])}),
                    1, all)) == 0)
        stop("'correlations$", .i, "$", .j, "' must have at least a ",
             "('<x>.cor', '<x>.metric') pair for the same field estimation")
      for (.k in colnames(correlations[[.i]][[.j]])) {

        if (!.k %in% .opt.cor.names[, "metric"] &&
            !is.numeric(correlations[[.i]][[.j]][, .k]))
          stop("Column '", .k,"' of 'correlations$", .i, "$", .j, "' must be ",
               "numeric")

      }

    }

  }

  # 'variables' must be c("N", "G", "V", "d", "dg", "d.0", "h", "h.0") (by
  # default) or a character vector with all or any of the preset estimations of
  # variables based on field data. Besides, 'correlations' elements must have at
  # least the columns corresponding to 'variables'
  if (!is.character(variables) || !is.vector(variables))
    stop("'variables' must be a character vector")
  if (length(variables) == 0 || all(!variables %in% .field.names))
    stop("'variables' must have at least one of the following character ",
         "strings: ", paste("'", .field.names, "'", sep = "", collapse = ", "))
  if (any(!variables %in% .field.names)) {

    variables <- variables[variables %in% .field.names]
    warning("There is any character string in 'variables' which do not ",
            "correspond with preset ones. It was not taken into account ",
            "during the execution")

  }
  for (.i in names(correlations)) {

    for (.j in names(correlations[[.i]])) {

      if (any(!as.vector(.opt.cor.names[variables, ]) %in%
              colnames(correlations[[.i]][[.j]])))
        stop("Any column corresponding to 'variables' is missing in ",
             "'correlations$", .i, "$", .j, "'")

    }

  }

  # 'dir.result' must be NULL (by default) or a character string containing the
  # absolute path to a existing directory
  if (!is.null(dir.result)) {

    if (!is.character(dir.result))
      stop("'dir.result' must be a character string")
    if (length(dir.result) > 1) {

      dir.result <- dir.result[1]
      warning("Only first value in 'dir.result' was taken into account during ",
              "the execution")

    }
    if (!dir.exists(dir.result)) stop("'dir.result' directory must exist")

  } else {

    # Define working directory as directory by default for saving files
    dir.result <- getwd()

  }


  # Loop for each correlation measure
  for (.i in names(correlations)) {

    # Define initial time, and print message
    t0 <- Sys.time()
    message("Plotting heatmap(s) for ",
            switch(.i, pearson = "optimal Pearson's correlations ",
                   spearman = "optimal Spearman's correlations "))


    # Loop for each plot design
    for (.j in names(correlations[[.i]])) {


      # Plot optimal correlations for each radius, k or BAF


      # Select data corresponding to the field variables, and add index values
      .col.names <- .opt.cor.names[rownames(.opt.cor.names) %in% variables, ,
                                   drop = FALSE]
      .index <- correlations[[.i]][[.j]][, .plot.design[.j]]
      .opt.cor <- correlations[[.i]][[.j]][, .col.names[, "cor"], drop = FALSE]
      .opt.metric <- correlations[[.i]][[.j]][, .col.names[, "metric"],
                                              drop = FALSE]

      # Define title, subtitle, and axis names
      .title <- switch(.i, pearson = "Pearson correlation",
                       spearman = "Spearman correlation")
      .subtitle <- paste("<br> <span style='font-size: 20px;'>",
                         switch(.j, fixed.area = "Circular fixed area plot",
                                k.tree = "K-tree plot",
                                angle.count = "Angle-count plot"),
                         "</span>", sep ="")
      .xaxis <- switch(.j, fixed.area = "Radius (m)",
                       k.tree = "K-tree (trees)",
                       angle.count = "BAF (m<sup>2</sup>/ha)")
      .yaxis <- "Variables"

      # Create heatmap
      fig <- plotly::plot_ly(x = .index, y = rownames(.col.names),
                             z = t(.opt.cor), type = "heatmap",
                             zmin = -1, zmax = 1, zmid = 0, colors = .color,
                             hoverinfo = "x+y+text",
                             text = matrix(paste("Cor:", round(t(.opt.cor), 3),
                                                 "<br>Metric:", t(.opt.metric)),
                                           nrow = ncol(.opt.cor),
                                           ncol = nrow(.opt.cor))) %>%
        plotly::layout(title = paste(.title, .subtitle, sep = ""), font = list(size = 25),
                       xaxis = list(title = .xaxis),
                       yaxis = list (title = .yaxis), margin = list(t = 100))

      # Save heatmap
      suppressWarnings(
        htmlwidgets::saveWidget(widget = fig,
                                file = file.path(dir.result,
                                                 paste("opt.correlations", .j,
                                                       .i, "html", sep = ".")),
                                selfcontained = TRUE,
                                libdir = file.path(dir.result,
                                                   "opt.correlations_files")))

    }

    # Define final time, and print message
    t1 <- Sys.time()
    message(" (", format(round(difftime(t1, t0, units = "secs"), 2)), ")")

  }


  return(invisible())

}

SUBROUTINE block_sub()
INTEGER :: res0,i,j,end_var
res0 = 0
add1 : BLOCK
  INTEGER :: res1
  res1 = res0 + 1
  BLOCK
    INTEGER :: res2,blockVar
    res2 = res1 + 1
    blockVar = res0 + 1
  END BLOCK
END BLOCK add1
!
outer: DO i=1,10
  DO j=1,i
    res0=res0+1
  END DO
END DO outer
!
IF(res0>10)THEN
  i=res0
END IF
!
ASSOCIATE( x=>1 )
  i=i+x
END ASSOCIATE
! Test variables/labels starting with "end"
end_var= 1
end_label: DO i=1,3
  end_var = end_var + i
END DO end_label
END SUBROUTINE block_sub

.onLoad <- function(libname, pkgname) {
  # Attempt to use pre-defined environment
  if (reticulate::virtualenv_exists("fortls_env")) {
    reticulate::use_virtualenv("fortls_env", required = FALSE)
  } else if (reticulate::condaenv_exists("fortls_env")) {
    reticulate::use_condaenv("fortls_env", required = FALSE)
  }
}

simulations <- function(tree.tls, tree.ds = NULL, tree.field,
                        plot.design = c("fixed.area", "k.tree", "angle.count"),
                        plot.parameters = data.frame(radius.max = 25,
                                                     k.max = 50, BAF.max = 4),
                        scan.approach = "single",
                        var.metr = list(tls = NULL, field = NULL),
                        v.calc = "parab", dbh.min = 4, h.min = 1.3,
                        max.dist = Inf, dir.data = NULL, save.result = TRUE,
                        dir.result = NULL) {

  # Call internal function
  sim <- .sim.calc(funct = "sim", tree.tls = tree.tls,
                   tree.ds = tree.ds,
                   tree.field = tree.field, plot.design = plot.design,
                   plot.parameters = plot.parameters,
                   scan.approach = scan.approach, var.metr = var.metr,
                   v.calc = v.calc, dbh.min = dbh.min, h.min = h.min,
                   max.dist = max.dist, dir.data = dir.data,
                   save.result = save.result, dir.result = dir.result)
  return(sim)

}

MODULE test_select
IMPLICIT NONE
!
TYPE :: parent
  INTEGER(4) :: n
END TYPE parent
!
TYPE, EXTENDS(parent) :: child1
  REAL(8) :: a
END TYPE child1
!
TYPE, EXTENDS(parent) :: child2
  COMPLEX(8) :: a
END TYPE child2
CONTAINS
!
SUBROUTINE test_select_sub(self)
CLASS(parent), INTENT(inout) :: self
! Select statement with binding
SELECT TYPE(this=>self)
TYPE IS(child1)
  this%a
CLASS IS(child2)
  this%a
CLASS DEFAULT
  this%n
END SELECT
! Select statement without binding
SELECT TYPE(self)
TYPE IS(child1)
  self%a
END SELECT
END SUBROUTINE test_select_sub
END MODULE test_select

module foo
   implicit none
   public :: length
   private
   integer :: len
   integer :: length
end module foo

program test_private
   use foo, only: length
   use test_vis_mod
   implicit none
   print*, some_var, length
end program test_private

tree.detection.multi.scan <- function(data, single.tree = NULL,
                                      dbh.min = 4, dbh.max = 200, h.min = 1.3,
                                      geo.dist = 0.1, tls.precision = NULL,
                                      stem.section = c(0.7, 3.5), stem.range = NULL, breaks = NULL,
                                      slice = 0.1, understory = NULL, bark.roughness = 1,
                                      den.type = 1, d.mer = NULL,
                                      segmentation = NULL,
                                      plot.attributes = NULL, plot = TRUE,
                                      threads = 1,
                                      dir.data = NULL, save.result = TRUE, dir.result = NULL){



  #### Transforming data (as data frame class) to data table class

  data <- data.table::setDT(data)


  #### Checking some function arguments ####

  # Obtaining working directory for loading files
  if(is.null(dir.data))
    dir.data <- getwd()


  # Obtaining working directory for saving files
  if(is.null(dir.result))
    dir.result <- getwd()


  # Checking tls.precision
  if(is.null(tls.precision))
    tls.precision <- 0.03


  # Conversion of the units of the arguments to units of the international system

  .dbh.min <- dbh.min / 100
  .dbh.max <- dbh.max / 100


  # Obtaining the Cartesian coordinates (x,y) of the plot center

  x.center <- as.numeric(data[data$point == 0, "x"])
  y.center <- as.numeric(data[data$point == 0, "y"])

  data <- data[-1, ]


  #### Detecting possible areas with trees in the point cloud ####


  if(!is.null(data$GLA)){
    woody <- data[data$GLA <= 0, ]
    woody <- woody[!is.na(woody$x) & !is.na(woody$y) & !is.na(woody$z), ]}
  else {woody <- data}


  if(!is.null(data$intensity) & suppressWarnings(mean(data$intensity, na.rm = T)) > 0 & is.null(data$GLA)){
    woody <- data[data$intensity > mean(data$intensity, na.rm = T), ]}


  if(!is.null(data$intensity) & suppressWarnings(mean(data$intensity, na.rm = T)) > 0 & !is.null(data$GLA)){
    woody <- woody[woody$intensity > mean(woody$intensity, na.rm = T), ]}



  # Statistical point cloud filtering ----

  # 1. Statistical Outliers Removal (SOR)

  message("Application of Statistical Outlier Removal (SOR) to the entire point cloud")

  woody <- woody[, c("x", "y", "z")]

  woody <- VoxR::filter_noise(data = data.table::setDT(woody), store_noise = TRUE, message = FALSE)

  woody <- woody[woody$Noise == 1, ]

  woody <- merge(data, woody[, c("x", "y", "z")], by = c("x", "y", "z"), all = FALSE)


  stem <- woody[woody$z > stem.section[1] & woody$z < stem.section[2], c("x", "y", "z")]

  stem <- voxel_grid_downsampling(as.matrix(stem[, c("x", "y", "z")]), voxel_size = tls.precision)

  stem <- data.frame(point = 1:nrow(stem),
                     x = stem[, 1],
                     y = stem[, 2],
                     z = stem[, 3])


  # 2 Computing geometric features (Verticality)

  message("Retention of points with high verticality & low surface variation")

  threads <- max(1, threads)

  GeometricFeatures <- geometric_features_dist(points = as.matrix(stem[, c("point", "x", "y", "z")]),
                                               dist = geo.dist,
                                               Verticality = TRUE,
                                               num_threads = threads)


  stem <- merge(stem, GeometricFeatures[, c("point", "Verticality")], by = "point")

  rm(GeometricFeatures)

  # 2.1 Retention of points with high verticality

  stem <- stem[!is.na(stem$Verticality), ]
  stem <- stem[stem$Verticality > 0.7, ]


  # 3 Computing geometric features (Surface_variation & Verticality)

  GeometricFeatures <- geometric_features_dist(points = as.matrix(stem[, c("point", "x", "y", "z")]),
                                               dist = geo.dist,
                                               Surface_variation = TRUE,
                                               Verticality = TRUE,
                                               num_threads = threads)

  stem <- merge(stem[, c("point", "x", "y", "z")],
                GeometricFeatures[, c("point", "Surface_variation", "Verticality")], by = "point")


  # 3.1 Retention of points with high verticality

  stem <- stem[!is.na(stem$Verticality), ]
  stem <- stem[stem$Verticality > 0.7, ]

  # 3.2 Retention of points with low surface variation

  stem$Surface_variation <- stem$Surface_variation / 0.33
  stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
  stem <- stem[stem$Surface_variation < stem$prob.ver, ]


  # 4. Retention of high points in the stem section defined (understory = TRUE)

  if(!is.null(understory)){

    stem$ver <- (stem$z - stem.section[1]) / (stem.section[2] - stem.section[1])
    stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
    stem <- stem[stem$ver > stem$prob.ver, ]

  }


  # 5. Remove cluster with fewer than 15 points

  stem$cluster <- dbscan::dbscan(stem[, c("x", "y", "z"), drop = FALSE], eps = tls.precision, minPts = 2)$cluster
  stem <- stem[stem$cluster > 0 & ave(stem$cluster, stem$cluster, FUN = length) > 20, , drop = FALSE]


  # 7. Detection of regions of high point density

  stem <- VoxR::vox(stem[, c("x", "y", "z")], res = tls.precision)

  stem <- VoxR::project_voxels(stem)

  stem <- stem[stem$ratio > 1, ]
  stem$cluster <- dbscan::dbscan(stem[, c("x", "y"), drop = FALSE], eps = tls.precision, minPts = 2)$cluster
  stem <- stem[stem$cluster > 0 & ave(stem$cluster, stem$cluster, FUN = length) > 1, , drop = FALSE]


  if(nrow(stem) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }


  # Detection of tree stem axes ----

  message("Detection of tree stem axes")


  # Creation of polygons to extract those projected areas in the original point cloud
  # where trees are probably located according to high point density regions


  buf <- sf::st_as_sf(data.frame(stem), coords = c("x","y"))
  buf <- sf::st_buffer(buf, max(.dbh.min, 0.25))
  buf <- sf::st_cast(sf::st_union(buf), "POLYGON")



  # Assigning points to trees previously detected

  # stem <- data.frame(woody[woody$prob.selec == 1, ])
  stem <- data.frame(woody[woody$z < stem.section[2], ])
  stem.3 <- sf::st_as_sf(stem, coords = c("x","y"))

  stem.3 <- sf::st_intersects(buf, stem.3)
  stem.3 <- data.table::setDT(as.data.frame(stem.3))
  colnames(stem.3) <- c("tree", "code")
  stem$code <- as.numeric(row.names(stem))
  stem <- merge(stem, stem.3, by = "code", all = FALSE)
  # stem <- subset(stem, select = -code)
  stem <- stem[, !(names(stem) %in% c("code"))]


  rm(stem.3)



  # Filtering stems axis

  stem.i <- do.call(rbind, lapply(split(stem, stem$tree), .n.w.ratio))
  stem.i <- stem.i[stem.i$z.p99 > stem.section[2] - 0.5, ]
  stem <- stem[stem$tree %in% stem.i$tree, ]

  rm(stem.i)


  # If there is only one tree in the point cloud

  if(!is.null(single.tree)){

    filter <- data.frame(table(stem$tree))
    filter <- filter[order(filter$Freq, decreasing = TRUE), ]
    stem <- stem[stem$tree == filter$Var1[1], ]

  }


  # Breaks argument

  if(is.null(breaks)){
    breaks <- c(0.2, seq(from = 0.4, to = max(woody$z), by = 0.3))
    breaks <- breaks[-length(breaks)]}


  # Defining stem axis

  # Downsampling stem for better axis estimation

  stem <- voxel_grid_downsampling(as.matrix(stem[, c("x", "y", "z", "tree")]), voxel_size = tls.precision)

  stem <- data.frame(x = stem[, 1],
                     y = stem[, 2],
                     z = stem[, 3],
                     tree = stem[, 4])

  eje <- do.call(rbind, lapply(split(stem, stem$tree), .stem.axis, breaks = breaks, scan.approach = "multi", tls.precision))

  eje <- eje[eje$tree %in% eje$tree, ]
  eje <- eje[eje$sec %in% as.character(breaks) & !is.na(eje$x), ]

  rm(stem)


  # Removing those axis very inclined (slope > 0.25)

  eje <- eje[abs(eje$slope) < 0.25, ]


  # Removing those axis ver very closed (0.25 m) to each other

  eje.2 <- eje[eje$sec == 1.3, ]
  dbscan <- dbscan::dbscan(eje.2[, c("x", "y"), drop = FALSE], eps = 0.25, minPts = 1)
  eje.2$cluster <- dbscan$cluster
  n.w.ratio <- tapply(eje.2$n.w.ratio, eje.2$cluster, max)
  eje <- eje[eje$n.w.ratio %in% n.w.ratio, ]

  rm(eje.2)


  # Assigning points to trees previously detected

  # Temporal!!!

  buf <- sf::st_as_sf(data.frame(eje[eje$sec > stem.section[1] & eje$sec < stem.section[2], ]), coords = c("x","y"))
  buf <- sf::st_buffer(buf, max(.dbh.min, 2.5))
  buf <- sf::st_cast(sf::st_union(buf), "POLYGON")


  woody.2 <- sf::st_intersects(buf, sf::st_as_sf(data.frame(woody), coords = c("x","y")))
  woody.2 <- data.table::setDT(as.data.frame(woody.2))
  colnames(woody.2) <- c("tree", "code")
  woody$code <- as.numeric(row.names(woody))
  woody <- merge(woody, woody.2, by = "code", all = FALSE)
  woody <- subset(woody, select = -code)
  # woody <- woody[, !(names(woody) %in% c("code"))]


  rm(buf, woody.2)
  gc()


  #### Starting with clustering process ####

  message("Computing sections")


  # Preallocate lists for efficiency
  .filteraux <- vector("list", length(breaks))


  slice <- slice / 2  # Adjust slice size


  # Set up parallel cluster
  cl <- parallel::makeCluster(max(1, threads))


  pb <- progress::progress_bar$new(total = length(breaks))


  for(i in seq_along(breaks)){

    pb$tick()

    cuts <- breaks[i]


    .cut <- woody[woody$z > (cuts - slice) & woody$z < (cuts + slice), , drop = FALSE]

    if(nrow(.cut) < 25){next}


    # DBSCAN clustering

    .error <- try(suppressMessages(dbscan::dbscan(.cut[, c("x", "y"), drop = FALSE], eps = tls.precision)))
    if(class(.error)[1] == "try-error"){
      message("No computed section: ", cuts, " m")
      next} else {
        .dbscan <- dbscan::dbscan(.cut[, c("x", "y"), drop = FALSE], eps = tls.precision)
        .cut$cluster <- .dbscan$cluster
        .cut <- .cut[.cut$cluster > 0 & ave(.cut$cluster, .cut$cluster, FUN = length) > 2, , drop = FALSE]
        }

    # Checking if there are clusters

    if(nrow(.cut) < 25){next}

    # Assigning section to the slice

    .cut$sec <- cuts

    # Selection of those cluster belonging to trees

    if (interactive()) {

    # Parallel processing within the loop
    .filter <- do.call(rbind, parallel::parLapply(cl, split(.cut, .cut$cluster), .sections.multi.scan,
                                                  tls.precision = tls.precision,
                                                  .dbh.min = .dbh.min, .dbh.max = .dbh.max,
                                                  slice = slice * 2, bark.roughness = bark.roughness,
                                                  x.center = x.center, y.center = y.center))
    } else {

      .filter <- do.call(rbind, lapply(split(.cut, .cut$cluster), .sections.multi.scan,
                                       tls.precision = tls.precision,
                                       .dbh.min = .dbh.min, .dbh.max = .dbh.max,
                                       slice = slice * 2, bark.roughness = bark.roughness,
                                       x.center = x.center, y.center = y.center))
    }


    .filteraux[[i]] <- .filter


    # Run garbage collection every 5 iterations
    if (i %% 5 == 0) gc()

  }# End of cuts loop



  # Stop cluster

  parallel::stopCluster(cl)


  .filter <- do.call(rbind, .filteraux)
  rm(.filteraux, woody)

  # Final garbage collection
  gc()



  #### Assigning sections to tree axis ####

  if(length(.filter) < 1) {

    # Generate a warning and create empty data.frame to be returned, if no row
    # was included in .filter

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }



  .filter <- .stem.assignment.multi.scan(.filter, eje, stem.section, x.center, y.center, single.tree)


  if(nrow(.filter) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }


  # Export all section detected

  # .filter <- .filter[order(.filter$tree, .filter$sec), , drop = FALSE]
  # .stem <- .filter[, c("tree", "sec", "center.x",  "center.y", "radius")]
  # .stem$radius <- .stem$radius * 200
  # colnames(.stem) <- c("tree2", "hi", "x",  "y", "dhi")

  # Export all section detected

  .filter <- .filter[order(.filter$tree, .filter$sec), , drop = FALSE]
  .stem <- .filter[, c("tree", "sec", "center.x",  "center.y", "radius")]
  .stem$radius <- .stem$radius * 200
  colnames(.stem) <- c("tree", "hi", "x",  "y", "dhi")


  # ESTE ES EL SITIO PARA IMPLEMENTAR LA CORRECCIÓN DE DIÁMETROS !!!!

  # .stem <- do.call(
  #   rbind,
  #   lapply(split(.stem, .stem$tree), .refine_diameters)
  # )
  # key_filter <- paste(.filter$tree, .filter$sec)
  # key_stem   <- paste(.stem$tree, .stem$hi)
  # idx <- match(key_filter, key_stem)
  # .filter$center.x <- .stem$x[idx]
  # .filter$center.y <- .stem$y[idx]
  # .filter$radius <- .stem$dhi[idx] / 200
  colnames(.stem) <- c("tree2", "hi", "x",  "y", "dhi")


  ####################################################################################
  ### Calculate of taper coefficient as the slope coefficient of linear regression ###
  ####################################################################################

  .taper <- .filter[, c("tree", "sec", "radius"), drop = FALSE]

  .slope.tree <- data.frame(tree = as.numeric(), slope = as.numeric())

  for(i in unique(.taper$tree)){

  .taper.i <- .taper[.taper$tree == i, ]

  if(nrow(.taper.i) < 2){

    .slope <- data.frame(tree = i, slope = NA)

  } else {

    .lm <- stats::lm(radius ~ sec, data = .taper.i)
    .slope <- data.frame(tree = i, slope = stats::coef(.lm)[2])

  }

  .slope.tree <- rbind(.slope.tree, .slope)

  }

  .filter <- merge(.filter, .slope.tree, by = "tree")


  # If a new column ("dif") is created containing the difference between dbh and
  # section from which radius is estimated, number of used sections (or cuts)
  # will not be important. An estimated radius will always exist

  .filter$dif <- 1.3 - .filter$sec
  .filter$slope <- ifelse(.filter$dif != 0 & is.na(.filter$slope), mean(.filter$slope, na.rm = TRUE), .filter$slope)
  .filter$radio.est <- ifelse(.filter$dif == 0, .filter$radius, .filter$radius + .filter$slope * .filter$dif)
  .filter <- .filter[!is.na(.filter$radio.est), ]



  # When there are not enough tree to the linear model, and tress were
  # detected at at different sections than 1.3 m, we assume these radius
  # as dbh. This could happen in very few situations.
  # .filter$radio.est <- ifelse(is.na(.filter$radio.est), .filter$radius, .filter$radio.est)
  # .filter$radio.est2 <- ifelse(is.na(.filter$radio.est2), .filter$radius2, .filter$radio.est2)

  top.lim <- max(1.3, abs(max(stem.section) - 1.3))

  .radio.est <- data.frame(tree = as.numeric(), radio.est = as.numeric())

  for (i in unique(.filter$tree)) {

    .dat <- .filter[which(.filter$tree == i), ]
    .dat$dif <- abs(.dat$dif)

    # if(min(.dat$dif) > top.lim)
    #   next

    .dat <- .dat[order(.dat$dif), ]

    if(.dat$dif[1] == 0 & .dat$circ[1] == 1){

       .dat <- .dat[1, ]

    } else if (nrow(.dat) > 1 & .dat$dif[1] == 0) {

      .dat <- .dat[.dat$dif == 0 | .dat$dif == .dat$dif[2], ]

    } else {

      .dat <- .dat[.dat$dif == min(.dat$dif), ]

    }


    .out <- data.frame(tree = i, radio.est = mean(.dat$radio.est, na.rm = TRUE))
    .radio.est <- rbind(.radio.est, .out)

  }



  # Tree normal section coordinates

  .filteraux <- data.frame(tree = as.numeric(),

                           filter = as.numeric(),

                           center.x = as.numeric(),
                           center.y = as.numeric(),
                           center.phi = as.numeric(),
                           center.rho = as.numeric(),
                           center.r = as.numeric(),
                           center.theta = as.numeric(),

                           sec.x = as.numeric(),
                           sec.y = as.numeric(),
                           sec.max = as.numeric(),

                           horizontal.distance = as.numeric(),
                           radius = as.numeric(),

                           partial.occlusion = as.numeric(),

                           n.pts = as.numeric(),
                           n.pts.red = as.numeric())



  for (i in unique(.filter$tree)) {

    .dat <- .filter[which(.filter$tree == i), ]

    if(length(breaks) > 3){

      if(nrow(.dat) < 3)
        next

      if (!any(suppressWarnings(diff(.dat$sec, lag = 2)) >= 0.6, na.rm = TRUE))
        next

      }

    .dat <- .dat[order(abs(.dat$dif)), ]

    .sec.x <- .dat$center.x[nrow(.dat)]
    .sec.y <- .dat$center.y[nrow(.dat)]
    .sec.max <- .dat$sec[nrow(.dat)]

    .dat$sec.x <- .sec.x
    .dat$sec.y <- .sec.y
    .dat$sec.max <- .sec.max

    .dat$filter <- nrow(.dat)


    if(nrow(.dat) > 3)
      .dat <- .dat[1:3, ]

    # if(min(abs(.dat$dif)) > top.lim)
    #   next

    .filteraux <- rbind(.filteraux, .dat)

  }


  if(nrow(.filteraux) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }

  # Genrating dendrometric variables

  # .filteraux <- subset(.filteraux, select = -radio.est)
  .filteraux <- .filteraux[, !(names(.filteraux) %in% c("radio.est"))]
  .filteraux <- merge(.filteraux, .radio.est, by = "tree")


  .tree <- data.frame(tree = tapply(.filteraux$tree, .filteraux$tree, mean, na.rm = TRUE),
                      filter = tapply(.filteraux$filter, .filteraux$tree, mean, na.rm = TRUE),

                      x = tapply(.filteraux$center.x, .filteraux$tree, mean, na.rm = TRUE),
                      y = tapply(.filteraux$center.y, .filteraux$tree, mean, na.rm = TRUE),

                      sec.x = tapply(.filteraux$sec.x, .filteraux$tree, mean, na.rm = TRUE),
                      sec.y = tapply(.filteraux$sec.y, .filteraux$tree, mean, na.rm = TRUE),
                      sec.max = tapply(.filteraux$sec.max, .filteraux$tree, mean, na.rm = TRUE),

                      phi = tapply(.filteraux$center.phi, .filteraux$tree, mean, na.rm = TRUE),
                      rho = tapply(.filteraux$center.rho, .filteraux$tree, mean, na.rm = TRUE),
                      r = tapply(.filteraux$center.r, .filteraux$tree, mean, na.rm = TRUE),
                      theta = tapply(.filteraux$center.theta, .filteraux$tree, mean,na.rm = TRUE),

                      horizontal.distance = tapply(.filteraux$center.rho, .filteraux$tree, mean, na.rm = TRUE), # repeated line
                      radius = tapply(.filteraux$radio.est, .filteraux$tree, mean, na.rm = TRUE),

                      partial.occlusion = tapply(.filteraux$arc.circ, .filteraux$tree, min, na.rm = TRUE),

                      n.pts = tapply(.filteraux$n.pts, .filteraux$tree, mean, na.rm = TRUE),
                      n.pts.red = tapply(.filteraux$n.pts.red, .filteraux$tree, mean, na.rm = TRUE))

  rm(.filteraux)


  # Cheking if there are negative radius

  .tree <- .tree[.tree$radius > 0, ]

  if(nrow(.tree) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }


  # Cheking minimum and maximum radius defined in the arguments

  .tree <- .tree[.tree$radius >= .dbh.min / 2 & .tree$radius <= .dbh.max / 2, ]

  if(nrow(.tree) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }


  # Selecting only those trees with more than one section detected when more than two breaks have been specified


  if(length(breaks) > 3)
    .tree <- .tree[.tree$filter >  2, ]

  if(nrow(.tree) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }


  # Ordering by distance and numbering trees from 1 to n trees

  .tree <- .tree[!duplicated(.tree$x) & !duplicated(.tree$y), ]
  .tree <- .tree[!duplicated(.tree$sec.x) & !duplicated(.tree$sec.y), ]

  if(nrow(.tree) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }


  # .tree <- .tree[order(.tree$rho), ]
  # .tree$tree <- 1:nrow(.tree)


  # Detecting possible trees overlaped

  # if(nrow(.tree) < 2 & !is.null(single.tree)){.tree.2 <- .tree}
  if(nrow(.tree) < 2){.tree.2 <- .tree}


  if(nrow(.tree) > 1 & is.null(single.tree)){

  .tree.2 <- data.frame(tree = as.numeric(), filter = as.numeric(),

                        x = as.numeric(), y = as.numeric(),

                        sec.x = as.numeric(), sec.y = as.numeric(), sec.max = as.numeric(),

                        phi = as.numeric(), rho = as.numeric(), r = as.numeric(), theta = as.numeric(),

                        horizontal.distance = as.numeric(), radius = as.numeric(),

                        partial.occlusion = as.numeric(),

                        n.pts = as.numeric(), n.pts.red = as.numeric())


  for (i in unique(.tree$tree)) {


    if(nrow(.tree[.tree$tree == i, ]) < 1)
      next

    .filt <- .tree[.tree$tree == i, ]
    .filteraux <- .tree[.tree$tree != i, ]

    if(nrow(.filteraux) < 1){
      .tree.2 <- rbind(.tree.2, .filt)
      next}


    .filteraux$dist <- sqrt((.filteraux$x - .filt$x) ^ 2 + (.filteraux$y - .filt$y) ^ 2) - .filteraux$radius - .filt$radius
    .filteraux$rho.dist <- abs(.filteraux$rho - .filt$rho) - .filteraux$radius - .filt$radius
    .filteraux$phi.dist <- abs(.filteraux$phi - .filt$phi)

    if(min(.filteraux$dist) < mean(.tree$radius) |
       .filteraux$rho.dist[.filteraux$dist == min(.filteraux$dist)] < mean(.tree$radius) &
       .filteraux$phi.dist[.filteraux$dist == min(.filteraux$dist)] < 0.1){

      .filteraux <- .filteraux[.filteraux$dist < 0 | .filteraux$rho.dist < mean(.tree$radius) & .filteraux$phi.dist < 0.1, ]

      .filteraux <- rbind(.filt, .filteraux[ , 1:(ncol(.filteraux)-3)])

      .filt <- .filteraux[.filteraux$filter == max(.filteraux$filter), ]

      if(nrow(.filt) > 1)
        .filt <- .filteraux[.filteraux$sec.max ==  min(.filteraux$sec.max), ]


      if(nrow(.filt) > 1)
        .filt <- .filt[.filt$partial.occlusion > 0, ]


      if(nrow(.filt) > 1)
        .filt <- .filt[1, ]

      .tree.remove <- .filteraux[.filteraux$tree != .filt$tree, ]$tree


      if(length(.tree.remove) < 1){.tree <- .tree} else {
        suppressWarnings(.tree <- .tree[.tree$tree != .tree.remove, ])
      }


      .tree.2 <- rbind(.tree.2, .filt)


    } else {

      .tree.2 <- rbind(.tree.2, .filt)
      .tree <- .tree[.tree$tree != .filt$tree, ]

    }

  }

  }

  .tree <- .tree.2
  rm(.tree.2)


  # .tree <- .tree[order(.tree$rho), ]
  # .tree$tree <- 1:nrow(.tree)

  # Indicate trees with partial occlusions, those for which none of the sections
  # was identified as circumference arch (ArcCirc)

  .tree$partial.occlusion <- ifelse(.tree$partial.occlusion == 0, 1, 0)

  # Compute dbh (cm)

  .tree$dbh <- .tree$radius * 200


  # Calculate points belonging to radius unit
  # Since it will be an estimation, select sections completely visible
  # (ArcCirc == 1) in section corresponding to 1.3 m (where dbh is estimated)

  .filter$filter <- ifelse(.filter$sec == 1.3 & .filter$circ == 1, 1, 0)
  .filter2 <- subset(.filter, .filter$filter == 1)

  if(nrow(.filter2) < 1)
    .filter2 <- .filter

  # Estimate number of points by cluster, with and without point cropping
  # process, corresponding to radius 1 m

  .filter2$points.radio <- .filter2$n.pts / .filter2$radius
  .filter2$points.radio.hom <- .filter2$n.pts.red / .filter2$radius

  # Average points after point cropping by m of radius

  .tree$points.m <- mean(.filter2$points.radio)
  .tree$points.m.hom <- mean(.filter2$points.radio.hom)

  # Finally, compute number of points estimated for each tree according to
  # radius

  .tree$n.pts.est <- .tree$points.m * .tree$radius
  .tree$n.pts.red.est <- .tree$points.m.hom * .tree$radius


  if(!is.null(plot) & is.null(segmentation))
    plotTree <- suppressMessages(lidR::plot(lidR::LAS(data[data$prob.selec == 1, c("x","y","z")]), size = 0.5))


  #### Estimating tree heights ####

  # Obtaining reduced point cloud

  # data <- data[data$z >= h.min, ]
  data <-
    suppressMessages(vroom::vroom(file.path(dir.data, data$file[1]),
                                  col_select = c("id", "file", "x", "y", "z", "rho"),
                                  progress = FALSE))

  data <- data.table::setDT(data)
  s <- sample(nrow(data), round(nrow(data)*0.1))
  data <- data[s, ]
  data <- data[, c("id", "file", "x", "y", "z", "rho")]



  # If only one tree is detected, Voronoi tessellation is not working

  .tree$tree2 <- .tree$tree
  .tree$tree <- 1:nrow(.tree)

  if(nrow(.tree) == 1){

    .P99 <- data.frame(tree = .tree$tree, h = stats::quantile(data$z, prob = 0.9999999999))

  } else {

    cloud_sf <- sf::st_as_sf(data, coords = c("x", "y"))
    stopifnot(nrow(cloud_sf) == nrow(data))
    bbox     <- sf::st_as_sfc(sf::st_bbox(cloud_sf))
    trees_sf <- sf::st_as_sf(.tree, coords = c("sec.x", "sec.y"))
    voronoi  <- sf::st_collection_extract(
      sf::st_voronoi(sf::st_union(trees_sf), envelope = bbox)
    )

    tree_idx <- vapply(
      sf::st_intersects(voronoi, trees_sf),
      function(x) if (length(x) == 0L) NA_integer_ else x[1L],
      integer(1)
    )
    voronoi <- sf::st_as_sf(data.frame(
      tree     = .tree$tree[tree_idx],
      geometry = sf::st_geometry(voronoi)
    ))

    int_idx       <- sf::st_intersects(cloud_sf, voronoi)
    cloud_sf$tree <- voronoi$tree[vapply(
      int_idx,
      function(x) if (length(x) == 0L) NA_integer_ else x[1L],
      integer(1)
    )]

    .P99 <- do.call(rbind, lapply(unique(voronoi$tree), function(id) {
      sec.max <- .tree$sec.max[match(id, .tree$tree)]
      z       <- cloud_sf$z[!is.na(cloud_sf$tree) & cloud_sf$tree == id & cloud_sf$z > sec.max]
      if (length(z) < 1L) return(data.frame(tree = id, h = 0))
      data.frame(tree = id, h = height_perc_cpp(rho_seq = Inf, z = z, rho = z)[, "P99.9"])
    }))

    rm(trees_sf, tree_idx, voronoi, cloud_sf, int_idx)

  }


  # Remove possible trees above "h.min" (1.3 m by default)

  .P99 <- .P99[.P99$h >= h.min, ]
  .tree <- merge(.tree, .P99, by = "tree", all = FALSE)

  rm(.P99)


  #### Estimating stem volume ####

  # Assigning dbh to stem dataset

  # .tree$tree2 <- .tree$tree
  .tree <- .tree[order(.tree$rho), ]
  .tree$tree <- 1:nrow(.tree)

  .stem <- merge(.stem, .tree[, c("tree", "tree2", "dbh", "h")], by = "tree2")
  .stem <- .stem[, c("tree", "hi", "x", "y", "dhi", "h", "dbh")]
  .tree <- .tree[, colnames(.tree) != "tree2"]

  # .stem <- merge(.stem, .tree[, c("tree", "h")], by = "tree")
  # .stem <- .stem[.stem$sec != 1.3, ]
  # .tree$sec <- 1.3
  # .stem <- rbind(.stem, .tree[, c("tree", "sec", "x", "y", "dbh", "h")])
  # .tree <- .tree[, -ncol(.tree)]
  # .stem <- .stem[order(.stem$tree, .stem$sec), , drop = FALSE]
  # colnames(.stem) <- c("tree", "sec", "x", "y", "dhi", "h")
  # .stem <- merge(.stem, .tree[, c("tree", "dbh")], by = "tree", all.x = TRUE)
  # colnames(.stem) <- c("tree", "hi", "x", "y", "dhi", "h", "dbh")

  # Cheking the trees h

  .stem <- merge(.stem[, c("tree", "hi", "x", "y", "dhi", "dbh")],
                 data.frame(tree = unique(.stem$tree),
                            h = do.call(rbind, lapply(split(.stem[, c("hi", "h")], .stem$tree), max))),
                 all.x = TRUE, by = "tree")

  # Compute volume (m3)

  if(length(table(.stem$hi)) > 3 & is.null(d.mer)){

  # Stem curve

  stem.v <- .volume(.stem, id = data$id[1], den.type = den.type)
  .tree <- merge(.tree, stem.v, all = TRUE)

  } else if (length(table(.stem$hi)) > 3 & !is.null(d.mer)) {

  stem.v <- .volume(.stem, d.mer, id = data$id[1], den.type = den.type)
  .tree <- merge(.tree, stem.v, all = TRUE)

  } else if (length(table(.stem$hi)) <= 3 & !is.null(d.mer)) {

  n <- den.type

  # Estimating volume according to dendrometric type

  .tree$v <- pi * (.tree[, "h"] ^ (n + 1) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)
  h.lim <- (((d.mer / 200) ^ 2) / ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)) ^ (1 / n)
  .tree$v.mer <- pi * ((.tree[, "h"] ^ (n + 1) - h.lim ^ (n + 1)) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)
  .tree$v.mer <- ifelse(.tree$v.mer < 0, 0, .tree$v.mer)
  .tree$h.mer <- h.lim

  } else {

  n <- den.type

  .tree$v <- pi * (.tree[, "h"] ^ (n + 1) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)

  }

  .stem <- .stem[, c("tree", "x", "y", "dhi", "dbh", "hi", "h")]
  .stem <- .stem[order(.stem$tree, .stem$hi), , drop = FALSE]

  if(!is.null(data$id)){
    .stem$id <- data$id[1]
    .stem <- .stem[, c("id", "tree", "x", "y", "dhi", "dbh", "hi", "h")]}



  # Straighness analysis

  straightness <- do.call(rbind, lapply(split(.stem, .stem$tree), .straightness, stem.range = stem.range))

  .tree <- merge(.tree, straightness, by = "tree")

  utils::write.csv(.stem,
                   file = file.path(dir.result, "stem.curve.csv"),
                   row.names = FALSE)

  rm(.stem)

  # If plot identification (id) is not available

  if(is.null(data$id) & is.null(.tree$v.mer)){

    .tree <- .tree[, c("tree", "x", "y", "phi", "horizontal.distance", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
    colnames(.tree) <- c("tree", "x", "y", "phi", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else if (is.null(data$id) & !is.null(.tree$v.mer)) {

    .tree <- .tree[, c("tree", "x", "y", "phi", "horizontal.distance", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
    colnames(.tree) <- c("tree", "x", "y", "phi", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")


  } else if (!is.null(data$id) & is.null(.tree$v.mer)) {

    # If plot identification (id) is available

    .tree$id <- data$id[1]
    .tree$file <- data$file[1]

    .tree <- .tree[, c("id", "file", "tree", "x", "y", "phi", "horizontal.distance", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
    colnames(.tree) <- c("id", "file", "tree", "x", "y", "phi", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else {

    # If plot identification (id) is available

    .tree$id <- data$id[1]
    .tree$file <- data$file[1]

    .tree <- .tree[, c("id", "file", "tree", "x", "y", "phi", "horizontal.distance", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
    colnames(.tree) <- c("id", "file", "tree", "x", "y", "phi", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  }

  .tree <- .tree[order(.tree$h.dist), ]
  .tree$tree <- 1:nrow(.tree)

  # Removing values of 0 in n.pts
  .tree$n.pts <- ifelse(.tree$n.pts < 1, 0.01, .tree$n.pts)
  .tree$n.pts.red <- ifelse(.tree$n.pts.red < 1, 0.01, .tree$n.pts.red)
  .tree$n.pts.est <- ifelse(.tree$n.pts.est < 1, 0.01, .tree$n.pts.est)
  .tree$n.pts.red.est <- ifelse(.tree$n.pts.red.est < 1, 0.01, .tree$n.pts.red.est)


  # Lastly, aggregate attributes table
  if(!is.null(plot.attributes))
    .tree <- merge(.tree, plot.attributes, by = "id", all = FALSE)


  if(isTRUE(save.result)){

    utils::write.csv(.tree,
                     file = file.path(dir.result, "tree.tls.csv"),
                     row.names = FALSE)
  }


  # if(isTRUE(save.result)){
  #
  #   .data.red <- noise[which(noise$prob.selec == 1), , drop = FALSE]
  #
  #   vroom::vroom_write(.data.red, file = file.path(dir.result, paste("noise_", .data.red$file[1], sep = "")), delim = ",", progress = FALSE)
  #
  # }


  # Tree segmentation

  if(!is.null(segmentation)){

    # treeLAS <- suppressMessages(lidR::LAS(data[, c("x","y","z")]))

    voro <- sf::st_as_sf(data, coords = c("x", "y", "z"))
    voronoi <- sf::st_as_sf(.tree, coords = c("x", "y"))

    voronoi <- sf::st_collection_extract(
      sf::st_voronoi(do.call(c, sf::st_geometry(voronoi))))

    voro$tree <- unlist(sf::st_intersects(voro, voronoi))

    coords <- as.data.frame(sf::st_coordinates(voro))
    coords$tree <- voro$tree


    if(!is.null(plot))
      plotTree <- suppressMessages(lidR::plot(lidR::LAS(coords[, c("X", "Y", "Z", "tree")]), size = 0.5, color = "tree"))


    for (i in .tree$tree) {

      id <- .tree[.tree$tree == i, "id"]

      coords <- as.data.frame(sf::st_coordinates(voro[voro$tree == i, ]))
      colnames(coords) <- c("x", "y", "z")

      suppressMessages(lidR::writeLAS(lidR::LAS(coords[, c("x","y","z")]),
                                      paste(dir.result, "/tree", id, i, ".laz", sep = "")))

    }

  }


  # Diameters

  if(!is.null(plot)){

    diameter <- data.frame(tree = as.numeric(),
                           x = as.numeric(),
                           y = as.numeric(),
                           z = as.numeric())

    phi <- seq(from = 0, to = 2 * pi, by = 2 * pi / 10000)


    for (i in .tree$tree) {

      tree <- rep(i, times = 10001)
      x <- .tree$x[i] + cos(phi) * ((.tree$dbh[i] / 100) / 2)
      y <- .tree$y[i] + sin(phi) * ((.tree$dbh[i] / 100) / 2)
      z <- stats::runif(10001, 1.2, 1.4)

      .diameter <- data.frame(tree = tree, x = x, y = y, z = z)

      diameter <- rbind(diameter, .diameter)

    }

    suppressMessages(lidR::plot(lidR::LAS(diameter[, c("x","y","z")]), add = plotTree, size = 5))

  }



  #####

  return(.tree)

}

module test_fortd
    implicit none

contains

    subroutine feed_pets(cats, dogs, food, angry)
        !! Feeds your cats and dogs, if enough food is available. If not enough
        !! food is available, some of your pets will get angry.

        ! Arguments
        integer, intent(in)  :: cats
            !! The number of cats to keep track of.
        integer, intent(in)  :: dogs
            !! The number of dogs to keep track of.
        real, intent(inout)  :: food
            !! The amount of pet food (in kilograms) which you have on hand.
        integer, intent(out) :: angry
            !! The number of pets angry because they weren't fed.

        return
    end subroutine feed_pets
end module test_fortd

# Calculate overlap area between two circles
.circle_overlap_area <- function(x1, y1, r1, x2, y2, r2) {
  # Distance between centers
  d <- sqrt((x2 - x1)^2 + (y2 - y1)^2)

  # Case 1: No overlap
  if (d >= r1 + r2) return(0)

  # Case 2: One circle contains the other
  if (d <= abs(r1 - r2)) return(pi * min(r1, r2)^2)

  # Case 3: Partial overlap (lens intersection)
  angle1 <- 2 * acos((d^2 + r1^2 - r2^2) / (2 * d * r1))
  angle2 <- 2 * acos((d^2 + r2^2 - r1^2) / (2 * d * r2))

  area1 <- 0.5 * r1^2 * (angle1 - sin(angle1))
  area2 <- 0.5 * r2^2 * (angle2 - sin(angle2))

  return(area1 + area2)
}


# Calculate union area of two circles
.circle_union_area <- function(x1, y1, r1, x2, y2, r2) {
  area1 <- pi * r1^2
  area2 <- pi * r2^2
  overlap <- .circle_overlap_area(x1, y1, r1, x2, y2, r2)
  return(area1 + area2 - overlap)
}


# Helper Functions ----


# Calculate overlap ratio between two consecutive measurements
.calculate_overlap_ratio <- function(row1, row2) {
  overlap <- .circle_overlap_area(
    row1$x, row1$y, row1$dhi / 2,
    row2$x, row2$y, row2$dhi / 2
  )
  union <- .circle_union_area(
    row1$x, row1$y, row1$dhi / 2,
    row2$x, row2$y, row2$dhi / 2
  )
  return(overlap / union)
}


# Calculate combined relative RMSE for diameter and position models
.calculate_combined_rmse <- function(data) {
  fit_d <- lm(dhi ~ hi, data = data)
  fit_x <- lm(x ~ hi, data = data)
  fit_y <- lm(y ~ hi, data = data)

  rel_rmse_d <- summary(fit_d)$sigma / mean(predict(fit_d))
  rel_rmse_x <- summary(fit_x)$sigma / mean(predict(fit_x))
  rel_rmse_y <- summary(fit_y)$sigma / mean(predict(fit_y))

  # rel_rmse_d <- summary(fit_d)$sigma / mean(data$dhi, na.rm = TRUE)
  # rel_rmse_x <- summary(fit_x)$sigma / mean(data$x, na.rm = TRUE)
  # rel_rmse_y <- summary(fit_y)$sigma / mean(data$y, na.rm = TRUE)

  return(mean(c(rel_rmse_d, rel_rmse_x, rel_rmse_y), na.rm = TRUE))
}


# Predict diameter and position at given height
.predict_at_height <- function(data, target_height) {
  fit_d <- lm(dhi ~ hi, data = data)
  fit_x <- lm(x ~ hi, data = data)
  fit_y <- lm(y ~ hi, data = data)

  return(c(
    dhi = coef(fit_d)[1] + coef(fit_d)[2] * target_height,
    x = coef(fit_x)[1] + coef(fit_x)[2] * target_height,
    y = coef(fit_y)[1] + coef(fit_y)[2] * target_height
  ))
}


# Main Function ----

# Refine tree diameter measurements
#
# Three-step process:
# 1. Find the most reliable segment (lowest RMSE)
# 2. Correct measurements above the segment
# 3. Correct measurements below the segment
#
# @param data Data frame with: tree, hi (height), dhi (diameter), x, y
# @param window_size Number of consecutive points for fitting (default: 10)
# @param overlap_threshold Min overlap ratio for valid measurements (default: 0.75)
# @param make_plots Create diagnostic plots (default: TRUE)
# @return Corrected data frame

.refine_diameters <- function(data,
                              window_size = 10,
                              overlap_threshold = 0.75,
                              make_plots = TRUE) {

  # Validation
  required_cols <- c("tree", "hi", "dhi", "x", "y")
  if (!all(required_cols %in% names(data))) {
    stop("Data must contain: ", paste(required_cols, collapse = ", "))
  }
  if (nrow(data) < window_size) {
    warning("Insufficient data. Returning original.")
    return(data)
  }

  corrected <- data

  # Step 1: Find best segment
  if (make_plots) {
    plot(data$dhi ~ data$hi,
         main = paste("Tree:", data$tree[1]),
         xlab = "Height", ylab = "Diameter",
         pch = 1, col = "gray60")
  }

  n_windows <- nrow(data) - window_size + 1
  rmse_vals <- sapply(1:n_windows, function(i) {
    .calculate_combined_rmse(data[i:(i + window_size - 1), ])
  })

  best_idx <- which.min(rmse_vals)
  best_segment <- data[best_idx:(best_idx + window_size - 1), ]

  if (make_plots) {
    points(best_segment$dhi ~ best_segment$hi, pch = 19, col = "darkgreen")
  }

  # Step 2: Forward pass (upward correction)
  last_good <- best_idx + window_size - 1
  if (last_good < nrow(data)) {
    for (i in last_good:(nrow(data) - 1)) {
      ratio <- .calculate_overlap_ratio(corrected[i, ], corrected[i + 1, ])

      if (ratio < overlap_threshold) {
        start <- max(1, i - window_size + 1)
        pred <- .predict_at_height(corrected[start:i, ], corrected[i + 1, "hi"])
        corrected[i + 1, c("dhi", "x", "y")] <- pred
      }
    }
  }

  # Step 3: Backward pass (downward correction)
  if (best_idx > 1) {
    for (i in best_idx:2) {
      ratio <- .calculate_overlap_ratio(corrected[i - 1, ], corrected[i, ])

      if (ratio < overlap_threshold) {
        end <- min(nrow(data), i + window_size - 1)
        pred <- .predict_at_height(corrected[i:end, ], corrected[i - 1, "hi"])
        corrected[i - 1, c("dhi", "x", "y")] <- pred
      }
    }
  }

  if (make_plots) {
    points(corrected$dhi ~ corrected$hi, col = "blue", pch = 19, cex = 0.5)
    legend("topright",
           legend = c("Original", "Best segment", "Corrected"),
           pch = c(1, 19, 19),
           col = c("gray60", "darkgreen", "blue"))
  }

  return(corrected)
}





# setwd("C:/pruebas")
#
# tree <- read.csv("stem.curve.csv")
#
# # Example
# for (i in unique(tree$tree)) {
#
# data <- tree[tree$tree == i, ]
#
# kk <- .refine_diameters(data)
#
# }





.straightness <- function(data, stem.range = NULL){


  n <- data$tree[1]


  # Retaining sections within the stem range defined in the arguments

  if(!is.null(stem.range) & nrow(data) > 2){

    data <- data[data$hi >= stem.range[1] & data$hi <= stem.range[2], ]

  }


  # SS.max, sinuosity and lean are not estimated if there are less than 3 sections

  if(nrow(data) < 3){

    SS.max <- NA
    sinuosity <- NA
    lean <- NA

  } else {

    # Filtering outliers

    data$dif.sec <- c(abs(diff(data$hi)), 0)
    data$dif <- c(diff(data$dhi),
                  data$dhi[nrow(data)] - data$dhi[nrow(data) - 1])
    data$dif <- ifelse(data$dif.sec > 1, data$dif / data$dif.sec, data$dif)

    while (suppressWarnings(max(abs(data$dif))) > 10) {

      data$filter <- ifelse(data$dif > 10 | data$dif < -10, 0, 1)
      data <- data[data$filter == 1, ]
      if(nrow(data) < 2) {next}

      data$dif.sec <- c(abs(diff(data$hi)), 0)
      data$dif <- c(diff(data$dhi),
                    data$dhi[nrow(data)] - data$dhi[nrow(data) - 1])
      data$dif <- ifelse(data$dif.sec > 1, data$dif / data$dif.sec, data$dif)

    }


  # SS.max, sinuosity and lean are not estimated if there are less than 3 sections

  if(nrow(data) < 3){

  SS.max <- NA
  sinuosity <- NA
  lean <- NA

  out <- data.frame(tree = n, SS.max = SS.max, sinuosity = sinuosity, lean = lean)

  return(out)

  }


  mod.x <- lm(data$x[c(1,nrow(data))]~data$hi[c(1,nrow(data))])
  mod.y <- lm(data$y[c(1,nrow(data))]~data$hi[c(1,nrow(data))])

  data$sagita.x <- coef(mod.x)[1] + coef(mod.x)[2] * data$hi
  data$sagita.y <- coef(mod.y)[1] + coef(mod.y)[2] * data$hi
  data$sagita <- sqrt((data$sagita.x-data$x)^2+(data$sagita.y-data$y)^2)

  S.max <- max(data$sagita)

  # h.range <- max(data$hi)-min(data$hi)
  # h.range <- sqrt((data$x[1]-data$x[nrow(data)]) ^ 2 + (data$y[1]-data$y[nrow(data)]) ^ 2 + (data$hi[1]-data$hi[nrow(data)]) ^ 2)
  h.range <- sqrt((data$x[nrow(data)]-data$x[1]) ^ 2 + (data$y[nrow(data)]-data$y[1]) ^ 2 + (data$hi[nrow(data)]-data$hi[1]) ^ 2)

  R <- (S.max ^ 2 + (h.range / 2) ^ 2) / (2 * S.max)

  if((R ^ 2 - (max(data$hi) / 2) ^ 2) > 0){
    SS.max <- R - sqrt(R ^ 2 - (max(data$hi) / 2) ^ 2)
    } else {SS.max <- S.max}


  # Sinuosity

  data$x.2 <- c(data$x[2:nrow(data)], data$x[nrow(data)])
  data$y.2 <- c(data$y[2:nrow(data)], data$y[nrow(data)])
  data$hi.2 <- c(data$hi[2:nrow(data)], data$hi[nrow(data)])

  data$l <- sqrt((data$x.2-data$x) ^ 2 + (data$y.2-data$y) ^ 2 + (data$hi.2-data$hi) ^ 2)

  sinuosity <- sum(data$l) / h.range

  # Lean

  pca <- stats::princomp(data[, c("x", "y", "hi")])
  XY <- sqrt(pca$loadings[, 1][1] ^ 2 + pca$loadings[, 1][2] ^ 2)

  if(XY == 0){XY <- 2e-16}
  lean <- atan(abs(pca$loadings[, 1][3]) / XY) * (360 / (2 * pi))

  }

  out <- data.frame(tree = data$tree[1], SS.max = SS.max, sinuosity = sinuosity, lean = lean)

  return(out)

}



# Example
# for (i in unique(tree$tree)) {
#
#   data <- tree[tree$tree == i, ]
#
#   kk <- .refine_diameters(data)
#
#   kk1 <- .straightness(kk)
#   kk2 <- .straightness(data)
#
# }

program test_where
   implicit none
   ! Example variables
   real:: A(5),B(5),C(5)
   A = 0.0
   B = 1.0
   C = [0.0, 4.0, 5.0, 10.0, 0.0]

   ! Oneliner
   WHERE(B .GT. 0.0)  B = SUM(A, DIM=1)

   ! Simple where construct use
   where (C/=0)
      A=B/C
   elsewhere
      A=0.0
   end where

   ! Named where construct
   named: where (C/=0)
      A=B/C
   elsewhere
      A=0.0
   end where named
end program test_where

metrics.variables <- function(tree.tls, tree.ds = NULL, tree.field = NULL,
                              plot.design = c("fixed.area", "k.tree", "angle.count"),
                              plot.parameters = data.frame(radius = 25, k = 10, BAF = 2),
                              scan.approach = "multi",
                              var.metr = list(tls = NULL, field = NULL),
                              v.calc = "parab", dbh.min = 4, h.min = 1.3,
                              max.dist = Inf, dir.data = NULL,
                              save.result = TRUE, dir.result = NULL) {

  if(!is.null(tree.field) & !is.null(var.metr$tls) & !is.null(var.metr$field))
    var.metr = list(tls = var.metr$tls, field = var.metr$field)

  if(!is.null(tree.field) & !is.null(var.metr$tls) & is.null(var.metr$field))
    var.metr = list(tls = var.metr$tls, field = NULL)

  if(!is.null(tree.field) & !is.null(var.metr$field) & is.null(var.metr$tls))
    var.metr = list(tls = NULL, field = var.metr$field)

  # Call internal function
  metr <- .sim.calc(funct = "metr", tree.tls = tree.tls,
                    tree.ds = tree.ds, tree.field = tree.field,
                    plot.design = plot.design,
                    plot.parameters = plot.parameters,
                    scan.approach = scan.approach,
                    var.metr = var.metr, v.calc = v.calc,
                    dbh.min = dbh.min, h.min = h.min, max.dist = max.dist,
                    dir.data = dir.data, save.result = save.result,
                    dir.result = dir.result)

  return(metr)

}

MODULE test_generic
TYPE :: test_gen_type
CONTAINS
  GENERIC :: my_gen => gen1,gen2
  GENERIC :: ASSIGNMENT(=) => assign1, assign2
  GENERIC :: OPERATOR(+) => plusop1, plusop2
  GENERIC, PRIVATE :: my_gen2 => gen3, gen4
END TYPE test_gen_type
CONTAINS
!
SUBROUTINE gen1(self,a,b)
CLASS(test_gen_type) :: self
REAL(8), INTENT(IN) :: a
REAL(8), INTENT(OUT) :: b
CALL self%
END SUBROUTINE gen1
!
SUBROUTINE gen2(self,a,b,c)
CLASS(test_gen_type) :: self
REAL(8), INTENT(IN) :: a,c
REAL(8), INTENT(OUT) :: b
END SUBROUTINE gen2
!
SUBROUTINE assign1(outvar,invar)
REAL(8) :: outvar
CLASS(test_gen_type) :: invar
END SUBROUTINE assign1
!
SUBROUTINE assign2(outvar,invar)
LOGICAL :: outvar
CLASS(test_gen_type) :: invar
END SUBROUTINE assign2
!
REAL(8) FUNCTION plusop1(var1,var2)
REAL(8) :: var1
CLASS(test_gen_type) :: var2
END FUNCTION plusop1
!
LOGICAL FUNCTION plusop2(var1,var2)
LOGICAL :: var1
CLASS(test_gen_type) :: var2
END FUNCTION plusop2
!
SUBROUTINE gen3(self,a,b)
CLASS(test_gen_type) :: self
REAL(8), INTENT(IN) :: a
REAL(8), INTENT(OUT) :: b
CALL self%
END SUBROUTINE gen3
!
SUBROUTINE gen4(self,a,b,c)
CLASS(test_gen_type) :: self
REAL(8), INTENT(IN) :: a,c
REAL(8), INTENT(OUT) :: b
END SUBROUTINE gen4
END MODULE test_generic

correlations <- function(simulations,
                         variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
                         method = c("pearson", "spearman"), save.result = TRUE,
                         dir.result = NULL) {

  # Checking if id columns are characters or factors, and converting to numeric in that cases
  if(!is.null(simulations$fixed.area) & is.character(simulations$fixed.area$id) | is.factor(simulations$fixed.area$id))
    simulations$fixed.area$id <- as.numeric(as.factor(simulations$fixed.area$id))

  if(!is.null(simulations$k.tree) & is.character(simulations$k.tree$id) | is.factor(simulations$k.tree$id))
    simulations$k.tree$id <- as.numeric(as.factor(simulations$k.tree$id))

  if(!is.null(simulations$angle.count) & is.character(simulations$angle.count$id) | is.factor(simulations$angle.count$id))
    simulations$angle.count$id <- as.numeric(as.factor(simulations$angle.count$id))


  # Define a character vector containing index name (radius, k or BAF) for each
  # available plot design
  .plot.design <- c(fixed.area = "radius", k.tree = "k", angle.count = "BAF")

  # Define character vectors containing the implemented field variables and TLS
  # metrics
  .field.names <- c(
    # Density (trees/ha), basal area (m2/ha) and volume (m3/ha)
    "N", "G", "V",

    # Volume (m3/ha) and biomass (Mg/ha) provided by user
    # if("W" %in% colnames(simulations$fixed.area))
    "V.user", "W.user",

    # Mean diameters (cm), and mean dominant diameters (cm)
    "d", "dg", "dgeom", "dharm",
    paste(c("d", "dg", "dgeom", "dharm"), "0", sep = "."),

    # Mean heights (m), and mean dominant heights (m)
    "h", "hg", "hgeom", "hharm",
    paste(c("h", "hg", "hgeom", "hharm"), "0", sep = "."))
  .tls.names <- c(
    # Density (trees/ha)
    "N.tls", "N.hn", "N.hr", "N.hn.cov", "N.hr.cov", "N.sh",
    "N.pam",

    # Number of points
    "n.pts", "n.pts.est", "n.pts.red", "n.pts.red.est",

    # Basal area (m2/ha)
    "G.tls", "G.hn", "G.hr", "G.hn.cov", "G.hr.cov", "G.sh",
    "G.pam",

    # Volume (m3/ha)
    "V.tls", "V.hn", "V.hr", "V.hn.cov", "V.hr.cov", "V.sh",
    "V.pam",

    # Mean diameters (cm), and mean dominant diameters (cm)
    paste(c("d", "dg", "dgeom", "dharm"), "tls", sep = "."),
    paste(c("d", "dg", "dgeom", "dharm"), "0.tls", sep = "."),

    # Mean heights (m), and mean dominant heights (m)
    paste(c("h", "hg", "hgeom", "hharm"), "tls", sep = "."),
    paste(c("h", "hg", "hgeom", "hharm"), "0.tls", sep = "."),

    # Height percentiles (m)
    sprintf("P%02i", c(1, 5, 10, 20, 25, 30, 40, 50, 60, 70, 75,
                       80, 90, 95, 99)),

    # Points metrics
    # Z coordinate
    "mean.z", "mean.q.z", "mean.g.z", "mean.h.z", "median.z", "mode.z",
    "max.z", "min.z", "var.z", "sd.z", "CV.z", "D.z", "ID.z",
    "kurtosis.z", "skewness.z",
    "p.a.mean.z", "p.a.mode.z", "p.a.2m.z",
    "p.b.mean.z", "p.b.mode.z", "p.b.2m.z", "CRR.z",
    "L2.z", "L3.z", "L4.z", "L3.mu.z", "L4.mu.z",
    "L.CV.z",
    "median.a.d.z", "mode.a.d.z",
    "weibull_c.z", "weibull_b.z",

    # Rho coordinate
    "mean.rho", "mean.q.rho", "mean.g.rho", "mean.h.rho", "median.rho", "mode.rho",
    "max.rho", "min.rho", "var.rho", "sd.rho", "CV.rho", "D.rho", "ID.rho",
    "kurtosis.rho", "skewness.rho",
    "p.a.mean.rho", "p.a.mode.rho",
    "p.b.mean.rho", "p.b.mode.rho", "CRR.rho",
    "L2.rho", "L3.rho", "L4.rho", "L3.mu.rho", "L4.mu.rho",
    "L.CV.rho",
    "median.a.d.rho", "mode.a.d.rho",
    "weibull_c.rho", "weibull_b.rho",

    # R coordinate
    "mean.r", "mean.q.r", "mean.g.r", "mean.h.r", "median.r", "mode.r",
    "max.r", "min.r", "var.r", "sd.r", "CV.r", "D.r", "ID.r",
    "kurtosis.r", "skewness.r",
    "p.a.mean.r", "p.a.mode.r",
    "p.b.mean.r", "p.b.mode.r", "CRR.r",
    "L2.r", "L3.r", "L4.r", "L3.mu.r", "L4.mu.r",
    "L.CV.r",
    "median.a.d.r", "mode.a.d.r",
    "weibull_c.r", "weibull_b.r")

  # Define a character vector containing the available correlation measurements
  .cor.method <- c("pearson", "spearman")


  # Check arguments


  # 'simulations' must be a list with all or any of the preset elements, and
  # being at least one of them no NULL
  if (!is.list(simulations)) stop("'simulations' must be a list")
  if (is.null(simulations) || all(!names(.plot.design) %in% names(simulations)))
    stop("'simulations' must have at least one of the following elements:",
         "'fixed.area', 'k.tree' or 'angle.count'")
  if (any(!names(simulations) %in% names(.plot.design))) {

    simulations <- simulations[names(simulations) %in% names(.plot.design)]
    warning("There is any element in 'simulations' which do not correspond ",
            "with preset ones. It was not taken into account during the ",
            "execution")

  }
  simulations <- simulations[!sapply(simulations, is.null)]
  if (length(simulations) == 0)
    stop("'simulations' must have at least one of the following elements ",
         "different from 'NULL': 'fixed.area', 'k.tree' or 'angle.count'")
  for (.i in names(simulations)) {

    # All elements in 'simulations' must be data frames with at least one row,
    # certain mandatory columns and most of them numeric
    if (!is.data.frame(simulations[[.i]]))
      stop("'simulations$", .i, "' must be a data.frame")
    if (nrow(simulations[[.i]]) < 1)
      stop("'simulations$", .i, "' must have at least one row")
    if (!"id" %in% colnames(simulations[[.i]]))
      stop("'simulations$", .i, "' must have a column named 'id'")
    if (!.plot.design[.i] %in% colnames(simulations[[.i]]))
      stop("'simulations$", .i, "' must have a column named '",
           .plot.design[.i], "'")
    if (all(!.field.names %in% colnames(simulations[[.i]])))
      stop("'simulations$", .i, "' must have at least one field estimations ",
           "column")
    if (all(!.tls.names %in% colnames(simulations[[.i]])))
      stop("'simulations$", .i, "' must have at least one TLS metrics column")
    for (.j in colnames(simulations[[.i]])) {

      if (.j != "id" && !is.numeric(simulations[[.i]][, .j]))
        stop("Column '", .j,"' of 'simulations$", .i, "' must be numeric")

    }

  }

  # 'variables' must be c("N", "G", "V", "d", "dg", "d.0", "h", "h.0") (by
  # default) or a character vector with all or any of the preset estimations of
  # variables based on field data. Besides, simulations elements must have at
  # least the columns corresponding to 'variables'
  if (!is.character(variables) || !is.vector(variables))
    stop("'variables' must be a character vector")
  if (length(variables) == 0 || all(!variables %in% .field.names))
    stop("'variables' must have at least one of the following character ",
         "strings: ", paste("'", .field.names, "'", sep = "", collapse = ", "))
  if (any(!variables %in% .field.names)) {

    variables <- variables[variables %in% .field.names]
    warning("There is any character string in 'variables' which do not ",
            "correspond with preset ones. It was not taken into account ",
            "during the execution")

  }
  for (.i in names(simulations)) {

    if (any(!variables %in% colnames(simulations[[.i]])))
      stop("Any column corresponding to 'variables' is missing in ",
           "'simulations$", .i, "'")

  }

  # 'method' must be c("pearson", "spearman") (by default) or a character vector
  # with all or any of the preset methods
  if (!is.character(method) || !is.vector(method))
    stop("'method' must be a character vector")
  if (length(method) == 0 || all(!method %in% .cor.method))
    stop("'method' must have at least one of the following character strings: ",
         paste("'", .cor.method, "'", sep = "", collapse = ", "))
  if (any(!method %in% .cor.method)) {

    method <- method[method %in% .cor.method]
    warning("There is any character string in 'method' which do not ",
            "correspond with preset ones. It was not taken into account ",
            "during the execution")

  }

  # 'save.result' must be TRUE (by default) or a logical
  if (!is.logical(save.result)) stop("'save.result' must be a logical")
  if (length(save.result) > 1) {

    save.result <- save.result[1]
    warning("Only first value in 'save.result' was taken into account during ",
            "the execution")

  }

  # If 'save.result' is TRUE, 'dir.result' must be NULL (by default) or a
  # character string containing the absolute path to a existing directory
  if (save.result) {

    if (!is.null(dir.result)) {

      if (!is.character(dir.result))
        stop("'dir.result' must be a character string")
      if (length(dir.result) > 1) {

        dir.result <- dir.result[1]
        warning("Only first value in 'dir.result' was taken into account ",
                "during the execution")

      }
      if (!dir.exists(dir.result)) stop("'dir.result' directory must exist")
    } else {

      # Define working directory as directory by default for saving files
      dir.result <- getwd()

    }

  }


  # Define values for certain auxiliary objects, and create empty lists where
  # results will be saved


  # Restrict field variables to them specified in 'variables' argument
  .field.names <- .field.names[.field.names %in% variables]

  # Define a character vector containing color palette for correlations plots,
  # if 'save.result' is TRUE
  if (save.result) {

    .tls.color <- substr(.tls.names, 1, 1)
    names(.tls.color) <- .tls.names
    # Density
    .tls.color[.tls.color == "N"] <-
      grDevices::hcl.colors(sum(.tls.color == "N") + 1,
                            "Reds")[-(sum(.tls.color == "N") + 1)]
    # Number of points
    .tls.color[.tls.color == "n"] <-
      grDevices::hcl.colors(sum(.tls.color == "n") + 1,
                            "Greens")[-(sum(.tls.color == "n") + 1)]
    # Basal area
    .tls.color[.tls.color == "G"] <-
      grDevices::hcl.colors(sum(.tls.color == "G") + 1,
                            "Blues")[-(sum(.tls.color == "G") + 1)]
    # Volume
    .tls.color[.tls.color == "V"] <-
      grDevices::hcl.colors(sum(.tls.color == "V") + 1,
                            "Oranges")[-(sum(.tls.color == "V") + 1)]
    # Diameters
    .tls.color[.tls.color == "d"] <-
      grDevices::hcl.colors(sum(.tls.color == "d") + 1,
                            "Purples")[-(sum(.tls.color == "d") + 1)]
    # Heights
    .tls.color[.tls.color == "h"] <-
      grDevices::hcl.colors(sum(.tls.color == "h"), "Magenta")

    # Percentiles
    .tls.color[.tls.color == "P"] <-
      grDevices::gray.colors(sum(.tls.color == "P"))

    # Means
    .tls.color[.tls.color == "m"] <-
      grDevices::hcl.colors(sum(.tls.color == "m"), "Reds2")

    # Percentages
    .tls.color[.tls.color == "p"] <-
      grDevices::hcl.colors(sum(.tls.color == "p"), "Greens2")

    # Weibull
    .tls.color[.tls.color == "w"] <-
      grDevices::hcl.colors(sum(.tls.color == "w"), "Blues2")


  }

  # Define character vectors containing names for correlation values and their
  # associated p-values, and names for optimal correlation values and their
  # corresponding TLS metric
  .cor.names <- c("estimate", "p.value")
  .opt.cor.names <- c("cor", "metric")

  # Define empty lists where correlations, p-values, and optimal correlations
  # and corresponding metrics will be saved
  correlations <- vector("list", length(method))
  names(correlations) <- method
  correlations.pval <-correlations
  opt.correlations <- correlations
  for (.i in names(correlations)) {

    correlations[[.i]] <- vector("list", length(simulations))
    names(correlations[[.i]]) <- names(simulations)
    correlations.pval[[.i]] <- correlations[[.i]]
    opt.correlations[[.i]] <- correlations[[.i]]

  }


  # Loop for each plot design in 'simulations' argument
  for (.i in names(simulations)) {

    # Define initial time, and print message
    t0 <- Sys.time()
    message("Computing correlations for ",
            switch(.i, fixed.area = "fixed area ", k.tree = "k-tree ",
                   angle.count = "angle-count "), "plots")


    # Rearrange simulated data ----


    # Convert data.frame containing simulated data into a matrix
    simulations[[.i]] <- as.matrix(simulations[[.i]])

    # Create an empty numeric 3-dimensional array where simulated data will be
    # rearranged
    .index.val <- sort(unique(simulations[[.i]][, .plot.design[.i]]))
    .index.dec <- .decimals(.index.val)
    names(.index.val) <- .format.numb(x = .index.val, dec = .index.dec)
    .col.names <- c(.field.names,
                    .tls.names[.tls.names %in% colnames(simulations[[.i]])])
    .id.val <- sort(unique(simulations[[.i]][, "id"]))
    names(.id.val) <- as.character(.id.val)
    .data <- array(numeric(), dim = c(length(.index.val), length(.col.names),
                                      length(.id.val)),
                   dimnames = list(names(.index.val), .col.names,
                                   names(.id.val)))

    # Rearrange simulated data into the previously created array
    for (.j in names(.id.val)) {

      .row.names <- simulations[[.i]][simulations[[.i]][, "id"] == .id.val[.j],
                                      .plot.design[.i]]
      .row.names <- .format.numb(x = .row.names, dec = .index.dec)
      .data[.row.names, , .j] <-
        simulations[[.i]][simulations[[.i]][, "id"] == .id.val[.j],
                          dimnames(.data)[[2]]]

    }


    # Compute correlations, and optimal correlations ----


    # Create a character matrix containing all the pairs of field variable and
    # TLS metric to be considered for correlation calculations
    .cor.pairs <- .tls.names[.tls.names %in% dimnames(.data)[[2]]]
    .cor.pairs <- cbind(field = rep(.field.names, each = length(.cor.pairs)),
                        tls = rep(.cor.pairs, length(.field.names)))
    rownames(.cor.pairs) <- apply(.cor.pairs, 1, paste, collapse = ".")


    # Loop for each method
    for (.j in method){


      # Compute correlations and associated p-values for each radius, k or BAF


      # Compute correlations
      .cor.data <-
        sapply(dimnames(.data)[[1]],
               function(index, data, cor.pairs, method, cor.names){
                 # Compute correlations for each pair
                 cor.data <-
                   sapply(1:nrow(cor.pairs),
                          function(nr, data, cor.pairs, method, cor.names) {
                            # Select simulated data
                            data <- matrix(data[cor.pairs[nr, ], ],
                                           nrow = ncol(cor.pairs),
                                           ncol = dim(data)[2],
                                           dimnames = list(cor.pairs[nr,],
                                                           dimnames(data)[[2]]))
                            # Select plots with non NA values
                            data <- data[, apply(!is.na(data), 2, all),
                                         drop = FALSE]
                            # Compute correlations only if number of plots is
                            # greater than 2 and data have non zero standard
                            # deviation
                            if (ncol(data) > 2 &
                                all(apply(data, 1, stats::sd) != 0)) {
                              # Compute correlations for each method
                              cor.data <- get(paste(".cor", method, sep ="."))(
                                x = data[cor.pairs[nr, "field"], ],
                                y = data[cor.pairs[nr, "tls"], ])
                              cor.data <- unlist(cor.data[cor.names])
                            } else {
                              cor.data <- rep(NA, length(cor.names))
                              names(cor.data) <- cor.names
                            }
                            return(cor.data)
                          },
                          data = matrix(data[index, , ], nrow = dim(data)[[2]],
                                        ncol = dim(data)[[3]],
                                        dimnames = dimnames(data)[2:3]),
                          cor.pairs = cor.pairs, method = method,
                          cor.names = cor.names)
                 return(cor.data)
               },
               data = .data, cor.pairs = .cor.pairs, method = .j,
               cor.names = .cor.names)

      # Rearrange correlations results into a numeric 3-dimensional array
      .cor.data <- array(.cor.data,
                         dim = c(length(.cor.names), nrow(.cor.pairs),
                                 dim(.data)[1]),
                         dimnames = list(.cor.names, rownames(.cor.pairs),
                                         dimnames(.data)[[1]]))
      .cor.data <- aperm(.cor.data, 3:1)

      # Remove results corresponding to first radius, k or BAF values if all
      # correlations are NA values
      .any.nona <- which(apply(!is.na(.cor.data), 1, any))
      if (length(.any.nona) == 0) {

        warning("All the computed ",
                switch(.j, pearson = "Pearson correlations",
                       spearman = "Spearman correlations"),
                " for ",
                switch(.i, fixed.area = "fixed area",
                       k.tree = "k-tree",
                       angle.count = "angle-count"),
                " plot design case are 'NA' values")
      } else {

        .cor.data <- .cor.data[min(.any.nona):dim(.cor.data)[1], , ,
                               drop = FALSE]

      }

      # Save index and correlations values in results list, and write a csv file
      # containing them if 'save.result' is TRUE
      correlations[[.j]][[.i]] <-
        matrix(c(.index.val[dimnames(.cor.data)[[1]]],
                 .cor.data[, , .cor.names[1]]),
               nrow = dim(.cor.data)[1], ncol = 1 + dim(.cor.data)[2],
               dimnames = list(NULL,
                               c(.plot.design[.i], dimnames(.cor.data)[[2]])))
      if (save.result)
        utils::write.csv(correlations[[.j]][[.i]],
                         file = file.path(dir.result,
                                          paste("correlations", .i, .j, "csv",
                                                sep =".")),
                         row.names = FALSE)

      # Save index and correlations p-values in results lists
      correlations.pval[[.j]][[.i]] <-
        matrix(c(.index.val[dimnames(.cor.data)[[1]]],
                 .cor.data[, , .cor.names[2]]),
               nrow = dim(.cor.data)[1], ncol = 1 + dim(.cor.data)[2],
               dimnames = list(NULL,
                               c(.plot.design[.i], dimnames(.cor.data)[[2]])))


      # Compute optimal correlations for each radius, k or BAF


      # Create an empty data.frame where optimal correlations for each field
      # variable, and the name of the corresponding TLS metric, will be saved
      # for each radius, k or BAF
      .col.names <- as.vector(t(outer(.field.names, .opt.cor.names, paste,
                                      sep = ".")))
      .opt.cor.data <- matrix(NA, nrow = dim(.cor.data)[1],
                              ncol = length(.col.names),
                              dimnames = list(dimnames(.cor.data)[[1]],
                                              .col.names))
      .opt.cor.data <- data.frame(.opt.cor.data, stringsAsFactors = FALSE)


      # Loop for each field variable
      for (.k in .field.names) {

        # Select pairs corresponding to the field variable
        .col.names <- rownames(.cor.pairs)[.cor.pairs[, "field"] == .k]
        .cor.data.k <- matrix(.cor.data[, .col.names, "estimate"],
                              nrow = dim(.cor.data)[1],
                              ncol = length(.col.names),
                              dimnames = list(dimnames(.cor.data)[[1]],
                                              .col.names))

        # Remove results corresponding to radius, k or BAF values such that all
        # correlations are NA values
        .cor.data.k <- .cor.data.k[apply(!is.na(.cor.data.k), 1, any), ,
                                   drop = FALSE]

        # Obtain and save optimal correlations and corresponding TLS metric
        if (nrow(.cor.data.k) > 0) {

          # Obtain optimal correlations and the corresponding TLS metric
          .opt.cor.data.k <-
            lapply(1:nrow(.cor.data.k),
                   function(nr, data, names) {
                     pos <- colnames(data)[which.max(abs(data[nr, ]))]
                     val <- data[nr, pos]
                     opt.cor <- data.frame(val = val, pos = pos,
                                           row.names = rownames(data)[nr],
                                           stringsAsFactors = FALSE)
                     return(opt.cor)
                   },
                   data = .cor.data.k, names = .opt.cor.names)
          .opt.cor.data.k <- do.call(rbind, .opt.cor.data.k)
          .opt.cor.data.k[, "pos"] <- .cor.pairs[.opt.cor.data.k[, "pos"],
                                                 "tls"]
          colnames(.opt.cor.data.k) <- paste(.k, .opt.cor.names, sep = ".")

          # Save optimal correlations and the corresponding TLS metric
          .opt.cor.data[rownames(.opt.cor.data.k), colnames(.opt.cor.data.k)] <-
            .opt.cor.data.k

        }


        # Plot correlations related to the field variable, if 'save.result' is
        # TRUE


        if (save.result) {

          # Convert correlations matrix to data.frame, and add index values
          .cor.data.k <- data.frame(.index.val[rownames(.cor.data.k)],
                                    .cor.data.k, stringsAsFactors = FALSE)
          colnames(.cor.data.k)[1] <- .plot.design[.i]

          # Define title, subtitle, and axis names, and create empty plot
          .title <-
            switch(.k, N = "Density (N, trees/ha)",
                   G = "Basal area (G, m<sup>2</sup>/ha)",
                   V = "Volume (V, m<sup>3</sup>/ha)",
                   V.user = "Volume (V, m<sup>3</sup>/ha) provided by user",
                   W.user = "Biomass (W, Mg/ha) provided by user",
                   d = "Arithmetic mean diameter (d, cm)",
                   dg = "Quadratic mean diameter (dg, cm)",
                   dgeom = "Geometric mean diameter (dgeom, cm)",
                   dharm = "Harmonic mean diameter (dharm, cm)",
                   d.0 = "Arithmetic mean dominant diameter (d.0, cm)",
                   dg.0 = "Quadratic mean dominant diameter (dg.0, cm)",
                   dgeom.0 = "Geometric mean dominant diameter (dgeom.0, cm)",
                   dharm.0 = "Harmonic mean dominant diameter (dharm.0, cm)",
                   h = "Arithmetic mean height (h, m)",
                   hg = "Quadratic mean height (hg, m)",
                   hgeom = "Geometric mean height (hgeom, m)",
                   hharm = "Harmonic mean height (hharm, m)",
                   h.0 = "Arithmetic mean dominant height (h.0, m)",
                   hg.0 = "Quadratic mean dominant height (hg.0, m)",
                   hgeom.0 = "Geometric mean dominant height (hgeom.0, m)",
                   hharm.0 = "Harmonic mean dominant height (hharm.0, m)")
          .subtitle <- paste("<br> <span style='font-size: 20px;'>",
                             switch(.i, fixed.area = "Circular fixed area plot",
                                    k.tree = "K-tree plot",
                                    angle.count = "Angle-count plot"),
                             "</span>", sep ="")
          .xaxis <- switch(.i, fixed.area = "Radius (m)",
                           k.tree = "K-tree (trees)",
                           angle.count = "BAF (m<sup>2</sup>/ha)")
          .yaxis <- switch(.j, pearson = "Pearson correlation",
                           spearman = "Spearman correlation")
          fig <-
            plotly::plot_ly(.cor.data.k, type = 'scatter', mode = 'lines') %>%
            plotly::layout(title = paste(.title, .subtitle, sep = ""), font = list(size = 25),
                           xaxis = list(title = .xaxis),
                           yaxis = list (title = .yaxis),
                           margin = list(t = 100))

          # Add traces
          for (.l in colnames(.cor.data.k)[colnames(.cor.data.k) !=
                                           .plot.design[.i]]) {

            .color <- .tls.color[.cor.pairs[.l, "tls"]]
            fig <- fig %>%
              plotly::add_trace(x = .cor.data.k[, .plot.design[.i]],
                                y = .cor.data.k[, .l],
                                name = .cor.pairs[.l, "tls"],
                                line = list(color = .color, width = 2,
                                            dash = 'dot'))

          }

          # Save correlations plot related to the field variable
          suppressWarnings(
            htmlwidgets::saveWidget(widget = fig,
                                    file = file.path(dir.result,
                                                     paste("correlations", .k,
                                                           .i, .j, "html",
                                                           sep = ".")),
                                    selfcontained = TRUE,
                                    libdir = file.path(dir.result,
                                                       "correlations_files")))

        }

      }

      # Save index, optimal correlations, and names of the corresponding TLS
      # metrics in results list, and write csv file containing them
      opt.correlations[[.j]][[.i]] <- cbind(.index.val[rownames(.opt.cor.data)],
                                            .opt.cor.data)
      rownames(opt.correlations[[.j]][[.i]]) <- NULL
      colnames(opt.correlations[[.j]][[.i]])[1] <- .plot.design[.i]
      if (save.result)
        utils::write.csv(opt.correlations[[.j]][[.i]],
                         file = file.path(dir.result, paste("opt.correlations",
                                                            .i, .j, "csv",
                                                            sep = ".")),
                         row.names = FALSE)

    }

    # Define final time, and print message
    t1 <- Sys.time()
    message(" (", format(round(difftime(t1, t0, units = "secs"), 2)), ")")

  }


  return(list(correlations = correlations,
              correlations.pval =  correlations.pval,
              opt.correlations = opt.correlations))

}

program preprocessor

#include "petscpc.h"
#ifdef PETSCPCDEF_H
    integer, parameter :: var = 1000
    PCType :: tmp
    print*, 999, 3.14, "some", var, PETSC_ERR_MEM
    print*, PETSC_ERR_INT_OVERFLOW, varVar
    ewrite(1,*) 'Assemble EP P1 matrix and rhs sytem'
    ewrite2(1,*) 'Assemble EP P1 matrix and rhs sytem'
    print*, SUCCESS

#endif
end program preprocessor

# Downsampling

voxel_grid_downsampling <- function(points, voxel_size) {

  reticulate::source_python(system.file("python", "voxel_grid_downsampling.py", package = "FORTLS"))

  points <- reticulate::np_array(points)

  reticulate::py$voxel_grid_downsampling(points, voxel_size)

}



# Geometric features

geometric_features_py <- function(data, dist = 0.1, geom_features_input = NULL, threads = 1) {

  reticulate::source_python(system.file("python", "geometric_features.py", package = "FORTLS"))

  if(is.null(geom_features_input)){
    geom_features_input <- c("verticality", "surface_variation", "planarity")}

  reticulate::py$compute_geom_features(input_df = data,
                                       dist = dist,
                                       features = geom_features_input,
                                       threads = as.integer(threads))

}

tree.detection.several.plots <- function(las.list, id.list = NULL, file = NULL,

                                         scan.approach = "multi",

                                         pcd.red = NULL, normalized = NULL,
                                         center.coord = NULL,
                                         x.side = NULL, y.side = NULL,
                                         max.dist = NULL, min.height = NULL, max.height = 50,
                                         algorithm.dtm = "knnidw", res.dtm = 0.2, csf = list(cloth_resolution = 0.5),
                                         intensity = NULL, RGB = NULL,
                                         voxel_size = NULL,

                                         single.tree = NULL,
                                         dbh.min = 4, dbh.max = 200, h.min = 1.3,
                                         geo.dist = 0.1,
                                         tls.resolution = NULL, tls.precision = NULL,
                                         stem.section = c(0.7, 3.5), stem.range = NULL, breaks = NULL,
                                         slice = 0.1, understory = NULL, bark.roughness = 1,
                                         den.type = 1, d.mer = NULL,
                                         plot.attributes = NULL, plot = NULL,

                                         threads = 1,

                                         dir.data = NULL, save.result = TRUE, dir.result = NULL,
                                         save.las = NULL){


  # Obtaining working directory for loading files
  if(is.null(dir.data))
    dir.data <- getwd()

  # Obtaining working directory for saving files
  if(is.null(dir.result))
    dir.result <- getwd()


  for (i in 1:length(las.list)) {

    # Assign id

    if(!is.null(id.list)){

      .id <- id.list[i]

    } else {

      .id <- as.integer(i)

    }

    # Assign coordinates

    if(!is.null(center.coord) & !is.null(center.coord$id)){

      x.center <- center.coord[center.coord$id == .id, "x"]
      y.center <- center.coord[center.coord$id == .id, "y"]

    } else if(!is.null(center.coord)) {

      x.center <- center.coord$x[i]
      y.center <- center.coord$y[i]

    } else {

      x.center = NULL
      y.center = NULL

    }



    # Messages

    message("Computing plot: ", .id)

    message("Normalizing")

    # File name

    if(!is.null(file)){

      .file <- paste(file[i], ".txt", sep = "")

    } else {

      .file <- paste(.id, ".txt", sep = "")

    }

    .data <- normalize(las = las.list[i], normalized = normalized,

                       x.center = x.center, y.center = y.center,

                       max.dist = max.dist, min.height = min.height, max.height = max.height,

                       algorithm.dtm = algorithm.dtm, res.dtm = res.dtm, csf = csf,

                       intensity = intensity, RGB = RGB,

                       voxel_size = voxel_size,

                       threads = threads,

                       scan.approach = scan.approach,

                       id = .id, file = .file, plot = NULL,

                       dir.data = dir.data, save.result = save.result, dir.result = dir.result,

                       save.las = save.las)

    if(!is.null(pcd.red))
      .data <- .data[.data$prob.selec == 1, ]

    message("Detecting trees")

    if(scan.approach == "single"){

    if(i > 1)
      .stem.curve <- read.csv(paste(dir.data, "/stem.curve.csv", sep = ""))

    .tree.tls.i <- tree.detection.single.scan(data = .data, single.tree = single.tree,

                                              dbh.min = dbh.min, dbh.max = dbh.max, h.min = h.min,

                                              geo.dist = geo.dist,

                                              tls.resolution = tls.resolution,

                                              stem.section = stem.section, stem.range = stem.range, breaks = breaks,

                                              slice = slice, understory = understory, bark.roughness = bark.roughness,

                                              den.type = den.type, d.mer = d.mer,

                                              plot.attributes = plot.attributes, plot = plot,

                                              threads = threads,

                                              save.result = FALSE, dir.result = dir.result)

    rm(.data)

    }

    if(scan.approach == "multi"){

    if(i > 1)
      .stem.curve <- read.csv(paste(dir.data, "/stem.curve.csv", sep = ""))

    .tree.tls.i <- tree.detection.multi.scan(data = .data, single.tree = single.tree,

                                             dbh.min = dbh.min, dbh.max = dbh.max, h.min = h.min,

                                             geo.dist = geo.dist,

                                             tls.precision = tls.precision,

                                             stem.section = stem.section, stem.range = stem.range, breaks = breaks,

                                             slice = slice, understory = understory, bark.roughness = bark.roughness,

                                             den.type = den.type, d.mer = d.mer,

                                             plot.attributes = plot.attributes, plot = plot,

                                             threads = threads,

                                             save.result = FALSE, dir.result = dir.result)

    rm(.data)

    }

    if(i < 2){

      .tree.tls <- .tree.tls.i

    } else {

      .tree.tls <- rbind(.tree.tls, .tree.tls.i)

      .stem.curve <- rbind(.stem.curve, read.csv(paste(dir.data, "/stem.curve.csv", sep = "")))

      utils::write.csv(.stem.curve,
                       file = file.path(dir.result, "stem.curve.csv"),
                       row.names = FALSE)

    }

    if(isTRUE(save.result)){

      utils::write.csv(.tree.tls,
                       file = file.path(dir.result, "tree.tls.csv"),
                       row.names = FALSE)

    }


  }


  #####
  return(.tree.tls)

}

program test_forall
   implicit none
   integer :: i, j, dim=3, a(10) = 2

   select case (dim)
    case(3)
      forall(i=1:10)
         a(i) = a(i) **2
         forall (j=1:i) a(j) = a(j) ** 2
      end forall
    case default
      call abort()
   end select
end program test_forall

program test_contains
    implicit none
    contains
    contains
end program test_contains
contains

module test_contains2
    subroutine foo()    ! Err: before contains
    end subroutine
    contains
end module test_contains2

module test_nonint_mod
   private
   integer, parameter, public :: DP = kind(0.0D0)
end module test_nonint_mod

program nonint
   use, non_intrinsic :: test_nonint_mod, only : DP
   implicit none
   real(DP) :: x
   x = 0.0_DP
end program nonint

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----include=FALSE------------------------------------------------------------
dir.data <- getwd()
download.file("https://www.dropbox.com/s/17yl25pbrapat52/PinusRadiata.laz?dl=1",
              destfile = file.path(dir.data, "PinusRadiata.laz"),
              mode = "wb")
download.file("https://www.dropbox.com/scl/fi/es5pfj87wj0g6y8414dpo/PiceaAbies.laz?rlkey=ayt21mbndc6i6fyiz2e7z6oap&dl=1",
              destfile = file.path(dir.data, "PiceaAbies.laz"),
              mode = "wb")
library(FORTLS)

## ----include=FALSE------------------------------------------------------------
singleLAS <- lidR::readLAS(paste(dir.data, "PinusRadiata.laz", sep = "/"))
lidR::plot(singleLAS, color = "RGB")

## ----eval = TRUE, include = TRUE----------------------------------------------
pcd.single <- normalize(las = "PinusRadiata.laz",
                        id = "PinusRadiata",
                        x.center = 0, y.center = 0,
                        max.dist = 10, 
                        scan.approach = "single",
                        threads = 2)

## ----eval = FALSE, include=FALSE----------------------------------------------
# head(pcd.single)

## ----echo=FALSE---------------------------------------------------------------
kableExtra::scroll_box(kable_input = kableExtra::kable(head(pcd.single), format = "html"), 
                       width = "100%")

## ----eval = FALSE, warning=FALSE, include=TRUE--------------------------------
# tls.resolution = list(point.dist = 6.34, tls.dist = 10)
# 
# tree.list.single.tls <- tree.detection.single.scan(data = pcd.single,
#                                                    tls.resolution = tls.resolution,
#                                                    threads = 2)

## ----eval = FALSE, include=FALSE----------------------------------------------
# head(tree.list.single.tls)

## ----echo=FALSE---------------------------------------------------------------
tree.list.single.tls <- read.csv(paste(dir.data, "tree.list.single.tls.csv", sep = "/"))
kableExtra::scroll_box(kable_input = kableExtra::kable(head(tree.list.single.tls), 
                                                       format = "html"), width = "100%")

## ----eval = FALSE, warning=FALSE, include=TRUE--------------------------------
# pcd.multi <- normalize(las = "PiceaAbies.laz",
#                        id = "PiceaAbies",
#                        x.center = 0, y.center = 0,
#                        scan.approach = "multi",
#                        threads = 2)
# 
# tree.list.multi.tls <- tree.detection.multi.scan(data = pcd.multi,
#                                                  d.mer = 20,
#                                                  threads = 2)

## ----eval = FALSE, include=TRUE-----------------------------------------------
# head(tree.list.multi.tls)

## ----echo=FALSE---------------------------------------------------------------
tree.list.multi.tls <- read.csv(paste(dir.data, "tree.list.multi.tls.csv", sep = "/"))
kableExtra::scroll_box(kable_input = kableExtra::kable(head(tree.list.multi.tls), 
                                                       format = "html"), width = "100%")

## ----eval = FALSE, include = FALSE--------------------------------------------
# diameter <- readLAS(paste(dir.data, "diameters.laz", sep = "/"))
# lidR::plot(singleLAS, color = "RGB", add = plot(diameter, color = "Intensity"))

## ----eval = FALSE, include= TRUE----------------------------------------------
# id <- c("PinusSylvestris1", "PinusSylvestris2")
# 
# center.coord <- data.frame(id = id,
#                            x = rep(0, length(id)),
#                            y = rep(0, length(id)))
# 
# tree.tls <- tree.detection.several.plots(las.list = c("PinusSylvestris1.laz",
#                                                       "PinusSylvestris2.laz"),
#                                          id.list = id,
#                                          center.coord = center.coord,
#                                          tls.resolution = list(point.dist = 7.67, tls.dist = 10),
#                                          max.dist = 7.5,
#                                          threads = 2)

tree.detection.single.scan <- function(data, single.tree = NULL,
                                       dbh.min = 4, dbh.max = 200, h.min = 1.3,
                                       geo.dist = 0.1, tls.resolution = list(), tls.precision = NULL,
                                       stem.section = c(0.7, 3.5), stem.range = NULL, breaks = NULL,
                                       slice = 0.1, understory = NULL, bark.roughness = 1,
                                       den.type = 1, d.mer = NULL,
                                       segmentation = NULL,
                                       plot.attributes = NULL, plot = TRUE,
                                       threads = 1,
                                       dir.data = NULL, save.result = TRUE, dir.result = NULL){



  #### Transforming data (as data frame class) to data table class

  data <- data.table::setDT(data)


  #### Checking some function arguments ####

  # Obtaining working directory for loading files
  if(is.null(dir.data))
    dir.data <- getwd()


  # Obtaining working directory for saving files
  if(is.null(dir.result))
    dir.result <- getwd()

  # Checking tls.precision
  if(is.null(tls.precision))
    tls.precision <- 0.03


  # Converting arguments to International System of Units
  # Arguments of the forest inventory

  .dbh.min <- dbh.min / 100
  .dbh.max <- dbh.max / 100



  # Arguments of the TLS precision
  # If resolution is defined by points distance at a certain distance from TLS (mm/m):

  if(is.null(tls.resolution))
    stop("The argument tls.resolution must be defined")

  .point.dist <- tls.resolution$point.dist / 1000
  .tls.dist <- tls.resolution$tls.dist

  # If resolution is defined by angles (?):
  .vertical.angle <- tls.resolution$vertical.angle * 2 * pi / 360
  .horizontal.angle <- tls.resolution$horizontal.angle * 2 * pi / 360

  # Define angle aperture according to available TLS resolution parameters:

  # Angular resolution:

  if(is.null(.point.dist)){

    .alpha.v <- .vertical.angle
    .alpha.h <- .horizontal.angle

  } else {

    .alpha.v <- atan((.point.dist / 2) / .tls.dist) * 2
    .alpha.h <- .alpha.v

  }

  # Obtaining Cartesian coordinates (x,y) from center

  x.center <- as.numeric(data[data$point == 0, "x"])
  y.center <- as.numeric(data[data$point == 0, "y"])

  data <- data[-1, ]



  #### Detecting possible areas with trees in the point cloud ####

  if(!is.null(data$GLA)){
    woody <- data[data$GLA <= 0, ]
    woody <- woody[!is.na(woody$x) & !is.na(woody$y) & !is.na(woody$z), ]}
  else {woody <- data}


  if(!is.null(data$intensity) & suppressWarnings(mean(data$intensity, na.rm = T)) > 0 & is.null(data$GLA)){
    woody <- data[data$intensity > mean(data$intensity, na.rm = T), ]}


  if(!is.null(data$intensity) & suppressWarnings(mean(data$intensity, na.rm = T)) > 0 & !is.null(data$GLA)){
    woody <- woody[woody$intensity > mean(woody$intensity, na.rm = T), ]}



  # Statistical point cloud filtering ----

  # 1. Statistical Outliers Removal (SOR)

  message("Application of Statistical Outlier Removal (SOR) to the entire point cloud")

  woody <- woody[, c("x", "y", "z")]
  woody <- VoxR::filter_noise(data = data.table::setDT(woody), store_noise = TRUE, message = FALSE)

  noise <- woody[woody$Noise == 2, ]
  woody <- woody[woody$Noise == 1, ]

  woody <- merge(data, woody[, c("x", "y", "z")], by = c("x", "y", "z"), all = FALSE)
  noise <- merge(data, noise, by = c("x", "y", "z"), all = FALSE)
  noise <- noise[, c("id", "file", "x", "y", "z", "rho")]

  stem <- woody[woody$z > stem.section[1] & woody$z < stem.section[2], c("x", "y", "z")]

  stem <- voxel_grid_downsampling(as.matrix(stem[, c("x", "y", "z")]), voxel_size = tls.precision)

  stem <- data.frame(point = 1:nrow(stem),
                     x = stem[, 1],
                     y = stem[, 2],
                     z = stem[, 3])



  # 2 Computing geometric features (Verticality)

  message("Retention of points with high verticality & low surface variation")

  threads <- max(1, threads)


  GeometricFeatures <- geometric_features_dist(points = as.matrix(stem[, c("point", "x", "y", "z")]),
                                               dist = geo.dist,
                                               Surface_variation = TRUE,
                                               Verticality = TRUE,
                                               num_threads = threads)

  stem <- merge(stem[, c("point", "x", "y", "z")],
                GeometricFeatures[, c("point", "Surface_variation", "Verticality")], by = "point")


  # 2.1 Retention of points with high verticality

  stem <- stem[!is.na(stem$Verticality), ]
  stem <- stem[stem$Verticality > 0.7, ]


  # 2.2 Retention of points with low surface variation

  stem$Surface_variation <- stem$Surface_variation / 0.33
  stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
  stem <- stem[stem$Surface_variation < stem$prob.ver, ]


  # 3. Retention of high points in the stem section defined (understory = TRUE)

  if(!is.null(understory)){

    stem$ver <- (stem$z - stem.section[1]) / (stem.section[2] - stem.section[1])
    stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
    stem <- stem[stem$ver > stem$prob.ver, ]

  }

  # 5. Remove cluster with fewer than 20 points

  stem$cluster <- dbscan::dbscan(stem[, c("x", "y", "z"), drop = FALSE], eps = tls.precision, minPts = 2)$cluster
  stem <- stem[stem$cluster > 0 & ave(stem$cluster, stem$cluster, FUN = length) > 20, , drop = FALSE]


  stem.2 <- NULL


  # 6. Detection of regions of high point density

  stem <- VoxR::vox(stem[, c("x", "y", "z")], res = tls.precision)

  stem <- VoxR::project_voxels(stem)

  stem <- stem[stem$ratio > 1, ]
  stem$cluster <- dbscan::dbscan(stem[, c("x", "y"), drop = FALSE], eps = tls.precision, minPts = 2)$cluster
  stem <- stem[stem$cluster > 0 & ave(stem$cluster, stem$cluster, FUN = length) > 1, , drop = FALSE]


  if(nrow(stem) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }




  # Creation of polygons to extract those projected areas in the original point cloud
  # where trees are probably located according to high point density regions


  buf <- sf::st_as_sf(data.frame(stem), coords = c("x","y"))
  buf <- sf::st_buffer(buf, max(.dbh.min, 1))
  buf <- sf::st_cast(sf::st_union(buf), "POLYGON")

  rm(stem)
  gc()


  # if(!is.null(understory) & is.null(single.tree)){
  #
  #   buf.2 <- sf::st_as_sf(stem.2, coords = c("x","y"))
  #   # sf::st_crs(buf) <- "+proj=utm +zone=19 +ellps=GRS80 +datum=NAD83 +unit=m"
  #   buf.2 <- sf::st_buffer(buf.2, max(.dbh.min, 0.5))
  #
  #
  #   buf.2 <- sf::st_difference(sf::st_union(buf.2), buf, 0.1)
  #
  #   # Detection of stem part without shrub vegetation and crown
  #
  #   buf.2 <- buf.2[sf::st_area(buf.2) > (pi / 4)*(.dbh.min + 0.1) ^ 2]
  #
  #   if(length(buf.2) < 1){
  #
  #     stem.2 <- NULL
  #
  #   } else {
  #
  #     stem.2 <- woody[woody$prob.selec == 1, ]
  #
  #     stem.3 <- sf::st_intersects(buf, sf::st_as_sf(stem.2, coords = c("x","y")))
  #     stem.3 <- data.table::setDT(as.data.frame(stem.3))
  #     colnames(stem.3) <- c("tree", "code")
  #     stem.2$code <- as.numeric(row.names(stem.2))
  #     stem.2 <- merge(stem.2, stem.3, by = "code", all = FALSE)
  #     # stem.2 <- subset(stem.2, select = -code)
  #     stem.2 <- stem.2[, 2:ncol(stem.2)]
  #
  #     rm(stem.3)
  #
  #
  #     if(nrow(stem.2) < 1)
  #       stem.2 <- NULL
  #
  #   }
  #
  #   rm(buf.2)
  #
  # }



  woody.2 <- sf::st_intersects(buf, sf::st_as_sf(data.frame(woody), coords = c("x","y")))
  woody.2 <- data.table::setDT(as.data.frame(woody.2))
  colnames(woody.2) <- c("tree", "code")
  woody$code <- as.numeric(row.names(woody))
  woody <- merge(woody, woody.2, by = "code", all = FALSE)
  woody <- subset(woody, select = -code)
  # woody <- woody[, !(names(woody) %in% c("code"))]

  woody <- woody[!is.na(woody$tree), ]


  rm(buf, woody.2)
  gc()

  # If there is only one tree in the point cloud

  if(!is.null(single.tree)){

    filter <- data.frame(table(stem$tree))
    filter <- filter[order(filter$Freq, decreasing = TRUE), ]
    stem <- stem[stem$tree == filter$Var1[1], ]

  }


  # Breaks argument

  if(is.null(breaks)){
    breaks <- seq(from = 0.4, to = max(woody$z), by = 0.3)
    # breaks <- c(0.2, seq(from = 0.4, to = max(woody$z), by = 0.3))
    breaks <- breaks[-length(breaks)]}

  gc()

  #### Starting with clustering process ####


  # data <- data[1, ]


  #### Starting with clustering process ####

  message("Computing sections")


  # Preallocate lists for efficiency
  .filteraux <- vector("list", length(breaks))
  .filteraux.2 <- vector("list", length(breaks))

  slice <- slice / 2  # Adjust slice size


  # Set up parallel cluster
  cl <- parallel::makeCluster(max(1, threads))


  pb <- progress::progress_bar$new(total = length(breaks))



  for(i in seq_along(breaks)){

    pb$tick()


    cuts <- breaks[i]


    .cut <- woody[woody$z > (cuts - slice) & woody$z < (cuts + slice), , drop = FALSE]

    if(nrow(.cut) < 25){next}



    # DBSCAN parameters

    .eps <- 2 * (tan(.alpha.h / 2) * (max(.cut$r, na.rm = TRUE) / cos(mean(.cut$slope, na.rm = TRUE))) * 2)


    # Clustering

    .error <- try(suppressMessages(dbscan::dbscan(.cut[, c("x", "y"), drop = FALSE], eps = .eps)))
    if(class(.error)[1] == "try-error"){
      message("No computed section: ", cuts, " m")
      next} else {
    .dbscan <- dbscan::dbscan(.cut[, c("x", "y"), drop = FALSE], eps = .eps)
    .cut$cluster <- .dbscan$cluster
    .cut <- .cut[.cut$cluster > 0 & ave(.cut$cluster, .cut$cluster, FUN = length) > 2, , drop = FALSE]
    }


    # Checking if there are clusters

    if(nrow(.cut) < 25){next}

    .cut$sec <- cuts


    if (interactive()) {

    # Parallel processing within the loop
    .filter <- do.call(rbind, parallel::parLapply(cl, split(.cut, .cut$cluster),
                                                  .sections.single.scan, .cut = .cut,
                                                  .alpha.v = .alpha.v, .alpha.h = .alpha.h,
                                                  .dbh.min = .dbh.min, .dbh.max = .dbh.max,
                                                  slice = slice * 2, bark.roughness = bark.roughness,
                                                  x.center = x.center, y.center = y.center))


    } else {


      .filter <- do.call(rbind, lapply(split(.cut, .cut$cluster), .sections.single.scan, .cut = .cut,
                                       .alpha.v = .alpha.v, .alpha.h = .alpha.h,
                                       .dbh.min = .dbh.min, .dbh.max = .dbh.max,
                                       slice = slice * 2, bark.roughness = bark.roughness,
                                       x.center = x.center, y.center = y.center))


    }


    .filteraux[[i]] <- .filter


    # Second...


    if(!is.null(stem.2)){

      .cut <- woody[woody$z > (cuts - slice) & woody$z < (cuts + slice), , drop = FALSE]

      if(nrow(.cut) < 25){next}


      # DBSCAN parameters

      .eps <- 2 * (tan(.alpha.h / 2) * (max(.cut$r, na.rm = TRUE) / cos(mean(.cut$slope, na.rm = TRUE))) * 2)

      # Clustering

      .error <- try(suppressMessages(dbscan::dbscan(.cut[, c("x", "y"), drop = FALSE], eps = .eps)))
      if(class(.error)[1] == "try-error"){
        message("No computed section: ", cuts, " m")
        next} else {
          .dbscan <- dbscan::dbscan(.cut[, c("x", "y"), drop = FALSE], eps = .eps)
          .cut$cluster <- .dbscan$cluster
          .cut <- .cut[.cut$cluster > 0 & ave(.cut$cluster, .cut$cluster, FUN = length) > 2, , drop = FALSE]
          # .cut <- .cut[which(.cut$cluster > 0), , drop = FALSE]
          }

      # Checking if there are clusters

      if(nrow(.cut) < 25){next}

      # Assigning section to the slice

      .cut$sec <- cuts


      # Selection of those cluster belonging to trees

      .filter <- do.call(rbind, lapply(split(.cut, .cut$cluster), .sections.single.scan, .cut = .cut,
                                       .alpha.v = .alpha.v, .alpha.h = .alpha.h,
                                       .dbh.min = .dbh.min, .dbh.max = .dbh.max,
                                       slice = slice * 2, bark.roughness = bark.roughness,
                                       x.center = x.center, y.center = y.center))

      .filteraux.2[[i]] <- .filter}


    # Run garbage collection only every 5 iterations
    if (i %% 5 == 0) gc()


  }# End of cuts loop


  # Stop cluster

  parallel::stopCluster(cl)

  .filteraux <- do.call(rbind, .filteraux)
  .filteraux.2 <- do.call(rbind, .filteraux.2)



  #### Assigning sections to tree axis ####

  if(length(.filteraux) < 1 & length(.filteraux.2) < 1) {

    warning("No tree was detected")

    .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

  }

  else {

    eje <- .stem.assignment.single.scan(.filteraux, stem.section, breaks)

    if(!is.null(.filteraux.2) & sum(c(.filteraux.2$circ, .filteraux.2$arc.circ)) > 0){

      # Assigning sections to tree axis

      eje.2 <- .stem.assignment.single.scan(.filteraux.2, stem.section, breaks)

      eje.2$tree <- eje.2$tree +  max(eje$tree)

      eje <- rbind(eje[, c("tree", "sec", "x", "y")], eje.2[, c("tree", "sec", "x", "y")])
      rm(eje.2)

      .filter <- rbind(.filteraux, .filteraux.2)
      rm(.filteraux, .filteraux.2)


    }  else {

      .filter <- .filteraux
      rm(.filteraux)

    }

    .filter <- .stem.assignment.single.scan.2(.filter, eje, stem.section, x.center, y.center, single.tree)


    if(nrow(.filter) < 1){

    warning("No tree was detected")

    .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
    return(.tree)

    }


    # Export all section detected
    .filter <- .filter[order(.filter$tree, .filter$sec), , drop = FALSE]
    .stem <- .filter[, c("tree", "sec", "center.x",  "center.y", "radius")]
    .stem$radius <- .stem$radius * 200
    colnames(.stem) <- c("tree2", "sec", "x",  "y", "dbh")


    # Calculate of taper coefficient as the slope coefficient of linear regression

    # .taper <- .filter[.filter$sec >= 1 & .filter$sec <= 1.6, c("tree", "sec", "radius"), drop = FALSE]
    .taper <- .filter[, c("tree", "sec", "radius"), drop = FALSE]

    .slope.tree <- data.frame(tree = as.numeric(), slope = as.numeric())

    for(i in unique(.taper$tree)){

      .taper.i <- .taper[.taper$tree == i, ]

      if(nrow(.taper.i) < 2){

        .slope <- data.frame(tree = i, slope = NA)

      } else {

        .lm <- stats::lm(radius ~ sec, data = .taper.i)
        .slope <- data.frame(tree = i, slope = stats::coef(.lm)[2])

      }

      .slope.tree <- rbind(.slope.tree, .slope)

    }

    .filter <- merge(.filter, .slope.tree, by = "tree")



    # If a new column ("dif") is created containing the difference between dbh and
    # section from which radius is estimated, number of used sections (or cuts)
    # will not be important. An estimated radius will always exist
    .filter$dif <- 1.3 - .filter$sec
    .filter$slope <- ifelse(.filter$dif != 0 & is.na(.filter$slope), mean(.filter$slope, na.rm = TRUE), .filter$slope)
    .filter$radio.est <- ifelse(.filter$dif == 0, .filter$radius, .filter$radius + .filter$slope * .filter$dif)
    .filter <- .filter[!is.na(.filter$radio.est), ]

    # When there are not enough tree to the linear model, and tress were
    # detected at at different sections than 1.3 m, we assume these radius
    # as dbh. This could happen in very few situations.
    # .filter$radio.est <- ifelse(is.na(.filter$radio.est), .filter$radius, .filter$radio.est)
    # .filter$radio.est2 <- ifelse(is.na(.filter$radio.est2), .filter$radius2, .filter$radio.est2)

    top.lim <- max(1.3, abs(max(stem.section) - 1.3))

    .radio.est <- data.frame(tree = as.numeric(), radio.est = as.numeric())

    for (i in unique(.filter$tree)) {

      .dat <- .filter[which(.filter$tree == i), ]
      .dat$dif <- abs(.dat$dif)

      if(min(.dat$dif) > top.lim)
        next

      .dat <- .dat[order(.dat$dif), ]

      if(.dat$dif[1] == 0 & .dat$arc.circ[1] == 1){

        .dat <- .dat[1, ]

      } else if (nrow(.dat) > 1 & .dat$dif[1] == 0) {

        .dat <- .dat[.dat$dif == 0 | .dat$dif == .dat$dif[2], ]

      } else{

        .dat <- .dat[.dat$dif == min(.dat$dif), ]

      }

      .out <- data.frame(tree = i, radio.est = mean(.dat$radio.est, na.rm = TRUE))
      .radio.est <- rbind(.radio.est, .out)

    }

    # Tree normal section coordinates
    .filteraux <- data.frame(tree = as.numeric(),
                             filter = as.numeric(),

                             center.x = as.numeric(),
                             center.y = as.numeric(),
                             center.phi = as.numeric(),
                             phi.left = as.numeric(),
                             phi.right = as.numeric(),
                             center.rho = as.numeric(),
                             center.r = as.numeric(),
                             center.theta = as.numeric(),

                             sec.x = as.numeric(),
                             sec.y = as.numeric(),
                             sec.max = as.numeric(),

                             horizontal.distance = as.numeric(), # repeated line
                             radius = as.numeric(),

                             partial.occlusion = as.numeric(),

                             n.pts = as.numeric(),
                             n.pts.red = as.numeric())



    for (i in unique(.filter$tree)) {

      .dat <- .filter[which(.filter$tree == i), ]
      .dat <- .dat[order(abs(.dat$dif)), ]

      .sec.x <- .dat$center.x[nrow(.dat)]
      .sec.y <- .dat$center.y[nrow(.dat)]
      .sec.max <- .dat$sec[nrow(.dat)]

      .dat$sec.x <- .sec.x
      .dat$sec.y <- .sec.y
      .dat$sec.max <- .sec.max

      .dat$filter <- nrow(.dat)

      if(nrow(.dat) > 3)
        .dat <- .dat[1:3, ]


      if(min(abs(.dat$dif)) > top.lim)
        next

      .filteraux <- rbind(.filteraux, .dat)

    }


    if(nrow(.filteraux) < 1){

      warning("No tree was detected")

      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
      return(.tree)

    }

    # .filteraux <- subset(.filteraux, select = -radio.est)
    .filteraux <- .filteraux[, !(names(.filteraux) %in% c("radio.est"))]
    .filteraux <- merge(.filteraux, .radio.est, by = "tree")

    # Dendrometric variables
    .tree <- data.frame(tree = tapply(.filteraux$tree, .filteraux$tree, mean, na.rm = TRUE),
                        filter = tapply(.filteraux$filter, .filteraux$tree, mean, na.rm = TRUE),

                        x = tapply(.filteraux$center.x, .filteraux$tree, mean, na.rm = TRUE),
                        y = tapply(.filteraux$center.y, .filteraux$tree, mean, na.rm = TRUE),

                        sec.x = tapply(.filteraux$sec.x, .filteraux$tree, mean, na.rm = TRUE),
                        sec.y = tapply(.filteraux$sec.y, .filteraux$tree, mean, na.rm = TRUE),
                        sec.max = tapply(.filteraux$sec.max, .filteraux$tree, mean, na.rm = TRUE),

                        phi = tapply(.filteraux$center.phi, .filteraux$tree, mean, na.rm = TRUE),
                        phi.left = tapply(.filteraux$phi.left, .filteraux$tree, mean, na.rm = TRUE),
                        phi.right = tapply(.filteraux$phi.right, .filteraux$tree, mean, na.rm = TRUE),
                        rho = tapply(.filteraux$center.rho, .filteraux$tree, mean, na.rm = TRUE),
                        r = tapply(.filteraux$center.r, .filteraux$tree, mean, na.rm = TRUE),
                        theta = tapply(.filteraux$center.theta, .filteraux$tree, mean,na.rm = TRUE),

                        horizontal.distance = tapply(.filteraux$center.rho, .filteraux$tree, mean, na.rm = TRUE), # repeated line
                        radius = tapply(.filteraux$radio.est, .filteraux$tree, mean, na.rm = TRUE),

                        partial.occlusion = tapply(.filteraux$arc.circ, .filteraux$tree, min, na.rm = TRUE),

                        n.pts = tapply(.filteraux$n.pts, .filteraux$tree, mean, na.rm = TRUE),
                        n.pts.red = tapply(.filteraux$n.pts.red, .filteraux$tree, mean, na.rm = TRUE))

    rm(.filteraux)


    # Cheking if there are negative radious

    .tree <- .tree[.tree$radius > 0, ]

    if(nrow(.tree) < 1){

      warning("No tree was detected")

      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
      return(.tree)

    }


    # Cheking minimum and maximum radius defined in the arguments

    .tree <- .tree[.tree$radius >= .dbh.min / 2 & .tree$radius <= .dbh.max / 2, ]


    if(nrow(.tree) < 1){

      warning("No tree was detected")

      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
      return(.tree)

    }


    # Selecting only those trees with more than one section detected when more than two breaks have been specified

    if(length(breaks) > 3)
      .tree <- .tree[.tree$filter > 1, ]


    if(nrow(.tree) < 1){

      warning("No tree was detected")

      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
      return(.tree)

    }


    # Remove duplicated trees and ordering by distance and numbering trees from 1 to n trees

    .tree <- .tree[!duplicated(.tree$x) & !duplicated(.tree$y), ]
    .tree <- .tree[!duplicated(.tree$sec.x) & !duplicated(.tree$sec.y), ]

    if(nrow(.tree) < 1){

      warning("No tree was detected")

      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
      return(.tree)

    }


    # Detecting possible trees overlaped

    # if(nrow(.tree) < 2 & !is.null(single.tree)){.tree.2 <- .tree}
    if(nrow(.tree) < 2){.tree.2 <- .tree}


    if(nrow(.tree) > 1 & is.null(single.tree)){

      .tree.2 <- data.frame(tree = as.numeric(), filter = as.numeric(),

                            x = as.numeric(), y = as.numeric(),

                            sec.x = as.numeric(), sec.y = as.numeric(), sec.max = as.numeric(),

                            phi = as.numeric(), rho = as.numeric(), r = as.numeric(), theta = as.numeric(),

                            horizontal.distance = as.numeric(), radius = as.numeric(),

                            partial.occlusion = as.numeric(),

                            n.pts = as.numeric(), n.pts.red = as.numeric())


      for (i in unique(.tree$tree)) {


        if(nrow(.tree[.tree$tree == i, ]) < 1)
          next

        .filt <- .tree[.tree$tree == i, ]
        .filteraux <- .tree[.tree$tree != i, ]

        if(nrow(.filteraux) < 1){

          .tree.2 <- rbind(.tree.2, .filt)
          next

        }

        .filteraux$dist <- sqrt((.filteraux$x - .filt$x) ^ 2 + (.filteraux$y - .filt$y) ^ 2) - .filteraux$radius - .filt$radius
        .filteraux$rho.dist <- abs(.filteraux$rho - .filt$rho) - .filteraux$radius - .filt$radius
        .filteraux$phi.dist <- abs(.filteraux$phi - .filt$phi)

        if(min(.filteraux$dist) < mean(.tree$radius) |
           .filteraux$rho.dist[.filteraux$dist == min(.filteraux$dist)] < mean(.tree$radius) &
           .filteraux$phi.dist[.filteraux$dist == min(.filteraux$dist)] < 0.1){

          .filteraux <- .filteraux[.filteraux$dist < mean(.tree$radius) | .filteraux$rho.dist < mean(.tree$radius) & .filteraux$phi.dist < 0.1, ]

          .filteraux <- rbind(.filt, .filteraux[ , 1:(ncol(.filteraux)-3)])

          .filt <- .filteraux[.filteraux$filter == max(.filteraux$filter), ]

          if(nrow(.filt) > 1)
            .filt <- .filteraux[.filteraux$sec.max ==  min(.filteraux$sec.max), ]


          if(nrow(.filt) > 1)
            .filt <- .filt[.filt$partial.occlusion > 0, ]


          if(nrow(.filt) > 1)
            .filt <- .filt[1, ]

          .tree.remove <- .filteraux[.filteraux$tree != .filt$tree, ]$tree


          if(length(.tree.remove) < 1){.tree <- .tree} else {
            suppressWarnings(.tree <- .tree[.tree$tree != .tree.remove, ])
          }


          .tree.2 <- rbind(.tree.2, .filt)


        } else {

          .tree.2 <- rbind(.tree.2, .filt)
          .tree <- .tree[.tree$tree != .filt$tree, ]

        }

      }

    }

    .tree <- .tree.2
    rm(.tree.2)


    # Indicate trees with partial occlusions, those for which none of the sections
    # was identified as circumference arch (ArcCirc)
    .tree$partial.occlusion <- ifelse(.tree$partial.occlusion == 0, 1, 0)

    # Compute dbh (cm)
    .tree$dbh <- .tree$radius * 200


    # Calculate points belonging to radius unit
    # Since it will be an estimation, select sections completely visible
    # (ArcCirc == 1) in section corresponding to 1.3 m (where dbh is estimated)
    .filter$filter <- ifelse(.filter$sec == 1.3 & .filter$arc.circ == 1, 1, 0)
    .filter2 <- subset(.filter, .filter$filter == 1)

    if(nrow(.filter2) < 1)
      .filter2 <- .filter

    # Estimate number of points by cluster, with and without point cropping
    # process, corresponding to radius 1 m
    .filter2$points.radio <- .filter2$n.pts / .filter2$radius
    .filter2$points.radio.hom <- .filter2$n.pts.red / .filter2$radius

    # Average points after point cropping by m of radius
    .tree$points.m <- mean(.filter2$points.radio)
    .tree$points.m.hom <- mean(.filter2$points.radio.hom)

    # Finally, compute number of points estimated for each tree according to
    # radius
    .tree$n.pts.est <- .tree$points.m * .tree$radius
    .tree$n.pts.red.est <- .tree$points.m.hom * .tree$radius


    data <- rbind(woody[, c("id", "file", "x", "y", "z", "rho")],
                  noise[, c("id", "file", "x", "y", "z", "rho")])

    rm(woody, noise)
    gc()


    if(!is.null(plot) & is.null(segmentation))
      plotTree <- suppressMessages(lidR::plot(lidR::LAS(data[, c("x","y","z")])))


    #### Estimating tree heights ####

    # Obtaining reduced point cloud

    # data <-
    #   suppressMessages(vroom::vroom(file.path(dir.data, data$file[1]),
    #                                 col_select = c("id", "file", "x", "y", "z", "rho"),
    #                                 progress = FALSE))


    s <- sample(nrow(data), round(nrow(data) * 0.1))
    data <- data[s, ]
    data <- data[, c("id", "file", "x", "y", "z", "rho")]


    # If only one tree is detected, Voronoi tessellation is not working

    .tree$tree2 <- .tree$tree
    .tree$tree <- 1:nrow(.tree)

    if(nrow(.tree) == 1){

      .P99 <- data.frame(tree = .tree$tree, h = stats::quantile(data$z, prob = 0.9999999999))

    } else {

      cloud_sf <- sf::st_as_sf(data, coords = c("x", "y"))
      stopifnot(nrow(cloud_sf) == nrow(data))
      bbox     <- sf::st_as_sfc(sf::st_bbox(cloud_sf))
      trees_sf <- sf::st_as_sf(.tree, coords = c("sec.x", "sec.y"))
      voronoi  <- sf::st_collection_extract(
        sf::st_voronoi(sf::st_union(trees_sf), envelope = bbox)
      )

      tree_idx <- vapply(
        sf::st_intersects(voronoi, trees_sf),
        function(x) if (length(x) == 0L) NA_integer_ else x[1L],
        integer(1)
      )
      voronoi <- sf::st_as_sf(data.frame(
        tree     = .tree$tree[tree_idx],
        geometry = sf::st_geometry(voronoi)
      ))

      int_idx       <- sf::st_intersects(cloud_sf, voronoi)
      cloud_sf$tree <- voronoi$tree[vapply(
        int_idx,
        function(x) if (length(x) == 0L) NA_integer_ else x[1L],
        integer(1)
      )]

      .P99 <- do.call(rbind, lapply(unique(voronoi$tree), function(id) {
        sec.max <- .tree$sec.max[match(id, .tree$tree)]
        z       <- cloud_sf$z[!is.na(cloud_sf$tree) & cloud_sf$tree == id & cloud_sf$z > sec.max]
        if (length(z) < 1L) return(data.frame(tree = id, h = 0))
        data.frame(tree = id, h = height_perc_cpp(rho_seq = Inf, z = z, rho = z)[, "P99.9"])
      }))

      rm(trees_sf, tree_idx, voronoi, cloud_sf, int_idx)

    }


    # Remove possible trees above "h.min" (1.3 m by default)

    .P99 <- .P99[.P99$h >= h.min, ]
    .tree <- merge(.tree, .P99, by = "tree", all = FALSE)

    rm(.P99)

    .tree <- .tree[order(.tree$rho), ]
    .tree$tree <- 1:nrow(.tree)


    # Assigning dbh to stem dataset

    .stem <- merge(.stem, .tree[, c("tree", "tree2", "h", "rho")], by = "tree2")
    .stem <- .stem[, colnames(.stem) != "tree2"]
    .tree <- .tree[, colnames(.tree) != "tree2"]
    # .stem <- .stem[.stem$sec != 1.3, ]
    # .tree$sec <- 1.3
    # .stem <- rbind(.stem, .tree[, c("tree", "sec", "x", "y", "dbh", "h", "rho")])
    # .tree <- .tree[, -ncol(.tree)]
    # .stem <- .stem[order(.stem$tree, .stem$sec), , drop = FALSE]
    # colnames(.stem) <- c("tree", "sec", "x", "y", "dhi", "h", "rho")
    .stem <- merge(.stem, .tree[, c("tree", "dbh")], by = "tree")
    colnames(.stem) <- c("tree", "hi", "x", "y", "dhi", "h", "rho", "dbh")

    # Cheking the trees h
    .stem <- merge(.stem[, c("tree", "hi", "x", "y", "dhi", "rho", "dbh")],
                   data.frame(tree = unique(.stem$tree),
                              h = do.call(rbind, lapply(split(.stem[, c("hi", "h")], .stem$tree), max))),
                   all.x = TRUE, by = "tree")

    # Compute volume (m3)

    if(length(table(.stem$hi)) > 3 & is.null(d.mer)){

      # Stem curve

      stem.v <- .volume(.stem, id = data$id[1], den.type = den.type)
      .tree <- merge(.tree, stem.v, all = TRUE)

    } else if (length(table(.stem$hi)) > 3 & !is.null(d.mer)) {

      stem.v <- .volume(.stem, d.mer, id = data$id[1], den.type = den.type)
      .tree <- merge(.tree, stem.v, all = TRUE)


    } else if (length(table(.stem$hi)) <= 3 & !is.null(d.mer)) {

      n <- den.type
      # Paraboloid volume

      .tree$v <- pi * (.tree[, "h"] ^ (n + 1) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)
      h.lim <- (((d.mer / 200) ^ 2) / ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)) ^ (1 / n)
      .tree$v.mer <- pi * ((.tree[, "h"] ^ (n + 1) - h.lim ^ (n + 1)) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)
      .tree$v.mer <- ifelse(.tree$v.mer < 0, 0, .tree$v.mer)
      .tree$h.mer <- h.lim

    } else {

      n <- den.type

      .tree$v <- pi * (.tree[, "h"] ^ (n + 1) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)

    }


    .tree <- .tree[order(.tree$rho), ]
    .tree$tree <- 1:nrow(.tree)

    .stem <- .stem[order(.stem$rho), ]
    .stem$tree <- rep(1:length(unique(.stem$tree)), times = table(.stem$rho))
    .stem <- .stem[, c("tree", "x", "y", "dhi", "dbh", "hi", "h")]
    .stem <- .stem[order(.stem$tree, .stem$hi), , drop = FALSE]

    if(!is.null(data$id)){
      .stem$id <- data$id[1]
      .stem <- .stem[, c("id", "tree", "x", "y", "dhi", "dbh", "hi", "h")]}


    # Straighness analysis

    straightness <- do.call(rbind, lapply(split(.stem, .stem$tree), .straightness, stem.range = stem.range))

    .tree <- merge(.tree, straightness, by = "tree", all.x = TRUE)


    utils::write.csv(.stem,
                     file = file.path(dir.result, "stem.curve.csv"),
                     row.names = FALSE)

    rm(.stem)


    # If plot identification (id) is not available
    if(is.null(data$id) & is.null(.tree$v.mer)){

      .tree <- .tree[, c("tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
      colnames(.tree) <- c("tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

    } else if (is.null(data$id) & !is.null(.tree$v.mer)) {

      .tree <- .tree[, c("tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
      colnames(.tree) <- c("tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "n.pts", "SS.max", "sinuosity", "lean", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")


    } else if (!is.null(data$id) & is.null(.tree$v.mer)) {

      # If plot identification (id) is available

      .tree$id <- data$id[1]
      .tree$file <- data$file[1]

      .tree <- .tree[, c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
      colnames(.tree) <- c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

    } else {

      # If plot identification (id) is available

      .tree$id <- data$id[1]
      .tree$file <- data$file[1]

      .tree <- .tree[, c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
      colnames(.tree) <- c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

    }

  }



  # Removing values of 0 in n.pts
  .tree$n.pts <- ifelse(.tree$n.pts < 1, 0.01, .tree$n.pts)
  .tree$n.pts.red <- ifelse(.tree$n.pts.red < 1, 0.01, .tree$n.pts.red)
  .tree$n.pts.est <- ifelse(.tree$n.pts.est < 1, 0.01, .tree$n.pts.est)
  .tree$n.pts.red.est <- ifelse(.tree$n.pts.red.est < 1, 0.01, .tree$n.pts.red.est)


  # Lastly, aggregate attributes table
  if(!is.null(plot.attributes))
    .tree <- merge(.tree, plot.attributes, by = "id", all = FALSE)


  if(isTRUE(save.result)){

    utils::write.csv(.tree,
                     file = file.path(dir.result, "tree.tls.csv"),
                     row.names = FALSE)
  }

  # if(isTRUE(save.result)){
  #
  #   .data.red <- noise[which(noise$prob.selec == 1), , drop = FALSE]
  #
  #   vroom::vroom_write(.data.red, file = file.path(dir.result, paste("noise_", .data.red$file[1], sep = "")), delim = ",", progress = FALSE)
  #
  # }


  # Tree segmentation

  if(!is.null(segmentation)){

    # treeLAS <- suppressMessages(lidR::LAS(data[, c("x","y","z")]))

    voro <- sf::st_as_sf(data, coords = c("x", "y", "z"))
    voronoi <- sf::st_as_sf(.tree, coords = c("x", "y"))

    voronoi <- sf::st_collection_extract(
      sf::st_voronoi(do.call(c, sf::st_geometry(voronoi))))

    voro$tree <- unlist(sf::st_intersects(voro, voronoi))

    coords <- as.data.frame(sf::st_coordinates(voro))
    coords$tree <- voro$tree


    if(!is.null(plot))
      plotTree <- suppressMessages(lidR::plot(lidR::LAS(coords[, c("X", "Y", "Z", "tree")]), size = 0.5, color = "tree"))


    for (i in .tree$tree) {

      id <- .tree[.tree$tree == i, "id"]

      coords <- as.data.frame(sf::st_coordinates(voro[voro$tree == i, ]))
      colnames(coords) <- c("x", "y", "z")

      suppressMessages(lidR::writeLAS(lidR::LAS(coords[, c("x","y","z")]),
                                      paste(dir.result, "/tree", id, i, ".laz", sep = "")))

    }

  }


  # Diameters

  if(!is.null(plot)){

    diameter <- data.frame(tree = as.numeric(),
                           x = as.numeric(),
                           y = as.numeric(),
                           z = as.numeric())

    phi <- seq(from = 0, to = 2*pi, by = 2 * pi / 10000)


    for (i in .tree$tree) {

      tree <- rep(i, times = 10001)
      x <- .tree$x[i] + cos(phi) * ((.tree$dbh[i] / 100) / 2)
      y <- .tree$y[i] + sin(phi) * ((.tree$dbh[i] / 100) / 2)
      z <- stats::runif(10001, 1.2, 1.4)

      .diameter <- data.frame(tree = tree,
                              x = x, y = y, z = z)

      diameter <- rbind(diameter, .diameter)

    }

    suppressMessages(lidR::plot(lidR::LAS(diameter[, c("x","y","z")]), add = plotTree, size = 5))

  }


  #####
  return(.tree)

}

#' Install Python dependencies required by FORTLS
#'
#' @param envname Name of the virtual environment or conda environment.
#' @param method Either "virtualenv", "conda", or "auto".
#' @export
install_fortls_python_deps <- function(envname = "fortls_env", method = "auto") {
  # Install Miniconda if using conda and it's not available
  if (method == "conda") {
    reticulate::install_miniconda()
  }

  reticulate::py_install(
    # packages = c("pandas", "numpy", "jakteristics"),  # Add more as needed
    packages = c("numpy"),  # Add more as needed
    envname = envname,
    method = method,
    pip = TRUE
  )

  message("Python environment and packages for FORTLS successfully installed.")
}

distance.sampling <- function(tree.tls,
                              id.plots = NULL,
                              strata.attributes = NULL){

  # Funciona con el data frame que se obtiene de la funci?n tree_detection()
  # El argumento plot.radius se refiere al radio de la parcela considerado
  .data <- tree.tls

  # Convert dbh (cm) to International System of Units (m)
  .data$dbh <- .data$dbh / 100


  # Selecting plots especifiesd in the argument id.plots (NULL by default)
  if(is.null(id.plots)) {

    .data <- .data

  } else {

    .data <- .data[is.element(.data$id, id.plots), , drop = FALSE]

  }


  if(is.null(.data$id)){

    # Si las el data de entrada no tiene una columna con la identificai?n de las parcelas,
    # y por tanto contendr?a una solo parcela, se le asigna el valor de 1 por defecto
    .data$id <- 1

  } else {

    # En caso contrario se mantiene la codificaci?n para distinguir entre las
    # distintas parcelas
    .data$id <- .data$id

  }

  # Por último se agrega la tabla de atributos
  if(!is.null(strata.attributes))
    .data <- merge(.data, strata.attributes, by = "stratum")

  # Stratum
  if(is.null(.data$stratum)){

    .data$stratum <- 1

  } else {

    .data$stratum <- .data$stratum

  }


  # According to ddf() funtion form "mrds" package:

  # Details

  # The fitting code has certain expectations about data.
  # It should be a dataframe with at least the following fields named and
  # defined as follows:

  # Study.Area:    name of the study area (by default "forest")
  # Region.Label: strata (if there are several stratum)
  # Sample.Label: point transect identifier (plot id)
  # Effort:       survey effort (1 for all points because they are visited a signle time)
  # object:       unique identifier for each detected tree
  # distance:     radial distance of detection from observer (TLS)
  # OBS:          initials of observer; TLS in this case
  # DBH:          diameter estimated at breast height (1.3 m)


  # Because of that, I am defining the next fields:

  # "Effort"
  .data$Effort <- 1

  # "OBS"
  .data$OBS <- "TLS"

  # "object"
  .data$object <- c(1:nrow(.data))

  # Naming the rest of fields according to ddf() function
  .data <- .data[, c("stratum", "id", "tree", "Effort", "h.dist", "OBS", "object", "dbh"), drop = FALSE]
  colnames(.data) <- c("Region.Label", "Sample.Label", "tree", "Effort", "distance", "OBS", "object", "dbh")


  # Deleting possible missing distances
  .data <- .data[!is.na(.data$distance), , drop = FALSE]


  # Defining empty data frames where results will be added

  # Trees detection probability

  .tree <- data.frame(stratum = as.numeric(), id = as.numeric(), tree = as.numeric(),
                      P.hn = as.numeric(), P.hn.cov = as.numeric(), P.hn.cov = as.numeric(), P.hr.cov = as.numeric())


  # Parameters of detection probability functions

  .par <- data.frame(P.hn.scale = as.numeric(),
                     P.hn.cov.scale.intercept = as.numeric(), P.hn.cov.dbh = as.numeric(),
                     P.hr.scale = as.numeric(), P.hr.shape = as.numeric(),
                     P.hr.cov.scale.intercept = as.numeric(), P.hr.cov.dbh = as.numeric(), P.hr.cov.shape = as.numeric())

  # AIC criterion of detection probability functions fitted
  .treeAIC <- data.frame(P.hn = as.numeric(), P.hn.cov = as.numeric(), P.hr = as.numeric(), P.hr.cov = as.numeric())



  ## Fitting detection probability functions by stratum

  for (i in unique(.data$Region.Label)) {

    # Defining plots location
    # graphics::par(mfrow = c(2, 2))


    # Selecting the correstponding stratum
    .dat <- .data[which(.data$Region.Label == i), , drop = FALSE]

    # Defining maximum distance for fitting detection probability functions
    if(is.null(strata.attributes$plot.radius)){

      .max.dist <- max(.dat$distance)

    } else {

      .max.dist <- strata.attributes$plot.radius[i]

    }


    .dat <- .dat[which(.dat$distance <= .max.dist), , drop = FALSE]


    # Ahora se utiliza la funci?n ddf() del paquete mrds,
    # el cual ha sido desarrollado para este tipo de muestreos

    # Half-Normal fucntion

    .error_hn <- try(suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hn", adjustment = NULL)))

    if(class(.error_hn)[1] == "try-error"){

      .hn <- data.frame(fitted = NA, criterion = NA)
      .hn <- list(ddf = .hn)

    } else {

      .hn <- suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hn", adjustment = NULL))

      # plot(.hn, main = paste("Half-normal - stratum", i, sep = " "))

    }

    # Half-Normal fucntion with dbh as coovariate

    .error_hn_cov <- try(suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hn", formula=~dbh, adjustment = NULL)))

    if(class(.error_hn_cov)[1] == "try-error"){

      .hn_cov <- data.frame(fitted = NA, criterion = NA)
      .hn_cov <- list(ddf = .hn_cov)

    } else {

      .hn_cov <- suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hn", formula=~dbh, adjustment = NULL))

      # plot(.hn_cov, main = paste("Half-normal with dbh as coovariate - Stratum", i, sep = " "))

    }

    # Hazard rate

    .error_hr <- try(suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hn", adjustment = NULL)))

    if(class(.error_hr)[1] == "try-error"){

      .hr <- data.frame(fitted = NA, criterion = NA)
      .hr <- list(ddf = .hr)

    } else {

      .hr <- suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hr", adjustment = NULL))

      # plot(.hr, main = paste("Hazard rate - stratum", i, sep = " "))

    }

    # Hazard rate with dbh as coovariate

    .error_hr_cov <- try(suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hr", formula=~dbh, adjustment = NULL)))

    if(class(.error_hr_cov)[1] == "try-error"){

      .hr_cov <- data.frame(fitted = NA, criterion = NA)
      .hr_cov <- list(ddf = .hr_cov)

    } else {

      .hr_cov <- suppressMessages(Distance::ds(.dat, truncation = list(left = 1, right = .max.dist), transect = "point", key = "hr", formula=~dbh, adjustment = NULL))

      # plot(.hr_cov,  main = paste("Hazard rate with dbh as coovariate - stratum", i, sep = " "))

    }

    # Final merge
    # Se obtiene un data frame con los valores de la probabilidad de detecci?n de cada arbol
    .salida <- data.frame(# object = .dat$object,
      P.hn = .hn$ddf$fitted, P.hn.cov = .hn_cov$ddf$fitted,
      P.hr = .hr$ddf$fitted, P.hr.cov = .hr_cov$ddf$fitted)
    .salida <- merge(data.frame(object = .dat$object), .salida, by = "row.names", all = TRUE)
    .salida$P.hn <- ifelse(is.na(.salida$P.hn),  mean(.salida$P.hn, na.rm = TRUE), .salida$P.hn)
    .salida$P.hn.cov <- ifelse(is.na(.salida$P.hn.cov),  mean(.salida$P.hn.cov, na.rm = TRUE), .salida$P.hn.cov)
    .salida$P.hr <- ifelse(is.na(.salida$P.hr),  mean(.salida$P.hr, na.rm = TRUE), .salida$P.hr)
    .salida$P.hr.cov <- ifelse(is.na(.salida$P.hr.cov),  mean(.salida$P.hr.cov, na.rm = TRUE), .salida$P.hr.cov)

    # Se asocian los valores obtenidos a sus respectivos ?rboles
    .tree_n <- merge(.dat, .salida, by = "object")

    .tree_n <- .tree_n[, c("Region.Label", "Sample.Label", "tree", "P.hn", "P.hn.cov", "P.hr", "P.hr.cov"), drop = FALSE]
    colnames(.tree_n) <- c("stratum", "id", "tree", "P.hn", "P.hn.cov", "P.hr", "P.hr.cov")

    .tree <- rbind(.tree, .tree_n)

    # Obtaining values of detection probability functions fitted
    .par_n <- data.frame(P.hn.scale = exp(.hn$ddf$par[1]),
                         P.hn.cov.scale.intercept = exp(.hn_cov$ddf$par[1]), P.hn.cov.dbh = .hn_cov$ddf$par[2],
                         P.hr.scale = exp(.hr$ddf$par[2]), P.hr.shape = exp(.hr$ddf$par[1]),
                         P.hr.cov.scale.intercept = exp(.hr_cov$ddf$par[2]), P.hr.cov.dbh = .hr_cov$ddf$par[3], P.hr.cov.shape = exp(.hr_cov$ddf$par[1]))

    .par <- rbind(.par, .par_n)

    # Data frame con el valor del AIC de los modelos ajustados
    .treeAIC_n <- data.frame(P.hn = .hn$ddf$criterion, P.hn.cov = .hn_cov$ddf$criterion, P.hr = .hr$ddf$criterion, P.hr.cov = .hr_cov$ddf$criterion)

    .treeAIC <- rbind(.treeAIC, .treeAIC_n)

  }

  # Final list
  .tree <- list(tree = .tree, par = .par, AIC = .treeAIC)

  ###
  return(.tree)

}

utils::globalVariables(c(
  "Z", "Classification", "z", "rho", "phi", "r", "theta", "point",
  ".I", "GLA", "G", "R", "B", "prob", "prob.random", ".N",
  "prob.selec"
))


normalize <- function(las, normalized = NULL,
                      x.center = NULL, y.center = NULL,
                      x.side = NULL, y.side = NULL,
                      xpoly = NULL, ypoly = NULL,
                      max.dist = NULL, min.height = NULL, max.height = 50,
                      algorithm.dtm = "knnidw", res.dtm = 0.2,
                      csf = list(cloth_resolution = 0.5),
                      intensity = NULL, RGB = NULL,
                      scan.approach = "multi",
                      voxel_size = NULL,
                      threads = 1,
                      id = NULL, file = NULL, plot = TRUE,
                      dir.data = NULL, save.result = TRUE, dir.result = NULL,
                      save.las = NULL){


  threads <- max(1, threads)
  lidR::set_lidr_threads(threads)


  if(is.null(normalized)){
  pb <- progress::progress_bar$new(total = 12)} else {
    pb <- progress::progress_bar$new(total = 6)}

  pb$tick()


  # Obtaining working directory for loading files
  if(is.null(dir.data))
    dir.data <- getwd()

  # Obtaining working directory for saving files
  if(is.null(dir.result))
    dir.result <- getwd()

  # Reading input (LAS file)

  if(inherits(las, "LAS")){

    las <- las

    } else {

    # Initialize the select option with a default value
    select_option <- "xyz"

    # Modify select_option based on RGB and intensity presence
    if (is.null(RGB) & is.null(intensity)) {

      select_option <- "xyz"

    } else if (is.null(RGB) & !is.null(intensity)) {

      select_option <- "xyzIntensity"

    } else if (!is.null(RGB) & is.null(intensity)) {

      select_option <- "xyzRGB"

    } else {

      select_option <- "xyzIntensityRGB"

    }

    # If normalized = TRUE, append "Classification"
    if (!is.null(normalized)) {
      select_option <- paste(select_option, "Classification")
    }

    # Read LAS file with the selected attributes
    # Suppress lidR's progress bar in non-interactive mode
    old_opt <- getOption("lidR.progress")
    options(lidR.progress = interactive())

    las <- suppressWarnings(suppressMessages(
      lidR::readLAS(file.path(dir.data, las), select = select_option)
    ))

    # Restore original option
    options(lidR.progress = old_opt)

  }

  # Filter duplicates and clean up immediately
  las <- lidR::filter_duplicates(las)


  pb$tick()

  # Establishing center

  if(is.null(x.center)) {

    x.center <- mean(c(las@header@PHB$`Max X`, las@header@PHB$`Min X`))

  }

  if(is.null(y.center)) {

    y.center <- mean(c(las@header@PHB$`Max Y`, las@header@PHB$`Min Y`))

  }


  # Giving the same scale factor to all coordinates

  las@header@PHB[["X scale factor"]] <- 0.001
  las@header@PHB[["Y scale factor"]] <- 0.001
  las@header@PHB[["Z scale factor"]] <- 0.001

  pb$tick()

  # Data filtering at horizontal distances larger than max_dist m in the horizontal plane

  if(!is.null(normalized)) {

    if (!is.null(max.dist)) {

      data <- lidR::clip_circle(las, x.center, y.center, max.dist)

    } else if (!is.null(x.side) && !is.null(y.side)) {

      data <- lidR::clip_rectangle(
        las,
        x.center - (x.side / 2), y.center - (y.side / 2),
        x.center + (x.side / 2), y.center + (y.side / 2)
      )

    } else if (!is.null(xpoly) && !is.null(ypoly)) {

      data <- lidR::clip_polygon(las, xpoly, ypoly)

    } else {

      data <- las

    }

    # Plot if requested
    if (!is.null(plot)) lidR::plot(data)


    # Save LAZ file if required
    if (!is.null(save.las)) {
      lidR::writeLAS(data, paste(dir.result, "/", id, ".laz", sep = ""))
    }

    data <- data.frame(data@data)
    data$slope <- 0
    pb$tick()

  } else {

  # Normalize

  data <- suppressWarnings(suppressMessages(lidR::classify_ground(las, algorithm = lidR::csf(cloth_resolution = csf$cloth_resolution, class_threshold  = 0.2), last_returns = FALSE)))

  pb$tick()


  # Generaion of Digital Terrain Model (DTM)


  if(algorithm.dtm == "knnidw")
    dtm <- suppressWarnings(suppressMessages(lidR::grid_terrain(data, res = res.dtm, algorithm = lidR::knnidw())))


  if(algorithm.dtm == "tin")
    dtm <- suppressWarnings(suppressMessages(lidR::grid_terrain(data, res = res.dtm, algorithm = lidR::tin())))


  dtm[dtm < min(data@data$Z)] <- NA


  pb$tick()

  # Estimating slope

  if(is.na(sf::st_crs(las))){

    raster::crs(dtm) <- "+proj=utm +zone=19 +ellps=GRS80 +datum=NAD83"

  } else {

    raster::crs(dtm) <- sf::st_crs(las)

  }

  slope <- raster::terrain(dtm, opt=c('slope'), unit='radians')

  pb$tick()

  if(mean(slope@data@values, na.rm = TRUE) > 0.25){


    # Normalize

    data <- suppressWarnings(suppressMessages(lidR::classify_ground(las, algorithm = lidR::csf(sloop_smooth = TRUE, class_threshold  = 0.2), last_returns = FALSE)))


    # Generaion of Digital Terrain Model (DTM)

    if(algorithm.dtm == "knnidw")
      dtm <- suppressWarnings(suppressMessages(lidR::grid_terrain(data, res = res.dtm, algorithm = lidR::knnidw())))


    if(algorithm.dtm == "tin")
      dtm <- suppressWarnings(suppressMessages(lidR::grid_terrain(data, res = res.dtm, algorithm = lidR::tin())))


    dtm[dtm < min(data@data$Z)] <- NA


  }


  rm(las)

  pb$tick()


  # Normalization of cloud data

  if(algorithm.dtm == "knnidw"){

      data <- suppressWarnings(suppressMessages(lidR::normalize_height(data, dtm, add_lasattribute = FALSE, na.rm = TRUE)))
      # data <- suppressWarnings(suppressMessages(lidR::normalize_height(data, algorithm = lidR::knnidw(), use_class = c(2L, 9L), dtm = NULL)))

  }

  if(algorithm.dtm == "tin"){

    data <- suppressWarnings(suppressMessages(lidR::normalize_height(data, dtm, add_lasattribute = FALSE, na.rm = TRUE)))
    # data <- suppressWarnings(suppressMessages(lidR::normalize_height(data, algorithm = lidR::tin(), use_class = c(2L, 9L), dtm = NULL)))

  }


  data <- lidR::filter_poi(data, Z >= 0)

  rm(dtm)

  pb$tick()



  # Data filtering at horizontal distances larger than max_dist m in the horizontal plane

  if(!is.null(max.dist)){

    data <- lidR::clip_circle(data, x.center, y.center, max.dist)

    } else if (!is.null(x.side) | !is.null(y.side)){

    data <- lidR::clip_rectangle(data, x.center - (x.side / 2), y.center - (y.side / 2),
                                  x.center + (x.side / 2), y.center + (y.side / 2))

    } else if (!is.null(xpoly) | !is.null(ypoly)){

      data <- lidR::clip_polygon(las, xpoly, ypoly)

  }

  pb$tick()


  # Plot

  if (!is.null(plot)) lidR::plot(data)



  # Saving laz file

  if(!is.null(save.las))
    lidR::writeLAS(data, paste(dir.result, "/", id, ".laz", sep = ""))



  # Assigning slope to point cloud

  data <- lidR::merge_spatial(data, slope, "slope")

  rm(slope)

  pb$tick()

  # Test for getting a smaller file - data frame

  data <- data.frame(data@data)
  data <- data.table::setDT(data)



  }

  # Removing points classified as ground - OPTIMIZED with data.table syntax
  data <- data[Classification == 1]



  # Extracting coordinates values - OPTIMIZED to reduce redundancy
  cols <- c("X", "Y", "Z", "slope")
  col_names <- c("x", "y", "z", "slope")

  if (!is.null(intensity)) {
    cols <- c(cols, "Intensity")
    col_names <- c(col_names, "intensity")
  }

  if (!is.null(RGB)) {
    cols <- c(cols, "R", "G", "B")
    col_names <- c(col_names, "R", "G", "B")
  }

  data <- data[, cols, with = FALSE]
  data.table::setnames(data, old = cols, new = col_names)


  # Low and high point filtering - OPTIMIZED with data.table chaining
  if(!is.null(min.height)) {
    data <- data[z > min.height]
  }

  if(!is.null(max.height)) {
    data <- data[z < max.height]
  }

  # Transformation to other coordinate systems - OPTIMIZED
  # Cylindrical coordinate system
  # Pre-compute differences to avoid redundant calculations
  dx <- data$x - x.center
  dy <- data$y - y.center

  data[, rho := sqrt(dx^2 + dy^2)]
  data[, phi := atan2(dy, dx)]
  data[phi < 0, phi := phi + 2 * pi]

  # Spherical coordinates system
  data[, r := sqrt(z^2 + rho^2)]
  data[, theta := atan2(z, rho)]

  data[, point := .I]


  # Green Leaf Algorithm (GLA) (Louhaichi et al., (2001))
  if(!is.null(RGB))
    data[, GLA := (2 * G - R - B) / (2 * G + R + B)]


  # Point crooping process (TLS single-scan)

  if(scan.approach == "single"){

    data[, prob := (r / max(r))^2]
    data[, prob.random := stats::runif(.N)]
    data[, prob.selec := as.integer(prob > prob.random)]
  }


  # For the rest of situations, point cloud is downsampled by voxelization

  if(scan.approach == "multi" & !is.null(voxel_size)){

    # Determine columns for voxelization
    voxel_cols <- c("x", "y", "z", "slope")
    voxel_names <- c("x", "y", "z", "slope")

    if (!is.null(intensity)) {
      voxel_cols <- c(voxel_cols, "intensity")
      voxel_names <- c(voxel_names, "intensity")
    }

    if (!is.null(RGB)) {
      voxel_cols <- c(voxel_cols, "R", "G", "B")
      voxel_names <- c(voxel_names, "R", "G", "B")
    }

    data <- as.data.frame(voxel_grid_downsampling(as.matrix(data[, voxel_cols, with = FALSE]), voxel_size))
    colnames(data) <- voxel_names
    data <- data.table::setDT(data)

    # Recalculate coordinates - OPTIMIZED (fixed bug: swapped arguments)
    dx <- data$x - x.center
    dy <- data$y - y.center

    data[, rho := sqrt(dx^2 + dy^2)]
    data[, phi := atan2(dy, dx)]  # BUGFIX: was atan2(dx, dy)
    data[phi < 0, phi := phi + 2 * pi]
    data[, r := sqrt(z^2 + rho^2)]
    data[, theta := atan2(z, rho)]

    data[, point := .I]
  }


  if(scan.approach == "multi"){
    data[, prob := stats::runif(.N)]
    data[, prob.selec := as.integer(prob > 0.5)]
  }


  # Assign id

  if(!is.null(id)){
    data[, id := id]
  } else {
    data[, id := 1L]
  }


  # File name

  if(!is.null(file)){
    data[, file := file]
  } else {
    data[, file := paste0(id[1], ".txt")]
  }

  # Reorder columns - OPTIMIZED to reduce redundancy
  final_cols <- c("id", "file", "point", "x", "y", "z", "rho", "phi", "r", "theta", "slope")

  if (!is.null(intensity)) {
    final_cols <- c(final_cols, "intensity")
  }

  if (!is.null(RGB)) {
    final_cols <- c(final_cols, "R", "G", "B", "GLA")
  }

  final_cols <- c(final_cols, "prob", "prob.selec")

  data <- data[, final_cols, with = FALSE]

  pb$tick()


  # Adding the point of the plot center - OPTIMIZED direct creation

  new_row <- data.table::data.table(
    id = data$id[1],
    file = data$file[1],
    point = 0L,
    x = x.center,
    y = y.center,
    z = 0,
    rho = 0,
    phi = 0,
    r = 0,
    theta = 0,
    slope = NA_real_,
    prob = 1,
    prob.selec = 1L
  )

  # Add optional columns
  if (!is.null(intensity)) {
    new_row[, intensity := NA_real_]
  }

  if (!is.null(RGB)) {
    new_row[, c("R", "G", "B", "GLA") := list(NA_real_, NA_real_, NA_real_, NA_real_)]
  }

  # Reorder to match data columns
  data.table::setcolorder(new_row, names(data))

  # Bind it to the top of the existing data
  data <- rbind(new_row, data)


  # Saving data

  if(isTRUE(save.result)){

    data.red <- data[prob.selec == 1]

    vroom::vroom_write(data.red, file = file.path(dir.result, data.red$file[1]), delim = ",", progress = FALSE)

  }

  pb$tick()

  return(data)

}

module mymod
   implicit none
   private
   public mytype, mytype2
   integer, public :: int1, int2, int3, int4, int5
   type :: mytype
      integer :: comp
   end type mytype
   type :: mytype2
      integer :: comp
   end type mytype2
   interface
      subroutine sub()
         import int1
         import mytype, int2
         type(mytype) :: some
      end subroutine sub
   end interface
end module mymod

program test_external
    implicit none
    REAL, EXTERNAL :: VAL
    REAL         VAR_A
    EXTERNAL     VAR_A
    EXTERNAL     VAR_B
    REAL         VAR_B
    EXTERNAL     VAR_B   ! throw error
    REAL         VAR_A   ! throw error
    EXTERNAL     VAR_C
end program test_external

.sections.single.scan <- function(cut,
                                  .cut,
                                  .alpha.v,
                                  .alpha.h,
                                  .dbh.min,
                                  .dbh.max,
                                  slice,
                                  bark.roughness,
                                  x.center,
                                  y.center){

  .filter <- data.frame(cluster = as.numeric(),

                        # Center coordinates
                        center.x = as.numeric(), center.y = as.numeric(),
                        center.phi = as.numeric(), center.rho = as.numeric(),
                        center.r = as.numeric(), center.theta = as.numeric(),

                        # Radius
                        radius = as.numeric(),

                        # Number of points belonging to cluster (raw and after point cropping)
                        n.pts = as.numeric(), n.pts.red = as.numeric(),

                        # Phi coordinates of left and right
                        phi.left = as.numeric(), phi.right = as.numeric(),

                        # Circumference arc
                        arc.circ = as.numeric(),

                        # Partial occlusion
                        sec = as.numeric())


  # Select cluster i
  .dat <- cut

  # First filter

  .n <- (slice / (tan(.alpha.v / 2) * (mean(.dat$r) / cos(mean(.cut$slope, na.rm = TRUE))) * 2))

  if(nrow(.dat) < .n){return(.filter)}

  # Second filter

  # Compute rho coordinates for section ends

  if((max(.dat$phi) - min(.dat$phi)) < pi){

    .dat.2 <- .dat[order(.dat$phi, decreasing = F), ]

  } else {

    .dat.2 <- .dat
    .dat.2$phi <- ifelse(.dat.2$phi < 1, .dat.2$phi + (2 * pi), .dat.2$phi)
    .dat.2 <- .dat.2[order(.dat.2$phi, decreasing = F), ]

  }

  .dat.3 <- .dat[order(.dat$theta, decreasing = F), ]

  .dist <- diff(.dat.2$phi)
  .dist.2 <- diff(.dat.3$theta)

  .dist <- sd(.dist) / .alpha.h
  .dist.2 <- sd(.dist.2) / .alpha.v
  .dist <- mean(c(.dist, .dist.2), na.rm = TRUE)


  if(.dist > 1){return(.filter)}

  rm(.dat.3, .dist, .dist.2)

  # Generate mesh

  .x.rang <- max(.dat$x) - min(.dat$x)
  .y.rang <- max(.dat$y) - min(.dat$y)

  # Compute centroids coordinates with respect to TLS

  .x.cent <- (.x.rang / 2) + min(.dat$x)
  .y.cent <- (.y.rang / 2) + min(.dat$y)

  # Obtain width for mesh to be applied in the cluster
  .ancho.malla <- (max(.x.rang, .y.rang) / 2) * 1.5

  # Maximal and minimal mesh coordinates
  .xmin <- .x.cent - .ancho.malla
  .ymin <- .y.cent - .ancho.malla
  .xmax <- .x.cent + .ancho.malla
  .ymax <- .y.cent + .ancho.malla


  # Filter

  .h <- 2 * (tan(.alpha.h / 2) * (mean(.dat$r) / cos(mean(.cut$slope, na.rm = TRUE))) * 2)

  .x.values <- seq(from = .xmin, to = .xmax, by = .h)
  .y.values <- seq(from = .ymin, to = .ymax, by = .h)

  .h <- .h / 2

  .density <- matrix(0, ncol = length(.x.values), nrow = length(.y.values))



  for(i in 1:length(.x.values)){
    for(j in 1:length(.y.values)){

      # .den <- .dat[.dat$x <= .x.values[i] + .h &
      #                .dat$x >  .x.values[i] - .h &
      #                .dat$y <= .y.values[j] + .h &
      #                .dat$y >  .y.values[j] - .h, , drop = FALSE]

      .den <- .dat[x <= .x.values[i] + .h &
                     x >  .x.values[i] - .h &
                     y <= .y.values[j] + .h &
                     y >  .y.values[j] - .h, ]

      # Discard cells with less than 2 points for computing mean points density by cell
      .density[j, i] <- ifelse(nrow(.den) < 1, NA, nrow(.den))
    }
  }



  # Estimate mean density by cell
  .threeshold <- stats::quantile(.density, prob = 0.1, na.rm = T)

  if(is.nan(.threeshold) | is.na(.threeshold)){return(.filter)}

  .density <- matrix(0, ncol = length(.x.values), nrow = length(.y.values))

  .remove <- data.frame(point = as.numeric())


  for(i in 1:length(.x.values)){
    for(j in 1:length(.y.values)){

      # .den <- .dat[.dat$x <= .x.values[i] + .h &
      #                .dat$x >  .x.values[i] - .h &
      #                .dat$y <= .y.values[j] + .h &
      #                .dat$y >  .y.values[j] - .h, , drop = FALSE]

      .den <- .dat[x <= .x.values[i] + .h &
                     x >  .x.values[i] - .h &
                     y <= .y.values[j] + .h &
                     y >  .y.values[j] - .h, ]

      # Discard cells with less than 2 points for computing mean density by
      # cell
      .density[j, i] <- ifelse(nrow(.den) < 1, NA, nrow(.den))

      if(nrow(.den) > .threeshold){

        .rem <- data.frame(point = .den$point)
        .remove <- rbind(.remove, .rem)
      }
    }
  }

  .dat <- merge(.dat, .remove, by = "point", all.y = TRUE)

  .dat <- .dat[!duplicated(.dat$point), ]


  if(nrow(.dat) < .n){return(.filter)}


  # Ratio points

  if(mean(.cut$slope, na.rm = TRUE) > 0.5){.n <- 0.7 * .n}


  # Obtain phi and rho coordinates corresponding to mesh intersections
  .x2.values <- seq(from = min(.dat$phi), to = max(.dat$phi), by = .alpha.h)

  # Define matrix where points number by cell will be stored
  .density <- vector(length = length(.x2.values))

  .remove <- data.frame(point = as.numeric())

  for(i in 1:length(.x2.values)){

    .den <- .dat[.dat$phi <= .x2.values[i] + (.alpha.h/2) &
                 .dat$phi >  .x2.values[i] - (.alpha.h/2), ]

    # Aquellas celdas con menos de 2 puntos no las tengo en cuenta
    # para luego mas tarde calcular la densidad media por celda
    .density[i] <- ifelse(nrow(.den) < 1, NA, nrow(.den))


    if(nrow(.den) > 1){

      .rem <- data.frame(point = .den$point)
      .remove <- rbind(.remove, .rem)

    }

  }


  # Estimate mean density by cell
  .density <- ifelse(is.nan(.density), NA, .density)

  if(is.nan(mean(.density, na.rm = TRUE))){return(.filter)}

  if(max(.density[!is.na(.density)], na.rm = T) < floor(.n)){return(.filter)}

  # Remove cells containing only 1 point
  .dat <- merge(.dat, .remove, by = "point", all.y = TRUE)

  # If no points remain in .dat after removing, go to next iteration
  if(nrow(.dat) < .n){return(.filter)}

  # Estimate points number for both the original cloud (.n.pts) and the
  # point cloud reduced by the point cropping process (.n.pts.red)
  .n.pts <- nrow(.dat)
  .n.pts.red <- nrow(.dat[.dat$prob.selec == 1, , drop = FALSE])

  # After this previous filtering, compute cluster centroid

  .x.values <- seq(from = .xmin, to = .xmax, by = 0.01)
  .y.values <- seq(from = .ymin, to = .ymax, by = 0.01)

  # Create an empty matrix where, for each mesh intersection, variance of
  # distances between points and corresponding intersection will be stored
  .matriz <- matrix(0, ncol = length(.x.values), nrow = length(.y.values))

  # for(i in 1:length(.x.values)){
  #   for(j in 1:length(.y.values)){
  #
  #     .variance <- stats::var(raster::pointDistance(cbind(.dat$x,.dat$y), c(.x.values[i], .y.values[j]), lonlat=FALSE))
  #     .matriz[j, i] <- .variance
  #
  #   }
  # }

  for(i in 1:length(.x.values)){
    for(j in 1:length(.y.values)){

      # rst_pnts = raster::pointDistance(p1 = cbind(.dat$x,.dat$y),
      #                                  p2 = c(.x.values[i], .y.values[j]),
      #                                  lonlat=FALSE)

      rst_pnts = sqrt((.dat[["x"]] - .x.values[i])^2 + (.dat[["y"]] - .y.values[j])^2)

      .variance <- stats::var(rst_pnts)
      .matriz[j, i] <- .variance

    }
  }

  # Consider as section center the intesection where variance is minimal
  .a <- which(.matriz == min(.matriz), arr.ind = TRUE)

  .center.x <- .x.values[.a[2]]
  .center.y <- .y.values[.a[1]]


  # Distances between points and center
  # .dat$dist <- raster::pointDistance(cbind(.dat$x,.dat$y), c(.x.values[.a[2]], .y.values[.a[1]]), lonlat = FALSE)
  .dat$dist = sqrt((.dat[["x"]] - .x.values[.a[2]])^2 + (.dat[["y"]] - .y.values[.a[1]])^2)


  # dat.i <- rep(list(as.matrix(.dat[, c("x", "y")])), 600)
  # kk <- try(do.call(rbind, (lapply(dat.i, .RANSAC))), silent = TRUE)
  # rm(dat.i)

  kk = try(iterations_RANSAC(data = as.matrix(.dat[, c("x", "y")]), n_iterations = 600) |>
             data.table::as.data.table(), silent = TRUE)


  if(class(kk)[1] == "try-error"){

    if(is.null(bark.roughness)){

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    } else if(bark.roughness == 1){

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    } else if(bark.roughness == 2){

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    } else {

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    }

    if(is.na(.cv) | .radio <= 0 | is.na(.radio)){return(.filter)}


  } else {

    colnames(kk) = c('x', 'y', 'radio', 'n', 'mae', 'cv')

    kk <- kk[kk$n >= max(kk$n), ]

    if(nrow(kk) > 1)
      kk <- kk[kk$mae <= min(kk$mae), ]

    kk <- kk[1, ]

    # .dat$distRANSAC <- raster::pointDistance(cbind(.dat$x,.dat$y), c(kk$x, kk$y), lonlat = FALSE)
    .dat$distRANSAC = sqrt((.dat[["x"]] - kk$x)^2 + (.dat[["y"]] - kk$y)^2)

    # Radius value as the mean distance
    if(is.null(bark.roughness)){
      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio
      .radioRANSAC <- kk$radio
      .cvRANSAC <- kk$cv
      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999}

    } else if(bark.roughness == 1){
      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio
      .radioRANSAC <- kk$radio
      .cvRANSAC <- kk$cv
      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999}

    } else if(bark.roughness == 2){
      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio
      .radioRANSAC <- kk$radio
      .cvRANSAC <- kk$cv
      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999}

    } else {
      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio
      # .radioRANSAC <- .centerRANSAC$radius
      .radioRANSAC <- kk$radio
      # .cvRANSAC <- stats::sd(.dat$distRANSAC[.dat$distRANSAC>stats::quantile(.dat$distRANSAC, prob = 0.75, na.rm = T) & .dat$distRANSAC<stats::quantile(.dat$distRANSAC, prob = 0.95, na.rm = T)]) / .radioRANSAC
      .cvRANSAC <- kk$cv
      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999}
    }

    if(is.na(.cv)){return(.filter)}

    if(1 * .cv >= .cvRANSAC){

      .radio <- .radioRANSAC
      .cv <- .cvRANSAC
      .dat$dist <- .dat$distRANSAC

      .center.x <- kk$x
      .center.y <- kk$y}

    .dat <- .dat[, 1:(ncol(.dat)-1)]

    if(.radio <= 0 | is.na(.radio)){return(.filter)}

  }

  .center.phi <- atan2(.center.y-y.center, .center.x-x.center)
  .center.phi <- ifelse(.center.phi < 0, .center.phi + (2 * pi), .center.phi)
  .center.rho <- sqrt((.center.x-x.center) ^ 2 + (.center.y-y.center) ^ 2)
  .center.r <- sqrt(.dat$sec[1] ^ 2 + .center.rho ^ 2)
  .center.theta <- atan2(.dat$sec[1], .center.rho)

  if(is.na(.cv) | .cv > 0.1 | length(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T)]) < 2) {return(.filter)}

  # Center behind tree surface
  if(stats::quantile(.dat$rho, prob = 0.05, na.rm = T) > .center.r) {return(.filter)}

  # At least 95 % of distances should be greater than .radio / 2
  if(stats::quantile(.dat$dist, prob = 0.05, na.rm = T) < (.radio / 2)) {return(.filter)}


  # Select 1st percentil, if necessary for strange points
  # It remains to be seen what happens if cluster is located in 0 +/- phi
  .pto.left <- stats::quantile(.dat.2$phi, prob = 0.01, na.rm = T)
  .rho.left <- mean(.dat.2$rho[which(.dat.2$phi <= .pto.left)])
  .phi.left <- mean(.dat.2$phi[which(.dat.2$phi <= .pto.left)])

  # Select 99th percentil, if necessary for strange points
  .pto.right <- stats::quantile(.dat.2$phi, prob = 0.99, na.rm = T)
  .rho.right <- mean(.dat.2$rho[which(.dat.2$phi >= .pto.right)])
  .phi.right <- mean(.dat.2$phi[which(.dat.2$phi >= .pto.right)])

  # For points in section center, select those in half the angle aperture
  # phi +/- TLS aperture .alpha
  .phi.cent <- max(.dat.2$phi) - ((max(.dat.2$phi) - min(.dat.2$phi)) / 2)
  .rho.cent <- mean(.dat.2$rho[which(round(.dat.2$phi, 3) >= round(.phi.cent - .alpha.h, 3) & round(.dat.2$phi, 3) <= round(.phi.cent + .alpha.h, 3))])

  if(is.nan(.rho.cent)){return(.filter)}

  # Check rho coordinates for ends are greater than center ones
  .arc.circ <- ifelse(.rho.left > .rho.cent & .rho.right > .rho.cent, 1, 0)

  # Convert original coordinates if cluster is located in 0 +/- phi
  .phi.left <- ifelse(.phi.left > (2 * pi), .phi.left - (2 * pi), .phi.left)
  .phi.right <- ifelse(.phi.right > (2 * pi), .phi.right - (2 * pi), .phi.right)

  # If the complete circumference arc is not detected (so previous criterion
  # is not satisfied), check if there is a partial occlusion.
  # If they belong to a tree section, they are found with a systematic
  # regularity so very high correlations between their correlative
  # numeration and phi must be found when they are ordered with respect to
  # phi
  .dat.2$n <- c(1:nrow(.dat.2))
  .cor <- try(stats::cor.test(x = .dat.2$n, y = .dat.2$phi, method = 'pearson'), silent = TRUE)

  # If error, go to next iteration
  if(methods::is(.cor) == "try-error"){return(.filter)} else{

    .occlusion <- .cor[[4]]

  }


  # Zhang et al., (2019)
  .n.w.ratio <- stats::sd(.dat$z) / sqrt(stats::sd(.dat$x) ^ 2 + stats::sd(.dat$y) ^ 2)

  if(.n.w.ratio > 1 | is.nan(.n.w.ratio)){return(.filter)}

  if(nrow(.dat) < .n){return(.filter)}


  # Results
  .filter <- data.frame(cluster = .dat$cluster[1],

                        center.x = .center.x, center.y = .center.y,
                        center.phi = .center.phi, center.rho = .center.rho,
                        center.r = .center.r, center.theta = .center.theta,

                        radius = .radio,

                        n.pts = .n.pts, n.pts.red = .n.pts.red,

                        phi.left = .phi.left, phi.right = .phi.right,

                        arc.circ = .arc.circ, occlusion = .occlusion)


  # Arch of circumference or partial arch of circumference?
  .filter$tree <- ifelse(.filter$arc.circ == 1, 1,
                         ifelse(.filter$arc.circ == 0 & .filter$occlusion > 0.975, 1, 0))
  .filter <- .filter[which(.filter$tree == 1), , drop = FALSE]


  if(nrow(.filter) < 1){

    .filter1.0 <- data.frame(cluster = as.numeric(),
                             center.x = as.numeric(), center.y = as.numeric(),
                             center.phi = as.numeric(), center.rho = as.numeric(),
                             center.r = as.numeric(), center.theta = as.numeric(),
                             radius = as.numeric(),
                             n.pts = as.numeric(), n.pts.red = as.numeric(),
                             phi.left = as.numeric(), phi.right = as.numeric(),
                             arc.cir = as.numeric(), sec = as.numeric())

  } else{

    .filter1.0 <- .filter[, c("cluster",
                              "center.x", "center.y", "center.phi", "center.rho", "center.r", "center.theta",
                              "radius", "n.pts", "n.pts.red", "phi.left", "phi.right", "arc.circ"), drop = FALSE]
    .filter1.0$sec <- cut$sec[1]

  }

  return(.filter1.0)

}


.sections.multi.scan <- function(cut,
                                 tls.precision,
                                 .dbh.min,
                                 .dbh.max,
                                 slice,
                                 bark.roughness,
                                 x.center,
                                 y.center) {

  .filter <- data.frame(cluster = as.numeric(),

                        # Center coordinates
                        center.x = as.numeric(), center.y = as.numeric(),
                        center.phi = as.numeric(), center.rho = as.numeric(),
                        center.r = as.numeric(), center.theta = as.numeric(),

                        # Radius
                        radius = as.numeric(),

                        # Number of points belonging to clusters (raw and after point cropping)
                        n.pts = as.numeric(), n.pts.red = as.numeric(),

                        # Circumference
                        circ = as.numeric(),

                        # Circumference arc
                        arc.circ = as.numeric(),

                        # Partial occlusion
                        occlusion = as.numeric())

  .dat <- cut


  if(nrow(.dat) < 10){return(.filter)}

  # Optional, just to see what you are doing here:
  # plot(cut$x, cut$y, asp = 1)

  # Generate a mesh over the points

  .x.rang <- max(.dat$x) - min(.dat$x)
  .y.rang <- max(.dat$y) - min(.dat$y)


  # Compute centroids coordinates with respect to TLS

  .x.cent <- (.x.rang / 2) + min(.dat$x)
  .y.cent <- (.y.rang / 2) + min(.dat$y)


  # Obtain width for mesh to be applied in the cluster

  .ancho.malla <- (max(.x.rang, .y.rang) / 2) * 1.5

  # Maximal and minimal mesh coordinates

  .xmin <- .x.cent - .ancho.malla
  .ymin <- .y.cent - .ancho.malla
  .xmax <- .x.cent + .ancho.malla
  .ymax <- .y.cent + .ancho.malla

  # Filter

  if(is.null(tls.precision)){.h <- 0.03} else {.h <- tls.precision * 2 }

  .x.values <- seq(from = .xmin, to = .xmax, by = .h)
  .y.values <- seq(from = .ymin, to = .ymax, by = .h)


  .h <- .h / 2

  # Optional, just to see what you are doing here:
  # abline(v = .x.values + .h, h = .y.values + .h)

  .density <- matrix(0, ncol = length(.x.values), nrow = length(.y.values))


  for(i in 1:length(.x.values)){
    for(j in 1:length(.y.values)){

      # .den <- .dat[.dat$x <= .x.values[i] + .h &
      #                .dat$x >  .x.values[i] - .h &
      #                .dat$y <= .y.values[j] + .h &
      #                .dat$y >  .y.values[j] - .h, , drop = FALSE]

      .den <- .dat[x <= .x.values[i] + .h &
                     x >  .x.values[i] - .h &
                     y <= .y.values[j] + .h &
                     y >  .y.values[j] - .h, ]

      # Discard cells with less than 2 points for computing mean points density by cell
      .density[j, i] <- ifelse(nrow(.den) < 1, NA, nrow(.den))
    }
  }


  # Estimate mean density by cell
  # .threeshold <- stats::median(.density, na.rm = T)
  # .threeshold <- mean(.density, na.rm = T)
  .threeshold <- stats::quantile(.density, prob = 0.1, na.rm = T)

  if(is.nan(.threeshold) | is.na(.threeshold)){return(.filter)}

  .density <- matrix(0, ncol = length(.x.values), nrow = length(.y.values))
  .remove <- data.frame(point = as.numeric())

  for(i in 1:length(.x.values)){
    for(j in 1:length(.y.values)){

      # .den <- .dat[.dat$x <= .x.values[i] + .h &
      #                .dat$x >  .x.values[i] - .h &
      #                .dat$y <= .y.values[j] + .h &
      #                .dat$y >  .y.values[j] - .h, , drop = FALSE]

      .den <- .dat[x <= .x.values[i] + .h &
                     x >  .x.values[i] - .h &
                     y <= .y.values[j] + .h &
                     y >  .y.values[j] - .h, ]

      # Discard cells with less than 2 points for computing mean density by
      # cell
      .density[j, i] <- ifelse(nrow(.den) < 1, NA, nrow(.den))

      if(nrow(.den) > .threeshold){

        .rem <- data.frame(point = .den$point)
        .remove <- rbind(.remove, .rem)
      }
    }
  }

  .dat <- merge(.dat, .remove, by = "point", all.y = TRUE)
  .dat <- .dat[!duplicated(.dat$point), ]

  # Optional, just to see what you are doing here:
  # points(.dat$x, .dat$y, col = "green")

  if(nrow(.dat) < 10){return(.filter)}

  # Estimate points number for both the original cloud (.n.pts) and the
  # point cloud reduced by the point cropping process (.n.pts.red)
  .n.pts <- nrow(.dat)
  .n.pts.red <- nrow(.dat[.dat$prob.selec == 1, , drop = FALSE])

  # After this previous filtering, compute cluster centroid

  .x.values <- seq(from = .xmin, to = .xmax, by = 0.01)
  .y.values <- seq(from = .ymin, to = .ymax, by = 0.01)

  # Optional, just to see what you are doing here:
  # plot(.dat$x, .dat$y, asp = 1)
  # abline(v = .x.values, h = .y.values)

  # Create an empty matrix where, for each mesh intersection, variance of
  # distances between points and corresponding intersection will be stored
  .matriz <- matrix(0, ncol = length(.x.values), nrow = length(.y.values))

  for(i in 1:length(.x.values)){
    for(j in 1:length(.y.values)){

      # rst_pnts = raster::pointDistance(p1 = cbind(.dat$x,.dat$y),
      #                                  p2 = c(.x.values[i], .y.values[j]),
      #                                  lonlat=FALSE)

      rst_pnts = sqrt((.dat[["x"]] - .x.values[i])^2 + (.dat[["y"]] - .y.values[j])^2)

      .variance <- stats::var(rst_pnts)
      .matriz[j, i] <- .variance

    }
  }

  # Consider as section center the intesection where variance is minimal,
  # which will be a center candidate

  .a <- which(.matriz == min(.matriz), arr.ind = TRUE)

  .center.x <- .x.values[.a[2]]
  .center.y <- .y.values[.a[1]]

  # Optional, just to see what you are doing here:
  # points(.center.x, .center.y, pch = 19, col = "red")

  # Distances between points and the condidate center
  # .dat$dist <- raster::pointDistance(cbind(.dat$x,.dat$y), c(.x.values[.a[2]], .y.values[.a[1]]), lonlat = FALSE)
  .dat$dist = sqrt((.dat[["x"]] - .x.values[.a[2]])^2 + (.dat[["y"]] - .y.values[.a[1]])^2)

  # data.table::fwrite(x = .dat, file = '/home/lampros/Desktop/Upwork/FREELANCER_Platform/2024_09_27_R_Package_Speed_Optimization_Expert/2024_11_16_UPDATED_TASK/benchmark/tmp.csv', row.names = F)


  # Application of the ransac algorithm to select another possible candidate for section circumference
  # We apply the function RANSAC() 600 times to generate 600 candidates

  # dat.i <- rep(list(as.matrix(.dat[, c("x", "y")])), 600)
  # kk <- try(do.call(rbind, (lapply(dat.i, .RANSAC))), silent = TRUE)
  #
  # rm(dat.i)

  kk = try(iterations_RANSAC(data = as.matrix(.dat[, c("x", "y")]), n_iterations = 600) |>
    data.table::as.data.table(), silent = TRUE)
  # colnames(kk) = c('x', 'y', 'radio', 'n', 'mae', 'cv')

  # If the RANSAC algorithm does not work, we retain the first candidate as circumference,
  # calculating the radio and coeficient of variation (cv).

  if(class(kk)[1] == "try-error"){

    if(is.null(bark.roughness)){

      # The radius of the circumference will be the average of all the distances from the center to the points of the section:

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    } else if(bark.roughness == 1){

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    } else if(bark.roughness == 2){

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    } else {

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

    }

    if(is.na(.cv) | .radio <= 0 | is.na(.radio)){return(.filter)}

    # If the RANSAC algorithm works, we retain both candidate as circumference,
    # calculating the radio and coeficient of variation (cv) for both.

  } else {

    # For RANSAC, we keep the circunference with lowest error (mae)
    # and maximum number of inliers (n)

    colnames(kk) = c('x', 'y', 'radio', 'n', 'mae', 'cv')

    kk <- kk[kk$n >= max(kk$n), ]

    if(nrow(kk) > 1)
      kk <- kk[kk$mae <= min(kk$mae), ]

    kk <- kk[1, ]

    # .dat$distRANSAC <- raster::pointDistance(cbind(.dat$x,.dat$y), c(kk$x, kk$y), lonlat = FALSE)
    .dat$distRANSAC = sqrt((.dat[["x"]] - kk$x)^2 + (.dat[["y"]] - kk$y)^2)

    if(is.null(bark.roughness)){

      # The radius of the circumference will be the average of all the distances from the center to the points of the section:

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.05, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

      # In the case of RANSAC, we already have an estimated radius:

      .radioRANSAC <- kk$radio
      .cvRANSAC <- kk$cv

      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999} # To avoid NA values

    } else if(bark.roughness == 1){

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

      .radioRANSAC <- kk$radio
      .cvRANSAC <- kk$cv

      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999}

    } else if(bark.roughness == 2){

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.5, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

      .radioRANSAC <- kk$radio
      .cvRANSAC <- kk$cv

      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999}

    } else {

      .radio <- mean(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)])
      .cv <- stats::sd(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.75, na.rm = T) & .dat$dist<stats::quantile(.dat$dist, prob = 0.99, na.rm = T)]) / .radio

      .radioRANSAC <- kk$radio
      .cvRANSAC <- kk$cv

      if(is.na(.cvRANSAC)){.cvRANSAC <- 9999}

    }

    if(is.na(.cv)){return(.filter)} # To avoid NA values

    # I keep the candidate with lowest coefficient of variation,
    # favoring RANSAC in case of equality

    if(.cv >= .cvRANSAC){

      .radio <- .radioRANSAC
      .cv <- .cvRANSAC
      .dat$dist <- .dat$distRANSAC

      .center.x <- kk$x
      .center.y <- kk$y}

    .dat <- .dat[, 1:(ncol(.dat)-1)]

    if(.radio <= 0 | is.na(.radio)){return(.filter)}

  }

  # Calculation of other coordinates of the center of the section
  # with respect to the plot center

  .center.phi <- atan2(.center.y-y.center, .center.x-x.center)
  .center.phi <- ifelse(.center.phi < 0, .center.phi + (2 * pi), .center.phi)
  .center.rho <- sqrt((.center.x-x.center) ^ 2 + (.center.y-y.center) ^ 2)
  .center.r <- sqrt(.dat$sec[1] ^ 2 + .center.rho ^ 2)
  .center.theta <- atan2(.dat$sec[1], .center.rho)


  # Application of some quality criteria to make sure that the point cloud belongs to a tree section

  if(is.na(.cv) | .cv > 0.15 | length(.dat$dist[.dat$dist>stats::quantile(.dat$dist, prob = 0.25, na.rm = T)]) < 2) {return(.filter)}



  # At least 95 % of distances should be greater than .radio / 2 for trees larger than 20 cm at dbh

  if(.radio > 0.1){if(stats::quantile(.dat$dist, prob = 0.05, na.rm = T) < (.radio / 2)){return(.filter)}}

  if(.radio < 0.1){if(stats::quantile(.dat$dist, prob = 0.01, na.rm = T) < (.radio / 2)){return(.filter)}}


  .dat.2 <- .dat[order(.dat$x, .dat$y, decreasing = F), ]

  .dat.2$a <- sqrt((.dat.2$x - .dat.2$x[1])^2+(.dat.2$y - .dat.2$y[1])^2)
  .dat.2$b <- sqrt((.dat.2$x[1] - .center.x)^2+(.dat.2$y[1] - .center.y)^2)
  .dat.2$c <- sqrt((.dat.2$x - .center.x)^2+(.dat.2$y - .center.y)^2)

  .dat.2$alpha <- suppressWarnings(acos(-(.dat.2$a^2-.dat.2$b^2-.dat.2$c^2)/(2*.dat.2$b*.dat.2$c)))

  # Checking if tree section belong to a full circumference
  # I check that there are points in the four quadrants of the circumference.

  k1 <- ifelse(nrow(.dat[.dat$x > .center.x & .dat$y > .center.y, ]) > 0, 1, 0)
  k2 <- ifelse(nrow(.dat[.dat$x > .center.x & .dat$y < .center.y, ]) > 0, 1, 0)
  k3 <- ifelse(nrow(.dat[.dat$x < .center.x & .dat$y > .center.y, ]) > 0, 1, 0)
  k4 <- ifelse(nrow(.dat[.dat$x < .center.x & .dat$y < .center.y, ]) > 0, 1, 0)

  # If there are points in all four quadrants, it is considered a complete circle:

  .circ <- ifelse(sum(k1, k2, k3, k4) == 4, 1, 0)


  # If there are only points in 3 or 2 quadrants, I consider it to be an arc of a circle:

  .arc.circ <- ifelse(sum(k1, k2, k3, k4) == 3 | sum(k1, k2, k3, k4) == 2, 1, 0)

  # If the complete circumference arc is not detected (so previous criterion
  # is not satisfied), check if there is a partial occlusion.
  # If they belong to a tree section, they are found with a systematic
  # regularity so very high correlations between their correlative
  # numeration and phi must be found when they are ordered with respect to phi

  .dat.2 <- .dat.2[order(.dat.2$alpha, decreasing = F), ]

  .dat.2$n <- c(1:nrow(.dat.2))
  .cor <- try(stats::cor.test(x = .dat.2$n, y = .dat.2$alpha, method = 'pearson'), silent = TRUE) # cor function could be used instead


  # If error, go to next iteration
  if(methods::is(.cor) == "try-error"){return(.filter)} else{

    .occlusion <- .cor[[4]]
  }

  # Zhang et al., (2019) - other selection criteria:

  .n.w.ratio <- stats::sd(.dat$z) / sqrt(stats::sd(.dat$x) ^ 2 + stats::sd(.dat$y) ^ 2)

  if(.n.w.ratio > 1 | .n.w.ratio < 0.1 | is.nan(.n.w.ratio)){return(.filter)}


  .densidad_radio <- .n.pts.red / .radio

  if(nrow(.dat) < 10){return(.filter)}

  # Results
  .filter <- data.frame(cluster = .dat$cluster[1],

                        center.x = .center.x, center.y = .center.y,
                        center.phi = .center.phi, center.rho = .center.rho,
                        center.r = .center.r, center.theta = .center.theta,

                        radius = .radio,

                        n.pts = .n.pts, n.pts.red = .n.pts.red,

                        circ = .circ, arc.circ = .arc.circ, occlusion = .occlusion,

                        density.radio = .densidad_radio)


  # Arch of circumference or partial arch of circumference?
  .Q1 <- stats::quantile(.filter$density.radio, prob = 0.25, na.rm = T)
  .Q3 <- stats::quantile(.filter$density.radio, prob = 0.75, na.rm = T)
  .outliers <- .Q1 - 1.5 * (.Q3 - .Q1)

  .filter$tree <- ifelse(.filter$circ == 1 & .filter$density.radio >= .outliers, 1,
                         ifelse(.filter$arc.circ == 1 & .filter$occlusion >= 0.95 & .filter$density.radio >= .outliers, 1,
                                ifelse(.filter$circ == 0 & .filter$arc.circ == 0 & .filter$occlusion >= 0.975 & .filter$density.radio >= .outliers, 1, 0)))
  .filter <- .filter[.filter$tree == 1, , drop = FALSE]

  if(nrow(.filter) < 1){

    .filter1.0 <- data.frame(cluster = as.numeric(),
                             center.x = as.numeric(), center.y = as.numeric(),
                             center.phi = as.numeric(), center.rho = as.numeric(),
                             center.r = as.numeric(), center.theta = as.numeric(),
                             radius = as.numeric(),
                             n.pts = as.numeric(), n.pts.red = as.numeric(),
                             circ = as.numeric(), arc.circ = as.numeric(), sec = as.numeric())

  } else{

    .filter1.0 <- .filter[, c("cluster",
                              "center.x", "center.y", "center.phi", "center.rho", "center.r", "center.theta",
                              "radius", "n.pts", "n.pts.red", "circ", "arc.circ"), drop = FALSE]

    .filter1.0$sec <- cut$sec[1]
  }

  return(.filter1.0)

}

.n.w.ratio <- function(stem){

  n.w.ratio <- stats::sd(stem$z) / sqrt(stats::sd(stem$x) ^ 2 + stats::sd(stem$y) ^ 2)
  out <- data.frame(tree = stem$tree[1],
                    n.w.ratio = n.w.ratio,
                    z.sd = sd(stem$z, na.rm = TRUE),
                    z.min = min(stem$z, na.rm = TRUE),
                    z.max = max(stem$z, na.rm = TRUE),
                    z.p99 <- stats::quantile(stem$z, probs = 0.99, na.rm = TRUE))
  return(out)

}



.volume <- function(data, d.mer = NULL, id, den.type){

  datos <- data.frame(tree = as.numeric(),
                      x = as.numeric(), y = as.numeric(),
                      hi = as.numeric(), dhi = as.numeric(),
                      h = as.numeric(), dbh = as.numeric())

  for (i in unique(data$tree)) {

    tree <- data[data$tree == i, ]
    tree <- tree[tree$hi < tree$h, ]

    if (nrow(tree) < 2 | suppressWarnings(min(abs(diff(tree$dhi)))) > 10) {

      datos <- rbind(datos, tree[tree$hi == 1.3, c("tree", "x", "y", "hi", "dhi", "h", "dbh")])

    } else {

      tree$dif.sec <- c(abs(diff(tree$hi)), 0)
      tree$dif <- c(diff(tree$dhi),
                    tree$dhi[nrow(tree)] - tree$dhi[nrow(tree) - 1])
      tree$dif <- ifelse(tree$dif.sec > 1, tree$dif / tree$dif.sec, tree$dif)

      while (suppressWarnings(max(abs(tree$dif))) > 10) {

        tree$filter <- ifelse(tree$dif > 10 | tree$dif < -10, 0, 1)
        tree <- tree[tree$filter == 1, ]
        if(nrow(tree) < 2) {next}

        tree$dif.sec <- c(abs(diff(tree$hi)), 0)
        tree$dif <- c(diff(tree$dhi),
                      tree$dhi[nrow(tree)] - tree$dhi[nrow(tree) - 1])
        tree$dif <- ifelse(tree$dif.sec > 1, tree$dif / tree$dif.sec, tree$dif)

      }

    }

    datos <- rbind(datos, tree[, c("tree", "x", "y", "hi", "dhi", "h", "dbh")])

    # Add additional values if the largest break is smaller than total height
    # minus 0.5 m
    if (all(tree$h - .5 > tree$hi)) {

      aux <- tree[which.max(tree$hi)[1], , drop = FALSE]
      datos <- rbind(datos,
                     data.frame(tree = aux$tree, x = aux$x, y = aux$y,
                                hi = aux$h - .5,
                                dhi = .5 * aux$dhi / (aux$h - aux$hi),
                                aux[, c("h", "dbh"), drop = FALSE]))

    }

  }

  # Removing NA's

  datos <- datos[!is.na(datos$hi), ]

  loes <- stats::lowess(datos$hi, datos$dhi)
  loes <- data.frame(hi = loes$x, dhi.mean = loes$y)
  loes <- loes[!duplicated(loes), ]
  datos <- merge(datos, loes, all = FALSE)
  datos$sep <- abs(datos$dhi - datos$dhi.mean)
  datos <- datos[datos$sep < stats::quantile(datos$sep, prob = 0.9, na.rm = T), ]

  datos$id <- id


  if(nrow(datos) < 3){

    if(is.null(d.mer)){

      n <- den.type

      v <- pi * (unique(data$h) ^ (n + 1) / (n + 1)) * ((unique(data$dbh) / 200) ^ 2 / (unique(data$h) - 1.3) ^ n)

      volume <- data.frame(tree = unique(data$tree), v = v)}

    else {

      n <- den.type

      v <- pi * (unique(data$h) ^ (n + 1) / (n + 1)) * ((unique(data$dbh) / 200) ^ 2 / (unique(data$h) - 1.3) ^ n)
      h.lim <- (((d.mer / 200) ^ 2) / ((unique(data$dbh) / 200) ^ 2 / (unique(data$h) - 1.3) ^ n)) ^ (1 / n)
      v.mer <- pi * ((unique(data$h) ^ (n + 1) - h.lim ^ (n + 1)) / (n + 1)) * ((unique(data$dbh) / 200) ^ 2 / (unique(data$h) - 1.3) ^ n)
      v.mer <- ifelse(v.mer < 0, 0, v.mer)

      volume <- data.frame(tree = unique(data$tree), v = v, v.mer = v.mer, h.mer = h.lim)}

  } else {


  ajuste <- stats::nls(dhi ~ dbh * ((h - hi) / (h - 1.3)) ** b1, data = datos,
                       start = c(b1 = 1), max)
  b1 <- stats::coef(ajuste)[1]

  # Height where diameter limit is reached (d.lim)
  h_d_lim <- function(dbh, h, d.lim, b1) {

    return(h - ((d.lim * (h - 1.3) ** b1) / dbh) ** (1 / b1))

  }

  # Volume between two heights
  vol_m3 <- function(dbh, h, hinf, hsup, b1) {

    return(pi * (dbh ** 2) *
             (((h - hinf) ** (2 * b1 + 1)) - ((h - hsup) ** (2 * b1 + 1))) /
             (40000 * ((h - 1.3) ** (2 * b1)) * (2 * b1 + 1)))

  }

  if (is.null(d.mer)) {

    volume <- data.frame(tree = as.numeric(), v = as.numeric())

    for (i in unique(data$tree)) {

      tree <- data[data$tree == i, ]
      volume.i <- data.frame(tree = i,
                             v = vol_m3(tree$dbh[1], tree$h[1], 0, tree$h[1],
                                        b1))
      volume <- rbind(volume, volume.i)

    }

  } else {

    volume <- data.frame(tree = as.numeric(), v = as.numeric(),
                         v.mer = as.numeric())

    for (i in unique(data$tree)) {

      tree <- data[data$tree == i, ]

      # If diameter at the bottom is smaller than 'd.mer' then 'h.lim' is forced
      # to be zero (consequently, 'v.mer' will be also zero)
      if (stats::predict(ajuste, data.frame(hi = 0, h = tree$h[1],
                                     dbh = tree$dbh[1])) < d.mer) {

        h.lim <- 0

      } else {

        h.lim <- h_d_lim(tree$dbh[1], tree$h[1], d.mer, b1)

      }

      volume.i <- data.frame(tree = i,
                             v = vol_m3(tree$dbh[1], tree$h[1], 0, tree$h[1], b1),
                             v.mer = vol_m3(tree$dbh[1], tree$h[1], 0, h.lim, b1),
                             h.mer = h.lim)

      volume <- rbind(volume, volume.i)

    }

  }}

  return(volume)

}


.stem.axis <- function(data, breaks, scan.approach = "single", tls.precision){

  if(scan.approach == "multi"){

    s <- sample(nrow(data), round(nrow(data)*0.5))
    data <- data[s, ]

    } else {

    data <- data[data$prob < 0.1 | data$prob > 0.9, ]}


  # if(nrow(data) < 100 | nrow(data) > 1000000 | min(data$z) > 1.3){
  if(nrow(data) < 50 | nrow(data) > 1000000){

    eje <- data.frame(tree = as.numeric(), sec = as.numeric(), x = as.numeric(), y = as.numeric(), n.w.ratio = as.numeric(), z.sd = as.numeric())

    } else {

  eje <- data.frame(tree = unique(data$tree), sec = seq(0, round(max(breaks), 1), by = 0.1))


  dbscan <- dbscan::dbscan(data[, c("x", "z"), drop = FALSE], eps = tls.precision * 2)
  data$cluster <- as.factor(dbscan$cluster)
  # plot(data$z, data$x, col = data$cluster, asp =1)

  cluster <- data.frame(table(data$cluster))
  colnames(cluster) <- c("cluster", "freq")
  cluster <- cluster[cluster$freq == max(cluster$freq), ]

  data <- merge(data, cluster, by = "cluster", all = FALSE)


  mod.x <- stats::lm(data = data, x ~ z)
  mod.y <- stats::lm(data = data, y ~ z)

  eje$x <- stats::coef(mod.x)[1] + stats::coef(mod.x)[2] * eje$sec
  eje$y <- stats::coef(mod.y)[1] + stats::coef(mod.y)[2] * eje$sec


  # plot(data$z, data$x, col = "red", asp =1)
  # points(eje$x~eje$sec)
  # plot(data$z, data$y, col = "red", asp =1)
  # points(eje$y~eje$sec)



  # n.w.ratio <- as.numeric(.n.w.ratio(data)[2])

  eje$n.w.ratio <- as.numeric(.n.w.ratio(data)[2])

  eje$z.sd <- sd(data$z, na.rm = TRUE)

  eje$slope <- max(stats::coef(mod.x)[2], stats::coef(mod.y)[2])

  }

  return(eje)


}





# Add quotes and paste a vector

.quot.past <- function(x, quot = "'", sep = "", collapse = ", ") {

  paste(quot, trimws(x), quot, sep = sep, collapse = collapse)

}


# Check object class and dimension, and values validity

.check.class <- function(x, x.class, name, col.of = NULL, n = NULL,
                         val = NULL, order.val = TRUE) {

  # Check object class
  if (x.class == "integer")
    .check <- is.numeric(x) && all(round(x[!is.na(x)], 0) == x[!is.na(x)])
  else
    .check <- get(paste("is", x.class, sep = "."))(x)
  if (!.check)
    stop(ifelse(is.null(col.of), .quot.past(name),
                paste("Column", .quot.past(name), "of", .quot.past(col.of))),
         " must be a ", x.class,
         ifelse(x.class %in% c("integer", "numeric", "character", "logical") &
                  !is.null(n) & n > 1, " vector", ""),
         ".")

  # Check object dimension
  if (!is.null(n)) {

    if (!x.class %in% c("data.frame")) {

      if (length(x) == 0)
        stop(ifelse(is.null(col.of), .quot.past(name),
                    paste("Column", .quot.past(name), "of",
                          .quot.past(col.of))),
             " must have length different from zero.")
      if (n == 1) {

        x <- unique(x)
        if (length(x) > 1) {

          warning("Only first value in ",
                  ifelse(is.null(col.of), .quot.past(name),
                         paste("column", .quot.past(name), "of",
                               .quot.past(col.of))),
                  " was taken into account during the execution.",
                  immediate. = TRUE)
          x <- x[1]

        }

      }

    } else {

      if (nrow(x) == 0) stop(.quot.past(name), " must have at least one row.")
      if (n == 1) {

        x <- unique(x)
        if (nrow(x) > 1) {

          warning("Only first row in ", .quot.past(name), " was taken into ",
                  "account during the execution.", immediate. = TRUE)
          x <- x[1, , drop = FALSE]

        }

      }

    }

  }

  # Check values and order
  if (!is.null(val)) {

    .inval <- x[!x %in% val]
    if (length(.inval) > 0)
      stop("Value(s) ", .quot.past(.inval), " in ",
           ifelse(is.null(col.of), .quot.past(name),
                  paste("column", .quot.past(name), "of",
                        .quot.past(col.of))),
           " is(are) not valid.\nValid value(s) is(are) ", .quot.past(val), ".")
    if (order.val)
      x <- val[val %in% x]


  }

  return(x)

}


# Check mandatory columns and their classes for a data.frame

.check.col <- function(x, x.mand, x.class, name, def = NULL,
                       na.action = c("stop", "warning")[1]) {

  # Check mandatory columns existence, assign values by default (if necessary),
  # and remove non necessary columns
  .miss <- colnames(x.mand)[apply(x.mand, 2, any)]
  .miss <- .miss[!.miss %in% colnames(x)]
  if (length(.miss) > 0)
    stop("According to specified arguments, ", .quot.past(name), " must have ",
         "column(s) named: ", .quot.past(.miss), ".")
  if (!is.null(def)) {

    .miss <- colnames(def)[!colnames(def) %in% colnames(x)]
    if (length(.miss) > 0)
      x <- cbind(x, do.call(cbind, lapply(def[, .miss], rep, times = nrow(x))))

  }
  x <- x[, colnames(x.mand), drop = FALSE]

  # Check mandatory columns classes
  for (.i in colnames(x)[colnames(x) %in% names(x.class)])
    x[, .i] <- .check.class(x = x[, .i], x.class = x.class[.i], name = .i,
                            col.of = name, n = nrow(x))

  # Check all values are no-NA
  .col.na <- colnames(x)[apply(is.na(x), 2, any)]
  if (length(.col.na) > 0) {

    if (na.action %in% "stop")
      stop("Column(s) ", .quot.past(.col.na), " of ", .quot.past(name),
           " has(have) missing values.")
    else if (na.action %in% "warning")
      warning("Column(s) ", .quot.past(.col.na),  "of ", .quot.past(name),
              " has(have) missing values.", immediate. = TRUE)

  }

  return(x)

}


# Compute number of decimals places

.decimals <- function(x){

  x <- format(x, scient = FALSE)
  ifelse(!base::grepl(".", x, fix = TRUE), 0,
         nchar(base::strsplit(x, ".", fix = TRUE)[[1]][2]))

}


# Compute radius, k and BAF, and tree variables according to plot design(s) and
# 'tree.var'. Currently available tree variables: basal area (g) and volume (v)

.tree.calc <- function(tree, plot.design, tree.var,
                       v.calc = c("coeff", "parab")[2]) {

  # Create data.frame where results will be saved
  .col.names <- plot.design
  if ("k.tree" %in% names(plot.design))
    .col.names <- c(.col.names,
                    paste("radius", plot.design["k.tree"], sep = "."))
  .col.names <- c(.col.names, tree.var)
  tree <- cbind(tree, matrix(nrow = nrow(tree), ncol = length(.col.names),
                             dimnames = list(NULL, .col.names)))

  # Compute radius (m) for fixed area plot design
  if (plot.design["fixed.area"] %in% colnames(tree))
    tree[, plot.design["fixed.area"]] <- tree[, "h.dist"]

  # Compute k (trees) and associated radius (m) for k-tree design
  if (plot.design["k.tree"] %in% colnames(tree)) {

    # Order trees by horizontal distance
    .ord <- order(tree[, "h.dist"], decreasing = F)
    # Save order as k
    tree[.ord, plot.design["k.tree"]] <- 1:nrow(tree)
    # Compute associated radius
    .dist <- c(tree[.ord[-1], "h.dist"],
               utils::tail(tree[.ord, "h.dist"], n = 1))
    tree[.ord, paste("radius", plot.design["k.tree"], sep = ".")] <-
      (tree[.ord, "h.dist"] + .dist) / 2

  }

  # Compute BAF (m2/ha) threshold for angle-count design
  if (plot.design["angle.count"] %in% colnames(tree))
    tree[, plot.design["angle.count"]] <- 2500 / (tree[, "h.dist"] /
                                                    tree[, "dbh"]) ^ 2

  # Compute basal area (m^2)
  if ("g" %in% colnames(tree)) tree[, "g"] <- (pi / 4) * tree[, "dbh"] ^ 2

  # Compute volume (m^3)
  if ("v" %in% colnames(tree)) tree[, "v"] <- tree[, "v"]

  # Compute comercial volume (m^3)
  if ("v.mer" %in% colnames(tree)) tree[, "v.mer"] <- tree[, "v.mer"]

  # if ("v" %in% colnames(tree)) {
  #
  #   if (v.calc == "coeff") {
  #
  #     # Coefficient of 0.45
  #     tree[, "v"] <- (pi / 4) * tree[, "dbh"] ^ 2 * tree[, "h"] * 0.45
  #
  #   } else if (v.calc == "parab") {
  #
  #     # Paraboloid
  #     tree[, "v"] <- pi * (tree[, "h"] ^ 2 / 2) *
  #       ((tree[, "dbh"] / 2) ^ 2 / (tree[, "h"] - 1.3))
  #
  #   } else
  #     stop("Argument for tree volume calculation must be 'coeff' or 'parab'.")
  #
  # }

  return(tree)

}


# Custom rounding of numbers

.customCeiling <- function(x, Decimals = 1) {

  ceiling(x * 10 ^ Decimals) / 10 ^ Decimals

}

.customFloor <- function(x, Decimals = 1) {

  floor(x * 10 ^ Decimals) / 10 ^ Decimals

}


# Custom format for numbers

.format.numb <- function(x, dec) {

  format(x, trim = TRUE, nsmall = dec)

}


# Compute several weighted mean functions for a numeric vector

.wmean.calculation <- function(data, w, mean.names) {

  sapply(mean.names,
         function(x, data, w) {
           get(paste("weighted_mean", x, sep = "_"))(data, w)
         },
         data = data, w = w)

}


# Compute expansion factors with occlusion corrections and estimate stand
# variables per ha, and compute mean diameters and heights

.stand.calc <- function(val, plot.design, tree, var.metr, ds.meth,
                        var.field.user, mean.d, mean.h) {

  # Select data according to specified radius/k/BAF value, and define radius
  if (names(plot.design) %in% c("fixed.area", "k.tree")) {

    tree <- tree[tree[, plot.design] <= val, , drop = FALSE]
    .radius <- switch(names(plot.design), fixed.area = val,
                      k.tree = tree[nrow(tree), paste("radius", plot.design,
                                                      sep = ".")])

  } else if (names(plot.design) %in% "angle.count")
    tree <- tree[tree[, plot.design] >= val, , drop = FALSE]

  # Add auxiliary column for density calculations
  .col.names <- c(paste("N", c("tls", names(ds.meth), "sh", "pam"), sep = "."),
                  "N")
  .col.names <- .col.names[.col.names %in% var.metr]
  if (length(.col.names) > 0) tree <- cbind(tree, n = 1)

  # Initialize data.frame where results will be saved: id, radius/k/BAF, and
  # variables and/or metrics according to 'var.metr'
  .col.names <- c("id", plot.design, var.metr)
  if ("stratum" %in% colnames(tree)) .col.names <- c("stratum", .col.names)
  stand <- data.frame(matrix(nrow = length(val), ncol = length(.col.names),
                             dimnames = list(names(val), .col.names)),
                      stringsAsFactors = FALSE)

  # Plot ID and radius/k/BAF
  stand[, "id"] <- unique(tree[, "id"])
  stand[, plot.design] <- val


  # Compute expansion factors to convert units per plot into units per ha, ----
  # with occlusion corrections, and estimate stand variables per ha: density
  # (trees/ha), basal area (m2/ha), volume (m3/ha), and optionally volume
  # (m3/ha) and biomass (Mg/ha) derived from trees attributes provided by the
  # user

  # Identify stand variables to be computed, and associated tree variable and
  # expansion factor
  .col.names <- c(sapply(c("N", "G", "V", "v.mer", "h.mer"), paste,
                         c("tls", names(ds.meth), "sh", "pam"), sep = "."),
                  "N", "G", "V", names(var.field.user))
  .col.names <- matrix("", nrow = length(.col.names), ncol = 2,
                       dimnames = list(.col.names, c("var", "ef")))
  for (.i in c("N", "G", "V"))
    .col.names[c(sapply(.i, paste, c("tls", names(ds.meth), "sh", "pam"),
                        sep = "."), .i), "var"] <-
    switch(.i, N = "n", G = "g", V = "v")

  .col.names[c("v.mer.tls", "v.mer.hn", "v.mer.hr", "v.mer.hn.cov", "v.mer.hr.cov", "v.mer.sh", "v.mer.pam"), "var"] <- "v.mer"
  .col.names[c("h.mer.tls", "h.mer.hn", "h.mer.hr", "h.mer.hn.cov", "h.mer.hr.cov", "h.mer.sh", "h.mer.pam"), "var"] <- "h.mer"

  for (.i in names(var.field.user))
    .col.names[.i, "var"] <- var.field.user[[.i]]
  .col.names[c(paste(c("N", "G", "V"), "tls", sep = "."), "N", "G", "V",
               names(var.field.user)), "ef"] <- "EF"

  .col.names["v.mer.tls", "ef"] <- "EF"
  .col.names["h.mer.tls", "ef"] <- "EF"

  for (.i in c(names(ds.meth), "sh", "pam"))
    .col.names[paste(c("N", "G", "V", "v.mer", "h.mer"), .i, sep = "."), "ef"] <- paste("EF", .i,
                                                                      sep =".")
  .col.names <- .col.names[rownames(.col.names) %in% colnames(stand), ,
                           drop = FALSE]
  if (nrow(.col.names) > 0) {

    # Compute expansion factor according to plot design (all plot designs)
    if (names(plot.design) %in% c("fixed.area", "k.tree"))
      .EF <- 10000 / (pi * .radius ^ 2)
    else if (names(plot.design) %in% "angle.count")
      .EF <- val / (pi * (tree[, "dbh"] / 2) ^ 2)
    .EF <- matrix(.EF, nrow = nrow(tree), dimnames = list(NULL, "EF"))

    # Compute expansion factor using distance sampling based correction (fixed
    # area and k-tree plot designs)
    .col.names.2 <- ds.meth[paste("EF", names(ds.meth), sep = ".") %in%
                              .col.names[, "ef"]]
    if (length(.col.names.2) > 0) {

      .P <- tree[nrow(tree), .col.names.2, drop = FALSE]
      colnames(.P) <- paste("EF", names(.col.names.2), sep = ".")
      if (nrow(tree) > 1) .P <- apply(.P, 2, rep, times = nrow(tree))
      .EF <- cbind(.EF, .EF[, "EF"] / .P)

    }

    # Compute expansion factor with correction of the shadowing effect (fixed
    # area and k-tree plot designs)
    if ("EF.sh" %in% .col.names[, "ef"]) {

      # Compute shadow area for each tree
      .sh <- ifelse(tree[, "partial.occlusion"] == 0,

                    # Non-occluded trees
                    ((((pi * .radius ^ 2) - (pi * tree[, "h.dist"] ^ 2)) /
                        (2 * pi)) * atan2(tree[, "dbh"], tree[, "h.dist"])) -
                      ((pi * (tree[, "dbh"] / 2) ^ 2) / 2),

                    # Occluded trees
                    ((((pi * .radius ^ 2) - (pi * tree[, "h.dist"] ^ 2)) /
                        (2 * pi)) * (tree[, "wide"])) -
                      (((pi * (tree[, "dbh"] / 2) ^ 2) * tree[, "wide"]) /
                         atan2(tree[, "dbh"], tree[, "h.dist"])))

      # Correction: sh = 0 if tree is not completely inside the plot
      .sh[tree[, "h.dist"] + tree[, "dbh"] / 2 > .radius] <- 0

      # Compute expansion factor
      .sh <- 1 + (sum(.sh, na.rm = TRUE) / (pi * .radius ^ 2))
      .EF <- cbind(.EF, EF.sh = .EF[, "EF"] * .sh)

    }

    # Compute expansion factor with gap probability attenuation correction
    # (angle-count plot design)
    if ("EF.pam" %in% .col.names[, "ef"]) {

      .De <- mean(tree[, "dbh"]) *
        (1 + (stats::sd(tree[, "dbh"], na.rm = TRUE) /
                mean(tree[, "dbh"], na.rm = TRUE)) ^ 2) ^ 0.5
      .t <- (((val / (pi * (tree[, "dbh"] / 2) ^ 2)) / 10000) * .De *
               tree[, "dbh"]) / (2 * sqrt(val / 10000))
      .Ft <- (2 / .t ^ 2) * (1 - exp(-.t) * (1 + .t))
      .EF <- cbind(.EF, EF.pam = .EF[, "EF"] / .Ft)

    }

    # Estimate variables per ha
    stand[, rownames(.col.names)] <-
      apply(tree[, .col.names[, "var"], drop = FALSE] *
              .EF[, .col.names[, "ef"], drop = FALSE], 2, sum)

  }


  # Compute mean diameters and heights ----

  # Compute mean diameters
  .col.names <- sapply(names(mean.d), paste, c("", ".tls"), sep = "")
  .col.names <- .col.names[apply(apply(.col.names, 1:2,  "%in%",
                                       colnames(stand)), 1, any), ]
  if (length(.col.names) > 0)
    stand[, .col.names] <- .wmean.calculation(data = tree[, "dbh"],
                                              w = rep(1, nrow(tree)),
                                              mean.names = mean.d)

  # Compute mean heights
  .col.names <- sapply(names(mean.h), paste, c("", ".tls"), sep = "")
  .col.names <- .col.names[apply(apply(.col.names, 1:2,  "%in%",
                                       colnames(stand)), 1, any), ]
  if (length(.col.names) > 0)
    stand[, .col.names] <- .wmean.calculation(data = tree[, "h"],
                                              w = rep(1, nrow(tree)),
                                              mean.names = mean.h)


  # Compute number of points
  .col.names <- paste("n.pts", c("", ".est", ".red", ".red.est"), sep = "")
  .col.names <- .col.names[.col.names %in% colnames(stand)]
  if (length(.col.names) > 0)
    stand[, .col.names] <- apply(tree[, .col.names, drop = FALSE], 2, sum)

  return(stand)

}


# LiDAR metrics: 'mean', 'max', 'min', 'sd', 'var', 'mode', 'kurtosis',
# 'skewness', 'p.a.mean', 'p.a.mode', 'weibull_b', 'weibull_c'

.getmode <- function(v) {

  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]

}

.c_function <- function(c, media, varianza){

  varianza - (media^2) * (gamma(1 + 2 / c) -
                            (gamma(1 + 1 / c))^2) / (gamma(1 + 1 / c))^2

}

.points.metrics <- function(rho_seq, data, metr) {

  # Restrict data
  data <- data[data[, "z"] > 0.1, , drop = FALSE]
  data <- data[data[, "rho"] > 0.1, , drop = FALSE]
  data <- data[data[, "r"] > 0.1, , drop = FALSE]

  # Compute metrics for coordinate z
  .metr <- lapply(rho_seq,
                  function(rho, data, metr) {

                    .sub <- data[data[, "rho"] <= rho, "z"]

                    .metr <- rep(NA, length(metr))
                    names(.metr) <- metr

                    if (any(c("mean.z", "p.a.mean.z", "weibull_c.z", "weibull_b.z") %in% names(.metr)))
                      .metr["mean.z"] <- mean(.sub)
                    if ("mean.q.z" %in% names(.metr))
                      .metr["mean.q.z"] <- sqrt(mean(.sub ^ 2))
                    if ("mean.g.z" %in% names(.metr))
                      .metr["mean.g.z"] <- exp(mean(log(.sub[.sub>0])))
                    if ("mean.h.z" %in% names(.metr))
                      .metr["mean.h.z"] <- length(.sub) / sum(1 / .sub)
                    if ("median.z" %in% names(.metr))
                      .metr["median.z"] <- stats::median(.sub)
                    if (any(c("mode.z", "p.a.mode.z") %in% names(.metr)))
                      .metr["mode.z"] <- .getmode(.sub)
                    if (any(c("max.z", "weibull_c.z", "weibull_b.z") %in% names(.metr)))
                      .metr["max.z"] <- max(.sub)
                    if (any(c("min.z", "weibull_c.z", "weibull_b.z") %in% names(.metr)))
                      .metr["min.z"] <- min(.sub)
                    if (any(c("var.z", "weibull_c.z", "weibull_b.z") %in% names(.metr)))
                      .metr["var.z"] <- var(.sub)
                    if ("sd.z" %in% names(.metr))
                      .metr["sd.z"] <- sd(.sub)
                    if ("CV.z" %in% names(.metr))
                      .metr["CV.z"] <- .metr["sd.z"] / .metr["mean.z"]
                    if ("D.z" %in% names(.metr))
                      .metr["D.z"] <- .metr["max.z"] - .metr["min.z"]
                    if ("ID.z" %in% names(.metr))
                      .metr["ID.z"] <- stats::quantile(.sub, prob = 0.75, na.rm = T) - stats::quantile(.sub, prob = 0.25, na.rm = T)

                    if ("kurtosis.z" %in% names(.metr))
                      .metr["kurtosis.z"] <- moments::kurtosis(.sub)
                    if ("skewness.z" %in% names(.metr))
                      .metr["skewness.z"] <- moments::skewness(.sub)

                    if ("p.a.mean.z" %in% names(.metr))
                      .metr["p.a.mean.z"] <- mean(.sub > .metr["mean.z"]) * 100
                    if ("p.a.mode.z" %in% names(.metr))
                      .metr["p.a.mode.z"] <- mean(.sub > .metr["mode.z"]) * 100
                    if ("p.a.2m.z" %in% names(.metr))
                      .metr["p.a.2m.z"] <- mean(.sub > 2) * 100

                    if ("p.b.mean.z" %in% names(.metr))
                      .metr["p.b.mean.z"] <- mean(.sub < .metr["mean.z"]) * 100
                    if ("p.b.mode.z" %in% names(.metr))
                      .metr["p.b.mode.z"] <- mean(.sub < .metr["mode.z"]) * 100
                    if ("p.b.2m.z" %in% names(.metr))
                      .metr["p.b.2m.z"] <- mean(.sub < 2) * 100

                    if ("CRR.z" %in% names(.metr))
                      .metr["CRR.z"] <- .metr["mean.z"] / .metr["max.z"]

                    if ("L2.z" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 1) - choose(length(.sub) - .sub, 1)) * .sub
                      .metr["L2.z"] <- (1 / (2 * choose(length(.sub), 2))) * sum(sumatorio)
                    if ("L3.z" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 2) - 2 * choose(.sub - 1, 1) * choose(length(.sub) - .sub, 1) + choose(length(.sub) - .sub, 2)) * .sub
                      .metr["L3.z"] <- (1 / (3 * choose(length(.sub), 3))) * sum(sumatorio)
                    if ("L4.z" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 3) - 3 * choose(.sub - 1, 2) * choose(length(.sub) - .sub, 1) +
                                      3 * choose(.sub - 1, 1) * choose(length(.sub) - .sub, 2) - choose(length(.sub) - .sub, 3)) * .sub
                      .metr["L4.z"] <- (1 / (4 * choose(length(.sub), 4))) * sum(sumatorio)


                    if ("L.CV.z" %in% names(.metr))
                      .metr["L.CV.z"] <- .metr["L2.z"] / .metr["mean.z"]
                    if ("L.skewness.z" %in% names(.metr))
                      .metr["L.skewness.z"] <- .metr["L3.z"] / .metr["L2.z"]
                    if ("L.kurtosis.z" %in% names(.metr))
                      .metr["L.kurtosis.z"] <- .metr["L4.z"] / .metr["L2.z"]


                    if ("median.a.d.z" %in% names(.metr))
                      .metr["median.a.d.z"] <- stats::median(abs(.sub - .metr["mean.z"]))
                    if ("mode.a.d.z" %in% names(.metr))
                      .metr["mode.a.d.z"] <- .getmode(abs(.sub - .metr["mean.z"]))


                    if (any(c("weibull_c.z", "weibull_b.z") %in% names(.metr))) {

                      .error <- try(stats::uniroot(.c_function, media = .metr["mean.z"],
                                                   varianza = .metr["var.z"],
                                                   interval = c(.metr["min.z"],
                                                                .metr["max.z"]))$root, silent = TRUE)

                      if(class(.error)[1] == "try-error"){

                        .metr["weibull_c.z"] <- NA
                        .metr["weibull_b.z"] <- NA

                      } else {

                        .metr["weibull_c.z"] <-
                          stats::uniroot(.c_function, media = .metr["mean.z"],
                                         varianza = .metr["var.z"],
                                         interval = c(.metr["min.z"],
                                                      .metr["max.z"]))$root

                      }

                      if ("weibull_b.z" %in% names(.metr)) {

                        .metr["weibull_b.z"] <-
                          .metr["mean.z"] / gamma(1 + 1 / .metr["weibull_c.z"])}

                    }



                    # Compute metrics for coordinate rho

                    .sub <- data[data[, "rho"] <= rho, "rho"]

                    if (any(c("mean.rho", "p.a.mean.rho", "weibull_c.rho", "weibull_b.rho")  %in% names(.metr)))
                      .metr["mean.rho"] <- mean(.sub)
                    if ("mean.q.rho" %in% names(.metr))
                      .metr["mean.q.rho"] <- sqrt(mean(.sub ^ 2))
                    if ("mean.g.rho" %in% names(.metr))
                      .metr["mean.g.rho"] <- exp(mean(log(.sub[.sub>0])))
                    if ("mean.h.rho" %in% names(.metr))
                      .metr["mean.h.rho"] <- length(.sub) / sum(1 / .sub)
                    if ("median.rho" %in% names(.metr))
                      .metr["median.rho"] <- stats::median(.sub)
                    if (any(c("mode.rho", "p.a.mode.rho") %in% names(.metr)))
                      .metr["mode.rho"] <- .getmode(.sub)
                    if (any(c("max.rho", "weibull_c.rho", "weibull_b.rho") %in% names(.metr)))
                      .metr["max.rho"] <- max(.sub)
                    if (any(c("min.rho", "weibull_c.rho", "weibull_b.rho") %in% names(.metr)))
                      .metr["min.rho"] <- min(.sub)
                    if (any(c("var.rho", "weibull_c.rho", "weibull_b.rho") %in% names(.metr)))
                      .metr["var.rho"] <- var(.sub)
                    if ("sd.rho" %in% names(.metr))
                      .metr["sd.rho"] <- sd(.sub)
                    if ("CV.rho" %in% names(.metr))
                      .metr["CV.rho"] <- .metr["sd.rho"] / .metr["mean.rho"]
                    if ("D.rho" %in% names(.metr))
                      .metr["D.rho"] <- .metr["max.rho"] - .metr["min.rho"]
                    if ("ID.rho" %in% names(.metr))
                      .metr["ID.rho"] <- stats::quantile(.sub, prob = 0.75, na.rm = T) - stats::quantile(.sub, prob = 0.25, na.rm = T)

                    if ("kurtosis.rho" %in% names(.metr))
                      .metr["kurtosis.rho"] <- moments::kurtosis(.sub)
                    if ("skewness.rho" %in% names(.metr))
                      .metr["skewness.rho"] <- moments::skewness(.sub)

                    if ("p.a.mean.rho" %in% names(.metr))
                      .metr["p.a.mean.rho"] <- mean(.sub > .metr["mean.rho"]) * 100
                    if ("p.a.mode.rho" %in% names(.metr))
                      .metr["p.a.mode.rho"] <- mean(.sub > .metr["mode.rho"]) * 100
                    if ("p.b.mean.rho" %in% names(.metr))
                      .metr["p.b.mean.rho"] <- mean(.sub < .metr["mean.rho"]) * 100
                    if ("p.b.mode.rho" %in% names(.metr))
                      .metr["p.b.mode.rho"] <- mean(.sub < .metr["mode.rho"]) * 100

                    if ("CRR.rho" %in% names(.metr))
                      .metr["CRR.rho"] <- .metr["mean.rho"] / .metr["max.rho"]

                    if ("L2.rho" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 1) - choose(length(.sub) - .sub, 1)) * .sub
                    .metr["L2.rho"] <- (1 / (2 * choose(length(.sub), 2))) * sum(sumatorio)
                    if ("L3.rho" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 2) - 2 * choose(.sub - 1, 1) * choose(length(.sub) - .sub, 1) + choose(length(.sub) - .sub, 2)) * .sub
                    .metr["L3.rho"] <- (1 / (3 * choose(length(.sub), 3))) * sum(sumatorio)
                    if ("L4.rho" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 3) - 3 * choose(.sub - 1, 2) * choose(length(.sub) - .sub, 1) +
                                      3 * choose(.sub - 1, 1) * choose(length(.sub) - .sub, 2) - choose(length(.sub) - .sub, 3)) * .sub
                    .metr["L4.rho"] <- (1 / (4 * choose(length(.sub), 4))) * sum(sumatorio)


                    if ("L.CV.rho" %in% names(.metr))
                      .metr["L.CV.rho"] <- .metr["L2.rho"] / .metr["mean.rho"]
                    if ("L.skewness.rho" %in% names(.metr))
                      .metr["L.skewness.rho"] <- .metr["L3.rho"] / .metr["L2.rho"]
                    if ("L.kurtosis.rho" %in% names(.metr))
                      .metr["L.kurtosis.rho"] <- .metr["L4.rho"] / .metr["L2.rho"]

                    if ("median.a.d.rho" %in% names(.metr))
                      .metr["median.a.d.rho"] <- stats::median(abs(.sub - .metr["mean.rho"]))
                    if ("mode.a.d.rho" %in% names(.metr))
                      .metr["mode.a.d.rho"] <- .getmode(abs(.sub - .metr["mean.rho"]))



                    if (any(c("weibull_c.rho", "weibull_b.rho") %in% names(.metr))) {

                      .error <- try(stats::uniroot(.c_function, media = .metr["mean.rho"],
                                                   varianza = .metr["var.rho"],
                                                   interval = c(.metr["min.rho"],
                                                                .metr["max.rho"]))$root, silent = TRUE)

                      if(class(.error)[1] == "try-error"){

                        .metr["weibull_c.rho"] <- NA
                        .metr["weibull_b.rho"] <- NA

                      } else {

                      .metr["weibull_c.rho"] <-
                        stats::uniroot(.c_function, media = .metr["mean.rho"],
                                       varianza = .metr["var.rho"],
                                       interval = c(.metr["min.rho"],
                                                    .metr["max.rho"]))$root

                      }

                    if ("weibull_b.rho" %in% names(.metr)) {

                      .metr["weibull_b.rho"] <-
                        .metr["mean.rho"] / gamma(1 + 1 / .metr["weibull_c.rho"])}

                    }


                    # Compute metrics for coordinate r

                    .sub <- data[data[, "rho"] <= rho, "r"]

                    if (any(c("mean.r", "p.a.mean.r", "weibull_c.r", "weibull_b.r") %in% names(.metr)))
                      .metr["mean.r"] <- mean(.sub)
                    if ("mean.q.r" %in% names(.metr))
                      .metr["mean.q.r"] <- sqrt(mean(.sub ^ 2))
                    if ("mean.g.r" %in% names(.metr))
                      .metr["mean.g.r"] <- exp(mean(log(.sub[.sub>0])))
                    if ("mean.h.r" %in% names(.metr))
                      .metr["mean.h.r"] <- length(.sub) / sum(1 / .sub)
                    if ("median.r" %in% names(.metr))
                      .metr["median.r"] <- stats::median(.sub)
                    if (any(c("mode.r", "p.a.mode.r") %in% names(.metr)))
                      .metr["mode.r"] <- .getmode(.sub)
                    if (any(c("max.r", "weibull_c.r", "weibull_b.r") %in% names(.metr)))
                      .metr["max.r"] <- max(.sub)
                    if (any(c("min.r", "weibull_c.r", "weibull_b.r") %in% names(.metr)))
                      .metr["min.r"] <- min(.sub)
                    if (any(c("var.r", "weibull_c.r", "weibull_b.r") %in% names(.metr)))
                      .metr["var.r"] <- var(.sub)
                    if ("sd.r" %in% names(.metr))
                      .metr["sd.r"] <- sd(.sub)
                    if ("CV.r" %in% names(.metr))
                      .metr["CV.r"] <- .metr["sd.r"] / .metr["mean.r"]
                    if ("D.r" %in% names(.metr))
                      .metr["D.r"] <- .metr["max.r"] - .metr["min.r"]
                    if ("ID.r" %in% names(.metr))
                      .metr["ID.r"] <- stats::quantile(.sub, prob = 0.75, na.rm = T) - stats::quantile(.sub, prob = 0.25, na.rm = T)

                    if ("kurtosis.r" %in% names(.metr))
                      .metr["kurtosis.r"] <- moments::kurtosis(.sub)
                    if ("skewness.r" %in% names(.metr))
                      .metr["skewness.r"] <- moments::skewness(.sub)

                    if ("p.a.mean.r" %in% names(.metr))
                      .metr["p.a.mean.r"] <- mean(.sub > .metr["mean.r"]) * 100
                    if ("p.a.mode.r" %in% names(.metr))
                      .metr["p.a.mode.r"] <- mean(.sub > .metr["mode.r"]) * 100
                    if ("p.b.mean.r" %in% names(.metr))
                      .metr["p.b.mean.r"] <- mean(.sub < .metr["mean.r"]) * 100
                    if ("p.b.mode.r" %in% names(.metr))
                      .metr["p.b.mode.r"] <- mean(.sub < .metr["mode.r"]) * 100

                    if ("CRR.r" %in% names(.metr))
                      .metr["CRR.r"] <- .metr["mean.r"] / .metr["max.r"]

                    if ("L2.r" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 1) - choose(length(.sub) - .sub, 1)) * .sub
                    .metr["L2.r"] <- (1 / (2 * choose(length(.sub), 2))) * sum(sumatorio)
                    if ("L3.r" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 2) - 2 * choose(.sub - 1, 1) * choose(length(.sub) - .sub, 1) + choose(length(.sub) - .sub, 2)) * .sub
                    .metr["L3.r"] <- (1 / (3 * choose(length(.sub), 3))) * sum(sumatorio)
                    if ("L4.r" %in% names(.metr))
                      sumatorio <- (choose(.sub - 1, 3) - 3 * choose(.sub - 1, 2) * choose(length(.sub) - .sub, 1) +
                                      3 * choose(.sub - 1, 1) * choose(length(.sub) - .sub, 2) - choose(length(.sub) - .sub, 3)) * .sub
                    .metr["L4.r"] <- (1 / (4 * choose(length(.sub), 4))) * sum(sumatorio)


                    if ("L.CV.r" %in% names(.metr))
                      .metr["L.CV.r"] <- .metr["L2.r"] / .metr["mean.r"]
                    if ("L.skewness.r" %in% names(.metr))
                      .metr["L.skewness.r"] <- .metr["L3.r"] / .metr["L2.r"]
                    if ("L.kurtosis.r" %in% names(.metr))
                      .metr["L.kurtosis.r"] <- .metr["L4.r"] / .metr["L2.r"]

                    if ("median.a.d.r" %in% names(.metr))
                      .metr["median.a.d.r"] <- stats::median(abs(.sub - .metr["mean.r"]))
                    if ("mode.a.d.r" %in% names(.metr))
                      .metr["mode.a.d.r"] <- .getmode(abs(.sub - .metr["mean.r"]))


                    if (any(c("weibull_c.r", "weibull_b.r") %in% names(.metr))) {

                      .error <- try(stats::uniroot(.c_function, media = .metr["mean.r"],
                                                   varianza = .metr["var.r"],
                                                   interval = c(.metr["min.r"],
                                                                .metr["max.r"]))$root, silent = TRUE)

                      if(class(.error)[1] == "try-error"){

                        .metr["weibull_c.r"] <- NA
                        .metr["weibull_b.r"] <- NA

                      } else {

                        .metr["weibull_c.r"] <-
                          stats::uniroot(.c_function, media = .metr["mean.r"],
                                         varianza = .metr["var.r"],
                                         interval = c(.metr["min.r"],
                                                      .metr["max.r"]))$root

                      }

                      if ("weibull_b.r" %in% names(.metr)) {

                        .metr["weibull_b.r"] <-
                          .metr["mean.r"] / gamma(1 + 1 / .metr["weibull_c.r"])}

                    }

                    return(.metr)

                  },

                  data = data, metr = metr)

  .metr <- do.call(rbind, .metr)

  return(.metr)

}


# Simulate plots and compute stand variables and metrics

.sim.calc <- function(funct, tree.tls, tree.ds, tree.field,
                      plot.design, plot.parameters, scan.approach, var.metr,
                      v.calc, dbh.min, h.min, max.dist, dir.data, save.result,
                      dir.result) {


  # Define available values for arguments and other auxiliary objects ----

  # Define a list containing mandatory trees' database(s) according to 'function
  # case'. Currently available 'function cases': 'sim', 'metr' and 'est'
  if(is.null(tree.field))
    .funct <- list(sim = c(field = "field", tls = "TLS"), metr = c(tls = "TLS"),
                   est = c(tls = "TLS"))

  else {

   .funct <- list(sim = c(field = "field", tls = "TLS"), metr = c(field = "field", tls = "TLS"),
                  est = c(tls = "TLS"))
  }


  # Define a character vector containing index name (radius, k or BAF) for each
  # available plot design. Currently available plot designs: 'fixed.area',
  # 'k.tree' and 'angle.count'
  .plot.design <- c(fixed.area = "radius", k.tree = "k", angle.count = "BAF")

  # Define a list containing mandatory columns in 'tree.tls' argument
  # associated to each available scan approach. Currently available scan
  # approaches: 'single' and 'multi'
  .scan.approach <- list(single = c("phi.left", "phi.right"),
                         multi = character())

  # Define a character vector containing mandatory columns in 'tree.ds'
  # argument associated to available distance sampling methodologies. Currently
  # available methodologies: 'hn' (half normal function), 'hn.cov' (half normal
  # function with dbh as covariate), 'hr' (half rate function) and 'hr.cov'
  # (half rate function with dbh as covariate)
  .ds.meth <- c(hn = "P.hn", hr = "P.hr", hn.cov = "P.hn.cov", hr.cov = "P.hr.cov")

  # Define character vectors containing the mean function to be used for each
  # mean diameter/height computation. Currently available means: arithmetic,
  # quadratic, geometric and harmonic
  .mean.d <- c(d = "arit", dg = "sqrt", dgeom = "geom", dharm = "harm")
  .mean.h <- c(h = "arit", hg = "sqrt", hgeom = "geom", hharm = "harm")

  # Define a numeric vector containing probabilities for height percentiles
  # calculation
  .prob = c(1, 5, 10, 20, 25, 30, 40, 50, 60, 70, 75, 80, 90, 95, 99)

  # Define a list containing mandatory columns in 'tree.field' argument
  # associated to each available field variables to be computed from optional
  # trees attributes provided by the user. Currently available field variables:
  # 'V.user' and 'W.user'
  .var.field.user <- list(V.user = "v.user", W.user = "w.user")

  # Define a list containing the available TLS variables and metrics according
  # to 'function case', 'tree.ds' availability, scan approach and plot
  # design(s), and the available field variables according to 'function case'
  # and trees variables provided by user in 'tree.field'.
  # Currently available:
  # - TLS variables and metrics (1): 'N.tls', 'N.hn' (2), 'N.hr' (2),
  #   'N.hn.cov' (2), 'N.hr.cov' (2), 'N.sh' (3), 'N.pam' (4); 'G.tls',
  #   'G.hn' (2), 'G.hr' (2), 'G.hn.cov' (2), 'G.hr.cov' (2), 'G.sh' (3),
  #   'G.pam' (4); 'V.tls', 'V.hn' (2), 'V.hr' (2), 'V.hn.cov' (2),
  #   'V.hr.cov'(2), 'V.sh' (3), 'V.pam' (4); 'd.tls', 'dg.tls', 'dgeom.tls',
  #   'dharm.tls'; 'h.tls', 'hg.tls', 'hgeom.tls', 'hharm.tls'; 'd.0.tls',
  #   'dg.0.tls', 'dgeom.0.tls', 'dharm.0.tls'; 'h.0.tls', 'hg.0.tls',
  #   'hgeom.0.tls', 'hharm.0.tls'; 'n.pts', 'n.pts.est',
  #   'n.pts.red', 'n.pts.red.est'; 'P01', 'P05', 'P10', 'P20', 'P25',
  #   'P30', 'P40', 'P50', 'P60', 'P70', 'P75', 'P80', 'P90', 'P95', 'P99';
  #   'mean', 'max', 'min', 'sd', 'var', 'mode', 'kurtosis', 'skewness',
  #   'p.a.mode', 'p.a.mean', 'weibull_c', 'weibull_b'.
  # - Field variables (5): 'N'; 'G'; 'V', 'V.user' (6); 'W.user' (6); 'd', 'dg',
  #   'dgeom', 'dharm'; 'h', 'hg', 'hgeom', 'hharm'; 'd.0', 'dg.0', 'dgeom.0',
  #   'dharm.0'; 'h.0', 'hg.0', 'hgeom.0', 'hharm.0'.
  # Remarks. (1) For function case 'est' only 'N.tls' and 'G.tls' are computed.
  # (2) Only for circular fixed area and k-tree plot designs if scan approach is
  # 'simple' and 'tree.ds' is not NULL. (3) Only for circular fixed area
  # and k-tree plot design if scan approach is 'simple'. (4) Only for
  # angle-count plot design if scan approach is 'simple'. (5) Only for function
  # case 'sim'. (6) Only if corresponding trees variable ('v.user' for 'V.user',
  # 'w.user' for 'W.user') are provided by user in 'tree.field' argument.
  .var.metr <- list(tls = NULL, field = NULL)
  .var.metr$tls <- c(sapply(if(length(tree.tls$v.mer) > 0){
                   c("N", "G", "V", "v.mer", "h.mer")} else{
                   c("N", "G", "V")}, paste,
                            c("tls", names(.ds.meth), "sh", "pam"), sep = "."),
                     t(sapply(names(c(.mean.d, .mean.h)),
                              paste, c(".tls", ".0.tls"), sep = "")),
                     paste("n.pts", c("", ".est", ".red", ".red.est"),
                           sep = ""),
                     sprintf("P%02i", .prob),

                     # Z coordinate
                     "mean.z", "mean.q.z", "mean.g.z", "mean.h.z", "median.z", "mode.z",
                     "max.z", "min.z", "var.z", "sd.z", "CV.z", "D.z", "ID.z",
                     "kurtosis.z", "skewness.z",
                     "p.a.mean.z", "p.a.mode.z", "p.a.2m.z",
                     "p.b.mean.z", "p.b.mode.z", "p.b.2m.z", "CRR.z",
                     "L2.z", "L3.z", "L4.z", "L.CV.z", "L.skewness.z", "L.kurtosis.z",
                     "median.a.d.z", "mode.a.d.z",
                     "weibull_c.z", "weibull_b.z",

                     # Rho coordinate
                     "mean.rho", "mean.q.rho", "mean.g.rho", "mean.h.rho", "median.rho", "mode.rho",
                     "max.rho", "min.rho", "var.rho", "sd.rho", "CV.rho", "D.rho", "ID.rho",
                     "kurtosis.rho", "skewness.rho",
                     "p.a.mean.rho", "p.a.mode.rho",
                     "p.b.mean.rho", "p.b.mode.rho", "CRR.rho",
                     "L2.rho", "L3.rho", "L4.rho", "L.skewness.rho", "L.kurtosis.rho",
                     "L.CV.rho",
                     "median.a.d.rho", "mode.a.d.rho",
                     "weibull_c.rho", "weibull_b.rho",

                     # R coordinate
                     "mean.r", "mean.q.r", "mean.g.r", "mean.h.r", "median.r", "mode.r",
                     "max.r", "min.r", "var.r", "sd.r", "CV.r", "D.r", "ID.r",
                     "kurtosis.r", "skewness.r",
                     "p.a.mean.r", "p.a.mode.r",
                     "p.b.mean.r", "p.b.mode.r", "CRR.r",
                     "L2.r", "L3.r", "L4.r", "L.CV.r", "L.skewness.r", "L.kurtosis.r",
                     "median.a.d.r", "mode.a.d.r",
                     "weibull_c.r", "weibull_b.r")

  .var.metr$field <- c("N", "G", names(.var.field.user),
                       t(sapply(names(c(.mean.d, .mean.h)), paste, c("", ".0"),
                                sep = "")))

  # Define available calculations for trees volume. Currently available
  # calculations: 'coeff' and 'parab'
  .v.calc <- c("coeff", "parab")

  # Define values by default for certain columns in 'plot.parameters'
  .plot.parameters <- data.frame(radius.incr = 0.1, k.incr = 1, BAF.incr = 0.1,
                                 num.trees = 100)


  # Check arguments 'funct', 'plot.design', 'scan.approach', 'v.calc', ----
  # 'dbh.min', 'h.min', 'max.dist', 'save.result', 'dir.data', 'dir.result'

  # 'funct' must be a character string containing the 'function  case'
  funct <- .check.class(x = funct, x.class = "character", name = "funct",
                        n = 1, val = names(.funct))

  # 'plot.design' must be a character vector containing the plot designs to be
  # computed
  plot.design <- .check.class(x = plot.design, x.class = "character",
                              name = "plot.design", n = Inf,
                              val = names(.plot.design))

  # 'scan.approach' must be a character string indicating the scan approach used
  # for obtaining LAS files
  scan.approach <- .check.class(x = scan.approach, x.class = "character",
                                name = "scan.approach", n = 1,
                                val = names(.scan.approach))

  # 'v.calc' must be a character string indicating how to calculate trees volume
  v.calc <- .check.class(x = v.calc, x.class = "character", name = "v.calc",
                         n = 1, val = .v.calc)

  # 'dbh.min', 'h.min' and 'max.dist' must be positive numeric values
  for (.i in c("dbh.min", "h.min", "max.dist")) {

    # Check object class and dimension
    assign(.i, .check.class(x = get(.i), x.class = "numeric", name = .i, n = 1))

    # Check value is  no-NA and strictly positive
    if (is.na(get(.i)) || get(.i) <= 0)
      stop(.quot.past(.i), " argument must be strictly positive.")

  }

  # 'save.result' must be a logical
  save.result <- .check.class(x = save.result, x.class = "logical",
                              name = "save.result", n = 1)

  # 'dir.data' and 'dir.result' must be NULL or a character string containing
  # the absolute path to a existing directory
  # Remark. If 'dir.data' is NULL, working directory is assigned by default. If
  # 'save.result' is FALSE, 'dir.result' is forced to be NULL; otherwise, if
  # 'dir.result' is NULL, working directory is assigned by default
  for (.i in c("dir.data", "dir.result")) {

    if (.i %in% "dir.result" & !save.result) assign(.i, NULL)
    else {

      if (is.null(get(.i))) assign(.i, getwd())
      else {

        # Check object class and dimension
        assign(.i, .check.class(x = get(.i), x.class = "character", name = .i,
                                n = 1))


        # Check directory existence
        if (!dir.exists(get(.i))) stop(.quot.past(.i), " directory must exist.")

      }

    }

  }


  # Check argument 'var.metr' ----

  # Restrict available TLS variables and metrics according to 'function case',
  # 'tree.ds', scan approach and plot design(s)
  if (!"tls" %in% names(.funct[[funct]])) .var.metr$tls <- NULL
  else {

    if (funct %in% "est")
      .var.metr$tls <- .var.metr$tls[.var.metr$tls %in%
                                       paste(c("N", "G"), "tls", sep = ".")]

    if (is.null(tree.ds))
      .var.metr$tls <- .var.metr$tls[!.var.metr$tls %in%
                                       sapply(c("N", "G", "V"), paste,
                                              names(.ds.meth), sep = ".")]

    if (is.null(tree.ds) & length(tree.tls$v.mer) > 0)
      .var.metr$tls <- .var.metr$tls[!.var.metr$tls %in%
                                       sapply(c("N", "G", "V", "v.mer", "h.mer"), paste,
                                              names(.ds.meth), sep = ".")]

    if (!scan.approach %in% "single")
      .var.metr$tls <- .var.metr$tls[!.var.metr$tls %in%
                                       sapply(c("N", "G", "V"), paste,
                                              c(names(.ds.meth), "sh", "pam"),
                                              sep = ".")]

    if (!scan.approach %in% "single" & length(tree.tls$v.mer) > 0)
      .var.metr$tls <- .var.metr$tls[!.var.metr$tls %in%
                                       sapply(c("N", "G", "V", "v.mer", "h.mer"), paste,
                                              c(names(.ds.meth), "sh", "pam"),
                                              sep = ".")]

    .var.metr$tls <- matrix(TRUE, nrow = length(plot.design),
                            ncol = length(.var.metr$tls),
                            dimnames = list(plot.design, .var.metr$tls))

    if (length(tree.tls$v.mer) > 0)
    .var.metr$tls[!rownames(.var.metr$tls) %in% c("fixed.area", "k.tree"),
                  colnames(.var.metr$tls) %in% sapply(c("N", "G", "V", "v.mer", "h.mer"), paste,
                                                      c(names(.ds.meth), "sh"),
                                                      sep = ".")] <- FALSE

    if (length(tree.tls$v.mer) < 1)
    .var.metr$tls[!rownames(.var.metr$tls) %in% c("fixed.area", "k.tree"),
                  colnames(.var.metr$tls) %in% sapply(c("N", "G", "V"), paste,
                                                      c(names(.ds.meth), "sh"),
                                                      sep = ".")] <- FALSE

    if (length(tree.tls$v.mer) > 0)
    .var.metr$tls[!rownames(.var.metr$tls) %in% "angle.count",
                  colnames(.var.metr$tls) %in% paste(c("N", "G", "V", "v.mer", "h.mer"), "pam",
                                                     sep = ".")] <- FALSE

    if (length(tree.tls$v.mer) < 1)
    .var.metr$tls[!rownames(.var.metr$tls) %in% "angle.count",
                  colnames(.var.metr$tls) %in% paste(c("N", "G", "V"), "pam",
                                                     sep = ".")] <- FALSE

    .var.metr$tls <- .var.metr$tls[, apply(.var.metr$tls, 2, any), drop = FALSE]

  }

  # Restrict field variables according to 'function case' and trees variables
  # provided by user in 'tree.field'
  if (!"field" %in% names(.funct[[funct]])) .var.metr$field <- NULL
  else {

    if (!is.data.frame(tree.field) ||
        any(!.var.field.user %in% colnames(tree.field)))
      .var.metr$field <-
        .var.metr$field[!.var.metr$field %in%
                          names(.var.field.user)[!.var.field.user %in%
                                                   colnames(tree.field)]]
    .var.metr$field <- matrix(TRUE, nrow = length(plot.design),
                              ncol = length(.var.metr$field),
                              dimnames = list(plot.design, .var.metr$field))

  }

  # 'var.metr' must be a list with one or two elements named 'tls' and 'field'
  var.metr <- .check.class(x = var.metr, x.class = class(.var.metr),
                           name = 'var.metr', n = Inf)
  # if (length(var.metr) != length(.var.metr))
  #   stop("'var.metr' must have length equal to ", length(.var.metr), ".")
  .miss <- names(.var.metr)[! names(.var.metr) %in% names(var.metr)]
  if (length(.miss) > 0)
    stop("Element(s) named ", .quot.past(.miss), " is(are) missing in ",
         "'var.metr' argument.")

  # 'var.metr$tls' must be NULL or a character vector containing the TLS
  # variables and metrics to be computed
  # Remark. If '.var.metr$tls' is NULL, 'var.metr$tls' is forced to be NULL;
  # otherwise, if 'var.metr$tls', all possible TLS variables and metrics
  # are assigned by default
  for (.i in names(var.metr)) {

    if (!.i %in% names(.var.metr)) var.metr[[.i]] <- NULL
    else {

      if (is.null(var.metr[[.i]]))
        var.metr[[.i]] <- colnames(.var.metr[[.i]])
      else
        var.metr[[.i]] <- .check.class(x = var.metr[[.i]],
                                       x.class = "character",
                                       name = paste("var.metr", .i, sep ="$"),
                                       n = Inf, val = colnames(.var.metr[[.i]]))

    }

  }

  # Remove plot design(s) with no metric/variable to be computed
  .inval <- do.call(cbind, .var.metr)[, do.call(c, var.metr), drop = FALSE]
  .inval <- rownames(.inval)[apply(!.inval, 1, all)]
  if (length(.inval) > 0) {

    warning("Plot design(s) ", .quot.past(.inval), " in 'plot.design' ",
            "argument was(were) discarded, since no associated metric or ",
            "variable needs to be computed.", immediate. = TRUE)
    plot.design <- plot.design[!plot.design %in% .inval]

  }


  # Check 'plot.parameters' argument ----

  # Define a logical value indicating if results will be computed by stratum
  # (only for function case 'metr' when 'stratum' column is included in
  # 'plot.parameters' argument)
  .by.stratum <- funct %in% "metr" &
    (is.data.frame(plot.parameters) & "stratum" %in% colnames(plot.parameters))

  # Define a logical vector containing all possible mandatory columns in
  # 'plot.parameters' according to 'function case', '.by.stratum' and plot
  # design(s)
  if (funct %in% c("sim", "est")) {

    .col.mand <- sapply(.plot.design[plot.design], paste, c("max", "incr"),
                        sep = ".")
    if (funct %in% "sim") .col.mand <- c(.col.mand, "num.trees")
    .col.mand <- matrix(FALSE, nrow = 1, ncol = length(.col.mand),
                        dimnames = list(NULL, .col.mand))
    .col.mand[, paste(.plot.design[plot.design], "max", sep = ".")] <- TRUE

  } else {

    .col.mand <- c(.plot.design[plot.design], "num.trees")
    .col.mand <- matrix(FALSE, nrow = 1, ncol = length(.col.mand),
                        dimnames = list(NULL, .col.mand))
    .col.mand[, .plot.design[plot.design]] <- TRUE

  }

  if (.by.stratum) .col.mand <- cbind("stratum" = TRUE, .col.mand)

  # Define a character vector containing (if necessary) the class of all
  # possible mandatory columns in 'plot.parameters'
  .col.class <- rep("numeric", ncol(.col.mand))
  names(.col.class) <- colnames(.col.mand)
  .col.class[names(.col.class) %in% "stratum"] <- NA
  .col.class[names(.col.class) %in%
               c(paste("k", c("max", "incr"), sep = "."), "num.trees")] <-
    "integer"
  .col.class <- .col.class[!is.na(.col.class)]

  # 'plot.parameters' must be a data.frame with at least one row
  plot.parameters <- .check.class(x = plot.parameters, x.class = "data.frame",
                                  name = "plot.parameters",
                                  n = ifelse(funct %in% c("sim", "est"), 1,
                                             Inf))

  # Check mandatory columns existence, assign values by default (if necessary),
  # remove non necessary columns, check mandatory columns classes and check all
  # values are no-NA
  plot.parameters <- .check.col(x = plot.parameters, x.mand = .col.mand,
                                x.class = .col.class, name = "plot.parameters",
                                def = .plot.parameters)

  # Check there are no duplicated strata
  if (.by.stratum) {

    # Define stratum code
    .stratum <- plot.parameters[, "stratum", drop = FALSE]
    .stratum <- cbind(code = apply(.stratum, 1, .quot.past), .stratum)

    # Check duplicated
    .dupl <- unique(.stratum[duplicated(.stratum[, "code"]), "code"])
    if (length(.dupl) > 0)
      stop("'plot.parameters' argument has several different rows for the ",
           "stratum(strata) ", .quot.past(.dupl, quot = ""), ".")

  }

  # Check 'radius', 'radius.max', 'k', 'k.max', 'BAF', 'BAF.max' and/or
  # 'num.trees' are strictly positive
  .inval <- c(sapply(.plot.design, paste, c("", ".max"), sep = ""), "num.trees")
  .inval <- colnames(plot.parameters)[colnames(plot.parameters) %in% .inval]
  .inval <- .inval[!apply(plot.parameters[, .inval, drop = FALSE] > 0, 2, all)]
  if (length(.inval) > 0)
    stop("All values in column(s) ", .quot.past(.inval), " of ",
         "'plot.parameters' argument must be strictly positive.")

  # Check 'radius.incr', 'k.incr" and 'BAF.incr' are positive
  .inval <- paste(.plot.design, "incr", sep = ".")
  .inval <- colnames(plot.parameters)[colnames(plot.parameters) %in% .inval]
  .inval <- .inval[!apply(plot.parameters[, .inval, drop = FALSE] >= 0, 2, all)]
  if (length(.inval) > 0)
    stop("All values in column(s) ", .quot.past(.inval), " of ",
         "'plot.parameters' argument must be positive.")

  # Add number of decimals places to be considered
  for (.i in .plot.design[plot.design]) {

    .dec <- sapply(plot.parameters[, ifelse(funct %in% c("sim", "est"),
                                            paste(.i, "incr", sep = "."), .i)],
                   .decimals)
    .dec <- matrix(.dec, ncol = 1,
                   dimnames = list(NULL, paste(.i, "dec", sep = ".")))
    plot.parameters <- cbind(plot.parameters, .dec)

  }


  # Check 'tree.tls' argument ----

  if (!"tls" %in% names(.funct[[funct]])) tree.tls <- NULL
  else {

    # Define a logical value indicating if results will be computed by stratum
    # (only for function case 'metr' when 'stratum' column is included in
    # 'plot.parameters' argument) or if strata will be differentiated in charts
    # with different colours (only for function case 'est' when 'stratum' column
    # is included in 'tree.tls' argument)
    .by.stratum.2 <- .by.stratum |
      (funct %in% "est" & (is.data.frame(tree.tls) &
                             "stratum" %in% colnames(tree.tls)))

    # Define a logical matrix containing all possible mandatory columns in
    # 'tree.tls' according to 'function case', '.by.stratum.2' and 'var.metr'
    if(length(tree.tls$v.mer) < 1)
    .col.mand <- c("id", "file", "tree", unlist(.scan.approach), "h.dist",
                   "dbh", "h", "v",
                   paste("n.pts", c("", ".est", ".red", ".red.est"),
                         sep = ""),
                   "partial.occlusion")

    if(length(tree.tls$v.mer) > 0)
      .col.mand <- c("id", "file", "tree", unlist(.scan.approach), "h.dist",
                     "dbh", "h", "h.mer", "v", "v.mer",
                     paste("n.pts", c("", ".est", ".red", ".red.est"),
                           sep = ""),
                     "partial.occlusion")

    if (.by.stratum.2) .col.mand <- c("stratum", .col.mand)
    .col.mand <- matrix(FALSE, nrow = length(var.metr$tls),
                        ncol = length(.col.mand),
                        dimnames = list(var.metr$tls, .col.mand))

    if(length(tree.tls$v.mer) < 1)
    .col.mand[, colnames(.col.mand) %in%
                c("stratum", "id", "tree", "h.dist", "dbh", "h", "v")] <- TRUE

    if(length(tree.tls$v.mer) > 0)
    .col.mand[, colnames(.col.mand) %in%
                c("stratum", "id", "tree", "h.dist", "dbh", "h", "h.mer", "v", "v.mer")] <- TRUE

    .col.mand[rownames(.col.mand) %in%
                c(sprintf("P%02i", .prob),
                  # Z coordinate
                  "mean.z", "mean.q.z", "mean.g.z", "mean.h.z", "median.z", "mode.z",
                  "max.z", "min.z", "var.z", "sd.z", "CV.z", "D.z", "ID.z",
                  "kurtosis.z", "skewness.z",
                  "p.a.mean.z", "p.a.mode.z", "p.a.2m",
                  "p.b.mean.z", "p.b.mode.z", "p.b.2m.z", "CRR.z",
                  "L2.z", "L3.z", "L4.z", "L.CV.z", "L.skewness.z", "L.kurtosis.z",
                  "median.a.d.z", "mode.a.d.z",
                  "weibull_c.z", "weibull_b.z",

                  # Rho coordinate
                  "mean.rho", "mean.q.rho", "mean.g.rho", "mean.h.rho", "median.rho", "mode.rho",
                  "max.rho", "min.rho", "var.rho", "sd.rho", "CV.rho", "D.rho", "ID.rho",
                  "kurtosis.rho", "skewness.rho",
                  "p.a.mean.rho", "p.a.mode.rho",
                  "p.b.mean.rho", "p.b.mode.rho", "CRR.rho",
                  "L2.rho", "L3.rho", "L4.rho", "L.skewness.rho", "L.kurtosis.rho",
                  "L.CV.rho",
                  "median.a.d.rho", "mode.a.d.rho",
                  "weibull_c.rho", "weibull_b.rho",

                  # R coordinate
                  "mean.r", "mean.q.r", "mean.g.r", "mean.h.r", "median.r", "mode.r",
                  "max.r", "min.r", "var.r", "sd.r", "CV.r", "D.r", "ID.r",
                  "kurtosis.r", "skewness.r",
                  "p.a.mean.r", "p.a.mode.r",
                  "p.b.mean.r", "p.b.mode.r", "CRR.r",
                  "L2.r", "L3.r", "L4.r", "L.CV.r", "L.skewness.r", "L.kurtosis.r",
                  "median.a.d.z", "mode.a.d.z",
                  "weibull_c.r", "weibull_b.r"),

              colnames(.col.mand) %in% "file"] <- TRUE

    if(length(tree.tls$v.mer) < 1)
    .col.mand[rownames(.col.mand) %in% paste(c("N", "G", "V"), "sh", sep = "."),
              colnames(.col.mand) %in%
                c(unlist(.scan.approach), "partial.occlusion")] <- TRUE

    if(length(tree.tls$v.mer) > 0)
    .col.mand[rownames(.col.mand) %in% paste(c("N", "G", "V", "v.mer", "h.mer"), "sh", sep = "."),
              colnames(.col.mand) %in%
                c(unlist(.scan.approach), "partial.occlusion")] <- TRUE

    for (.i in paste("n.pts", c("", ".est", ".red", ".red.est"), sep = ""))
      .col.mand[rownames(.col.mand) %in% .i,
                colnames(.col.mand) %in% .i] <- TRUE

    .col.mand <- .col.mand[, apply(.col.mand, 2, any), drop = FALSE]

    # Define a character vector containing (if necessary) the class of all
    # possible mandatory columns in 'tree.tls'
    .col.class <- rep("numeric", ncol(.col.mand))
    names(.col.class) <- colnames(.col.mand)
    .col.class[names(.col.class) %in% c("stratum", "id", "file", "tree")] <- NA
    .col.class <- .col.class[!is.na(.col.class)]

    # 'tree.tls' must be a data.frame with at least one row
    tree.tls <- .check.class(x = tree.tls, x.class = "data.frame",
                                  name = "tree.tls", n = Inf)

    # Check mandatory columns existence, remove non necessary columns, check
    # mandatory columns classes and check all values are no-NA
    tree.tls <- .check.col(x = tree.tls, x.mand = .col.mand,
                                x.class = .col.class, name = "tree.tls")

    # Define code for plot IDs
    .id <- unique(tree.tls[, "id", drop = FALSE])
    .id <- cbind(code = apply(.id, 1, .quot.past), .id)

    # Check strata
    if (.by.stratum.2) {

      # Check strata are included in 'plot.parameters' (only for function case
      # 'metr' when 'stratum' column is included in 'plot.parameters' argument)
      if (.by.stratum) {

        .miss <- apply(unique(tree.tls[, "stratum", drop = FALSE]), 1,
                       .quot.past)
        .miss <- .miss[!.miss %in%
                         .stratum[.stratum[, "code"] %in% .miss, "code"]]
        if (length(.miss) > 0)
          stop("Stratum(strata) ", .quot.past(.miss, quot = ""), " in ",
               "'tree.tls' argument is(are) missing in 'plot.parameters'",
               " argument.")

      }

      # Define code for unique pairs ('stratum', 'id')
      .stratum.id <- unique(tree.tls[, c("stratum", "id"), drop = FALSE])
      .stratum.id <- cbind(code = paste("(", apply(.stratum.id, 1, .quot.past),
                                        ")", sep = ""),
                           .stratum.id)

      # Check each plot ID has an unique associated stratum
      .dupl <- unique(.stratum.id[duplicated(.stratum.id[, "id"]), "id"])
      if (length(.dupl) > 0)
        stop("Plot ID(s) ", .quot.past(.dupl), " has(have) several different ",
             "strata associated in 'tree.tls' argument.")

      # Save code for pairs ('stratum', 'id'), and remove from 'tree.tls'
      .stratum.id <- .stratum.id[match(.id[, "id"], .stratum.id[, "id"]), ,
                                 drop = FALSE]
      .id <- cbind(.id, stratum.id.code = .stratum.id[, "code"],
                   stratum = .stratum.id[, "stratum"])
      tree.tls <- tree.tls[, !colnames(tree.tls) %in% "stratum",
                                     drop = FALSE]

    }

    # Check TXT files
    if ("file" %in% colnames(tree.tls)) {

      # Define code for unique pairs ('id', 'file')
      .id.file <- unique(tree.tls[, c("id", "file"), drop = FALSE])
      .id.file <- cbind(code = paste("(", apply(.id.file, 1, .quot.past), ")",
                                     sep = ""),
                        .id.file)

      # Check each plot ID has an unique associated TXT file
      .dupl <- unique(.id.file[duplicated(.id.file[, "id"]), "id"])
      if (length(.dupl) > 0)
        stop("Plot ID(s) ", .quot.past(.dupl), " has(have) several different ",
             "TXT files associated in 'tree.tls' argument.")

      # Check each TXT file has an unique associated plot ID
      .dupl <- unique(.id.file[duplicated(.id.file[, "file"]), "file"])
      if (length(.dupl) > 0)
        stop("TXT file(s) ", .quot.past(.dupl), " has(have) several different ",
             "plot IDs associated in 'tree.tls' argument.")

      # Check TXT files existence
      .miss <- .id.file[, "file"]
      .miss <- .miss[!file.exists(file.path(dir.data, .miss))]
      if (length(.miss) > 0)
        stop("TXT file(s) ", .quot.past(.miss), " in 'tree.tls' argument ",
             "does(do) not exists in the directory specified in 'dir.data' ",
             "argument.")

      # Save code for pairs ('id', 'file'), and remove file from 'tree.tls'
      .id.file <- .id.file[match(.id[, "id"], .id.file[, "id"]), , drop = FALSE]
      .id <- cbind(.id, id.file.code = .id.file[, "code"],
                   file = .id.file[, "file"])
      tree.tls <- tree.tls[, !colnames(tree.tls) %in% "file",
                                     drop = FALSE]

    }

    # Define code for pairs ('id', 'tree') and check they are not duplicated
    .id.tree <- tree.tls[, c("id", "tree"), drop = FALSE]
    .id.tree <- cbind(code = paste("(", apply(.id.tree, 1, .quot.past), ")",
                                   sep = ""),
                      .id.tree)
    .dupl <- unique(.id.tree[duplicated(.id.tree[, "code"]), "code"])
    if (length(.dupl) > 0)
      stop("'tree.tls' argument has several different rows for the ",
           "pair(s) of plot ID and tree ", .quot.past(.dupl, quot = ""), ".")

    # Check 'dbh, 'h', 'n.pts', 'n.pts.red', 'n.pts.est' and/or
    # 'n.pts.red.est' are strictly positive
    .inval <- c("dbh", "h", paste("n.pts",
                                  c("", ".est", ".red", ".red.est"), sep = ""))
    .inval <- colnames(tree.tls)[colnames(tree.tls) %in% .inval]
    .inval <- .inval[!apply(tree.tls[, .inval, drop = FALSE] > 0, 2, all)]
    if (length(.inval) > 0)
      stop("All values in column(s) ", .quot.past(.inval), " of ",
           "'tree.tls' argument must be strictly positive.")

    # Check 'n.pts.red'/'n.pts.red.est' is greater or equal to
    # 'n.pts'/'n.pts.est'
    .inval <- t(sapply(paste("n.pts", c("", ".red"), sep = ""), paste,
                       c("", ".est"), sep = ""))
    .inval <- .inval[, apply(apply(.inval, 1:2, "%in%",
                                   colnames(tree.tls)), 2, all),
                     drop = FALSE]
    if (ncol(.inval) > 0) {

      .inval <- .inval[, apply(.inval, 2,
                               function(dat.par, dat) {
                                 any(dat[, dat.par[1]] < dat[, dat.par[2]])
                               },
                               dat = tree.tls),
                       drop = FALSE]
      if (ncol(.inval) > 0)
        stop(paste("All values in column(s) ", .quot.past(.inval[1, ]), " of ",
                   "'tree.tls' argument must be greater or equal to the ",
                   "corresponding ones in column(s) ", .quot.past(.inval[2, ]),
                   ". ", sep = ""))

    }

    # Check 'h.dist' is positive
    .inval <- "h.dist"
    .inval <- colnames(tree.tls)[colnames(tree.tls) %in% .inval]
    .inval <- .inval[!apply(tree.tls[, .inval, drop = FALSE] >= 0, 2, all)]
    if (length(.inval) > 0)
      stop("All values in column(s) ", .quot.past(.inval), " of ",
           "'tree.tls' argument must be positive.")

    # Check 'phi.left' and 'phi.right' belong to the interval [0, 2 * pi]
    .inval <- unlist(.scan.approach)
    .inval <- colnames(tree.tls)[colnames(tree.tls) %in% .inval]
    .inval <- .inval[!apply(tree.tls[, .inval, drop = FALSE] >= 0 &
                              tree.tls[, .inval, drop = FALSE] <= 2 * pi,
                            2, all)]
    if (length(.inval) > 0)
      stop("All values in column(s) ", .quot.past(.inval), " of ",
           "'tree.tls' argument must be in the interval [0, 2 * pi].")

    # Check 'partial.occlusion' is 0 or 1
    .inval <- "partial.occlusion"
    .inval <- colnames(tree.tls)[colnames(tree.tls) %in% .inval]
    .inval <- .inval[!apply(tree.tls[, .inval, drop = FALSE] == 0 |
                              tree.tls[, .inval, drop = FALSE] == 1,
                            2, all)]
    if (length(.inval) > 0)
      stop("All values in column(s) ", .quot.past(.inval), " of ",
           "'tree.tls' argument must be 0 or 1.")

  }


  # Check 'tree.ds' argument ----

  if (!"tls" %in% names(.funct[[funct]])) tree.ds <- NULL
  else if (!is.null(tree.ds)) {

    # Define a logical matrix containing all possible mandatory columns in
    # 'tree.ds' according to 'var.metr'
    .col.mand <- c("id", "tree", .ds.meth)
    .col.mand <- matrix(FALSE, nrow = length(var.metr$tls),
                        ncol = length(.col.mand),
                        dimnames = list(var.metr$tls, .col.mand))
    for (.i in names(.ds.meth))
      .col.mand[rownames(.col.mand) %in% paste(c("N", "G", "V", "v.mer", "h.mer"), .i, sep = "."),
                colnames(.col.mand) %in%
                  c("id", "tree", paste("P", .i, sep = "."))] <- TRUE
    .col.mand <- .col.mand[, apply(.col.mand, 2, any), drop = FALSE]

    # Define a character vector containing (if necessary) the class of all
    # possible mandatory columns in 'tree.ds'
    .col.class <- rep("numeric", ncol(.col.mand))
    names(.col.class) <- colnames(.col.mand)
    .col.class[names(.col.class) %in% c("id", "tree")] <- NA
    .col.class <- .col.class[!is.na(.col.class)]


    # If there are no mandatory columns, 'tree.ds' is forced to be NULL;
    # otherwise, checking process is continued
    if (ncol(.col.mand) == 0) tree.ds <- NULL
    else {

      # 'tree.ds' must be a list with at least an element named 'tree'
      tree.ds <- .check.class(x = tree.ds, x.class = "list",
                                   name = "tree.ds", n = Inf)
      .miss <- c("tree")[!c("tree") %in% names(tree.ds)]
      if (length(.miss) > 0)
        stop("Element(s) named ", .quot.past(.miss), " is(are) missing in ",
             "'tree.ds' argument.")
      tree.ds <- tree.ds$tree

      # 'tree.ds$tree' must be a data.frame with at least one row
      tree.ds <- .check.class(x = tree.ds, x.class = "data.frame",
                                   name = "tree.ds$tree", n = Inf)

      # Check mandatory columns existence, remove non necessary columns, check
      # mandatory columns classes and check all values are no-NA
      tree.ds <- .check.col(x = tree.ds, x.mand = .col.mand,
                                 x.class = .col.class,
                                 name = "tree.ds$tree")

      # Discard rows corresponding to plot IDs not included in 'tree.tls'
      .miss <- unique(tree.ds[!tree.ds[, "id"] %in% .id[, "id"],
                                   "id"])
      if (length(.miss) > 0) {

        warning("Row(s) of 'tree.ds$tree' argument associated to plot ",
                "ID(s) ", .quot.past(.miss, quot = ""), " was(were) discarded ",
                "because this(these) plot(s) is(are) missing in ",
                "'tree.tls' argument.", immediate. = TRUE)
        tree.ds <- tree.ds[!tree.ds[, "id"] %in% .miss, ,
                                     drop = FALSE]

      }

      # Check plot IDs included in 'tree.tls' are no missing
      .miss <- .id[, "id"]
      .miss <- .miss[!.miss %in% unique(tree.ds[, "id"])]
      if (length(.miss) > 0)
        stop("Plot ID(s) ", .quot.past(.miss), " in 'tree.tls' ",
             "argument is(are) missing in 'tree.ds$tree' argument.")

      # Check there are no duplicated pairs ('id', 'tree')
      .dupl <- tree.ds[, c("id", "tree"), drop = FALSE]
      .dupl <- paste("(", apply(.dupl, 1, .quot.past), ")", sep = "")
      .dupl <- unique(.dupl[duplicated(.dupl)])
      if (length(.dupl) > 0)
        stop("'tree.ds$tree' argument has several different rows for the ",
             "pair(s) of plot ID and tree ", .quot.past(.dupl, quot = ""), ".")

      # Check pairs ('id', 'tree') in 'tree.tls' are included in
      # 'tree.ds'
      .miss <- tree.ds[, c("id", "tree"), drop = FALSE]
      .miss <- paste("(", apply(.miss, 1, .quot.past), ")", sep = "")
      .miss <- .id.tree[, "code"][!.id.tree[, "code"] %in% .miss]
      if (length(.miss) > 0)
        stop("Pair(s) of plot ID and tree ", .quot.past(.miss, quot = ""),
             " in 'tree.tls' argument is(are) missing in ",
             "'tree.ds$tree' argument.")

      # Check 'P.hn', 'P.hr', 'P.hn.cov' and 'P.hr.cov' belong to the interval
      # [0, 1]
      .inval <- .ds.meth
      .inval <- colnames(tree.ds)[colnames(tree.ds) %in% .inval]
      .inval <- .inval[!apply(tree.ds[, .inval, drop = FALSE] >= 0 &
                                tree.ds[, .inval, drop = FALSE] <= 1,
                              2, all)]
      if (length(.inval) > 0)
        stop("All values in column(s) ", .quot.past(.inval), " of ",
             "'tree.ds$tree' argument must be in the interval [0, 1].")

    }

  }


  # Check 'tree.field' argument ----

  if (!"field" %in% names(.funct[[funct]])) tree.field <- NULL
  else {

    # Define a logical matrix containing all possible mandatory columns in
    # 'tree.field' according to 'var.metr'
    .col.mand <- c("id", "tree", "h.dist", "dbh", "h", unlist(.var.field.user))
    .col.mand <- matrix(FALSE, nrow = length(var.metr$field),
                        ncol = length(.col.mand),
                        dimnames = list(var.metr$field, .col.mand))
    .col.mand[, colnames(.col.mand) %in%
                c("id", "tree", "h.dist", "dbh", "h")] <- TRUE
    for (.i in names(.var.field.user))
      .col.mand[rownames(.col.mand) %in% .i,
                colnames(.col.mand) %in% .var.field.user[[.i]]] <- TRUE
    .col.mand <- .col.mand[, apply(.col.mand, 2, any), drop = FALSE]

    # Define a character vector containing (if necessary) the class of all
    # possible mandatory columns in 'tree.field'
    .col.class <- rep("numeric", ncol(.col.mand))
    names(.col.class) <- colnames(.col.mand)
    .col.class[names(.col.class) %in% c("id", "tree")] <- NA
    .col.class <- .col.class[!is.na(.col.class)]

    # 'tree.field' must be a data.frame with at least one row
    tree.field <- .check.class(x = tree.field, x.class = "data.frame",
                                    name = "tree.field", n = Inf)

    # Check mandatory columns existence, remove non necessary columns, check
    # mandatory columns classes and check all values are no-NA
    tree.field <- .check.col(x = tree.field, x.mand = .col.mand,
                                  x.class = .col.class,
                                  name = "tree.field")

    # Discard rows corresponding to plot IDs not included in 'tree.tls'
    .miss <- unique(tree.field[!tree.field[, "id"] %in% .id[, "id"],
                                    "id"])
    if (length(.miss) > 0) {

      warning("Row(s) of 'tree.field' argument associated to plot ID(s) ",
              .quot.past(.miss, quot = ""), " was(were) discarded because ",
              "this(these) plot(s) is(are) missing in 'tree.tls' ",
              "argument.", immediate. = TRUE)
      tree.field <- tree.field[!tree.field[, "id"] %in% .miss, ,
                                         drop = FALSE]

    }

    # Check plot IDs included in 'tree.tls' are no missing
    .miss <- .id[, "id"]
    .miss <- .miss[!.miss %in% unique(tree.field[, "id"])]
    if (length(.miss) > 0)
      stop("Plot ID(s) ", .quot.past(.miss), " in 'tree.tls' ",
           "argument is(are) missing in 'tree.field' argument.")

    # Check there are no duplicated pairs ('id', 'tree')
    .dupl <- tree.field[, c("id", "tree"), drop = FALSE]
    .dupl <- paste("(", apply(.dupl, 1, .quot.past), ")", sep = "")
    .dupl <- unique(.dupl[duplicated(.dupl)])
    if (length(.dupl) > 0)
      stop("'tree.field' argument has several different rows for the ",
           "pair(s) of plot ID and tree ", .quot.past(.dupl, quot = ""), ".")

    # Check 'dbh, 'h', 'v.user' and/or 'w.user' are strictly positive
    .inval <- c("dbh", "h", unlist(.var.field.user))
    .inval <- colnames(tree.field)[colnames(tree.field) %in% .inval]
    .inval <- .inval[!apply(tree.field[, .inval, drop = FALSE] > 0, 2,
                            all)]
    if (length(.inval) > 0)
      stop("All values in column(s) ", .quot.past(.inval), " of ",
           "'tree.field' argument must be strictly positive.")

    # Check 'h.dist' is positive
    .inval <- "h.dist"
    .inval <- colnames(tree.field)[colnames(tree.field) %in% .inval]
    .inval <- .inval[!apply(tree.field[, .inval, drop = FALSE] >= 0, 2,
                            all)]
    if (length(.inval) > 0)
      stop("All values in column(s) ", .quot.past(.inval), " of ",
           "'tree.field' argument must be positive.")

  }


  # Create a list containing empty data.frames where results will be saved ----
  # and restrict trees' database(s) according to 'dbh.min', 'h.min' and/or
  # 'max.dist'

  # Create a list containing empty data.frames where results will be saved for
  # each plot design
  stand <- vector("list", length(plot.design))
  names(stand) <- plot.design
  for (.i in names(stand)) {

    .col.names <- c("id", .plot.design[.i])
    for (.j in names(.funct[[funct]]))
      .col.names <- c(.col.names,
                      var.metr[[.j]][.var.metr[[.j]][.i, var.metr[[.j]]]])
    if (.by.stratum.2) .col.names <- c("stratum", .col.names)
    stand[[.i]] <- data.frame(matrix(numeric(), ncol = length(.col.names),
                                     dimnames = list(NULL, .col.names)),
                              stringsAsFactors = FALSE)

  }

  # Restrict trees' database(s) according to 'dbh.min', 'h.min' and/or
  # 'max.dist'
  for (.i in names(.funct[[funct]])) {

    .tree <- get(paste("tree", .i, sep ="."))
    .inval <- which(.tree[, "dbh"] < dbh.min | .tree[, "h"] < h.min |
                      .tree[, "h.dist"] > max.dist)
    if (length(.inval) > 0) {

      warning(length(.inval), " tree(s) in 'tree.list.", .i, "' argument ",
              "was(were) discarded, since dbh or height were less than minima ",
              "values indicated in 'dbh.min' and 'h.min' arguments, or ",
              "horizontal distance is greater than 'max.dist' argument.",
              immediate. = TRUE)
      .tree <- .tree[- .inval, , drop = FALSE]
      assign(paste("tree.list", .i, sep ="."), .tree)

      # Remove plot ID(s) without trees for calculations below
      .miss <- .id[!.id[, "id"] %in% unique(.tree[, "id"]), "id"]
      if (length(.miss) > 0) {

        if (length(.miss) < nrow(.id)) {

          warning("Plot ID(s) ", .quot.past(.miss), " in 'tree.list.", .i,
                  "' argument was(were) discarded, since all its(their) trees ",
                  "have dbh or height less than minima values indicated in ",
                  "'dbh.min' and 'h.min' arguments, or horizontal distance ",
                  "greater than 'max.dist' argument.", immediate. = TRUE)
          .id <- .id[!.id[, "id"] %in% .miss, , drop = FALSE]

        } else
          stop("All plot ID(s) in 'tree.list.", .i, "' argument was(were) ",
               "discarded, since all trees have dbh or height less than ",
               "minima values indicated in 'dbh.min' and 'h.min' arguments, ",
               "or horizontal distance greater than 'max.dist' argument.")

      }

    }

  }


  # Loop for each TLS plot ----

  for (.i in 1:nrow(.id)) {

    # Define initial time, and print message with the plot ID
    t0 <- Sys.time()
    message("Computing ",
            ifelse(funct %in% c("sim", "est"), "simulations", "metrics"),
            " for plot: ", .id[.i, "code"])

    # Define list where results associated to the plot will be saved
    .stand <- vector("list", length(.funct[[funct]]))
    names(.stand) <- names(.funct[[funct]])

    # Select plot parameters according to '.by.stratum'
    .str <- ifelse(.by.stratum,
                   match(.id[.i, "stratum"], plot.parameters[, "stratum"]), 1)
    .par <- plot.parameters[.str, , drop = FALSE]


    # Loop for each trees' database (TLS and field data) ----

    for (.j in names(.stand)) {

      # Define list where results associated to the plot will be saved
      .stand[[.j]] <- vector("list", length(plot.design))
      names(.stand[[.j]]) <- plot.design


      # Create trees' database (TLS and field data) ----

      # Select data corresponding to the plot from the complete trees' database
      .tree <- get(paste("tree", .j, sep ="."))
      .tree <- .tree[.tree[, "id"] %in% .id[.i, "id"], , drop = FALSE]
      rownames(.tree) <- NULL

      # Convert dbh (cm) to International System of Units (m)
      .tree[, "dbh"] <- .tree[, "dbh"] / 100

      # Compute radius, k and BAF, and tree variables according to plot
      # design(s) and 'tree.var'. Currently available tree variables: basal area
      # (g) and volume (v)
      .col.mand <- c("g", "v", "v.mer", "h.mer")

      .col.mand <- matrix(FALSE, nrow = length(var.metr[[.j]]),
                          ncol = length(.col.mand),
                          dimnames = list(var.metr[[.j]], .col.mand))
      .col.mand[rownames(.col.mand) %in%
                  c(paste("G", c("tls", names(.ds.meth), "sh", "pam"),
                          sep = "."), "G"),
                colnames(.col.mand) %in% "g"] <- TRUE
      .col.mand[rownames(.col.mand) %in%
                  c(paste("V", c("tls", names(.ds.meth), "sh", "pam"),
                          sep = "."), "V"),
                colnames(.col.mand) %in% "v"] <- TRUE
      .col.mand[rownames(.col.mand) %in%
                  c(paste("v.mer", c("tls", names(.ds.meth), "sh", "pam"),
                          sep = "."), "v.mer"),
                colnames(.col.mand) %in% "v.mer"] <- TRUE
      .col.mand[rownames(.col.mand) %in%
                  c(paste("h.mer", c("tls", names(.ds.meth), "sh", "pam"),
                          sep = "."), "h.mer"),
                colnames(.col.mand) %in% "v.mer"] <- TRUE
      .col.mand <- colnames(.col.mand)[apply(.col.mand, 2, any)]
      .tree <- .tree.calc(tree = .tree, plot.design = .plot.design[plot.design],
                          tree.var = .col.mand, v.calc = v.calc)

      # Compute angular aperture (TLS data)
      if (.j == "tls" & all(c("phi.left", "phi.right") %in% colnames(.tree))) {

        # Compute angular aperture
        .wide <- .tree[, "phi.right"] - .tree[, "phi.left"]
        .wide <- ifelse(.wide < 0, (2 * pi) + .wide, .wide)

        # Remove angular coordinates, and add angular aperture
        .tree <- .tree[, !colnames(.tree) %in% c("phi.left", "phi.right"),
                       drop = FALSE]
        .tree <- cbind(.tree, wide = .wide)

      }

      # Add detection probability from distance sampling database (TLS data)
      if (.j == "tls" & !is.null(tree.ds)) {

        .ds <- tree.ds[tree.ds[, "id"] %in% .id[.i, "id"], ,
                            drop = FALSE]
        .tree <- cbind(.tree, .ds[match(.tree[, "tree"], .ds[, "tree"]),
                                  colnames(.ds) %in% .ds.meth, drop = FALSE])

      }


      # Create points database(s) (TLS data) ----

      .data.tls <- NULL
      if (.j == "tls" & "file" %in% colnames(.id)) {

        # Read points' database
        .data.tls <-
          suppressMessages(vroom::vroom(file.path(dir.data, .id[.i, "file"]),
                                        col_select = c("x", "y", "z", "rho", "r"),
                                        progress = FALSE))
        .data.tls <- as.matrix(.data.tls)

        # Discard points according to 'max.dist'
        .inval <- which(.data.tls[, "rho"] > max.dist)
        if (length(.inval) > 0) {

          warning(length(.inval), " point(s) in point cloud associated to ",
                  "plot ", .id[.i, "code"], " was(were) discarded, since ",
                  "horizontal distance is greater than 'max.dist' argument.",
                  immediate. = TRUE)
          .data.tls <- .data.tls[- .inval, , drop = FALSE]

        }

        # Order .data.tls by rho, select columns required for calculations
        # below, and convert to matrix
        .data.tls <- .data.tls[order(.data.tls[, "rho"], decreasing = FALSE),
                               c("z", "rho", "r"), drop = FALSE]

      }


      # Loop for each plot design (TLS and field data) ----

      for (.k in plot.design) {

        # Order trees' database by radius/k/BAF in order to simplify
        # calculations below
        if (.k %in% c("fixed.area", "k.tree"))
          .tree <- .tree[order(.tree[, .plot.design[.k]], decreasing = FALSE), ,
                         drop = FALSE]
        else if (.k %in% "angle.count")
          .tree <- .tree[order(.tree[, .plot.design[.k]], decreasing = TRUE), ,
                         drop = FALSE]


        # Create radius/k/BAF sequence according to 'function case', ----
        # 'plot.parameters' argument, and radius/k/BAF values in trees' database

        # Select number of decimals
        .par.dec <- .par[, paste(.plot.design[.k], "dec", sep = ".")]

        # Compute range of values in trees' database
        .funct.rang <- switch(.k, fixed.area = ".customCeiling",
                              k.tree = ".customCeiling",
                              angle.count = ".customFloor")
        .db.rang <- range(get(.funct.rang)(.tree[, .plot.design[.k]],
                                           Decimals = .par.dec))

        # Select minimum value (ensuring at least one tree is always selected
        # for fixed area and k-tree plot designs)
        .par.min <- .db.rang[1]

        # Select maximum value
        if (funct %in% c("sim", "est"))
          .par.max <- .par[, paste(.plot.design[.k], "max", sep = ".")]
        else .par.max <- .par[, .plot.design[.k]]
        # Adjust maximum value according to 'max.dist' for fixed area plot
        # design, number of trees for k-tree plot design, and maximum BAF for
        # angle-count plot design
        .ref.max <- switch(.k, fixed.area = max.dist, k.tree = .db.rang[2],
                           angle.count = .db.rang[2])
        if (.par.max > .ref.max) {

          .par.max <- .ref.max
          warning("For ", .funct[[funct]][.j], " plot ", .id[.i, "code"], ", ",
                  ifelse(funct %in% c("sim", "est"), "maximum ", ""),
                  .quot.past(.plot.design[.k]), " was reduced to ",
                  .par.max,
                  switch(.k,
                         fixed.area = paste(", since this is the maximum",
                                            "horizontal distance indicated in",
                                            "'max.dist' argument"),
                         k.tree = paste(", since this is the number of trees",
                                        "in the plot"),
                         angle.count = paste(" to ensure that at least one",
                                             "tree is always selected")),
                  ".", immediate. = TRUE)

        }

        # Adjust maximum value according to minimum one
        if (.par.min > .par.max) {

          .par.max <- .par.min
          warning("For ", .funct[[funct]][.j], " plot ", .id[.i, "code"], ", ",
                  ifelse(funct %in% c("sim", "est"), "maximum ", ""),
                  .quot.past(.plot.design[.k]), " was increased to ", .par.max,
                  switch(.k,
                         fixed.area = paste(" to ensure that at least one",
                                            "tree is always selected"),
                         k.tree = paste(" to ensure that at least one",
                                        "tree is always selected"),
                         angle.count = paste(", since this ensures all trees",
                                             "in the plot are selected")),
                  ".", immediate. = TRUE)

        }

        # Force coincidence between minimum and maximum value, and select
        # increment according to 'function case'
        if (!funct %in% c("sim", "est")) {

          .par.min <- .par.max
          .par.incr <- 0

        } else .par.incr <- .par[, paste(.plot.design[.k], "incr", sep = ".")]

        # Create sequence
        .par.seq <- seq(from = .par.max, to = .par.min, by = - .par.incr)
        .par.seq <- sort(unique(round(.par.seq, .par.dec)))
        names(.par.seq) <- .format.numb(x = .par.seq, dec = .par.dec)


        # Compute stand variables and/or metrics according to plot design  ----
        # and 'var.metr'

        # Compute expansion factors with occlusion corrections and estimate
        # stand variables per ha, and compute mean diameters and heights
        .col.names <- var.metr[[.j]][.var.metr[[.j]][.k, var.metr[[.j]]]]
        .col.names <-
          .col.names[.col.names %in%
                       c(sapply(c("N", "G", "V", "v.mer", "h.mer"), paste,
                                c("tls", names(.ds.meth), "sh", "pam"),
                                sep = "."),
                         paste(names(c(.mean.d, .mean.h)), "tls", sep = "."),
                         paste("n.pts", c("", ".est", ".red", ".red.est"),
                               sep = ""),
                         "N", "G", "V", names(.var.field.user),
                         names(c(.mean.d, .mean.h)))]
        .stand[[.j]][[.k]] <- lapply(.par.seq, .stand.calc,
                                     plot.design = .plot.design[.k],
                                     tree = .tree, var.metr = .col.names,
                                     ds.meth = .ds.meth,
                                     var.field.user = .var.field.user,
                                     mean.d = .mean.d, mean.h = .mean.h)
        .stand[[.j]][[.k]] <- do.call(rbind, .stand[[.j]][[.k]])

        # Compute mean dominant diameters and heights
        .col.names <- paste(names(c(.mean.d, .mean.h)), ".0", sep = "")
        if (.j %in% "tls") .col.names <- paste(.col.names, .j, sep = ".")
        names(.col.names) <- paste(names(c(.mean.d, .mean.h)), ".0", sep = "")
        .col.names <-
          .col.names[.col.names %in%
                       var.metr[[.j]][.var.metr[[.j]][.k, var.metr[[.j]]]]]
        if (length(.col.names) > 0) {

          if (.k %in% "fixed.area")
            .dh.0 <- fixed_area_cpp(radius_seq = .par.seq,
                                    hdist = .tree[, "h.dist"],
                                    d = .tree[, "dbh"], h = .tree[, "h"],
                                    num = .par[, "num.trees"])
          else if (.k %in% "k.tree")
            .dh.0 <- k_tree_cpp(k_seq = .par.seq,
                                radius_seq = .tree[, "radius.k"],
                                k = .tree[, "k"], d = .tree[, "dbh"],
                                h = .tree[, "h"], num = .par[, "num.trees"])
          else if (.k %in% "angle.count")
            .dh.0 <- angle_count_cpp(baf_seq = .par.seq, baf = .tree[, "BAF"],
                                     d = .tree[, "dbh"], h = .tree[, "h"],
                                     num = .par[, "num.trees"])
          rownames(.dh.0) <- .format.numb(x = .dh.0[, .plot.design[.k]],
                                          dec = .par.dec)
          colnames(.dh.0)[colnames(.dh.0) %in% names(.col.names)] <-
            .col.names[colnames(.dh.0)[colnames(.dh.0) %in% names(.col.names)]]
          .stand[[.j]][[.k]] <- cbind(.stand[[.j]][[.k]],
                                      .dh.0[rownames(.stand[[.j]][[.k]]),
                                            .col.names, drop = FALSE])

        }

        # Compute rho sequence for z coordinate metrics
        if (.k %in% "fixed.area") .rho.seq <- .par.seq
        else if (.k %in% "k.tree")
          .rho.seq <- sapply(.par.seq,
                             function(k, tree) {
                               max(tree[tree[, "k"] <= k, "radius.k"])
                             },
                             tree = .tree)
        else if (.k %in% "angle.count")
          .rho.seq <- sapply(.par.seq,
                             function(BAF, tree) {
                               max(tree[tree[, "BAF"] >= BAF, "h.dist"])
                             },
                             tree = .tree)

        # Compute percentiles of z coordinate
        .col.names <- sprintf("P%02i", .prob)
        .col.names <-
          .col.names[.col.names %in%
                       var.metr[[.j]][.var.metr[[.j]][.k, var.metr[[.j]]]]]
        if (length(.col.names) > 0) {

          .perc <- height_perc_cpp(rho_seq = .rho.seq, z = .data.tls[, "z"],
                                   rho = .data.tls[, "rho"])
          rownames(.perc) <- names(.rho.seq)
          .stand[[.j]][[.k]] <- cbind(.stand[[.j]][[.k]],
                                      .perc[rownames(.stand[[.j]][[.k]]),
                                            .col.names, drop = FALSE])

        }

        # Compute descriptive statistics of z coordinate, percentage of points
        # above mode and mean, and parameters of fitted Weibull distribution
        .col.names <- c("mean.z", "mean.q.z", "mean.g.z", "mean.h.z", "median.z", "mode.z",
                        "max.z", "min.z", "var.z", "sd.z", "CV.z", "D.z", "ID.z",
                        "kurtosis.z", "skewness.z",
                        "p.a.mean.z", "p.a.mode.z", "p.a.2m.z",
                        "p.b.mean.z", "p.b.mode.z", "p.b.2m.z", "CRR.z",
                        "L2.z", "L3.z", "L4.z", "L.CV.z", "L.skewness.z", "L.kurtosis.z",
                        "median.a.d.z", "mode.a.d.z",
                        "weibull_c.z", "weibull_b.z",

                        # Compute descriptive statistics of rho coordinate
                        "mean.rho", "mean.q.rho", "mean.g.rho", "mean.h.rho", "median.rho", "mode.rho",
                        "max.rho", "min.rho", "var.rho", "sd.rho", "CV.rho", "D.rho", "ID.rho",
                        "kurtosis.rho", "skewness.rho",
                        "p.a.mean.rho", "p.a.mode.rho",
                        "p.b.mean.rho", "p.b.mode.rho", "CRR.rho",
                        "L2.rho", "L3.rho", "L4.rho", "L.CV.rho", "L.skewness.rho", "L.kurtosis.rho",
                        "median.a.d.rho", "mode.a.d.rho",
                        "weibull_c.rho", "weibull_b.rho",

                        # Compute descriptive statistics of r coordinate
                        "mean.r", "mean.q.r", "mean.g.r", "mean.h.r", "median.r", "mode.r",
                        "max.r", "min.r", "var.r", "sd.r", "CV.r", "D.r", "ID.r",
                        "kurtosis.r", "skewness.r",
                        "p.a.mean.r", "p.a.mode.r",
                        "p.b.mean.r", "p.b.mode.r", "CRR.r",
                        "L2.r", "L3.r", "L4.r", "L.CV.r", "L.skewness.r", "L.kurtosis.r",
                        "median.a.d.r", "mode.a.d.r",
                        "weibull_c.r", "weibull_b.r")

        .col.names <-
          .col.names[.col.names %in%
                       var.metr[[.j]][.var.metr[[.j]][.k, var.metr[[.j]]]]]
        if (length(.col.names) > 0) {

          .pts.met <- .points.metrics(rho_seq = .rho.seq, data = .data.tls,
                                      metr = .col.names)
          .stand[[.j]][[.k]] <- cbind(.stand[[.j]][[.k]],
                                      .pts.met[rownames(.stand[[.j]][[.k]]),
                                               .col.names, drop = FALSE])

        }


        # Convert diameters from International System of Units (m) to cm
        .col.names <- sapply(names(.mean.d), paste,
                             c(".tls", ".0.tls", "", ".0"), sep = "")
        .col.names <-
          colnames(.stand[[.j]][[.k]])[colnames(.stand[[.j]][[.k]]) %in%
                                         .col.names]
        .stand[[.j]][[.k]][.col.names] <- .stand[[.j]][[.k]][.col.names] * 100

        # Add stratum
        if (.by.stratum.2)
          .stand[[.j]][[.k]] <- cbind(stratum = .id[.i, "stratum"],
                                      .stand[[.j]][[.k]])

      }

    }


    # Save fixed area/k-tree/angle-count plot results, and write csv files ----
    # containing them if 'save.result' is TRUE

    for (.j in plot.design) {


      # Merge fixed area/k-tree/angle-count plot results for all trees'
      # database(s)

      # Results for first trees' database(s)
      .stand.j <- .stand[[1]][[.j]]

      # Add results for the rest of trees' databases (if any)
      for (.k in names(.stand)[-1]) {

        # Restrict rows to those existing in trees' database to be added
        .stand.j <- .stand.j[rownames(.stand.j) %in% rownames(.stand[[.k]][[.j]]), ,
                             drop = FALSE]

        # Add new columns
        .stand.j <- cbind(.stand.j,
                          .stand[[.k]][[.j]][rownames(.stand.j),
                                             !colnames(.stand[[.k]][[.j]]) %in%
                                               colnames(.stand.j),
                                             drop = FALSE])
      }

      # Save reordered merged results
      rownames(.stand.j) <- NULL
      stand[[.j]] <- rbind(stand[[.j]],
                           .stand.j[, colnames(stand[[.j]]), drop = FALSE])
      stand[[.j]] <- stand[[.j]][order(stand[[.j]][, .plot.design[[.j]]],
                                       stand[[.j]][, "id"]), ,
                                 drop = FALSE]
      rownames(stand[[.j]]) <- NULL

      # Write CSV file
      if (save.result) {

        .file <- paste(switch(funct, sim = "simulations",
                              metr = "metrics.variables",
                              est = "estimation.plot.size"),
                       .j, "plot.csv", sep = ".")
        utils::write.csv(stand[[.j]], file = file.path(dir.result, .file),
                         row.names = FALSE)

      }

    }

    # Define final time, and print message
    t1 <- Sys.time()
    message(" (", format(round(difftime(t1, t0, units = "secs"), 2)), ")")

  }

  return(stand)

}


# Compute Pearson/Spearman correlations

.cor.pearson <- function(x, y){

  stats::cor.test(x = x, y = y, method = 'pearson')

}

.cor.spearman <- function(x, y){

  stats::cor.test(x = x, y = y, method = 'spearman', exact = FALSE)

}





# When no trees are detected ----



.no.trees.detected.single <- function(data, d.mer, plot.attributes, dir.result, save.result){

  if(is.null(data$id) & is.null(d.mer)){

    # If plot identification (id) is not available

    .colnames <- c("tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else if (is.null(data$id) & !is.null(d.mer)) {

    # If plot identification (id) is not available

    .colnames <- c("tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "n.pts", "SS.max", "sinuosity", "lean", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else if (!is.null(data$id) & is.null(d.mer)) {

    # If plot identification (id) is available

    .colnames <- c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else {

    # If plot identification (id) is available

    .colnames <- c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  }

  .tree <- data.frame(matrix(nrow = 1, ncol = length(.colnames), dimnames = list(NULL, .colnames)))

  if(!is.null(data$id)){

    .tree$id <- data$id[1]
    .tree$file <- data$file[1]

  }

  # Lastly, aggregate attributes table
  if(!is.null(plot.attributes))
    .tree <- merge(.tree, plot.attributes, by = "id", all = FALSE)


  if(isTRUE(save.result)){

    utils::write.csv(.tree,
                     file = file.path(dir.result, "tree.tls.csv"),
                     row.names = FALSE)
  }


  #####
  return(.tree)


}



.no.trees.detected.multi <- function(data, d.mer, plot.attributes, dir.result, save.result){

  if(is.null(data$id) & is.null(d.mer)){

    # If plot identification (id) is not available

    .colnames <- c("tree", "x", "y", "phi", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else if (is.null(data$id) & !is.null(d.mer)) {

    # If plot identification (id) is not available

    .colnames <- c("tree", "x", "y", "phi", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else if (!is.null(data$id) & is.null(d.mer)) {

    # If plot identification (id) is available

    .colnames <- c("id", "file", "tree", "x", "y", "phi", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  } else {

    # If plot identification (id) is available

    .colnames <- c("id", "file", "tree", "x", "y", "phi", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")

  }

  .tree <- data.frame(matrix(nrow = 1, ncol = length(.colnames), dimnames = list(NULL, .colnames)))


  if(!is.null(data$id)){

    .tree$id <- data$id[1]
    .tree$file <- data$file[1]

  }

  # Lastly, aggregate attributes table
  if(!is.null(plot.attributes))
    .tree <- merge(.tree, plot.attributes, by = "id", all = FALSE)


  if(isTRUE(save.result)){

    utils::write.csv(.tree,
                     file = file.path(dir.result, "tree.tls.csv"),
                     row.names = FALSE)
  }


  #####
  return(.tree)


}

      double precision function myfun(n,xval)
      integer i,n
c     **********
      double precision xval
      integer ieq1(2), ieq2(2)
      double precision req(2)
      character*(LEN=200) bob
      character dave*(20)
      equivalence (req(1),ieq1(1))
      equivalence (req(2),ieq2(1))
c
      data req(1) /1.0000000d-16/
      data req(2) /1.0000000d-308/
c
      myfun = xval
      bob(1:20) = dave
      do 10 i = 1, n
   10    myfun = myfun + xval
      return
c
      end
c
      subroutine glob_sub(n,xval,yval)
      integer i,n
c     **********
      double complex xval,yval
c
      yval = xval
      do 20 i = 1, n
         yval = yval + xval
   20    continue
      return
c
      end

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

geometric_features_dist <- function(points, dist, Anisotropy = FALSE, Eigenentropy = FALSE, Eigenvalue_sum = FALSE, First_eigenvalue = FALSE, Linearity = FALSE, Normal_x = FALSE, Normal_y = FALSE, Normal_z = FALSE, Number_of_points = FALSE, Omnivariance = FALSE, PCA_1 = FALSE, PCA_2 = FALSE, Planarity = FALSE, Second_eigenvalue = FALSE, Sphericity = FALSE, Surface_density = FALSE, Surface_variation = FALSE, Third_eigenvalue = FALSE, Verticality = FALSE, Volume_density = FALSE, num_threads = 0L, use_spatial_index = TRUE, grid_cell_size = -1.0) {
    .Call(`_FORTLS_geometric_features_dist`, points, dist, Anisotropy, Eigenentropy, Eigenvalue_sum, First_eigenvalue, Linearity, Normal_x, Normal_y, Normal_z, Number_of_points, Omnivariance, PCA_1, PCA_2, Planarity, Second_eigenvalue, Sphericity, Surface_density, Surface_variation, Third_eigenvalue, Verticality, Volume_density, num_threads, use_spatial_index, grid_cell_size)
}

fit_circle_cpp_modified <- function(points) {
    .Call(`_FORTLS_fit_circle_cpp_modified`, points)
}

is_one_row_all_na <- function(df) {
    .Call(`_FORTLS_is_one_row_all_na`, df)
}

sample_indices <- function(total_number_obs, return_obs) {
    .Call(`_FORTLS_sample_indices`, total_number_obs, return_obs)
}

internal_ransac <- function(dat, dist = 0.05) {
    .Call(`_FORTLS_internal_ransac`, dat, dist)
}

RANSAC_cpp <- function(data, dist = 0.05) {
    .Call(`_FORTLS_RANSAC_cpp`, data, dist)
}

iterations_RANSAC <- function(data, n_iterations = 600L) {
    .Call(`_FORTLS_iterations_RANSAC`, data, n_iterations)
}

weighted_mean_arit <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_arit`, x, w)
}

weighted_mean_sqrt <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_sqrt`, x, w)
}

weighted_mean_geom <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_geom`, x, w)
}

weighted_mean_harm <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_harm`, x, w)
}

height_perc_cpp <- function(rho_seq, z, rho) {
    .Call(`_FORTLS_height_perc_cpp`, rho_seq, z, rho)
}

fixed_area_cpp <- function(radius_seq, hdist, d, h, num) {
    .Call(`_FORTLS_fixed_area_cpp`, radius_seq, hdist, d, h, num)
}

k_tree_cpp <- function(k_seq, radius_seq, k, d, h, num) {
    .Call(`_FORTLS_k_tree_cpp`, k_seq, radius_seq, k, d, h, num)
}

angle_count_cpp <- function(baf_seq, baf, d, h, num) {
    .Call(`_FORTLS_angle_count_cpp`, baf_seq, baf, d, h, num)
}

module sig_help_markdown
  implicit none
  private

contains
  !> Top level Doc
  subroutine sub2call(arg1, arg2)
    integer, intent(in) :: arg1 !< Doc for arg1
    integer, intent(in), optional :: arg2 !< Doc for arg2
    print*, "sub2call: arg1=", arg1
    if (present(arg2)) print*, "sub2call: arg2=", arg2
  end subroutine sub2call

  !> Top level Doc
  function fun2fcall(arg1, arg2) result(res)
    integer, intent(in) :: arg1 !< Doc for arg1
    integer, intent(in), optional :: arg2 !< Doc for arg2
    integer :: res
    res = arg1
    if (present(arg2)) res = res + arg2
  end function fun2fcall

  subroutine calling()
    call sub2call(1, 2)
    print*, "fun2fcall(1, 2)=", fun2fcall(1, 2)
  end subroutine calling

end module sig_help_markdown

# This function assigns detected sections to their stem axis

.stem.assignment.single.scan <- function(data, stem.section, breaks){


    kk <- data.frame(sec = table(data$sec[data$arc.circ == 1]))


    kk[, 1] <- as.numeric(as.character(kk[, 1]))
    kk <- kk[kk[,2] == max(kk[,2]), ]


    if(nrow(kk) > 1){
      kk$dif <- abs(kk[, 1] - 1.3)
      kk <- kk[kk$dif == min(kk$dif), ]}


    if(nrow(kk) > 1){
      kk <- kk[kk[, 1] == min(kk[, 1]), ]}


    if(kk[, 1] > 1){
      eje <- data[data$sec < kk[, 1] + 1 & data$sec > kk[, 1] - 1,
                    c("center.x", "center.y", "center.rho", "center.phi", "radius", "sec")]} else {

      eje <- data[data$sec < as.numeric(as.character(kk[, 1])) + 1,
                    c("center.x", "center.y", "center.rho", "center.phi", "radius", "sec")]}


    .dbscan <- dbscan::dbscan(eje[, c("center.x", "center.y"), drop = FALSE],
                              eps = mean(eje$radius[eje$sec == as.character(kk[, 1])], na.rm = TRUE), minPts = 1)

    eje$tree <- .dbscan$cluster
    eje <- eje[, c("tree", "sec", "center.x", "center.y", "center.rho", "center.phi")]
    colnames(eje) <- c("tree", "sec", "x", "y", "rho", "phi")
    eje <- eje[order(eje$tree), ]

    eje.2 <- data.frame(tree = as.numeric(), sec = as.numeric(),
                        x = as.numeric(), y = as.numeric(),
                        rho = as.numeric(), phi = as.numeric())

    for (i in unique(eje$tree)) {

      kk <- eje[eje$tree == i, ]
      kk <- merge(kk, data.frame(sec = breaks), all.y = TRUE)

      kk$tree <- i
      kk$x <- ifelse(is.na(kk$x), mean(kk$x, na.rm = TRUE), kk$x)
      kk$y <- ifelse(is.na(kk$y), mean(kk$y, na.rm = TRUE), kk$y)
      kk$rho <- ifelse(is.na(kk$rho), mean(kk$rho, na.rm = TRUE), kk$rho)
      kk$phi <- ifelse(is.na(kk$phi), mean(kk$phi, na.rm = TRUE), kk$phi)

      eje.2 <- rbind(eje.2, kk)}

    return(eje.2)

}


.stem.assignment.single.scan.2 <- function(filter, eje, stem.section, x.center, y.center, single.tree){

  eje$sec <- as.character(eje$sec)

  eje$phi <- atan2(eje$y - y.center, eje$x - x.center)
  eje$phi <- ifelse(eje$phi < 0, eje$phi + (2 * pi), eje$phi)
  eje$rho <- sqrt((eje$x - x.center) ^ 2 + (eje$y - y.center) ^ 2)

  filter$cluster <- 1:nrow(filter)
  filter$sec <- as.character(filter$sec)
  filter <- merge(eje, filter, by = "sec", all.y = TRUE)
  filter$dist <- sqrt((filter$center.x - filter$x) ^ 2 + (filter$center.y - filter$y) ^ 2)
  filter$dist.rho <- abs(filter$center.rho - filter$rho)
  filter$dist.phi <- abs(filter$center.phi - filter$phi)
  filter$sec <- as.numeric(filter$sec)

  filter <- filter[!is.na(filter$dist) | !is.na(filter$dist.rho) | !is.na(filter$dist.phi), ]

  filteraux <- data.frame(tree = as.numeric(), sec = as.numeric(),
                          center.x = as.numeric(), center.y = as.numeric(),
                          center.phi = as.numeric(), center.rho = as.numeric(),
                          center.r = as.numeric(), center.theta = as.numeric(),
                          radius = as.numeric(),
                          n.pts = as.numeric(), n.pts.red = as.numeric(),
                          circ = as.numeric(), arc.cir = as.numeric())

  if(!is.null(single.tree)){

    for (i in unique(filter$tree)) {

      .filt.tree <- filter[filter$tree == i, ]

      for (j in unique(.filt.tree$sec)) {

        .filt <- .filt.tree[.filt.tree$sec == j, ]

        .filt$dist.total <- .filt$dist / max(.filt$dist)

        .filt$dist.total <- ifelse(is.nan(.filt$dist.total), 0, .filt$dist.total)

        .filt <- .filt[.filt$dist.total == min(.filt$dist.total), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[1, ]

        if(.filt$dist > 2.5){next}

        .filt <- .filt[, c("tree", "cluster", "sec", "dist",
                           "center.x", "center.y", "center.phi",
                           "center.rho", "center.r", "center.theta",
                           "radius", "n.pts", "n.pts.red",
                           "phi.left",  "phi.right", "arc.circ")]

        filteraux <- rbind(filteraux, .filt)

        filter <- filter[filter$cluster != .filt$cluster, ]

      }

    }

  } else {

    for (i in unique(filter$tree)) {

      .filt.tree <- filter[filter$tree == i, ]

      for (j in unique(.filt.tree$sec)) {

        .filt <- .filt.tree[.filt.tree$sec == j, ]

        .filt$dist.total <- .filt$dist / max(.filt$dist) +
          .filt$dist.rho / max(.filt$dist.rho) +
          .filt$dist.phi / max(.filt$dist.phi)

        .filt$dist.total <- ifelse(is.nan(.filt$dist.total), 0, .filt$dist.total)

        .filt <- .filt[.filt$dist.total == min(.filt$dist.total), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[.filt$dist.rho == min(.filt$dist.rho), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[.filt$dist.phi == min(.filt$dist.phi), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[1, ]

        if(.filt$dist > 2.5 | .filt$dist.rho > 2.5 | .filt$dist.phi > pi/2){next}


        .filt <- .filt[, c("tree", "cluster", "sec", "dist",
                           "center.x", "center.y", "center.phi",
                           "center.rho", "center.r", "center.theta",
                           "radius", "n.pts", "n.pts.red",
                           "phi.left",  "phi.right", "arc.circ")]

        filteraux <- rbind(filteraux, .filt)

        filter <- filter[filter$cluster != .filt$cluster, ]

      }

    }

  }

  # Removing repeated cluster

  filteraux <- filteraux[!duplicated(filteraux$cluster), ]


  filteraux <- filteraux[order(filteraux$tree, filteraux$sec), , drop = FALSE]


  filter <- data.frame(tree = as.numeric(), sec = as.numeric(), dist = as.numeric(),
                       center.x = as.numeric(), center.y = as.numeric(),
                       center.phi = as.numeric(), center.rho = as.numeric(),
                       center.r = as.numeric(), center.theta = as.numeric(),
                       radius = as.numeric(),
                       n.pts = as.numeric(), n.pts.red = as.numeric(),
                       circ = as.numeric(), arc.cir = as.numeric())

  for (i in unique(filteraux$tree)) {

    .filt <- filteraux[filteraux$tree == i, ]

    .filt <- .filt[.filt$dist < mean(.filt$dist, na.rm = TRUE) + mean(.filt$radius, na.rm = TRUE), ]


    if(nrow(.filt) < 1){next}


    if(nrow(.filt) == 1){

      .filt$dif <- NA
      filter <- rbind(filter, .filt)
      next

    }


    .filt$dif <- c(diff(.filt$radius), diff(.filt$radius)[length(diff(.filt$radius))])
    .filt$dif.sec <- c(abs(diff(.filt$sec)), abs(diff(.filt$sec))[length(abs(diff(.filt$sec)))])
    .filt$dif <- ifelse(.filt$dif.sec > 1, .filt$dif * .filt$dif.sec, .filt$dif)

    threshold <- mean(.filt$radius) / 2

    .filt <- .filt[.filt$dif < threshold & .filt$dif > - threshold, ]

    if(nrow(.filt) < 1){next}

    if(nrow(.filt) == 1){

      .filt <- .filt[, -ncol(.filt)]
      filter <- rbind(filter, .filt)
      next

    }


    while (max(.filt$dif) >= threshold / 2 & min(.filt$dif) <= - threshold) {

      .filt <- .filt[.filt$dif < threshold / 2 & .filt$dif > - threshold, ]


      .filt$dif <- c(diff(.filt$radius), diff(.filt$radius)[length(diff(.filt$radius))])
      .filt$dif.sec <- c(abs(diff(.filt$sec)), abs(diff(.filt$sec))[length(abs(diff(.filt$sec)))])
      .filt$dif <- ifelse(.filt$dif.sec > 1, .filt$dif * .filt$dif.sec, .filt$dif)


    }


    .filt <- .filt[, -ncol(.filt)]
    filter <- rbind(filter, .filt)



  }

  return(filter)

}


# This function assigns detected sections to their stem axis

.stem.assignment.multi.scan <- function(.filter, eje, stem.section, x.center, y.center, single.tree){


  # eje$sec <- as.character(eje$sec)

  eje$phi <- atan2(eje$y - y.center, eje$x - x.center)
  eje$phi <- ifelse(eje$phi < 0, eje$phi + (2 * pi), eje$phi)
  eje$rho <- sqrt((eje$x - x.center) ^ 2 + (eje$y - y.center) ^ 2)



  .filter$cluster <- 1:nrow(.filter)
  .filter$sec <- as.character(.filter$sec)
  .filter <- merge(eje, .filter, by = "sec", all.y = TRUE)
  .filter$dist <- sqrt((.filter$center.x - .filter$x) ^ 2 + (.filter$center.y - .filter$y) ^ 2)
  .filter$dist.rho <- abs(.filter$center.rho - .filter$rho)
  .filter$dist.phi <- abs(.filter$center.phi - .filter$phi)
  # .filter$sec <- as.numeric(.filter$sec)


  .filter <- .filter[!is.na(.filter$dist) | !is.na(.filter$dist.rho) | !is.na(.filter$dist.phi), ]

  filteraux <- data.frame(tree = as.numeric(), sec = as.numeric(),
                          center.x = as.numeric(), center.y = as.numeric(),
                          center.phi = as.numeric(), center.rho = as.numeric(),
                          center.r = as.numeric(), center.theta = as.numeric(),
                          radius = as.numeric(),
                          n.pts = as.numeric(), n.pts.red = as.numeric(),
                          circ = as.numeric(), arc.cir = as.numeric())

  if(!is.null(single.tree)){

    for (i in unique(.filter$tree)) {

      .filt.tree <- .filter[.filter$tree == i, ]

      for (j in unique(.filt.tree$sec)) {

        .filt <- .filt.tree[.filt.tree$sec == j, ]

        .filt$dist.total <- .filt$dist / max(.filt$dist)

        .filt$dist.total <- ifelse(is.nan(.filt$dist.total), 0, .filt$dist.total)

        .filt <- .filt[.filt$dist.total == min(.filt$dist.total), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[1, ]

        if(.filt$dist > 2.5){next}

        .filt <- .filt[, c("tree", "cluster", "sec", "dist",
                           "center.x", "center.y", "center.phi",
                           "center.rho", "center.r", "center.theta",
                           "radius", "n.pts", "n.pts.red", "circ", "arc.circ")]

        filteraux <- rbind(filteraux, .filt)

        .filter <- .filter[.filter$cluster != .filt$cluster, ]

      }

    }

  } else {


    for (i in unique(.filter$tree)) {

      .filt.tree <- .filter[.filter$tree == i, ]

      for (j in unique(.filt.tree$sec)) {

        .filt <- .filt.tree[.filt.tree$sec == j, ]

        .filt$dist.total <- .filt$dist / max(.filt$dist) +
          .filt$dist.rho / max(.filt$dist.rho) +
          .filt$dist.phi / max(.filt$dist.phi)


        .filt$dist.total <- ifelse(is.nan(.filt$dist.total), 0, .filt$dist.total)

        .filt <- .filt[.filt$dist.total == min(.filt$dist.total), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[.filt$dist.rho == min(.filt$dist.rho), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[.filt$dist.phi == min(.filt$dist.phi), ]

        if(nrow(.filt) > 1)
          .filt <- .filt[1, ]

        if(.filt$dist > 2.5 | .filt$dist.rho > 2.5 | .filt$dist.phi > pi/2){next}


        .filt <- .filt[, c("tree", "cluster", "sec", "dist",
                           "center.x", "center.y", "center.phi",
                           "center.rho", "center.r", "center.theta",
                           "radius", "n.pts", "n.pts.red", "circ", "arc.circ")]

        filteraux <- rbind(filteraux, .filt)

        .filter <- .filter[.filter$cluster != .filt$cluster, ]

      }

    }

  }

  # Removing repeated cluster

  filteraux <- filteraux[!duplicated(filteraux$cluster), ]


  filteraux <- filteraux[order(filteraux$tree, filteraux$sec), , drop = FALSE]


  .filter <- data.frame(tree = as.numeric(), sec = as.numeric(), dist = as.numeric(),
                        center.x = as.numeric(), center.y = as.numeric(),
                        center.phi = as.numeric(), center.rho = as.numeric(),
                        center.r = as.numeric(), center.theta = as.numeric(),
                        radius = as.numeric(),
                        n.pts = as.numeric(), n.pts.red = as.numeric(),
                        circ = as.numeric(), arc.cir = as.numeric())

  for (i in unique(filteraux$tree)) {

    .filt <- filteraux[filteraux$tree == i, ]


    .filt <- .filt[.filt$dist < mean(.filt$dist, na.rm = TRUE) + mean(.filt$radius, na.rm = TRUE), ]



    if(nrow(.filt) < 1){next}


    if(nrow(.filt) == 1){

      .filt$dif <- NA
      .filter <- rbind(.filter, .filt)
      next

      }


    .filt$dif <- c(diff(.filt$radius), diff(.filt$radius)[length(diff(.filt$radius))])
    .filt$dif.sec <- c(abs(diff(.filt$sec)), abs(diff(.filt$sec))[length(abs(diff(.filt$sec)))])
    .filt$dif <- ifelse(.filt$dif.sec > 1, .filt$dif * .filt$dif.sec, .filt$dif)

    threshold <- mean(.filt$radius) / 2

    .filt <- .filt[.filt$dif < threshold & .filt$dif > -threshold, ]

    if(nrow(.filt) < 1){next}

    if(nrow(.filt) == 1){

        .filt <- .filt[, -ncol(.filt)]
        .filter <- rbind(.filter, .filt)
        next

      }


    while (max(.filt$dif) >= threshold / 2 & min(.filt$dif) <= -threshold) {

      .filt <- .filt[.filt$dif < threshold / 2 & .filt$dif > -threshold, ]


      .filt$dif <- c(diff(.filt$radius), diff(.filt$radius)[length(diff(.filt$radius))])
      .filt$dif.sec <- c(abs(diff(.filt$sec)), abs(diff(.filt$sec))[length(abs(diff(.filt$sec)))])
      .filt$dif <- ifelse(.filt$dif.sec > 1, .filt$dif * .filt$dif.sec, .filt$dif)


    }


    .filt <- .filt[, -ncol(.filt)]
    .filter <- rbind(.filter, .filt)


  }

  return(.filter)

}

module points
  type :: point
     real :: x, y
  end type point

  interface
     module function point_dist(a, b) result(distance)
       type(point), intent(in) :: a, b
       real :: distance
     end function point_dist

     module logical function is_point_equal_a(a, b)
      type(point), intent(in) :: a, b
    end function is_point_equal_a

    module subroutine is_point_equal_sub(a, b, test)
      type(point), intent(in) :: a, b
      logical, intent(out) :: test
    end subroutine is_point_equal_sub
  end interface
contains
  logical function is_point_equal(a, b)
    type(point), intent(in) :: a, b
    is_point_equal = merge(.true., .false., a%x == b%x .and. a%y == b%y)
  end function is_point_equal
end module points
#define __PARENT_MOD__ points
submodule (__PARENT_MOD__) points_a
contains
  module function point_dist(a, b)
    type(point), intent(in) :: a, b
    distance = sqrt((a%x - b%x)**2 + (a%y - b%y)**2)
  end function point_dist

  module procedure is_point_equal_a
    type(point) :: c
    is_point_equal_a = merge(.true., .false., a%x == b%x .and. a%y == b%y)
  end procedure is_point_equal_a

  module procedure is_point_equal_sub
    type(point) :: c
    test = is_point_equal(a,b)
  end procedure is_point_equal_sub
end submodule points_a

estimation.plot.size <- function(tree.tls,
                                 plot.parameters = data.frame(radius.max = 25,
                                                              k.max = 50,
                                                              BAF.max = 4),
                                 dbh.min = 4, average = FALSE,
                                 all.plot.designs = FALSE) {


  # Define stratum as 1 (by default) when tree.list$stratum is missed
  if(is.null(tree.tls$stratum) | all.plot.designs)
    tree.tls$stratum <- 1

  if(is.factor(tree.tls$stratum))
    tree.tls$stratum <- as.numeric(tree.tls$stratum)

  if(is.character(tree.tls$stratum))
    tree.tls$stratum <- as.numeric(as.factor(tree.tls$stratum))



  # Call internal function
  .est <- .sim.calc(funct = "est", tree.tls = tree.tls,
                    tree.ds = NULL, tree.field = NULL,
                    plot.design = c("fixed.area", "k.tree", "angle.count"),
                    plot.parameters = plot.parameters, scan.approach = "single",
                    var.metr = list(tls = c("N.tls", "G.tls")), v.calc = "parab",
                    dbh.min = dbh.min,  h.min = 1.3, max.dist = Inf,
                    dir.data = NULL, save.result = FALSE, dir.result = NULL)
  .fixed.area.plot <- .est$fixed.area
  .k.tree.plot <- .est$k.tree
  .angle.count.plot <- .est$angle.count

  # Create labels for plot axis
  .fixed.area.plot.axis <- list(main = "Circular fixed area plots", xlab = "Radius (m)")
  .k.tree.plot.axis <- list(main = "k-tree plots", xlab = "k-tree (trees)")
  .angle.count.plot.axis <- list(main = "Angle-count plots",
                                 xlab = expression("BAF" ~ (m^{2}/ha)))



  # Obtaining mean values and standard deviation when mean argument
  # is specified in the function ----

  if (average | all.plot.designs) {

    .fixed.area.plot.i <- split(.fixed.area.plot, .fixed.area.plot$stratum)

    .agregate <- function(data){

      .out <- data.frame(n = tapply(data$radius, data$radius, length),
                         radius = tapply(data$radius, data$radius, mean,
                                         na.rm = TRUE),
                         N = tapply(data$N, data$radius, mean, na.rm = TRUE),
                         N.sd = tapply(data$N, data$radius, stats::sd,
                                       na.rm = TRUE),
                         G = tapply(data$G, data$radius, mean, na.rm = TRUE),
                         G.sd = tapply(data$G, data$radius, stats::sd,
                                       na.rm = TRUE))

      .out$stratum <- data$stratum[1]

      return(.out)

    }

    .fixed.area.plot <- do.call(rbind, lapply(.fixed.area.plot.i, .agregate))

    .k.tree.plot.i <- split(.k.tree.plot, .k.tree.plot$stratum)

    .agregate <- function(data){

      .out <- data.frame(n = tapply(data$k, data$k, length),
                         k = tapply(data$k, data$k, mean, na.rm = TRUE),
                         N = tapply(data$N, data$k, mean, na.rm = TRUE),
                         N.sd = tapply(data$N, data$k, stats::sd, na.rm = TRUE),
                         G = tapply(data$G, data$k, mean, na.rm = TRUE),
                         G.sd = tapply(data$G, data$k, stats::sd, na.rm = TRUE))

      .out$stratum <- data$stratum[1]

      return(.out)

    }

    .k.tree.plot <- do.call(rbind, lapply(.k.tree.plot.i, .agregate))


    .angle.count.plot.i <- split(.angle.count.plot, .angle.count.plot$stratum)

    .agregate <- function(data){

      .out <- data.frame(n = tapply(data$BAF, data$BAF, length),
                         BAF = tapply(data$BAF, data$BAF, mean, na.rm = TRUE),
                         N = tapply(data$N, data$BAF, mean, na.rm = TRUE),
                         N.sd = tapply(data$N, data$BAF, stats::sd,
                                       na.rm = TRUE),
                         G = tapply(data$G, data$BAF, mean, na.rm = TRUE),
                         G.sd = tapply(data$G, data$BAF, stats::sd,
                                       na.rm = TRUE))

      .out$stratum <- data$stratum[1]

      return(.out)

    }

    .angle.count.plot <- do.call(rbind, lapply(.angle.count.plot.i,
                                               .agregate))

  }

  # List with plots simulatioins and axis information

  .plots <- list(.fixed.area.plot = list(.fixed.area.plot,
                                         .fixed.area.plot.axis),
                 .k.tree.plot = list(.k.tree.plot, .k.tree.plot.axis),
                 .angle.count.plot = list(.angle.count.plot,
                                          .angle.count.plot.axis))


  ##### PLOTS ####

  # Individual plots ----


  if (!average & !all.plot.designs) {

    # Defining margins

    oldpar <- graphics::par(no.readonly = TRUE)
    on.exit(graphics::par(oldpar))

    graphics::par(mfrow = c(3, 2))
    graphics::par(mar = c(4, 5, 2, 1))



    for (i in c(1:length(.plots))) {

      .data <- .plots[[i]]

      .axis <- .data[[2]]
      .data <- .data[[1]]

      # Density (tree/ha)

      plot(.data[, 3], .data$N, type = "n",
           main = .axis[[1]],
           xlab = .axis[[2]],
           ylab = expression("N.tls" ~ (trees/ha)),
           xlim = c(min(.data[, 3]), max(.data[, 3])),
           ylim = c(min(.data$N), max(.data$N)))

      # The rest of the plots

      for (.i in unique(.data$id)) {

        .dat <- subset(.data, .data$id == .i)

        .x <- .dat[, 3]
        .y <- .dat$N

        graphics::lines(.x, .y, col = scales::alpha(.dat$stratum, 0.5))

      }

      graphics::legend("topright",
                       legend = unique(as.character(.data$stratum)),
                       text.col = unique(.data$stratum),
                       bty = "n", pch = 15,
                       col = unique(.data$stratum))

      # Basal area (m2/ha)

      plot(.data[, 3], .data$G, type = "n",
           main = .axis[[1]],
           xlab = .axis[[2]],
           ylab = expression("G.tls" ~ (m^{2}/ha)),
           xlim = c(min(.data[, 3]), max(.data[, 3])),
           ylim = c(min(.data$G), max(.data$G)))

      # The rest of the plots

      for (.i in unique(.data$id)) {

        .dat <- subset(.data, .data$id == .i)

        .x <- .dat[, 3]
        .y <- .dat$G

        graphics::lines(.x, .y, col = scales::alpha(.dat$stratum, 0.5))

      }

      graphics::legend("topright",
                       legend = unique(as.character(.data$stratum)),
                       text.col = unique(.data$stratum),
                       bty = "n", pch = 15,
                       col = unique(.data$stratum))

    }
  }

  # Mean values ----

  if (average) {

    # Defining margins

    graphics::par(mfrow = c(3, 2))
    graphics::par(mar = c(4, 5, 2, 4))

    for (i in c(1:length(.plots))) {

      .data <- .plots[[i]]

      .axis <- .data[[2]]
      .data <- .data[[1]]

      .data$N.min <- .data$N - (.data$N.sd / 2)
      .data$N.max <- .data$N + (.data$N.sd / 2)

      .data$G.min <- .data$G - (.data$G.sd / 2)
      .data$G.max <- .data$G + (.data$G.sd / 2)


      plot(.data[, 2], .data$N, type = "n",
           main = .axis[[1]],
           xlab = .axis[[2]],
           ylab = expression("N.tls" ~ (trees/ha)),
           xlim = c(min(.data[, 2]), max(.data[, 2])),
           ylim = c(min(.data[, c("N", "N.min", "N.max")], na.rm = TRUE),
                    max(.data[, c("N", "N.min", "N.max")], na.rm = TRUE)))


      for (i in unique(.data$stratum)) {

        .dat <- subset(.data, .data$stratum == i)

        .area <- .dat[which(!is.na(.dat$N.sd)), , drop = FALSE]

        graphics::par(new=T)

        plot(.data[, 2], .data$N, type = "n",
             xlab = "", ylab = "",
             xlim = c(min(.data[, 2]), max(.data[, 2])),
             ylim = c(min(.data[, c("N", "N.min", "N.max")], na.rm = TRUE),
                      max(.data[, c("N", "N.min", "N.max")], na.rm = TRUE)))

        graphics::polygon(c(.area[, 2], rev(.area[, 2])),
                          c(.area$N.max, rev(.area$N.min)),
                          col = scales::alpha(.dat$stratum, 0.5), border = NA)

        graphics::lines(.dat[, 2], .dat$N, col = .dat$stratum)

        graphics::par(new=T)

        plot(.dat[, 2], .dat$n,
             ylim = c(0, max(.data$n)),
             xaxt="n", yaxt="n",
             ylab = "", xlab = "",
             type="l", lty = 3, col = .dat$stratum)

        graphics::axis(4)

        graphics::mtext(side = 4, line = 3, "No plots", col = "black",
                        cex = 0.75)

      }

      graphics::legend("topright",
                       legend = unique(as.character(.data$stratum)),
                       text.col = unique(.data$stratum),
                       bty = "n", pch = 15,
                       col = unique(.data$stratum))

      # Basal area (m2/ha)

      plot(.data[, 2], .data$G, type = "n",
           main = .axis[[1]],
           xlab = .axis[[2]],
           ylab = expression("G.tls" ~ (m^{2}/ha)),
           xlim = c(min(.data[, 2]), max(.data[, 2])),
           ylim = c(min(.data[, c("G", "G.min", "G.max")], na.rm = TRUE),
                    max(.data[, c("G", "G.min", "G.max")], na.rm = TRUE)))


      for (i in unique(.data$stratum)) {

        .dat <- subset(.data, .data$stratum == i)

        .area <- .dat[which(!is.na(.dat$G.sd)), , drop = FALSE]

        graphics::par(new=T)

        plot(.data[, 2], .data$G, type = "n",
             xlab = "", ylab = "",
             xlim = c(min(.data[, 2]), max(.data[, 2])),
             ylim = c(min(.data[, c("G", "G.min", "G.max")], na.rm = TRUE),
                      max(.data[, c("G", "G.min", "G.max")], na.rm = TRUE)))


        graphics::polygon(c(.area[, 2], rev(.area[, 2])),
                          c(.area$G.max, rev(.area$G.min)),
                          col = scales::alpha(.dat$stratum, 0.5), border = NA)

        graphics::lines(.dat[, 2], .dat$G, col = .dat$stratum)

        graphics::par(new=T)

        plot(.dat[, 2], .dat$n,
             ylim = c(0, max(.data$n)),
             xaxt="n", yaxt="n",
             ylab = "", xlab = "",
             type="l", lty = 3, col = .dat$stratum)

        graphics::axis(4)

        graphics::mtext(side = 4, line = 3, "No plots", col = "black",
                        cex = 0.75)

      }

      graphics::legend("topright",
                       legend = unique(as.character(.data$stratum)),
                       text.col = unique(.data$stratum),
                       bty = "n", pch = 15,
                       col = unique(.data$stratum))

    }

  }

  # All plot design together ----

  if (all.plot.designs) {

    if (average) grDevices::dev.new()

    # Defining margins

    graphics::par(mfrow = c(1, 2))
    graphics::par(mar = c(7, 5, 2, 1))

    .N.max <- max(.plots$.fixed.area.plot[[1]]$N,
                  .plots$.k.tree.plot[[1]]$N,
                  .plots$.angle.count.plot[[1]]$N)

    .N.min <- min(.plots$.fixed.area.plot[[1]]$N,
                  .plots$.k.tree.plot[[1]]$N,
                  .plots$.angle.count.plot[[1]]$N)



    for (i in c(1:length(.plots))) {

      .data <- .plots[[i]]

      .axis <- .data[[2]]
      .data <- .data[[1]]

      .data$N.min <- .data$N - (.data$N.sd / 2)
      .data$N.max <- .data$N + (.data$N.sd / 2)

      if (i > 1)
        graphics::par(new=T)

      plot(.data[, 2], .data$N, type = "n",
           main = "Density",
           xlab = "",
           xaxt = "n",
           ylab = expression("N.tls" ~ (trees/ha)),
           xlim = c(0, max(.data[, 2])),
           ylim = c(.N.min, .N.max))

      graphics::par(new = T)

      if(i == 1)
        j <- 1

      if(i == 2)
        j <- 3

      if(i == 3)
        j <- 5

      graphics::axis(1, line = j, col = i, col.ticks = i, col.axis = i)

      graphics::mtext(.axis[[2]], side = 1, line = j, at = 0, adj = 1, col = i)

      .area <- .data[which(!is.na(.data$N.sd)), , drop = FALSE]

      graphics::polygon(c(.area[, 2], rev(.area[, 2])),
                        c(.area$N.max, rev(.area$N.min)),
                        col = scales::alpha(i, 0.25), border = NA)

      graphics::lines(.data[, 2], .data$N, col = scales::alpha(i, 0.75),
                      lwd = 2)

    }


    graphics::legend("topright",
                     legend = c(.plots$.fixed.area.plot[[2]]$main,
                                .plots$.k.tree.plot[[2]]$main,
                                .plots$.angle.count.plot[[2]]$main),
                     text.col = c(1:length(.plots)), cex = 0.75,
                     bty = "n", pch = 15,
                     col = c(1:length(.plots)))


    # Basal area (m2/ha)

    .G.max <- max(.plots$.fixed.area.plot[[1]]$G,
                  .plots$.k.tree.plot[[1]]$G,
                  .plots$.angle.count.plot[[1]]$G)

    .G.min <- min(.plots$.fixed.area.plot[[1]]$G,
                  .plots$.k.tree.plot[[1]]$G,
                  .plots$.angle.count.plot[[1]]$G)


    for (i in c(1:length(.plots))) {

      .data <- .plots[[i]]

      .axis <- .data[[2]]
      .data <- .data[[1]]

      .data$G.min <- .data$G - (.data$G.sd / 2)
      .data$G.max <- .data$G + (.data$G.sd / 2)

      if (i > 1)
        graphics::par(new=T)

      plot(.data[, 2], .data$G, type = "n",
           main = "Basal Area",
           xlab = "",
           xaxt = "n",
           ylab = expression("G.tls" ~ (m^{2}/ha)),
           xlim = c(0, max(.data[, 2])),
           ylim = c(.G.min, .G.max))

      graphics::par(new=T)

      if(i == 1)
        j <- 1

      if(i == 2)
        j <- 3

      if(i == 3)
        j <- 5

      graphics::axis(1, line = j, col = i, col.ticks = i, col.axis = i)

      graphics::mtext(.axis[[2]], side = 1, line = j, adj = 1, at = 0, col = i)

      .area <- .data[which(!is.na(.data$G.sd)), , drop = FALSE]

      graphics::polygon(c(.area[, 2], rev(.area[, 2])),
                        c(.area$G.max, rev(.area$G.min)),
                        col = scales::alpha(i, 0.25), border = NA)

      graphics::lines(.data[, 2], .data$G, col = scales::alpha(i, 0.75),
                      lwd = 2)

    }

    graphics::legend("topright",
                     legend = c(.plots$.fixed.area.plot[[2]]$main,
                                .plots$.k.tree.plot[[2]]$main,
                                .plots$.angle.count.plot[[2]]$main),
                     text.col = c(1:length(.plots)), cex = 0.75,
                     bty = "n", pch = 15,
                     col = c(1:length(.plots)))

  }

  return(invisible())

}

module mixed_case_interface_sub_child
  implicit none

contains
  subroutine foo(Func)
    interface
      function Func()
      end function Func
    end interface
  end subroutine foo
end module mixed_case_interface_sub_child

relative.bias <- function(simulations,
                          variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
                          save.result = TRUE, dir.result = NULL) {



  # Checking if id columns are characters or factors, and converting to numeric in that cases
  if(!is.null(simulations$fixed.area) & is.character(simulations$fixed.area$id) | is.factor(simulations$fixed.area$id))
    simulations$fixed.area$id <- as.numeric(as.factor(simulations$fixed.area$id))

  if(!is.null(simulations$k.tree) & is.character(simulations$k.tree$id) | is.factor(simulations$k.tree$id))
    simulations$k.tree$id <- as.numeric(as.factor(simulations$k.tree$id))

  if(!is.null(simulations$angle.count) & is.character(simulations$angle.count$id) | is.factor(simulations$angle.count$id))
    simulations$angle.count$id <- as.numeric(as.factor(simulations$angle.count$id))


  # Define a character vector containing index name (radius, k or BAF) for each
  # available plot design
  .plot.design <- c(fixed.area = "radius", k.tree = "k", angle.count = "BAF")

  # Define character vectors containing the implemented field variables and TLS
  # metrics
  .field.names <- c(
    # Density (trees/ha), basal area (m2/ha) and volume (m3/ha)
    "N", "G", "V",

    # Volume (m3/ha) provided by user
    "V.user",

    # Mean diameters (cm), and mean dominant diameters (cm)
    "d", "dg", "dgeom", "dharm",
    paste(c("d", "dg", "dgeom", "dharm"), "0", sep = "."),

    # Mean heights (m), and mean dominant heights (m)
    "h", "hg", "hgeom", "hharm",
    paste(c("h", "hg", "hgeom", "hharm"), "0", sep = "."))
  .tls.names <- c(
    # Density (trees/ha)
    "N.tls", "N.hn", "N.hr", "N.hn.cov", "N.hr.cov", "N.sh",
    "N.pam",

    # Number of points
    "n.pts", "n.pts.est", "n.pts.red", "n.pts.red.est",

    # Basal area (m2/ha)
    "G.tls", "G.hn", "G.hr", "G.hn.cov", "G.hr.cov", "G.sh",
    "G.pam",

    # Volume (m3/ha)
    "V.tls", "V.hn", "V.hr", "V.hn.cov", "V.hr.cov", "V.sh",
    "V.pam",

    # Mean diameters (cm), and mean dominant diameters (cm)
    paste(c("d", "dg", "dgeom", "dharm"), "tls", sep = "."),
    paste(c("d", "dg", "dgeom", "dharm"), "0.tls", sep = "."),

    # Mean heights (m), and mean dominant heights (m)
    paste(c("h", "hg", "hgeom", "hharm"), "tls", sep = "."),
    paste(c("h", "hg", "hgeom", "hharm"), "0.tls", sep = "."),

    # Height percentiles P99 (m)
    # Remark: only for relative bias regarding heights and
    # dominant heights
    sprintf("P%02i", 99))


  # Check arguments


  # 'simulations' must be a list with all or any of the preset elements, and
  # being at least one of them no NULL
  if (!is.list(simulations)) stop("'simulations' must be a list")
  if (is.null(simulations) || all(!names(.plot.design) %in% names(simulations)))
    stop("'simulations' must have at least one of the following elements:",
         "'fixed.area', 'k.tree' or 'angle.count'")
  if (any(!names(simulations) %in% names(.plot.design))) {

    simulations <- simulations[names(simulations) %in% names(.plot.design)]
    warning("There is any element in 'simulations' which do not correspond ",
            "with preset ones. It was not taken into account during the ",
            "execution")

  }
  simulations <- simulations[!sapply(simulations, is.null)]
  if (length(simulations) == 0)
    stop("'simulations' must have at least one of the following elements ",
         "different from 'NULL': 'fixed.area', 'k.tree' or 'angle.count'")
  for (.i in names(simulations)) {

    # All elements in 'simulations' must be data frames with at least one row,
    # certain mandatory columns and most of them numeric
    if (!is.data.frame(simulations[[.i]]))
      stop("'simulations$", .i, "' must be a data.frame")
    if (nrow(simulations[[.i]]) < 1)
      stop("'simulations$", .i, "' must have at least one row")
    if (!"id" %in% colnames(simulations[[.i]]))
      stop("'simulations$", .i, "' must have a column named 'id'")
    if (!.plot.design[.i] %in% colnames(simulations[[.i]]))
      stop("'simulations$", .i, "' must have a column named '",
           .plot.design[.i], "'")
    if (all(!.field.names %in% colnames(simulations[[.i]])))
      stop("'simulations$", .i, "' must have at least one field estimations ",
           "column")
    if (all(!.tls.names %in% colnames(simulations[[.i]])))
      stop("'simulations$", .i, "' must have at least one TLS metrics column")
    for (.j in colnames(simulations[[.i]])) {

      if (.j != "id" && !is.numeric(simulations[[.i]][, .j]))
        stop("Column '", .j,"' of 'simulations$", .i, "' must be numeric")

    }

  }

  # 'variables' must be c("N", "G", "V", "d", "dg", "d.0", "h", "h.0") (by
  # default) or a character vector with all or any of the preset estimations of
  # variables based on field data. Besides, simulations elements must have at
  # least the columns corresponding to 'variables'
  if (!is.character(variables) || !is.vector(variables))
    stop("'variables' must be a character vector")
  if (length(variables) == 0 || all(!variables %in% .field.names))
    stop("'variables' must have at least one of the following character ",
         "strings: ", paste("'", .field.names, "'", sep = "", collapse = ", "))
  if (any(!variables %in% .field.names)) {

    variables <- variables[variables %in% .field.names]
    warning("There is any character string in 'variables' which do not ",
            "correspond with preset ones. It was not taken into account ",
            "during the execution")

  }
  for (.i in names(simulations)) {

    if (any(!variables %in% colnames(simulations[[.i]])))
      stop("Any column corresponding to 'variables' is missing in ",
           "'simulations$", .i, "'")

  }

  # 'save.result' must be TRUE (by default) or a logical
  if (!is.logical(save.result)) stop("'save.result' must be a logical")
  if (length(save.result) > 1) {

    save.result <- save.result[1]
    warning("Only first value in 'save.result' was taken into account during ",
            "the execution")

  }

  # If 'save.result' is TRUE, 'dir.result' must be NULL (by default) or a
  # character string containing the absolute path to a existing directory
  if (save.result) {

    if (!is.null(dir.result)) {

      if (!is.character(dir.result))
        stop("'dir.result' must be a character string")
      if (length(dir.result) > 1) {

        dir.result <- dir.result[1]
        warning("Only first value in 'dir.result' was taken into account ",
                "during the execution")

      }
      if (!dir.exists(dir.result)) stop("'dir.result' directory must exist")
    } else {

      # Define working directory as directory by default for saving files
      dir.result <- getwd()

    }

  }


  # Define values for certain auxiliary objects, and create empty lists where
  # results will be saved


  # Restrict field variables to them specified in 'variables' argument
  .field.names <- .field.names[.field.names %in% variables]

  # Define a character vector containing color palette for relative bias plots,
  # if 'save.result' is TRUE
  if (save.result) {

    .tls.color <- substr(.tls.names[substr(.tls.names, 1, 1) != "P"], 1, 1)
    names(.tls.color) <- .tls.names[substr(.tls.names, 1, 1) != "P"]
    # Density
    .tls.color[.tls.color == "N"] <-
      grDevices::hcl.colors(sum(.tls.color == "N") + 1,
                            "Reds")[-(sum(.tls.color == "N") + 1)]
    # Number of points
    .tls.color[.tls.color == "n"] <-
      grDevices::hcl.colors(sum(.tls.color == "n") + 1,
                            "Greens")[-(sum(.tls.color == "n") + 1)]
    # Basal area
    .tls.color[.tls.color == "G"] <-
      grDevices::hcl.colors(sum(.tls.color == "G") + 1,
                            "Blues")[-(sum(.tls.color == "G") + 1)]
    # Volume
    .tls.color[.tls.color == "V"] <-
      grDevices::hcl.colors(sum(.tls.color == "V") + 1,
                            "Oranges")[-(sum(.tls.color == "V") + 1)]
    # Diameters
    .tls.color[.tls.color == "d"] <-
      grDevices::hcl.colors(sum(.tls.color == "d") + 1,
                            "Purples")[-(sum(.tls.color == "d") + 1)]
    # Heights
    .tls.color[.tls.color == "h"] <-
      grDevices::hcl.colors(sum(.tls.color == "h"), "Magenta")
    # Percentiles
    .perc.names <- outer(.field.names[substr(.field.names, 1, 1) == "h"],
                         .tls.names[substr(.tls.names, 1, 1) == "P"], paste,
                         sep = ".")
    .perc.names <- as.vector(.perc.names)
    .perc.color <-  grDevices::gray.colors(length(.perc.names))
    names(.perc.color) <- .perc.names
    .tls.color <- c(.tls.color, .perc.color)

  }

  # Define an empty list where relative bias will be saved
  RB <- vector("list", length(simulations))
  names(RB) <- names(simulations)


  # Loop for each plot design in 'simulations' argument
  for (.i in names(simulations)) {

    # Define initial time, and print message
    t0 <- Sys.time()
    message("Computing relative bias for ",
            switch(.i, fixed.area = "fixed area ", k.tree = "k-tree ",
                   angle.count = "angle-count "), "plots")


    # Rearrange simulated data ----


    # Convert data.frame containing simulated data into a matrix
    simulations[[.i]] <- as.matrix(simulations[[.i]])

    # Create an empty numeric 3-dimensional array where simulated data will be
    # rearranged
    .index.val <- sort(unique(simulations[[.i]][, .plot.design[.i]]))
    .index.dec <- .decimals(.index.val)
    names(.index.val) <- .format.numb(x = .index.val, dec = .index.dec)
    .col.names <- c(.field.names,
                    .tls.names[.tls.names %in% colnames(simulations[[.i]])])
    .id.val <- sort(unique(simulations[[.i]][, "id"]))
    names(.id.val) <- as.character(.id.val)
    .data <- array(numeric(), dim = c(length(.index.val), length(.col.names),
                                      length(.id.val)),
                   dimnames = list(names(.index.val), .col.names,
                                   names(.id.val)))

    # Rearrange simulated data into the previously created array
    for (.j in names(.id.val)) {

      .row.names <- simulations[[.i]][simulations[[.i]][, "id"] == .id.val[.j],
                                      .plot.design[.i]]
      .row.names <- .format.numb(x = .row.names, dec = .index.dec)
      .data[.row.names, , .j] <-
        simulations[[.i]][simulations[[.i]][, "id"] == .id.val[.j],
                          dimnames(.data)[[2]]]

    }


    # Compute relative bias ----


    # Create a character matrix containing all the pairs of field variable and
    # TLS metric to be considered for relative bias calculations
    .RB.pairs <- .tls.names[.tls.names %in% dimnames(.data)[[2]]]
    .RB.pairs <- cbind(field = rep(.field.names, each = length(.RB.pairs)),
                       tls = rep(.RB.pairs, length(.field.names)))
    rownames(.RB.pairs) <- apply(.RB.pairs, 1, paste, collapse = ".")
    # Restrict possible pairs to comparable ones: field and corresponding TLS
    # counterpart; or field heights and TLS percentiles
    .row.names <-
      (substr(.RB.pairs[, "tls"], 1, nchar(.RB.pairs[, "field"])) ==
         .RB.pairs[, "field"]) |
      (substr(.RB.pairs[, "tls"], 1, nchar(
        gsub(".user", "", .RB.pairs[, "field"], fixed = TRUE))) ==
         gsub(".user", "", .RB.pairs[, "field"], fixed = TRUE)) |
      ((substr(.RB.pairs[, "field"], 1, 1) == "h" &
          substr(.RB.pairs[, "tls"], 1, 1) == "P"))
    .RB.pairs <- .RB.pairs[.row.names, , drop = FALSE]
    # Restrict pairs related to diameters and heights to those corresponding to
    # the same mean function: different from diameters and heights; or
    # diameters/heights corresponding to the same mean function; or field
    # heights and TLS percentiles
    .row.names <-
      (!substr(.RB.pairs[, "field"], 1, 1) %in% c("d", "h")) |
      (substr(.RB.pairs[, "field"], 1, 1) %in% c("d", "h") &
         paste(.RB.pairs[, "field"], "tls", sep = ".") == .RB.pairs[, "tls"]) |
      (substr(.RB.pairs[, "field"], 1, 1) == "h" &
         substr(.RB.pairs[, "tls"], 1, 1) == "P")
    .RB.pairs <- .RB.pairs[.row.names, , drop = FALSE]
    # Check if all field estimations have at least one TLS counterpart
    if (any(!.field.names %in% .RB.pairs[, "field"]))
      stop("'simulations$", .i, "' must have at least one TLS counterpart for ",
           "each element of 'variables' argument")


    # Compute relative bias for each radius, k or BAF


    # Compute relative bias
    .RB.data <-
      lapply(dimnames(.data)[[1]],
             function(index, data, RB.pairs){
               # Compute relative bias for each pair
               RB.data <-
                 sapply(1:nrow(RB.pairs),
                        function(nr, data, RB.pairs) {
                          # Select simulated data
                          data <- matrix(data[RB.pairs[nr, ], ],
                                         nrow = ncol(RB.pairs),
                                         ncol = dim(data)[2],
                                         dimnames = list(RB.pairs[nr,],
                                                         dimnames(data)[[2]]))
                          # Select plots with non NA values
                          data <- data[, apply(!is.na(data), 2, all),
                                       drop = FALSE]
                          # Compute relative bias only if number of plots is
                          # greater than 0
                          if (ncol(data) > 0) {
                            RB.data <-
                              ((mean(data[RB.pairs[nr, "tls"], ]) -
                                  mean(data[RB.pairs[nr, "field"], ])) /
                                 mean(data[RB.pairs[nr, "field"], ])) * 100
                          } else {
                            RB.data <- NA
                          }
                          return(RB.data)
                        },
                        data = matrix(data[index, , ], nrow = dim(data)[[2]],
                                      ncol = dim(data)[[3]],
                                      dimnames = dimnames(data)[2:3]),
                        RB.pairs = RB.pairs)
               return(RB.data)
             },
             data = .data, RB.pairs = .RB.pairs)
    .RB.data <- do.call(rbind, .RB.data)
    rownames(.RB.data) <- dimnames(.data)[[1]]
    colnames(.RB.data) <- rownames(.RB.pairs)

    # Remove results corresponding to first radius, k or BAF values if all
    # relative bias are NA values
    .any.nona <- which(apply(!is.na(.RB.data), 1, any))
    if (length(.any.nona) == 0) {

      warning("All the computed relative bias for ",
              switch(.i, fixed.area = "fixed area", k.tree = "k-tree",
                     angle.count = "angle-count"),
              " plot design case are 'NA' values")

    } else {

      .RB.data <- .RB.data[min(.any.nona):dim(.RB.data)[1], , drop = FALSE]

    }

    # Save index and relative bias values in results list, and write csv file
    # containing them if 'save.result' is TRUE
    RB[[.i]] <- matrix(c(.index.val[rownames(.RB.data)], .RB.data),
                       nrow = nrow(.RB.data), ncol = 1 + ncol(.RB.data),
                       dimnames = list(NULL, c(.plot.design[.i],
                                               colnames(.RB.data))))
    if (save.result)
      utils::write.csv(RB[[.i]],
                       file = file.path(dir.result,
                                        paste("RB", .i, "csv", sep =".")),
                       row.names = FALSE)


    # Loop for plotting relative bias related to each field variable if
    # 'save.result' is TRUE
    # Remark: all field variables related to diameters are plotted in the same
    # interactive graphic, and the same for heights


    if (save.result) {

      .j.seq <- .field.names[!substr(.field.names, 1, 1) %in% c("d", "h")]
      if (any(substr(.field.names, 1, 1) == "d")) .j.seq <- c(.j.seq, "d")
      if (any(substr(.field.names, 1, 1) == "h")) .j.seq <- c(.j.seq, "h")

      for (.j in .j.seq) {

        # Select pairs corresponding to the field variable
        if (!.j %in% c("d", "h")) {

          .RB.data.j <-
            .RB.data[, rownames(.RB.pairs)[.RB.pairs[, "field"] == .j],
                     drop = FALSE]
          colnames(.RB.data.j) <- .RB.pairs[colnames(.RB.data.j), "tls"]
          .color <- .tls.color[colnames(.RB.data.j)]

        } else {

          .col.names <-
            rownames(.RB.pairs)[substr(.RB.pairs[, "field"], 1, 1) == .j &
                                  substr(.RB.pairs[, "tls"], 1, 1) != "P"]
          .RB.data.j <- .RB.data[, .col.names, drop = FALSE]
          .color <- .tls.color[.RB.pairs[.col.names, "tls"]]
          names(.color) <- colnames(.RB.data.j)

          # Add percentiles
          .col.names <-
            rownames(.RB.pairs)[substr(.RB.pairs[, "field"], 1, 1) == .j &
                                  substr(.RB.pairs[, "tls"], 1, 1) == "P"]
          if (length(.col.names) > 0) {

            .RB.data.j <- cbind(.RB.data.j,
                                .RB.data[, .col.names, drop = FALSE])
            .color <- c(.color, .tls.color[.col.names])

          }

        }

        # Remove results corresponding to radius, k or BAF values such that all
        # relative bias are NA values
        .RB.data.j <- .RB.data.j[apply(!is.na(.RB.data.j), 1, any), ,
                                 drop = FALSE]

        # Convert relative bias matrix to data.frame, and add index values
        .RB.data.j <- data.frame(.index.val[rownames(.RB.data.j)], .RB.data.j,
                                 stringsAsFactors = FALSE)
        colnames(.RB.data.j)[1] <- .plot.design[.i]

        # Define title, subtitle, and axis names, and create empty plot
        .title <- switch(.j, N = "Density (N, trees/ha)",
                         G = "Basal area (G, m<sup>2</sup>/ha)",
                         V = "Volume (V, m<sup>3</sup>/ha)",
                         V.user = paste("Volume (V, m<sup>3</sup>/ha) provided",
                                        "by user"),
                         d = "Mean diameters (cm)",
                         h = "Mean heights (m)")
        .subtitle <- paste("<br> <span style='font-size: 20px;'>",
                           switch(.i, fixed.area = "Circular fixed area plot",
                                  k.tree = "K-tree plot",
                                  angle.count = "Angle-count plot"),
                           "</span>", sep ="")
        .xaxis <- switch(.i, fixed.area = "Radius (m)",
                         k.tree = "K-tree (trees)",
                         angle.count = "BAF (m<sup>2</sup>/ha)")
        .yaxis <- "Relative bias (%)"
        fig <-
          plotly::plot_ly(.RB.data.j, type = 'scatter', mode = 'lines') %>%
          plotly::layout(title = paste(.title, .subtitle, sep = ""), font = list(size = 25),
                         xaxis = list(title = .xaxis),
                         yaxis = list (title = .yaxis), margin = list(t = 100))

        # Add traces
        for (.k in
             colnames(.RB.data.j)[colnames(.RB.data.j) != .plot.design[.i]]) {

          fig <- fig %>%
            plotly::add_trace(x = .RB.data.j[, .plot.design[.i]],
                              y = .RB.data.j[, .k], name = .k,
                              line = list(color = .color[.k], width = 2,
                                          dash = 'dot'))

        }

        # Save relative bias plot related to the field variable
        suppressWarnings(
          htmlwidgets::saveWidget(widget = fig,
                                  file = file.path(dir.result,
                                                   paste("RB", .j, .i, "html",
                                                         sep = ".")),
                                  selfcontained = TRUE,
                                  libdir = file.path(dir.result, "RB_files")))

      }

    }

    # Define final time, and print message
    t1 <- Sys.time()
    message(" (", format(round(difftime(t1, t0, units = "secs"), 2)), ")")

  }


  return(RB)

}

module parent_mod
  implicit none
  type :: typ
    real(kind=8) :: value
  contains
    procedure :: method1 => submod_method1
  end type typ
  interface
    module subroutine submod_method1(this)
      class(typ), intent(inout) :: this
    end subroutine submod_method1
    module subroutine submod_method2(this, value)
      class(typ), intent(inout) :: this
      real, intent(in) :: value
    end subroutine submod_method2
  end interface
end module parent_mod
submodule(parent_mod) submod
contains
  module subroutine submod_method1(this)
    class(typ), intent(inout) :: this
    this%value = 0
  end subroutine submod_method1
end submodule submod

MODULE test_abstract
ABSTRACT INTERFACE
  SUBROUTINE abs_interface(a,b)
  INTEGER(4), DIMENSION(3,6), INTENT(in) :: a
  REAL(8), INTENT(out) :: b(4)
  END SUBROUTINE abs_interface
END INTERFACE
PROCEDURE(abs_interface) :: test
END MODULE test_abstract

module test_int

   implicit none

   contains

   subroutine foo(f, arg2)
      interface
         function f(x)
            real, intent(in) :: x
            real :: f
         end function
      end interface
      integer, intent(in) :: arg2
      real :: y
      y = 1.
      print*, f(y)
   end subroutine foo

   function foo2(f, g, h) result(arg3)
      interface
         function f(x) result(z)
            real, intent(in) :: x
            real :: z
         end function
         function g(x) result(z)
            real, intent(in) :: x
            real :: z
         end function
      end interface
      interface
         function h(x) result(z)
            real, intent(in) :: x
            real :: z
         end function h
      end interface
      real :: y
      real :: arg3
      y = 1.
      arg3 = f(g(h(y)))
   end function foo2

end module test_int

program test_nan
   use, intrinsic :: iso_fortran_env, only: sp=>real32, dp=>real64, qp=>real128
   use, intrinsic :: ieee_arithmetic, only: ieee_value, ieee_quiet_nan, ieee_is_nan
   implicit none

   complex(qp) :: nan_zp

   nan_zp = ieee_value(1.,ieee_quiet_nan)
   print '(A4,2X,F5.1,6X,L1,2X,Z32)','zp',real(nan_zp), ieee_is_nan(real(nan_zp)),nan_zp
end program test_nan

module rename_mod1
real(8) :: var1
end module rename_mod1
!
module rename_mod2
use rename_mod1, only: renamed_var1 => var1
integer :: originalname
end module rename_mod2
!
subroutine test_rename_sub()
use rename_mod2, only : localname => originalname, renamed_var2 => renamed_var1
implicit none
!
localname = 4
renamed_var2 = 4    
end subroutine test_rename_sub

program params
    implicit none
    integer, parameter :: var = &
                          1000, &
                          var2 = 23, var3 = &
                          var*var2, &
                          var4 = 123
    double precision, parameter :: somevar = 23.12, some = 1e-19
    logical(kind=8), parameter :: long_bool = .true.
    character(len=5), parameter :: sq_str = '12345'
    character(len=5), parameter :: dq_str = "12345"
    integer, parameter :: var_no_space=123
    integer, parameter :: var_more_space  =   123
    integer, parameter :: var_sum1  = 1 + 23
    integer, parameter :: var_ex1  = 1 - 23
    integer, parameter :: var_mul1  = 1  *   23
    integer, parameter :: var_div1  = 1/1
    INTEGER, PARAMETER :: var_multi2 = 1 *   &
                                        23 + &
                                        2 /1        ! comment
    INTEGER(4), PARAMETER :: SIG$ERR   = -1
end program params

subroutine preprocessor_else(var)

#if 0
#define MYTYPE logical
#else
#define MYTYPE integer
#endif

MYTYPE :: var0

#undef MYTYPE

#if 1
#define MYTYPE real
#else
#define MYTYPE character
#endif

MYTYPE :: var1

endsubroutine preprocessor_else

module import_mod
  implicit none
  type :: type1
    real(kind=8) :: value
  contains
    procedure :: abs_int => abs_int1
  end type type1
  type :: type2
    type(type1) :: t
  end type type2
  interface
    subroutine abs_int1(this)
      import type1
      class(type1), intent(inout) :: this ! only type1
    end subroutine abs_int1
    subroutine abs_int2(this)
      import, only: type2
      class(type2), intent(inout) :: this ! only type2
    end subroutine abs_int2
    subroutine abs_int3(this)
      import, none
      class(type1), intent(inout) :: this ! no comp results
    end subroutine abs_int3
    subroutine abs_int4(this)
      import, all
      class(type1), intent(inout) :: this ! type1 and type2
    end subroutine abs_int4
    subroutine abs_int5(this)
      import
      class(type1), intent(inout) :: this ! type1 and type2
    end subroutine abs_int5
    subroutine abs_int6(this)
      import type1
      import type2
      class(type1), intent(inout) :: this ! type1 and type2
    end subroutine abs_int6
    subroutine abs_int7(this)
      import :: type1, type2
      class(type1), intent(inout) :: this ! type1 and type2
    end subroutine abs_int7
  end interface
end module import_mod

program main
  use import_mod
  type(type1) :: obj
  call obj%abs_int()
end program main

module var_shadowing_keyword_arg
  character(len=6), parameter :: TEST = "4.10.4"
  character(len=6, kind=4), parameter :: TEST2 = "4.10.4"
  real(kind=8) :: a
end module var_shadowing_keyword_arg

program program_var_shadowing_keyword_arg
  use var_shadowing_keyword_arg
  integer :: len
  integer :: kind
end program program_var_shadowing_keyword_arg

module use_mod
integer :: val1, val2, val3
contains
end module use_mod
module use_mod_all
integer :: val4, val5
contains
end module use_mod_all

program use_main
use use_mod, only: val1, val2
use use_mod, only: val3_renamed => val3
use use_mod_all, only: val4
use use_mod_all, only: val4, val5
print*, val3_renamed
print*, val4
end program use_main

program test_arg_names_as_keywords
    implicit none
    integer, parameter :: impure = 8
contains
    subroutine foo(recursive, ierr)
        integer, intent(in) :: recursive
        integer, intent(out) :: ierr
        print*, recursive
    end subroutine foo
    real(8) impure elemental function foo2(recursive, elemental) result(pure)
        integer, intent(in) :: recursive, elemental
    end function foo2
    real( kind = impure ) pure elemental function foo3(recursive) result(pure)
        integer, intent(in) :: recursive
    end function foo3
    subroutine foo4(&
        recursive, &
        ierr)
        integer, intent(in) :: recursive
        integer, intent(out) :: ierr
        print*, recursive
    end subroutine foo4
    pure real(impure) function foo5(recursive) result(val)
        integer, intent(in) :: recursive
    end function foo5
end program test_arg_names_as_keywords

!> module doc for doxygen_doc_mod
!!
!! with info
module doxygen_doc_mod
    implicit none

    !> Doc for a_t
    type :: a_t
    end type
end module


module ford_doc_mod
    !! Doc for ford_doc_mod
    implicit none

    type :: b_t
        !! Doc for b_t
    end type

end module


program main
    use doxygen_doc_mod
    use ford_doc_mod

    type(a_t) :: a
    type(b_t) :: b
end program

      program comment_as_reference
C     Comment with variable name gets picked as a ref: variable_to_reference
      real variable_to_reference
      variable_to_reference = 1
      end program comment_as_reference

module some_mod
    implicit none

    type, abstract :: base_t
    end type

    type, abstract, extends(base_t) :: extends_t
    end type

    type, extends(extends_t) :: a_t
    end type
end module

subroutine normal_kinds()
    integer, parameter :: r15 = selected_real_kind(15)
    integer(kind=4) :: a, b(3,4)
    integer*8 aa, bb(3,4)
    integer(8) :: aaa, bbb(3,4)
    real(kind=r15) :: r
    real(kind(0.d0)) :: rr
end subroutine normal_kinds

real*8 function foo(val) result(r)
    real(8), intent(in) :: val
    r = val
end function foo

real(kind=8) function phi(val) result(r)
    real(8), intent(in) :: val
    r = val
end function phi

subroutine character_len_parsing(input)
    ! global variable_type * length variable_name1, variable_name2,...
    CHARACTER*17 A, B(3,4), V(9)
    CHARACTER*(6+3) C
    CHARACTER*10D(3,4)
    CHARACTER*(LEN(B))DD(3,4)
    ! local variable_type variable_name1 * length, variable_name2 * length,...
    CHARACTER AA*17, BB(3,4)*17, VV(9)*17
    CHARACTER CC*(6+3)
    CHARACTER AAA*(LEN(A))
    CHARACTER INPUT(*)*10
    ! explicit len and kind for characters
    CHARACTER(LEN=200) F
    CHARACTER(KIND=4, LEN=200) FF(3,4)
    CHARACTER(KIND=4, LEN=200) AAAA(3,4)*100

    ! override global length with local length
    CHARACTER*10 BBB(3,4)*(LEN(B))      ! has the length of len(b)
    CHARACTER*10CCC(3,4)*(LEN(B))       ! no-space
    CHARACTER(KIND=4) BBBB(3,4)*(LEN(B))      ! cannot have *10(kind=4) or vice versa

    INTEGER((4)) INT_KIND_IMP   ! FIXME: (()) trips up the regex
end subroutine character_len_parsing

subroutine preprocessor_if_elif_else()

! This file, as used in test_preproc, and together with the two similar files,
! tests that when there is an if-elif-elif-else, only the first branch that
! evaluates to true is used, and the others ignored. Also when multiple
! conditions evaluate to true.

#if 0
#define PART1 0
#elif 0
#define PART2 0
#elif 0
#define PART3 0
#else
#define PART4 1
#endif

#ifndef PART1
#define PART1 0
#endif
#ifndef PART2
#define PART2 0
#endif
#ifndef PART3
#define PART3 0
#endif
#ifndef PART4
#define PART4 0
#endif

integer, parameter :: res = PART1+PART2+PART3+PART4

endsubroutine preprocessor_if_elif_else

program multiline_tests
  implicit none
  integer :: result
  character(len=100) :: str

  ! Test: Simple multi-line continuation
  result = 1 + &
    2 + &
    3

  ! Test: Multi-line continuation with a preprocessor directive
  result = 10 + &
#ifdef TEST
    20 + &
#endif
    30

  ! Test: Multi-line continuation with string concatenation
  str = 'Hello' // &
  & ' ' // &
  &  'World'

  ! Test: Multi-line continuation with mixed preprocessor and arithmetic operations
  result = &
#ifdef MULT
    (10*2) + &
#else
    (10 * 3) + &
#endif
  & 10 * 4

  ! Test: Multi-line continuation with C preprocessor && sequence
  result = 100 + &
#if defined(TEST) && defined(MULT)
  &(20) + &
#endif
  &10

  ! Test: multiplee Multi-line continuation with C preprocessor and comments
  result = 1000 + & ! Comment 0
#if defined( TEST ) && defined( MULT )
  &100 + &  ! Comment 1
  &200+&    !! Comment 2
#else
    500 + & !!! Comment 3
#endif
  &600

end program multiline_tests

subroutine intent(arg1, arg2, arg3, arg4, arg5)
    implicit none
    integer(4), intent(in) :: arg1
    integer, intent(out) :: arg2
    integer(4), intent(inout) :: arg3
    integer(4), intent(in out) :: arg4
    real, optional, intent(in) :: arg5
end subroutine intent

! Tests that the parser will not break, when parsing incomplete variables
! constructs. This is particularly important for autocompletion.
program test_incomplete_dims
    implicit none
    integer   :: dim_val(1, 2
    character :: char_val*(10
    integer   :: (
end program test_incomplete_dims

subroutine preprocessor_if_nested()

! This file, as used in test_preproc, tests that when there are nested
! if-else preprocessor blocks, only the branches are used where ALL
! statements leading to the definition evaluate to true.

#if 0
#if 1
#define PART1 1
#else
#define PART2 1
#endif
#else
#if 1
#define PART3 1
#else
#define PART4 1
#endif
#endif

#ifndef PART1
#define PART1 0
#endif
#ifndef PART2
#define PART2 0
#endif
#ifndef PART3
#define PART3 0
#endif
#ifndef PART4
#define PART4 0
#endif

integer, parameter :: res = PART1+PART2+PART3+PART4

endsubroutine preprocessor_if_nested

module some_mod
  implicit none
  private
  public :: some_sub
  interface some_sub
    module procedure a_subroutine
    module procedure b_subroutine
  end interface
contains
  subroutine a_subroutine(x)
    integer, intent(in) :: x
    write(*,*) 'x = ', x
  end subroutine a_subroutine
  subroutine b_subroutine(x, y)
    integer, intent(in) :: x, y
    write(*,*) 'x = ', x
    write(*,*) 'y = ', y
  end subroutine b_subroutine
end module some_mod

program main
    use some_mod, only: some_sub
    implicit none
end program main

subroutine preprocessor_elif(var, var3, var4, var5, var6)

! This file, as used in test_preproc, checks that
! 1. the steps after the preprocessor parsing has fully finished, are only
! using content from the parts within the preprocessor if-elif-else that
! should be used. To do this, it has some regular fortran code within the
! #if and #elif.
! 2. the #endif correctly concludes the if-elif, so any new #define statements
! that come after the #endif, are picked up during the preprocessor parsing.

#if 0
integer, intent(in) :: var
#elif 1
integer, intent(inout) :: var
var = 3
#else
integer, intent(out) :: var
var = 5
#endif

#define OTHERTYPE integer

OTHERTYPE :: var2

PRINT*, var

endsubroutine preprocessor_elif

PROGRAM test_associate_block
    IMPLICIT NONE
    REAL :: A(5), B(5,5), C, III = 1
    ASSOCIATE (X => A, Y => C)
        PRINT*, X, Y, III
    END ASSOCIATE
    ASSOCIATE (X => 1)
        PRINT*, X
    END ASSOCIATE
    ASSOCIATE (ARRAY => B(:,1))
        ARRAY (3) = ARRAY (1) + ARRAY (2)
    END ASSOCIATE
END PROGRAM test_associate_block

---
title: "Plot design optimization"
author: "Anika Seppelt"
#date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Plot design optimization}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The function `metrics.variables` used for the calculation of stand-level variables and metrics (see vignette "Stand-level") requires arguments specifying the plot designs and sizes. If the optimal plot design and size for the calculation of stand-level variables is not known, the optimal plots design for the corresponding TLS data can be determined by two different approaches implemented in FORTLS. The approaches depend on whether field data for the sample plots is available or not. 

```{r include=FALSE}
dir.data <- system.file("data", package="FORTLS")
setwd(dir.data)
load("Rioja.data.RData")
load("Rioja.simulations.RData")
tree.tls <- Rioja.data$tree.tls
tree.field <- Rioja.data$tree.field
dir.data <- system.file("exdata", package="FORTLS")
library(FORTLS)
```

## Estimating optimal plot size without field data

If no field data is available, the function **`estimation.plot.size`** can be applied to determine the optimal plot design and size. This function uses the data frame containing the list of detected trees (introduced in **`tree.tls`**) and estimates stand-level density ($N$, trees/ha) and basal area ($G$, m$^2$/ha) for many simulated differently-sized plots and the three plot designs (circular fixed area, k-tree and angle-count) by increasing continuously their sizes. 

Thus, circular fixed area plots with increasing radius (increment of 0.1 m) to the maximum radius defined by **`radius.max`** in **`plot.parameters`** (by default set to 25, if radius is larger than furthest tree, the horizontal distance to this furthest tree is considered as maximum radius) will be simulated and for each plot, the variables (N and G) are estimated. Similarly, k-tree plots with tree numbers ($k$) ranging from 1 to **`k.max`** (specified in `plot.parameters`, default value set to 50 or total number of trees in the plot) and angle-count plots with increasing basal area factor (BAF, increments of 0.1 m$^2$/ha) to the maximum value specified by **`BAF.max`** in `plot.parameters` (set to 4 by default) are simulated and the respective stand-level variables are calculated. Optionally, the minimum diameter at breast height (**`dbh.min`**, in cm) to include the trees in the estimations can be defined. By default the minimum $dbh$ is set to 4 cm.

The function generates size-estimation charts i.e., plots showing the estimated stand-level density ($N$) and basal area ($G$) on the $y$ axes respective to the different plot sizes ($x$ axes). The estimations will be performed for simulated plots corresponding to all sample plots. By default the output graphs will contain one line for each sample plot. When **`average`** is set to `TRUE`, the average of all estimations (for all plots) as a continuous line and the standard deviation as grey shaded area will be drawn instead of multiple lines for each sample plot. One chart for each plot design is drawn by default. If **`all.plot.designs`** is set to `TRUE`, the line charts of all three plot design will be drawn in one graph with different colours for each plot design.

```{r fig.height=7, fig.width=7, fig.align = "center", message=FALSE, warning=FALSE,fig.alt = "Line charts output obtained with the estimation.plot.size function"}
estimation.plot.size(tree.tls = tree.tls,
                     plot.parameters = data.frame(radius.max = 25, k.max = 50, BAF.max = 4),
                     dbh.min = 4,
                     average = TRUE, all.plot.designs = FALSE)
```

The continuous line represents the average over all sample plots (i.e. 16 plots in the example shown here) of the estimated density ($N$) on the left and the basal area ($G$) on the right. The dotted line indicates the number of sample plots. This figure helps to find suitable plot designs for the calculation of stand-level metrics and variables. The optimal plot design and size should be chosen within a range where the estimated values for $N$ and $G$ reach a stable level. A too small plot leads to high errors of estimation, since only few trees enter the plot and therefore the sample is too small. In the example above, the basal area estimated for fixed area plots with radius smaller than 5 m is much higher (around 40-50 m$^2$/ha) than the true value (around 20 m$^2$/ha). On the other hand, too large plots come along with systematic errors due to occlusion of trees. Therefore, the basal area in the same example of fixed area plots with radius bigger than 20 m is estimated lower than the true value. In order to avoid both types of errors, the figure helps to find a plot size range with stable values.

## Validation with field data and optimizing plot design

When data from field measurements are available for the same sample plots, the TLS-based estimates can be validated and the optimal plot designs can be found applying functions implemented in FORTLS. In the first step of the optimization process, the function **`simulations`** simulates plots with incremental size and computes the corresponding stand-level metrics and variables (similar to the function `metrics.variables`, see "Stand-level" vignette). Based on the simulated data, two different processes can be performed. First, the bias between TLS data and field data for each individual estimated variable can be assessed with the function **`relative.bias`**. Second, correlations between all estimated variables and metrics based on TLS-data (output data of the `simulations` function) and the variables estimated from field data can be calculated with the **`correlations`** function. This function calculates both the Pearson and Spearman correlation coefficients. To visualize the correlation coefficients, heat maps can be drawn with the **`optimize.plot.design`** function.

### Plot simultaion and estimation of metrics and variables

The **`simulations`** function is applied as follows:

```{r eval=FALSE, include=TRUE}
simulations <- simulations(tree.tls = tree.tls, tree.ds = tree.ds, tree.field = tree.field,
            plot.design = c("fixed.area", "k.tree", "angle.count"),
            plot.parameters = data.frame(radius.max = 25, k.max = 50, BAF.max = 4),
            scan.approach = "single", var.metr = list(tls = NULL, field = NULL),
            dbh.min = 4, h.min = 1.3, max.dist = Inf,
            dir.data = dir.data, save.result = FALSE, dir.result = NULL)
```

#### The input data frames

Both TLS and field data from the same sample plots are required to compute the function. The TLS data introduced in **`tree.tls`** should have the same format as the data frame returned from the `tree.detection.single.scan` and `tree.detection.multi.scan` functions. Thus, each row must correspond to a detected tree and it must contain at least the following columns: `id`, `file`, `tree`, `x`, `y`, `phi.left`, `phi.right`, `horizontal.distance`, `dbh`, `num.points`, `num.points.hom`, `num.points.est`, `num.points.hom.est` and `partial.occlusion`. The data frame containing the field data must be inserted in the argument **`tree.field`**. Similar to the TLS data table, each row must correspond to a tree (specified in the columns `id` and `tree`) and the values for `horizontal.distance`, `dbh`, `total.height` and an integer value indicating whether the tree is dead (1) or alive (NA, specified in `dead`) must be included in the data frame. 

When the distance sampling method for correction of occlusion effects was applied (function `distance.sample`, see "Stand-level" vignette), a list with the results in the output data frames from the aforesaid function can be introduced in **`tree.ds`**. The list must contain at least the data frame `tree` with the detection probabilities (`P.hn`, `P.hn.cov`, `P.hr` and `P.hr.cov`) for each tree. By default `tree.ds` is set to `NULL` and as a result, the calculations of the variables based on occlusion correction will not be performed.

#### Specifying designs of simulated plots

A vector containing the names of the plot designs can specify the plot designs (`"fixed.area"`, `"k.tree"`, `"angle.count"`) that are to be considered for the simulations in **`plot.design`**. By default, this argument is set to `NULL` and all three plots designs will be considered. 

Furthermore the argument **`plot.parameters`** allows for manually specifying the design of the simulated plots. Many differently-sized plots of the plot designs specified in `plot.design` are simulated. The list introduced in `plot.parameters` can include the following elements to customize the generated plots. The elements `radius.max`, `k.tree.max` and `BAF.max` define the maximum radius (in m), the maximum number of trees and the maximum BAF (in m$^2$/ha) respectively to which the sizes of circular fixed area, k-tree and angle-count plots respectively should increase. By default the values are set to `radius.max = 25`, `k.tree.max = 50` and `BAF.max = 4`. The increment by which the sizes of circular fixed area and angle-count plots sequentially increase can also be customized by specifying the elements `radius.increment` and `BAF.increment` respectively. The default settings are `radius.increment = 0.1` (in m) and `BAF.increment = 0.1` (in m$^2$/ha). An additional element of the list can be `num.trees` defining the number of dominant trees per hectare (trees/ha). This value is needed for the calculation of dominant diamters and heights and is set to 100 trees/ha by default.

#### Further adjustable arguments

Similar to the other functions, the scan approach can be specified in **`scan.approach`** and is by default set to `"multi"`. Metrics and variables of interest can be defined as a vector in **`var.metr`**. Thus, only those metrics and variables named in the vector are calculated. If not specified, `var.metr` is set to `NULL` and all possible metrics and variables are computed. The arguments **`dbh.min`**, **`h.min`** and **`max.dist`** can optionally define the minimum $dbh$, height and maximum distance of a tree to be included in the calculations. The default values are `dbh.min = 4` (in cm), `h.min = 1.3` (in m) and `max.dist = NULL` (no maximal distance is considered). 

The argument **`dir.data`** should specify the working directory of the .txt files with the normalized reduced point clouds (output of `normalize` function). If not specified, it is set to `NULL` and the current working directory is assigned to it. The output files will be saved by default, since the argument **`save.result`** is set to `TRUE`, to the directory path indicated in **`dir.result`**. For each plot design (circular fixed area, k-tree and angle-count plots), a .csv file is created using the `write.csv` function from the [utils](https://CRAN.R-project.org/package=R.utils) package.

#### Output of the simulations function

The `simulations` function generates a list with one element for each plot design. The elements are data frames containing the simulated plot. Each row represents a simulated plot defined by their respective plot identification number `id` and their size determined by either `radius`, `k` or `BAF` depending on the plot design. The columns `N`, `G`, `V`, `V.user`, `W.user`, `d`, `dg`, `dgeom`, `dharm`, `h`, `hg`, `hgeom`, `hharm`, `d.0`, `dg.0`, `dgeom.0`, `dharm.0`, `h.0`, `hg.0`, `hgeom.0` and `hharm.0` display the stand-level variables based on the field data. The remaining columns contain the stand-level variables and metrics estimated for each plot based on the TLS data. As an example the data frame for circular fixed area plots is shown below.

```{r eval=FALSE, include=TRUE}
head(simulations$fixed.area)
```

```{r echo=FALSE}
kableExtra::scroll_box(kable_input = kableExtra::kable(head(Rioja.simulations$fixed.area), 
                                                       format = "html"), width = "100%")
```

As described above, plots with increasing sizes are estimated and the corresponding variables an metrics are calculated. The plots are ordered from the smallest to the biggest. Plots with small radius can not be simulated for all sample plots (see table above), since trees not always enter, because the plots are too small. But plots with e.g. a radius of 20 m, can be simulated for all sample plots (see end of table below).

```{r eval=FALSE, include=TRUE}
tail(simulations$fixed.area)
```

```{r echo=FALSE}
kableExtra::scroll_box(kable_input = kableExtra::kable(tail(Rioja.simulations$fixed.area), 
                                                       format = "html"), width = "100%")
```

### Calculation of relative bias

In order to estimate the goodness of the TLS-based estimation of the variables, the function **`relative.bias`** computes the relative bias between the variables estimated from field data and their respective TLS-based estimates. The relative bias is calculated for each sample plot and each simulated plot (i.e. different plot sizes and designs). Therefore, the input data for this function (introduced in **`simulations`**) must be a list of data frames containing the estimated variables (based on field and TLS data) for all the simulated plots. Thus, a similar list to the output of the `simulations` function (see above) is required, that has the same description and format. 

Optionally, the variables for which the relative bias will be computed can be specified in a vector in **`variables`**. Only, the names of the field data based estimates can be introduced. If not otherwise specified, the argument will be set to `c("N", "G", "V", "d", "dg", "d.0", "h", "h.0")` by default. Other possible variables are `dgeom`, `dharm`, `dg.0`, `dgeom.0`, `dharm.0`, `hg`, `hgeom`, `hharm`, `hg.0`, `hgeom.0` or `hharm.0`. 

The arguments **`save.result`** and **`dir.result`** define whether and to which directory the output files should be saved. Two different output files are generated. First, the data frames for each plot design (as shown below for circular fixed areas) are saved as .csv files using the `write.csv` function from the [utils](https://CRAN.R-project.org/package=R.utils) package. Second, interactive line charts representing the relative biases are saved as .html files by means of the `saveWidget` function in the [htmlwidgets](https://CRAN.R-project.org/package=htmlwidgets) package. An example of these interactive line charts is provided below.

```{r eval=TRUE}
bias <- relative.bias(simulations = Rioja.simulations,
              variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
              save.result = FALSE, dir.result = NULL)
```

The function calculates the relative bias between the field data estimates (specified in `variables`) and the counterpart variables that are estimated based on TLS data. The TLS counterparts for the density (`N`) are the variables `N.tls`, `N.hn`, `N.hr`, `N.hn.cov`, `N.hr.cov` and `N.sh` for circular fixed area and k-tree plots, and `N.tls` and `N.pam` for angle-count plots. The same pattern applies to the basal area (`G`) and the volume (`V`) where the corresponding TLS-based estimates are `G.tls`, `G.hn`, `G.hr`, `G.hn.cov`, `G.hr.cov`, `G.sh` and `G.pam`, and `V.tls`, `V.hn`, `V.hr`, `V.hn.cov`, `V.hr.cov`, `V.sh` and `V.pam` respectively. In case of mean and dominant diameters (`d`, `dg`, `dgeom`, `dharm`, `d.0`, `dg.0`, `dgeom.0`, and `dharm.0`) and heights (`h`, `hg`, `hgeom`, `hharm`, `h.0`, `hg.0`, `hgeom.0` and `hharm.0`), for all three plot designs their respective counterpart variables are `d.tls`, `dg.tls`, `dgeom.tls`, `dharm.tls`, `d.0.tls`, `dg.0.tls`, `dgeom.0.tls` and `dharm.0.tls` (for the diameter), and `h.tls`, `hg.tls`, `hgeom.tls`, `hharm.tls`, `h.0.tls`, `hg.0.tls`, `hgeom.0.tls`, `hharm.0.tls` and in addition `P99` (for the height). The relative bias are calculated as follows

$$ 
\frac{\frac{1}{n} \sum_{i=1}^{n}y_i - \frac{1}{n}\sum_{i=1}^{n}x_i}{\sum_{i=1}^{n}x_i}
$$

where $x_i$ is the value of the field estimate and $y_i$ the value of its TLS counterpart corresponding to plot $i$ of $n$ sample plots. For each plot size defined by the radius, k or BAF, the biases are calculated and stored as a data frame (shown below). Each row represents the a simulated plot of a certain size (here defined by `radius`) and the columns contain the calculate bias between the variables indicated in the column names. The two compared variables are joint with `.` as separation, e.g. `N.N.tls` means that the bias between `N` and `N.tls` was calculated. 

```{r eval=FALSE, include=TRUE}
head(bias$fixed.area)
```

```{r echo=FALSE}
kableExtra::scroll_box(kable_input = kableExtra::kable(head(bias$fixed.area), 
                                                       format = "html"), width = "100%")
```

For better visualization, line charts are created that show the relative bias of a given variable and plot design. As an example, the interactive graphic showing the relative bias of basal area estimations (`G`) for fixed are plots can be seen when following the link ([RB.G.fixed.area.html](https://www.dropbox.com/s/8a0hwh1xpqlprjc/RB.G.fixed.area.plot.html?dl=0)).

### Functions facilitating model-based or model-assisted sampling approaches 

To facilitate the application of model-based sampling, two additional functions are included in the FORTLS package. The function **`correlations`** computes the correlations between variables estimated from field data and those estimated from TLS data and calculates the respective Pearson and Spearman correlation coefficients. The results are saved as .csv files and represented as line charts and heat maps (when applying the function **`optimize.plot.design`**). 

#### Computing correlations

The **`correlations`** function computes the correlations for all the plot designs that are introduces as elements of a list in **`simulations`**. The format and description must be the same as the output list of the `simulations` function. Also similar to the function `relative.bias`, the variables for which the correlations are to be calculated can be specified in **`variables`**. By default, this argument is set to `variables = c("N", "G", "V", "d", "dg", "d.0", "h", "h.0")`. If only one of the two above-mentioned correlation measures should be calculated, it can be specified in **`method`**. This argument is set to `method = c("pearson", "spearman")` by default and both correlation coefficients are computed. 

```{r}
fixed.area.simulations <- list(fixed.area = Rioja.simulations$fixed.area[Rioja.simulations$fixed.area$radius < 7.5, ])
cor <- correlations(simulations = fixed.area.simulations,
             variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
             method = c("pearson", "spearman"), 
             save.result = FALSE, dir.result = NULL)
```

In addition to the calculation of the correlation measures, the function also performs tests of association and returns the p-values. The output of this function is a list containing the following three elements `correlations`, `correlations.pval` and `opt.correlations`. Each of them is a list including, if not otherwise specified (in `method`), two elements `pearson` and `spearman`. These two elements are lists again that include separate data frames for each plot design (circular fixed area, k-tree and angle-count plots). The data frames contain the corresponding correlation coefficients (in `correlations`), the calculated p-values (in `correlations.pval`) and the optimal correlations for a given plot size and field data estimate (in `opt.correlations`).

```{r eval=FALSE, include=TRUE}
cor$correlations$pearson$fixed.area[20:26,1:15]
```

```{r echo=FALSE}
kableExtra::scroll_box(kable_input = kableExtra::kable(
  cor$correlations$pearson$fixed.area[20:26,1:15], format = "html"), width = "100%")
```

All mentioned data frames are divided into rows each of which represent a given plot size defined by `radius` (for circular fixed area plots), `k` (for k-tree plots) or `BAF` (for angle-count plots). The columns of the data frames in `correlations` (shown above) and `correlations.pval` (shown below) contain the calculated coefficients and p-values respectively for the corresponding correlation. The column names are composed of the two variables (e.g. `N` and `N.tls`) separated by `.` (giving `N.N.tls`) that were correlated as described for the `relative.bias` function.

```{r eval=FALSE, include=TRUE}
cor$correlations.pval$pearson$fixed.area[20:26,1:15]
```

```{r echo=FALSE}
kableExtra::scroll_box(kable_input = kableExtra::kable(
  cor$correlations.pval$pearson$fixed.area[20:26,1:15], format = "html"), width = "100%")
```

The data frames of the `opt.correlations` list (example shown below) are also divided into rows that represent the different plot sizes. For a given plot size and variable (specified in the argument `variables`), the best correlating TLS-based estimate and the corresponding correlation coefficient is displayed in this table. The columns named `<variable>.metric` (with `<variable>` being here `N`, `G`, `d`, `dg`, `d.0`, `h` and `h.0`) contain the TLS-based variable or metric that yielded the best correlation with the respective field data-based variable of the column name for a certain plot radius. The columns `<variable>.cor` display the measures of the respective correlations. That means, in the example shown here, the TLS-based estimate that yielded the best correlation with the field data based variable density (`N`) for circular fixed area plots with a radius of 4.4 m is `ID.rho`. And the correlation coefficient is 0.7286.

```{r eval=FALSE, include=TRUE}
cor$opt.correlations$pearson$fixed.area[20:26,]
```

```{r echo=FALSE}
kableExtra::scroll_box(kable_input = kableExtra::kable(
  cor$opt.correlations$pearson$fixed.area[20:26,], format = "html"), width = "100%")
```

The `correlations` functions creates different files and saves them (if `save.result = TRUE`, default setting) to the directory indicated in `dir.result`. These files are, on the one hand, .csv files of the data frames in the lists `correlations` and `opt.correlations` created by means of the `write.csv` function from the [utils](https://CRAN.R-project.org/package=R.utils) package. These .csv files will be named as `correlations.<plot design>.<method>.csv` and `opt.correlations.<plot design>.plot.<method>.csv` with `<plot design>` being `fixed.area.plot`, `k.tree.plot` or `angle.count.plot` and `<method>` being `pearson` or `spearman`. On the other hand, interactive line charts representing the correlation coefficients will be created for each variable (selected in `variables`) as .html file using the `saveWidget` function in the [htmlwidgets](https://CRAN.R-project.org/package=htmlwidgets) package. As an example, the interactive line chart for the variable height (`h`) and fixed area plots (pearson measure) is shown ([correlations.h.fixed.area.pearson.html](https://www.dropbox.com/s/etriqgcmbahab7z/correlations.h.fixed.area.pearson.html?dl=0)).

#### Visualizing correlations

In order to visualize the optimal correlations, the function **`optimize.plot.design`** creates heat maps and is applied as follows:

```{r eval=FALSE}
optimize.plot.design(correlations = cor$opt.correlations,
                     variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
                     dir.result = NULL)
```

The function creates the heat maps based on the optimal correlation list (`opt.correlations` from the output of the `correlations` function) introduced in **`correlations`**. The introduced list must have the same format and description as the `opt.correlation` list. Similar to the other functions described above, the variables of interest can be selected in **`variables`** (default setting: `variables = c("N", "G", "V", "d", "dg", "d.0", "h", "h.0")`). This function generates interactive heat maps with the `saveWidget` function of the [htmlwidgets](https://CRAN.R-project.org/package=htmlwidgets) package and saves these graphics to the directory indicated in **`dir.result`** (or by default to the working directory). For each plot design and correlation measure a plot is generated and named as `opt.correlations.<plot design>.<method>.html` where `<plot design>` can be `fixed.area.plot`, `k.tree.plot` or `angle.count.plot` and `<method>` either `pearson` or `spearman` according to the plot design and correlation measure. 

As an example, the heat map of the pearson correlation coefficient for fixed area plots and pearson measure is provided and can be seen when opening the link ([opt.correlations.fixed.area.pearson.html](https://www.dropbox.com/s/e0wzq32ltjjeren/opt.correlations.fixed.area.pearson.html?dl=0)).

---
name: Bug report
about: Create a report to help us improve
title: ''
labels: bug
assignees: ''

---

**Describe the bug**
A clear and concise description of what the bug is.

**To Reproduce**
Try and reproduce the `fortls` error through the debug interface, for more see `fortls --debug_help`. Usually debug requests start like `fortls --debug_filepath your_file.f90 --debug_rootpath . ...`.

Start with posting:
1. a **Minimal Working Example** to demonstrate the bug
2. the `fortls` command to reproduce the issue, or your `fortls`
3. the output of the `fortls` command
4. Any additional JSONRPC requests like the ones produced with `--debug_log`

Alternatively, you can try and describe the steps that you followed to encounter the bug:
1. Go to '...'
2. Click on '....'
3. Scroll down to '....'
4. See error

**Expected behavior**
A clear and concise description of what you expected to happen.

**Screenshots & Animations**
If applicable, add screenshots or GIF/MP4 animations to help explain your problem.

**Setup information (please complete the following information):**
 - OS: [e.g. Linux, Mac]
 - Python Version [e.g. 3.10]
 - fortls Version [e.g. 2.3]
 - Code editor used [e.g. VS Code, Vim]
 - the Fortran extension for the code editor and its version [e.g. Modern Fortran v3.0.0] (if applicable)

**Configuration information (please complete the following information):**
- Your `.fortlsrc` or `.fortls.json` or `.fortls` configuration file OR any other JSON config being used (if any)
- Any settings specified through your extension [e.g. for VS Code settings from `settings.json`]

**Additional context**
Add any other context about the problem here.

# Contributing

👍🎉 Thank you for taking the time to contribute! 🎉👍

In this file you will find all the steps necessary to guide you through your first contribution to the project.

Please note our [Code of Conduct](https://github.com/fortran-lang/fortls/blob/master/CODE_OF_CONDUCT.md) and adhere to it in all your interactions with this project.

## 📚 Getting Started

A good place to start is the [Issues tab](https://github.com/fortran-lang/fortls/issues) on GitHub. Look for any issues with the `help wanted` tag.

### Downloading ⬇️

Firstly, fork the repository from <https://github.com/fortran-lang/fortls>.

Then clone the forked repository into your local machine.

```sh
git@github.com:<YOUR-USERNAME>/fortls.git
```

Where `<YOUR-USERNAME>` should be your GitHub username.

### Dependencies

To build this project you will need [Python](https://www.python.org/) `>= 3.7` and [pip](https://www.python.org/) `>= 21.0`.
To install all Python dependencies open a terminal go into the `fortls` cloned folder and run:

```sh
pip install -e ".[dev,docs]"
```

### Testing 🧪

To verify that your cloning of the GitHub repository worked as expected open a terminal and run:

```sh
pytest -v
```

This will run the entire unit test suite. You can also run this to verify that you haven't broken anything in the code.

👉 **Tip!** You can run individual tests by selecting the path to the Python file and the method

```sh
pytest test/test_interface.py::test_version_update_pypi
```

### Developing & Debugging 🐞️

❗️ Before you start developing, open a terminal inside `fortls` and run:

```sh
pre-commit install
```

This will ensure that all you commits meet the formatting standards of the project.

---

You can now start writing code! Your local `fortls` version will be updated with every code change you make, so you can use your normal code editor to checkout the `fortls` features that you have implemented.
It is however considerably easier to create compact unittests to check if your changes have worked.

A `fortls` test normally involves writing a Python function which sends a JSONRPC request to the server and then test checks for the correct server response.
Often times small bits of Fortran source code also have to be submited to be used by the test.
You can find varisous test examples in the `tests` directory.

👉 **Tip!** You can attach a debugger to the main `fortls` source code during unittesting which should allow you to pause, break, step into, etc. while testing, thus making it easier to find mistakes.

### Merging

To merge your changes to the main `fortls` repository push your branch on GitHub and open a [Pull Request](https://github.com/fortran-lang/fortls/pulls). Ping `@gnikit` to review your PR.

# Security Policy

## Supported Versions

`fortls` supports **ONLY** the latest Release. An autoupdate function is enabled by default to fetch the newest updates from `PyPi`.
For Anaconda environments the autoupdate functionality is disabled and it is up to the user to update to the latest version.


## Reporting a Vulnerability

The codebase is regularly scanned and patched for any potential security vulnerabilities.
If you manage to find a vulnerability in the Language Server please open an [Bug Report](https://github.com/fortran-lang/fortls/issues) with prefix: **SECURITY:**.

---
title: "Tree-level variables"
author: "Anika Seppelt & Juan Alberto Molina Valero"
#date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tree-level variables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

FORTLS is used for processing of point cloud data derived from terrestrial-based technologies such as Terrestrial Laser Scanning (TLS) or Simultaneous Localization and Mapping (SLAM). Point cloud data must be provided as .las or .laz files. The first obligatory step is the normalization of the point cloud applying the function `normalize`. The obtained normalized point clouds serve as input data for the tree detection functions `tree.detection.single.scan`, `tree.detection.multi.scan` and `tree.detection.several.plots`. The function `tree.detection.single.scan` detects trees from normalized TLS single-scan data and `tree.detection.multi.scan` from normalized TLS multi-scan (or SLAM) data. If data from more than one plot are to be analyzed automatically, the function `tree.detection.several.plots` should be used, which includes both the normalization and the tree detection functions and executes these functions on each input plot sequentially. 

```{r include=FALSE}
dir.data <- getwd()
download.file("https://www.dropbox.com/s/17yl25pbrapat52/PinusRadiata.laz?dl=1",
              destfile = file.path(dir.data, "PinusRadiata.laz"),
              mode = "wb")
download.file("https://www.dropbox.com/scl/fi/es5pfj87wj0g6y8414dpo/PiceaAbies.laz?rlkey=ayt21mbndc6i6fyiz2e7z6oap&dl=1",
              destfile = file.path(dir.data, "PiceaAbies.laz"),
              mode = "wb")
library(FORTLS)
```

## Normalization

The aim of the normalization process is to obtain the coordinates relative to the plot's center and the ground level. In this process, the functions `readLAS`, `clip_circle`, `classify_ground`, `grid_terrain` and `normalize_height` from the [lidR](https://CRAN.R-project.org/package=lidR) package are used internally (Roussel et al., 2020^[Roussel, J.R., Auty, D., Coops, N.C., Tompalski, P., Goodbody, T.R.H., Sanchez Meador, A., Bourdon, J.F., de Boissieu, F., Achim, A., 2020. lidR: an R package for analysis of Airborne Laser Scanning (ALS) data. Remote Sens. Environ. 251, 112061 https://doi.org/10.1016/j.rse.2020.112061.]). The following steps are executed:

- Classification of points as "ground" or "not ground" by the cloth simulation filter (CSF) algorithm (Zhang et al., 2016^[Zhang, W., Qi, J., Wan, P., Wang, H., Xie, D., Wang, X., Yan, G., 2016. An easy-to-use airborne LiDAR data filtering method based on Cloth simulation. Rem. Sens. 8 (6), 501. https://doi.org/10.3390/rs8060501.])
- Generation of a digital terrain model (DTM) by spatial interpolation of the points classified as "ground"
- Normalization of the point cloud by subtracting the DTM and computation of the Cartesian, cylindrical and spherical coordinates 
- Reduction of the density of the point cloud. In case of single-scan approach data, the densitiy is reduced by the point cropping (PCP) algorithm (Molina-Valero et al., 2019^[Molina Valero, J.A., Ginzo Villamayor, M.J., Novo Pérez, M.A., Álvarez-González, J.G., Pérez-Cruzado, C., 2019. Estimación del área basimétrica en masas maduras de Pinus sylvestris en base a una única medición del escáner láser terrestre (TLS). Cuad. Soc. Esp. Cienc. For. 45 (3), 97–116. https://doi.org/10.31167/csecfv0i45.19887.]) and in the case of multi-scan approach data, the point density reduction is based on random selection of points

The following figure shows the normalized point cloud which is used as example data below. The function `plot` from the [lidR](https://CRAN.R-project.org/package=lidR) package is used to generate the figure. 

```{r include=FALSE}
singleLAS <- lidR::readLAS(paste(dir.data, "PinusRadiata.laz", sep = "/"))
lidR::plot(singleLAS, color = "RGB")
```

<img src="pointcloud1.jpg" alt="drawing" width="300"/>

### The arguments of `normalize`

The `normalize` function is applied as follows:

```{r eval = TRUE, include = TRUE}
pcd.single <- normalize(las = "PinusRadiata.laz",
                        id = "PinusRadiata",
                        x.center = 0, y.center = 0,
                        max.dist = 10, 
                        scan.approach = "single",
                        threads = 2)
```

The name of the .las or .laz file containing the point cloud data is introduced in **`las`** argument and must include the .las/.laz extension. Optionally, the plot identification number (**`id`**) and the file name (**`file`**) can be defined. Both are set to `NULL` by default, which assigns `1` to the plot identification number and `1.txt` (same name as the identification number) to the reduced point cloud saved in the working directory specified in `dir.result`.

The directory of the input .las/.laz files and the output file can be specified in **`dir.data`** and **`dir.result`** respectively. If not specified, the current working directory is used. The output .txt files containing the reduced point clouds will be saved if not otherwise specified in **`save.result`** (`save.result = TRUE` by default).

If the point cloud in the input file was already normalized, the argument **`normalized`** can be set to `normalized = TRUE` (default setting `normalized = NULL`). As a result, one part of the internal normalization process is skipped. Furthermore the scanning approach applied for data collection must be specified in **`scan.approach`** with `"single"` (set by default) indicating the TLS single-scan approach and `"multi"` indicating the TLS multi-scan and SLAM point clouds approaches.

#### Defining the point cloud

The planimetric coordinates $x$ and $y$ of the center are by default `x.center = 0` and `y.center = 0`. If this does not coincide with the point cloud data, the coordinates of the plot center must be specified by **`x.center`** and **`y.center`**. 

Furthermore the size of the point cloud can be reduced by the arguments **`max.dist`**, **`min.height`** and **`max.height`**. If the maximum horizontal distance in meter to the plot center (`max.dist`) is set, points that are further away are discarded. Similarly, the minimum and maximum height in meters (`min.height`, `max.height` respectively) defines which points are discarded, that are those below the minimum height and those above the maximum height relative to the ground level. The default value for all three arguments is `NULL`. Hence, no points are discarded from the point cloud after normalization.

#### Adjusting the algorithms applied in `normalize` function

In order to generate the DTM, two different algorithms can be applied specified by **`algorithm.dtm`**. Spatial interpolation based on a k-nearest neighbor approach with inverse-distance weighting (`knnidw`) is selected by default. The second method is the Delaunay triangulation (`tin`). The resolution of the DTM (**`res.dtm`**) is set to 0.2 m by default but can be adjusted manually. 

To adjust the CSF algorithm, a list with parameters (e.g. the cloth resolution which is set to 0.5 by default) can be introduced in **`csf`**.

When the point clouds are colorized, the RGB values can be used to improve the normalization and tree detection process (**`RGB`**). The colors serve to distinguish leaf from ground and stem points by the Green Leaf Algorithm (GLA, Louhaichi et al., 2001^[Louhaichi, M., Borman, M. M., & Johnson, D. E. (2001). Spatially located platform and aerial photography for documentation of grazing impacts on wheat. Geocarto International, 16(1), 65-70. doi:10.1080/10106040108542184]). If the GLA algorithm should be applied to remove some points from the point cloud (i.e. leaf points), it must be indicated by `RGB = TRUE`.

### The output data frame

The `normalize` function generates the data frame shown below. Each row corresponds to one point of the point cloud of the input data. The columns `id`, `file` and `point` indicate the plot identification number, the file name and the point number respectively. The following columns contain the normalized Cartesian, cylindrical and spherical coordinates `x` (distance on x axis in m), `y` (distance on y axis in m), `z` (height relative to ground level in m), `rho` (horizontal distance in m), `phi` (angle in rad), `r` (radial distance in m) and `theta` (polar angle in rad). The column `slope` displays the slope of the terrain given in rad. If the GLA algorithm was used, the column `GLA` shows the results of that algorithm. Furthermore, a selection probability is assigned to each point by applying the PCP algorithm (`prob`) and the column `prob.select` shows the selected plots (indicated with 1) and discarded points (indicated with 0).

```{r eval = FALSE, include=FALSE}
head(pcd.single)
```

```{r echo=FALSE}
kableExtra::scroll_box(kable_input = kableExtra::kable(head(pcd.single), format = "html"), 
                       width = "100%")
```

The .txt file saved to the directory indicated by `dir.result` (if `save.result = TRUE`) contains a similar data frame to that shown above. However, the data frame will only include the reduced point cloud, i.e. only the selected points (`prob.select = 1`). The data frame is saved without row names as .txt file by using the `vroom_write` function of the [vroom](https://CRAN.R-project.org/package=vroom) package.

## Tree detection

The tree detection functions include algorithms to detect as many trees as possible in the point clouds. Additionally, the diameter at 1.3 m above ground level (diameter at breast height, $dbh$) is estimated and the coordinates of the tree's center are calculated for each detected tree. Depending on the TLS approach, different tree detection functions should be used.

### Data from TLS single-scan approach

When the single-scan approach was used to collect the data, the function `tree.detection.single.scan` can be applied as follows:

```{r eval = FALSE, warning=FALSE, include=TRUE}
tls.resolution = list(point.dist = 6.34, tls.dist = 10)

tree.list.single.tls <- tree.detection.single.scan(data = pcd.single,
                                                   tls.resolution = tls.resolution,
                                                   threads = 2)
```

The normalized and reduced point cloud, i.e. the output of the `normalize` function, is the input data frame for this function (**`data`**). The different arguments that can be specified are explained below.

#### Defining the range of diameters and heights of possible trees

With **`dbh.min`** and **`dbh.max`**, the range of possible tree diameters can be specified. Hence, only  cluster of points with a bigger diameter than `dbh.min` and a smaller diameter than `dbh.max` will be considered as possible trees. Additionally, **`min.height`** defines the minimum height of a possible tree or point cluster to be considered as a tree. If not manually specified, the values are set to `dbh.min = 4`, `dbh.max = 200` (values in cm) and `h.min = 1.3` (value in m).

#### Resolution of the TLS 

The resolution of the TLS scan (**`tls.resolution`**) can be defined either by the aperture angle or the distance between to consecutive points. The aperture angle is determined by the horizontal and vertical aperture angles (`horizontal.angle` and `vertical.angle`). When choosing the angle to define the TLS resolution, both elements must be part of the list required in `tls.resolution = list(horizontal.angle, vertical.angle)`. The second option to determine the resolution considers the distance of two consecutive points (`point.dist`) at a certain distance from the TLS device (`tls.dist`) also given in a list as it is shown in the example above.

#### Including further information about the plots

In **`plot.attributes`** a data frame with attributes at plot level (e.g. strata) can be inserted for additional information. This data frame must contain a column named `id` coinciding with that used in the `id` argument of the function `normalize`. If there are strata, the column specifying the strata must be named `stratum` (numeric) for other functions (e.g., `estimation.plot.size` or `metrics.variables`). If this argument is not specified, it will be set to NULL by default and the function will not add possible plot attributes.

#### Algorithm to distinguish stem points and foliage points

In order to distinguish stem points from points belonging to thin branches or foliage, the local surface variation, also known as normal change rate (NCR) is calculated for each point. This is a quantitative measure of the curvature feature (Pauly et al., 2002^[Pauly, M., Gross, M., & Kobbelt, L. P., (2002). Efficient simplification of point-sampled surfaces. In IEEE Conference on Visualization. (pp. 163-170). Boston, USA. https://doi.org/10.1109/VISUAL.2002.1183771]). For each point, the NCR index is estimated in a local neighborhood with a radius of 5 cm. This radius is considered as suitable for the stem separation in forests (Ma et al. 2015^[Ma, L., Zheng, G., Eitel, J. U., Moskal, L. M., He, W., & Huang, H. (2015). Improved salient feature-based approach for automatically separating photosynthetic and nonphotosynthetic components within terrestrial lidar point cloud data of forest canopies. IEEE Transactions Geoscience Remote Sensing, 54(2), 679-696. https://doi.org/10.1109/TGRS.2015.2459716]; Xia et al., 2015^[Xia, S., Wang, C., Pan, F., Xi, X., Zeng, H., & Liu, H. (2015). Detecting stems in dense and homogeneous forest using single-scan TLS. Forests. 6(11), 3923-3945. https://doi.org/10.3390/f6113923]). Higher NCR values indicate more curved surfaces e.g. branches and foliage. Therefore, a threshold (**`ncr.threshold`**) is established, which can be modified manually. By defalut it is set to 0.1 according to Zhang et al. (2019^[Zhang, W., Wan, P., Wang, T., Cai, S., Chen, Y., Jin, X., & Yan, G. (2019). A novel approach for the detection of standing tree stems from plot-level terrestrial laser scanning data. Remote Sens. 11(2), 211. https://doi.org/10.3390/rs11020211]), meaning that points with a higher NCR value than that threshold are discarded. 

#### Algorithms for identification of trees

In order to improve the detection of trees, the point cloud is reduced by removing parts of it with no trees. The argument **`stem.section`** serves to identify the part of the point cloud, i.e. a range of the coordinate $z$, which contains less bushes, branches or other disruptive points. Hence, a range of the coordinate $z$ and therefore a belt-like area is selected, either by defining the range manually or by an internal algorithm. This belt-like area includes predominantly the stems of the trees. Within this horizontal area, point clusters with higher density are chosen, which are supposedly the stems of the trees. Applying a circular buffer around the stems, vertical cylinders are created, which contain the stems. In the following algorithms only these vertical cylindrical parts of the point cloud are used to detect trees.

After the cylinders have been selected from the point cloud, **`breaks`** defines the height (in m) of horizontal slices on which the tree detection algorithms are applied. If not otherwise specified, slices are taken every 0.3 m starting at a height of 0.4 m until reaching the maximum height. The slices have a extension of 0.1 m (height of slice +/- 5 cm). On each slice the following algorithms are applied:

- **Removal of branches and foliage:** The NCR values are calculated for each point and only those points are kept as stem points with a NCR value lower than the predefined threshold (`ncr.threshold`)
- **Clustering of the points:** The clustering process is applied on the horizontal projection of the point's Cartesian coordinates i.e., only their $x$ and $y$ coordinates are considered. The Density-Based Spatial Clustering of Applications based on the Noise (DBSCAN) method (Ester et al., 1996^[Ester, M., Kriegel, H.P., Sander, J., Xu, X., (1996). A density-based algorithm for discovering clusters in large spatial databases with noise. In Kdd 96 (34), 226–231.]) implemented by the `dbscan` function of the [dbscan](https://CRAN.R-project.org/package=dbscan) package is used to perform the clustering. The radius of the epsilon neighborhood (`eps`) is defined as the minimum distance between two consecutive points at the furthest distance from the plot center in the respective horizontal slice
- **Removing of points belonging to remaining branches and foliage:** Two characteristics are used to distinguish stem and branch clusters. First, the density of stem sections is higher than of branch and foliage sections. And second, stem points should have a predominant vertical distribution. Hence, when the point cloud is vertical projected and dissected into a grid, cells over stems have a higher point density than cells over leafs or branches. Points in cells with a lower density than the median density are removed from the point cloud
- **Calculation of the center of the potential tree section:** The center is considered as the point in which the variance of the distances between all cluster points and the potential center point reaches its lowest value
- **Classification of clusters:** In order to decide whether the clusters belong to a tree section, different geometric features are considered. First, the calculated center of the tree section must be located behind the stem point clusters relative to the TLS position. Second, no points should be located behind the tree surface relative to the TLS position.  And third, the clusters should form an arc, which implicates that extreme points should lay further away from the TLS than points in the middle.

As explained above, these algorithms are applied on all horizontal slices (defined by `breaks`). Thus, tree sections are identified at different heights. Those sections that belong to the same tree are joined by applying the DBSCAN algorithm on the horizontal projection of the different sections. Thereafter, tree attributes can be estimated.

#### Estimation of tree attributes

The diameter of the detected trees ($dbh$) is obtained at 1.3 m as the double of radius. If the tree is not detected in the section at 1.3 m, the $dbh$ is estimated by fitting a linear taper function with radius as response variable and the section heights as explanatory variables. Thus, this function allows to estimate the radius at 1.3 m and to calculate $dbh$.

The argument **`d.mer`** defines the top stem diameter (in cm), which is used for the calculation of the commercial stem volume. If this argument is not specified, the merchantable tree volume volume (`v.mer`) is not calculated.

#### The output data frame

```{r eval = FALSE, include=FALSE}
head(tree.list.single.tls)
```

```{r echo=FALSE}
tree.list.single.tls <- read.csv(paste(dir.data, "tree.list.single.tls.csv", sep = "/"))
kableExtra::scroll_box(kable_input = kableExtra::kable(head(tree.list.single.tls), 
                                                       format = "html"), width = "100%")
```

The data frame shown above is the output of the `tree.detect.single.scan` function. Each row represents a detected tree (consecutively numbered in the column `tree`). The columns `id` and `file` display the plot identification number and the file name respectively equal to the columns in the `normalize` output table. The coordinates of the detected trees are given as Cartesian coordinates of the tree's center (`x` and `y`, in m) and azimuthal angles of the center (`phi` in rad), the left border (`phi.left` in rad), the right border (`phi.right` in rad) and the horizontal distance from the tree's center to the plot's center (`h.dist` in m).

Furthermore, the tree attributes `dbh` (diameter at breast height in cm), `h` (total height in m), `v` (tree stem volume in m$^3$) are estimated. If `d.mer` was defined as argument, the volume of the stem from the ground to the height of the diameter given in `d.mer` is estimated (merchantable volume, `v.mer` in m$^3$). 

For each tree, the number of points of the normal section slice (1.3 m +/- 0.05 m) of the original point cloud and the reduced point cloud (`n.pts` and `n.pts.red` respectively) are calculated and also estimated (`n.pts.est` and `n.pts.red.est` respectively). The column `partial.occlusion` describes whether the the detected tree is partially occluded (1) or not (0).

The data frame is saved as .csv file without row names using the `write.csv` function from the [utils](https://CRAN.R-project.org/package=R.utils) package.

### Data from TLS multi-scan approach

When multiple scans were performed in the same sampling plot (multi-scan approach) or SLAM devices were used, the function `tree.detection.multi.scan` can be applied as follows below. Additionally, the function `normalize` must be adjusted by specifying `scan.approach = "multi"`.

```{r eval = FALSE, warning=FALSE, include=TRUE}
pcd.multi <- normalize(las = "PiceaAbies.laz",
                       id = "PiceaAbies",
                       x.center = 0, y.center = 0,
                       scan.approach = "multi",
                       threads = 2)

tree.list.multi.tls <- tree.detection.multi.scan(data = pcd.multi,
                                                 d.mer = 20,
                                                 threads = 2)
```

The function `tree.detection.multi.scan` comes along with the same arguments as the function `tree.detection.single.scan`, which are described in "Data from TLS single-scan approach". However, instead of specifying the resolution, the precision of the TLS (in m) can be defined in `tls.precision`. If not defined, the default value is 0.03 m. The procedure remains the same and the output data frame contains the all the columns explained above:

```{r eval = FALSE, include=TRUE}
head(tree.list.multi.tls)
```

```{r echo=FALSE}
tree.list.multi.tls <- read.csv(paste(dir.data, "tree.list.multi.tls.csv", sep = "/"))
kableExtra::scroll_box(kable_input = kableExtra::kable(head(tree.list.multi.tls), 
                                                       format = "html"), width = "100%")
```

The following figure shows the same point cloud (`"PiceaAbies.laz"`) as above. The trees that were detected by the `tree.detection.multi.scan` function are labeled with a red belt at 1.3 m. 

```{r eval = FALSE, include = FALSE}
diameter <- readLAS(paste(dir.data, "diameters.laz", sep = "/"))
lidR::plot(singleLAS, color = "RGB", add = plot(diameter, color = "Intensity"))
```

<img src="pointcloud1diameter.jpg" alt="drawing" width="300"/>

### Automatic normalization and tree detection of several plots

If data from multiple plots are to be analysed, the function `tree.detection.several.plots` can be used. This function conducts both normalization and tree detection processes for each plot automatically. The result tables (as explained above) are stored directly and separately for each plot. Hence, if an error occurs in one plot, the results of the previously analysed plots are stored. 

In the function, the arguments for both the `normalize` and `tree.detection` functions must be specified as explained above. The function is applied as follows:

```{r eval = FALSE, include= TRUE}
id <- c("PinusSylvestris1", "PinusSylvestris2")

center.coord <- data.frame(id = id,
                           x = rep(0, length(id)),
                           y = rep(0, length(id)))

tree.tls <- tree.detection.several.plots(las.list = c("PinusSylvestris1.laz",
                                                      "PinusSylvestris2.laz"),
                                         id.list = id,
                                         center.coord = center.coord,
                                         tls.resolution = list(point.dist = 7.67, tls.dist = 10),
                                         max.dist = 7.5,
                                         threads = 2)
```

The names of the .las files have to be introduced as a character vector in **`las.list`**. Optionally, vectors with the plot identification numbers and the file names can be specified in **`id`** and **`file`**. If not specified, the plots will be named with correspondent numbers from 1 to n plots (`id`) and their respective `id` in "id.txt" as file names. The other arguments can be specified as explained in the sections above. 

The input files are analysed successively. After finishing the analysis of one plot, the reduced point cloud as .txt file and the tree list as .csv file are saved to the directory indicated in **`dir.results`**. 

\encoding{UTF-8}


\name{optimize.plot.design}


\alias{optimize.plot.design}


\title{
Optimize Plot Design Based on Optimal Correlations
}


\description{
Generation of interactive heatmaps graphically represent the optimal
correlations between variables estimated from field data, and metrics
derived from TLS data. These data must be derived from any of the three different
plot designs currently available (circular fixed area, k-tree and angle-count) and
correspond to plots with incremental values for the plot design parameter
(radius, k and BAF, respectively). In addition, correlation measures that are currently
admissible are Pearson's correlation coefficient and/or Spearman's \emph{rho}.
}


\usage{
optimize.plot.design(correlations,
                     variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
                     dir.result = NULL)
}


\arguments{

  \item{correlations}{
  List including the optimal correlations between field estimations and TLS
  metrics. The structure and format must be analogous to the
  \code{opt.correlations} element in the output returned by the
  \code{\link{correlations}} function. In particular, the list must include at least one
  of the following named elements:
    \itemize{
      \item \code{pearson}: list containing the optimal Pearson's correlations,
      and the names of the TLS metrics to which they correspond. It must include
      at least one of the following named elements:
        \itemize{
          \item \code{fixed.area.plot}: data frame containing the optimal
          Pearson's correlations, and the names of the TLS metrics to which
          they correspond, under circular fixed area plot design. Each row will
          correspond to a radius value, and the data frame will include the following columns:
            \itemize{
              \item \code{radius}: radius (m) of the simulated plots used for
              computing the estimated correlations.
              \item Columns \sQuote{\code{<x>.cor}} and
              \sQuote{\code{<x>.metric}}: the former, numeric column(s)
              containing optimal Pearson's correlations between
              \sQuote{\code{<x>}}, a field estimate, and the available TLS
              metrics when \code{\link{correlations}} function was executed;
              and the latter, character column(s) containing names of the TLS
              metrics to which they correspond.
            }
          If \code{fixed.area.plot} is included in \code{pearson} element, it
          must contain at least the \code{radius} column and a
          (\sQuote{\code{<x>.cor}}, \sQuote{\code{<x>.metric}}) pair of
          columns corresponding to the same field estimation.
          \item \code{k.tree.plot}: data frame including the optimal Pearson's
          correlations and the names of the TLS metrics to which they
          correspond, under the k-tree plot design. Each row will
          correspond to a k value, and the following columns will be included:
            \itemize{
              \item \code{k}: number of trees (trees) of the simulated plots
              used for computing the estimated correlations.
              \item Columns \sQuote{\code{<x>.cor}} and
              \sQuote{\code{<x>.metric}}: same description and format as
              indicated in \code{correlations$pearson$fixed.area.plot} element.
            }
          If \code{k.tree.plot} is included in \code{pearson} element, it must
          contain at least \code{k} column, and a (\sQuote{\code{<x>.cor}},
          \sQuote{\code{<x>.metric}}) pair of columns corresponding to the same
          field estimation.
          \item \code{angle.count.plot}: data frame including the optimal
          Pearson's correlations and the names of the TLS metrics to which
          they correspond, for the angle-count plot design. Each row will
          correspond to a BAF value, and the data frame will include the following columns:
            \itemize{
              \item \code{BAF}: BAF (\eqn{{m}^{2}/ha}{m^2/ha}) of the simulated plots used to
              compute the estimated correlations.
              \item Columns \sQuote{\code{<x>.cor}} and
              \sQuote{\code{<x>.metric}}: same description and format as
              indicated in \code{correlations$pearson$fixed.area.plot} element.
            }
          If the \code{angle.count.plot} is included in the \code{pearson} element, it
          must include at least the \code{BAF} column and a
          (\sQuote{\code{<x>.cor}}, \sQuote{\code{<x>.metric}}) pair of columns
          corresponding to the same field estimation.
        }

      \item \code{spearman}: list containing the optimal Spearman's
      correlations, and the names of the TLS metrics to which they correspond.
      The structure and format will be analogous to that indicated for the
      previous element but with optimal Pearson's correlations replaced by
      Spearman's correlations.
    }
  }

  \item{variables}{
  Optional character vector naming field estimations whose optimal correlations
  will be represented graphically in the heatmaps generated during the
  execution. If this argument is specified by the user, it must include at
  least one of the following character strings: \dQuote{\code{N}},
  \dQuote{\code{G}}, \dQuote{\code{V}}, \dQuote{\code{d}}, \dQuote{\code{dg}},
  \dQuote{\code{dgeom}}, \dQuote{\code{dharm}}, \dQuote{\code{d.0}},
  \dQuote{\code{dg.0}}, \dQuote{\code{dgeom.0}}, \dQuote{\code{dharm.0}},
  \dQuote{\code{h}}, \dQuote{\code{hg}}, \dQuote{\code{hgeom}},
  \dQuote{\code{hharm}}, \dQuote{\code{h.0}}, \dQuote{\code{hg.0}},
  \dQuote{\code{hgeom.0}}, or \dQuote{\code{hharm.0}}. If this argument is not
  specified by the user, it will be set to
  \code{c("N", "G", "V", "d", "dg", "d.0", "h", "h.0")} by default. In both
  cases, all data frames in the \code{correlations}
  argument must have at least the (\sQuote{\code{<x>.cor}},
  \sQuote{\code{<x>.metric}}) pairs corresponding to the field estimations
  specified in the \code{variables} argument.
  }

  \item{dir.result}{
  Optional character string naming the absolute path of an existing directory
  where files described in \sQuote{Output Files} section will be saved.
  \code{.Platform$file.sep} must be used as the path separator in
  \code{dir.result}. If this argument is not specified by the user, it will be set
  to \code{NULL} by default and, as consequence, the current working directory
  of the \R process will be assigned to \code{dir.result} during the execution.
  }
}


\details{
This function represents graphically, by means of interactive heatmaps, the strongest correlations (positive or negative) for each plot design and size simulated, between the
estimated variables based on field data specified in the \code{variables}
argument, and metrics derived from TLS data, under circular fixed area, k-tree and/or
angle-count plot designs.

Two correlation measures are implemented at present: Pearson’s correlation
coefficient and Spearman’s rho. Hence, only optimal correlations based on \code{correlations} arguments will be taken into account during the
execution.

For each correlation measure and plot design, at least one no missing value for optimal correlations must be represented;
otherwise, execution will be stopped, and an error message will appear. In
addition, at least two different no missing values for
optimal correlations are required to ensure that the colour palette is
correctly applied when the heatmap is generated.
}


\value{
Invisible \code{NULL}.
}


\section{Output Files}{
During the execution, interactive heatmaps graphically representing optimal
correlations values between field estimations and TLS metrics are created and
saved in \code{dir.result} directory by means of the \link[htmlwidgets]{saveWidget}
function in the \CRANpkg{htmlwidgets} package. The widgets generated allow users
to consult optimal correlations values and TLS metrics to which they correspond
directly on the plots, to zoom and scroll, and so on. The pattern used for
naming these files is \file{opt.correlations.<plot design>.<method>.html},
where \file{<plot design>} equals \dQuote{\code{fixed.area.plot}},
\dQuote{\code{k.tree.plot}} or \dQuote{\code{angle.count.plot}} according to
plot design, and \file{<method>} equals \dQuote{\code{pearson}} or
\dQuote{\code{spearman}} according to correlation measure.
}


\note{
This function is key to choosing the best possible plot design (in terms of
correlation measures) considering all variables of interest before establishing
definitive sampling design.
}


\author{
Juan Alberto Molina-Valero and
Adela Martínez-Calvo.}


\seealso{
\code{\link{correlations}}.
}


\examples{
\donttest{

# Load field estimations and TLS metrics corresponding to Rioja data set

data("Rioja.simulations")


# Compute correlations between field estimations and TLS metrics corresponding
# to Rioja example, and select optimal correlations results

corr <- correlations(simulations = Rioja.simulations,
                     variables = c("N", "G", "d", "dg", "dgeom","dharm",
                                   "d.0", "dg.0", "dgeom.0", "dharm.0", "h",
                                   "hg", "hgeom", "hharm", "h.0", "hg.0",
                                   "hgeom.0", "hharm.0"),
                     save.result = FALSE)

opt.corr <- corr$opt.correlations


# Establish directory where optimal correlations heatmaps corresponding to Rioja
# example will be saved. For instance, current working directory

dir.result <- tempdir()


# Generate heatmaps for optimal correlations between field estimations and TLS
# metrics corresponding to Rioja example
# Optimal Pearson's and Spearman's correlations for variables by default

# optimize.plot.design(correlations = opt.corr, dir.result = dir.result)


# Optimal Pearson's and Spearman's correlations for variables 'N' and 'G'

optimize.plot.design(correlations = opt.corr, variables = c("N", "G"),
                     dir.result = dir.result)


  }
}

\encoding{UTF-8}

\name{geometric_features_py}

\alias{geometric_features_py}

\title{This function obtains geometric features at point level}

\description{This function obtains geometric features at point level}


\usage{
geometric_features_py(data,
                      dist = 0.1,
                      geom_features_input = NULL,
                      threads = 1)}

\arguments{

  \item{data}{data}

  \item{dist}{dist}

  \item{geom_features_input}{geom_features_input}

  \item{threads}{threads}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}

\name{tree.detection.several.plots}

\alias{tree.detection.several.plots}

\title{Tree-Level Variables Estimation for Several Plots}

\description{This function integrates both, the \code{\link{normalize}} and \code{\link{tree.detection.single.scan}} or \code{\link{tree.detection.multi.scan}} functions, generating the same \sQuote{Output Files} as indicated for these, and it returs the same \sQuote{Value} as described for \code{\link{tree.detection.single.scan}} or \code{\link{tree.detection.multi.scan}} respectively. However, this function is designed for working with several plots, producing a list of all scans considered automatically from LAS files.}

\usage{
tree.detection.several.plots(las.list, id.list = NULL, file = NULL,

                             scan.approach = "multi",
                             pcd.red = NULL, normalized = NULL,
                             center.coord = NULL,
                             x.side = NULL, y.side = NULL,
                             max.dist = NULL, min.height = NULL, max.height = 50,
                             algorithm.dtm = "knnidw", res.dtm = 0.2,
                             csf = list(cloth_resolution = 0.5),
                             intensity = NULL, RGB = NULL, voxel_size = NULL,

                             single.tree = NULL,
                             dbh.min = 4, dbh.max = 200, h.min = 1.3,
                             geo.dist = 0.1,
                             tls.resolution = NULL, tls.precision = NULL,
                             stem.section = c(0.7, 3.5), stem.range = NULL, breaks = NULL,
                             slice = 0.1, understory = NULL, bark.roughness = 1,
                             den.type = 1, d.mer = NULL,

                             plot.attributes = NULL, plot = NULL,

                             threads = 1,

                             dir.data = NULL, save.result = TRUE, dir.result = NULL,
                             save.las = NULL)}

\arguments{

  \item{las.list}{Character vector containing the names of all LAS files for analysis and belonging to TLS point cloud, including .las extension (see \sQuote{Examples})}

  \item{id.list}{Optional vector with plots identification encoded as character string or numeric. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, the plots will be encoded with correlative numbers from 1 to n plots.}

  \item{file}{Optional vector containing files name identification encoded as character string or numeric value. If it is null, file will be encoded as \code{id} by default.}

  \item{scan.approach}{Character parameter indicating TLS single-scan (\sQuote{single}) or TLS multi-scan approach or SLAM point clouds (\sQuote{multi}) approaches. If this argument is not specified by the user, it will be set to \sQuote{multi} approach.}

  \item{pcd.red}{Optional argument to indicate if point cloud density must be reduced to detect trees.}

  \item{normalized}{Optional argument to establish as \code{TRUE} when point cloud is already normalized.}

  \item{center.coord}{Planimetric x and y center coordinate of the plots. They has to be introduced as a data frame object with the following columns names: 'id',  'x' and 'y'. They represent plot id, and center coordinates respectively.}

  \item{x.side}{x-side (m) of the plot when the plot is square or rectangular.}

  \item{y.side}{y-side (m) of the plot when the plot is square or rectangular.}

  \item{max.dist}{Optional maximum horizontal distance (m) considered from the plot centre. All points farther than \code{max.dist} will be discarded after the normalization process. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, all points will be used in processing, with \code{max.dist} representing the farthest point.}

  \item{min.height}{Optional minimum height (m) considered from ground level. All points below \code{min.height} will be discarded after the normalization process. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, all points will be used in processing, with \code{min.height} representing the lowest point.}

  \item{max.height}{Optional maximum height (m) considered from ground level. All points above \code{max.height} will be discarded after the normalization process. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, all points will be used in processing, with \code{max.height} representing the highest point.}

    \item{algorithm.dtm}{Algorithm used to generate the digital terrain model (DTM) from the TLS point cloud. There are two posible options based on spatial interpolation: \sQuote{tin} and \sQuote{knnidw} (see \sQuote{Details}). If this argument is not specified by the user, it will be set to \sQuote{knnidw} algorithm.}

    \item{res.dtm}{Numeric parameter. Resolution of the DTM generated to normalize point cloud (see \sQuote{Details}). If this argument is not specified by the user, it will be set to 0.2 m.}

    \item{csf}{List containing parameters of CSF algorithm:}

\itemize{

  \item {\code{cloth_resolution}}: by default 0.5.}

  \item{intensity}{Logical parameter useful when point clouds have intensity values. It may be useful in some internal process to filter data.}

  \item{RGB}{Logical parameter useful when point clouds are colorized, thus including values of RGB colors. It is based on the Green Leaf Algorithm (GLA) (see \sQuote{Details}).}

  \item{voxel_size}{Defines the size of the 3D grid cells used for downsampling.}

  \item{single.tree}{Optional argument to indicate if there is only one tree.}

  \item{dbh.min}{Optional minimum \emph{dbh} (cm) considered for detecting trees. By default it will be set at 4 cm.}

  \item{dbh.max}{Optional maximum \emph{dbh} (cm) considered for detecting trees. By default it will be set at 200 cm.}

  \item{h.min}{Optional minimum \emph{h} (m) considered for detecting trees. By default it will be set at 1.3 m.}

  \item{geo.dist}{Local surface variation (also known as normal change rate, NCR). By default it will be set as 0.1. For better understanding of this argument see \sQuote{Details}.}

  \item{tls.resolution}{List containing parameters of TLS resolution. This can be defined by the angle aperture:}

\itemize{

  \item {\code{horizontal.angle}}: horizontal angle aperture (degrees).

  \item {\code{vertical.angle}}: vertical angle aperture (degrees).

  \item {\code{point.dist}}: distance (mm) between two consecutive points.

  \item {\code{tls.dist}}: distance (m) from TLS at which two consecutive points are separated by \code{point.dist}.

  If this argument is not specified by the user, it will be set to NULL by default and, as a consequence the function will stop giving an error message.}

  \item{tls.precision}{Optional argument indicating the average point cloud precision in cm.}

  \item{stem.section}{Section free of noise (shurb, branches, etc.) considered to detect trees. If not specified, an automatic internal algorithm will be applied (see \sQuote{Details}).}

  \item{breaks}{Height above ground level (m) of slices considered for detecting trees. By default it will be considered all possible sections from 0.1 m to maximum height by 0.3 m intervals (+/- 5 cm).}

  \item{stem.range}{Section considered to estimate straightness tree attributes.}

  \item{slice}{Slice width considered for detecting trees. By default it will be considered as 0.1 m.}

  \item{understory}{Optional argument to indicate if there is dense understory vegetation.}

  \item{bark.roughness}{Bark roughness established in 3 degrees (1 < 2 < 3). By default it will be considered as 2.}

  \item{den.type}{Numeric argument indicating the dendrometic type used to estimate volumen when there are not sections enough to fit a taper equation. Dendrometrics types available are the following: cylinder = 0, paraboloid = 1 (by default), cone = 2 and neiloid = 3.}

  \item{d.mer}{Top stem diameter (cm) considered to estimate commercial timber volume.}

  \item{plot.attributes}{Data frame with attributes at plot level. It must contain a column named \code{id} (character string or numeric value) with encoding coinciding with that used in \code{id} argument of \code{\link{normalize}} for identifying plots. If there are strata, another column named \sQuote{stratum} (numeric) will be required for other functionalities of \pkg{FORTLS} (see, for instance, \code{\link{estimation.plot.size}} or \code{\link{metrics.variables}}). If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, the function will not add these possible plot attributes.}

  \item{plot}{Optional logical which indicates whether or not the normalized point cloud will be plot. If this argument is not specified by the user, it will be set to \code{TRUE} by default and, as consequence, the normalized point cloud will be plot.}

  \item{threads}{Number of threads.}

  \item{dir.data}{Optional character string naming the absolute path of the directory where LAS files containing TLS point clouds are located. \code{.Platform$file.sep} must be used as the path separator in \code{dir.dat}. If this argument is not specified by the user, it will be set to \code{NULL} by default and, as consequence, the current working directory of the \R process will be assigned to \code{dir.dat} during the execution.}

  \item{save.result}{Optional logical which indicates whether or not the output files described in \sQuote{Output Files} section should be saved in the \code{dir.result}. If this argument is not specified by the user, it will be set to \code{TRUE} by default and, as a consequence, the output files will be saved.}

  \item{dir.result}{Optional character string naming the absolute path of an existing directory where files described in \sQuote{Output Files} section will be saved. \code{.Platform$file.sep} must be used as the path separator in \code{dir.result}. If this argument is not specified by the user, and \code{save.result} is \code{TRUE}, it will be set to \code{NULL} by default and, as a consequence, the current working directory of the \R process will be assigned to \code{dir.result} during the execution.}

  \item{save.las}{Optional logical which indicates whether or not the imput point cloud must be saved in \code{dir.result} as LAZ file.}}


\details{

See \code{\link{normalize}}, \code{\link{tree.detection.single.scan}} and \code{\link{tree.detection.multi.scan}} for further details.}

\value{Data frame with the same description and format as \code{\link{tree.detection.single.scan}} and \code{\link{tree.detection.multi.scan}} \sQuote{Values}. In this case, the \code{id} of plots will be encoded with correlative numbers from 1 to n, where n is the number of LAS files included in \code{files} argument, and \code{file} column will be encoded as \code{id}, but including .las extension.}


\section{Output Files}{

At the end of the tree detection process, if the \code{save.result} argument is \code{TRUE}, the function will save both, the reduced point clouds as TXT files encoded according to \code{file} column of \sQuote{Value}; and the data frame with the tree list described in \sQuote{Value} as CSV file (see \code{\link{normalize}} and \code{\link{tree.detection.single.scan}} or \code{\link{tree.detection.multi.scan}} \sQuote{Output files}). All outputs are written without row names in the \code{dir.result} directory using \link[vroom]{vroom_write} function from \CRANpkg{vroom} package.}


\note{

This function has been developed for working with several plots, which will be the most common situation in forest inventory approaches. Nevertheless, several LAS files are not provided as examples due to problems with memory capacity.}


\author{
Juan Alberto Molina-Valero
and Adela Martínez-Calvo.}

\seealso{
\code{\link{normalize}}, \code{\link{tree.detection.single.scan}},\code{\link{tree.detection.multi.scan}}, \code{\link{distance.sampling}}, \code{\link{estimation.plot.size}}, \code{\link{simulations}}, \code{\link{metrics.variables}}.}

\examples{

\donttest{

# Establishment of working directories (optional)
# Establishment of working directories using tempdir() to avoid leaving files in
# user filespace

dir.data <- tempdir()
dir.result <- tempdir()

# Loading example data (LAZ files) to dir.data

download.file(
"www.dropbox.com/scl/fi/hzzrt0a39crdy6uvcj9el/PinusSylve1.laz?rlkey=svpwvorkm8889fgbnj14ns1f2&dl=1",
              destfile = file.path(dir.data, "PinusSylvestris1.laz"),
              mode = "wb")

download.file(
"www.dropbox.com/scl/fi/zeszze31jh5m1g4o3ns1o/PinusSylve2.laz?rlkey=wx72bi6ggdc7wedwgzupekp9k&dl=1",
              destfile = file.path(dir.data, "PinusSylvestris2.laz"),
              mode = "wb")


# Tree detection (TLS single-scan aproach)

id <- c("PinusSylvestris1", "PinusSylvestris2")

center.coord <- data.frame(id = id,
                           x = rep(0, length(id)),
                           y = rep(0, length(id)))

tree.tls <- tree.detection.several.plots(las.list = c("PinusSylvestris1.laz",
                                                      "PinusSylvestris2.laz"),

                                         id.list = id,

                                         scan.approach = "single",

                                         center.coord = center.coord,

                                         tls.resolution = list(point.dist = 7.67, tls.dist = 10),

                                         max.dist = 7.5,

                                         breaks = 1.3,

                                         threads = 2,

                                         dir.data = dir.data, dir.result = dir.result)

  }
}

\encoding{UTF-8}

\name{fixed_area_cpp}

\alias{fixed_area_cpp}

\title{Calculate dominant diameters and heights for simulations for angle-count plots.}

\description{Calculate dominant diameters and heights for simulations for angle-count plots.}

\usage{
  fixed_area_cpp(radius_seq, hdist, d, h, num)}

\arguments{

  \item{radius_seq}{BAF sequence for simulations}

  \item{hdist}{BAF masimum for simulations}

  \item{d}{vector with diameters opf trees}

  \item{h}{vector with heights of trees}

  \item{num}{number of trees considered for calculating diameter and dominant height}}

\value{Data frame.}


\author{
  Juan Alberto Molina-Valero,
  María José Ginzo Villamayor,
  Manuel Antonio Novo Pérez,
  Adela Martínez-Calvo,
  Juan Gabriel Álvarez-González,
  Fernando Montes and
  César Pérez-Cruzado.}

\keyword{internal}

\encoding{UTF-8}

\name{is_one_row_all_na}

\alias{is_one_row_all_na}

\title{This function was updated to return also the input data with the computed voxels}

\description{This function was updated to return also the input data with the computed voxels}


\usage{
is_one_row_all_na(df)}

\arguments{

  \item{df}{df}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}

\name{sample_indices}

\alias{sample_indices}

\title{Sample_indices}

\description{Sample_indices}


\usage{
sample_indices(total_number_obs, return_obs)}

\arguments{

  \item{total_number_obs}{total_number_obs}

  \item{return_obs}{return_obs}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}

\name{iterations_RANSAC}

\alias{iterations_RANSAC}

\title{Function that performs the "RANSAC_cpp" N-times}

\description{Function that performs the "RANSAC_cpp" N-times}


\usage{
iterations_RANSAC(data, n_iterations = 600)}

\arguments{

  \item{data}{data}

  \item{n_iterations}{n_iterations}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}


\name{relative.bias}


\alias{relative.bias}


\title{
Relative Bias Between Field Estimations and TLS metrics
}


\description{
Computes relative bias between variables estimated from field data and
their TLS counterparts derived from TLS data. Field estimates and TLS metrics
for a common set of plots are required in order to compute relative bias. These
data must come from any of the three different plot designs currently available
(circular fixed area, k-tree and angle-count) and correspond to plots with incremental
values for the plot design parameter (radius, k and BAF, respectively). In
addition to computing relative bias, interactive line charts graphically
representing the values obtained between each field estimate and its related
TLS metrics are also generated.
}


\usage{
relative.bias(simulations,
              variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
              save.result = TRUE, dir.result = NULL)
}


\arguments{

  \item{simulations}{
  List containing variables estimated from field data and metrics
  derived from TLS data. The structure and format must be analogous to output
  returned by the \code{\link{simulations}} function. In particular, teh list must include
  at least one of the following named elements:
    \itemize{
      \item \code{fixed.area.plot}: same description and format as indicated
      for same named element in \code{simulations} argument of
      \code{\link{correlations}} function. The only difference is the
      columns required when it is included in the argument: in adittion to \code{id},
      \code{radius} and at least one of the field estimate columns, it must
      include at least one TLS counterpart for each field estimate condidered.

      \item \code{k.tree.plot}: same description and format as indicated for
      same named element in \code{simulations} argument of
      \code{\link{correlations}} function. The only difference is the columns required when it is included in the argument: in adittion to \code{id}, \code{k}
      and at least one of the field estimate columns, it must include at
      least one TLS counterpart for each field estimate considered.

      \item \code{angle.count.plot}: same description and format as indicated
      for the same named element in the \code{simulations} argument of
      \code{\link{correlations}} function. The only difference is the columns that are required when this element is included in the argument: in adittion to \code{id},
      \code{BAF} and at least one of the field estimate columns, it must
      contain at least one TLS counterpart for each field estimate considered.
    }
  }

  \item{variables}{
  Optional character vector naming field estimates for which the relative bias
  between them and all their available TLS counterparts will be computed. If
  this argument is specified by the user, it must contain at least one of the
  following character strings: \dQuote{\code{N}}, \dQuote{\code{G}},
  \dQuote{\code{V}}, \dQuote{\code{d}}, \dQuote{\code{dg}},
  \dQuote{\code{dgeom}}, \dQuote{\code{dharm}}, \dQuote{\code{d.0}},
  \dQuote{\code{dg.0}}, \dQuote{\code{dgeom.0}}, \dQuote{\code{dharm.0}},
  \dQuote{\code{h}}, \dQuote{\code{hg}}, \dQuote{\code{hgeom}},
  \dQuote{\code{hharm}}, \dQuote{\code{h.0}}, \dQuote{\code{hg.0}}, \cr
  \dQuote{\code{hgeom.0}}, or \dQuote{\code{hharm.0}}. If this argument is not
  specified by the user, it will be set to
  \code{c("N", "G", "V", "d", "dg", "d.0", "h", "h.0")} by default. In both
  cases, all the elements in \code{simulations} argument
  must include at least the columns corresponding to the field estimations specified in the
  \code{variables} argument.
  }

  \item{save.result}{
  Optional logical which indicates whether or not the output files described in
  \sQuote{Output Files} section must be saved in \code{dir.result}. If
  this argument is not specified by the user, it will be set to \code{TRUE} by
  default and, as a consequence, the output files will be saved.
  }

  \item{dir.result}{
  Optional character string naming the absolute path of an existing directory
  where files described in \sQuote{Output Files} section will be saved.
  \code{.Platform$file.sep} must be used as the path separator in
  \code{dir.result}. If this argument is not specified by the user, and
  \code{save.result} is \code{TRUE}, it will be set to \code{NULL} by default and,
  as a consequence, the current working directory of the \R process will be
  assigned to \code{dir.result} during the execution.
  }
}


\details{
For each radius, k or BAF value (according to the currently available plot
designs: circular fixed area, k-tree and angle-count), this function computes the relative
bias between each variable estimated from field data, and specified in the
\code{variables} argument, and their counterparts derived from TLS data, and
existing in the data frames included in the \code{simulations} argument. TLS
metrics considered \emph{counterparts} for each field estimate are
detailed below (see \code{\link{simulations}} \sQuote{Value} function
for details about used notation):
  \itemize{
    \item TLS counterparts for \code{N} are \code{N.tls}, \code{N.hn},
    \code{N.hr}, \code{N.hn.cov}, \code{N.hr.cov} and \code{N.sh} in the fixed
    area and k-tree plot designs; and \code{N.tls} and \code{N.pam} in the
    angle-count plot design.
    \item TLS counterparts for \code{G} are \code{G.tls}, \code{G.hn},
    \code{G.hr}, \code{G.hn.cov}, \code{G.hr.cov} and \code{G.sh} in the fixed
    area and k-tree plot designs; and \code{G.tls} and \code{G.pam} in the
    angle-count plot design.
    \item TLS counterparts for \code{V} are \code{V.tls}, \code{V.hn},
    \code{V.hr}, \code{V.hn.cov}, \code{V.hr.cov} and \code{V.sh} in the fixed
    area and k-tree plot designs; and \code{V.tls} and \code{V.pam} in the
    angle-count plot design.
    \item TLS counterparts for \code{d}, \code{dg}, \code{dgeom}, \code{dharm},
    \code{d.0}, \code{dg.0}, \code{dgeom.0}, and \code{dharm.0} are,
    respectively: \code{d.tls}, \code{dg.tls}, \code{dgeom.tls},
    \code{dharm.tls}, \code{d.0.tls}, \code{dg.0.tls}, \code{dgeom.0.tls}, \cr
    and \code{dharm.0.tls} in any of the three available plot designs.
    \item TLS counterparts for \code{h}, \code{hg}, \code{hgeom}, \code{hharm},
    \code{h.0}, \code{hg.0}, \code{hgeom.0}, and \code{hharm.0} are,
    respectively \code{h.tls}, \code{hg.tls}, \code{hgeom.tls},
    \code{hharm.tls}, \code{h.0.tls}, \code{hg.0.tls}, \code{hgeom.0.tls}, and
    \cr \code{hharm.0.tls} in any of the three available plot designs. In adittion, \code{P99} is also taken into account as a counterpart for all
    these field estimates.
  }

The relative bias between a field estimation and any of its TLS counterparts is
estimated as follows
\deqn{\frac{\frac{1}{n}\sum_{i = 1}^{n}{y_{i}} -
\frac{1}{n}\sum_{i = 1}^{n}{x_{i}}}{\frac{1}{n}\sum_{i = 1}^{n}{x_{i}}} * 100}{
((\sum(y_i)/n - \sum(x_i)/n) / (\sum(x_i)/n) * 100,}
where \eqn{x_{i}}{x_i} and \eqn{y_{i}}{y_i}  are the values of the field
estimate and its TLS counterpart, respectively, corresponding to plot
\eqn{i}{i} for \eqn{i = 1, \ldots, n}.
}


\value{

  \item{fixed.area.plot}{
  If no \dQuote{\code{fixed.area.plot}} element exists in \code{simulations}
  argument, missing; otherwise, the matrix will include estimates of the the relative bias
  under circular fixed area plot design between each field estimation specified in
  \code{variables} argument and its TLS counterpart(s) existing in the
  \dQuote{\code{fixed.area.plot}} element in the \code{simulations} argument. Each
  row will correspond to a radius value, and the following columns will be included:
    \itemize{
      \item \code{radius}: radius (m) of the simulated plots used for computing
      the estimated relative bias.
      \item Column(s) \sQuote{\code{<x>.<y>}}: numeric column(s) containing
      estimated relative bias between \sQuote{\code{<x>}}, a field estimation,
      and \sQuote{\code{<y>}}, a TLS counterpart.
    }
  }

  \item{k.tree.plot}{
  If no \dQuote{\code{k.tree.plot}} element exists in the \code{simulations}
  argument, missing; otherwise, the matrix will include the relative bias estimated
  in the k-tree plot design between each field estimation specified in
  \code{variables} argument and its TLS counterpart(s) existing in
  \dQuote{\code{k.tree.plot}} element in \code{simulations} argument. Each row will correspond to a k value, and the following columns will be included:
    \itemize{
      \item \code{k}: number of trees (trees) of the simulated plots used for
      computing the estimated relative bias.
      \item Column(s) \sQuote{\code{<x>.<y>}}: numeric column(s) containing
      estimated relative bias between \sQuote{\code{<x>}}, a field estimation,
      and \sQuote{\code{<y>}}, a TLS counterpart.
    }
  }

  \item{angle.count.plot}{
  If no \dQuote{\code{angle.count}} element exists in \code{simulations}
  argument, missing; otherwise, the matrix will contain estimated relative bias under
  angle-count plot design between each field estimation specified in the
  \code{variables} argument and its TLS counterpart(s) existing in the
  \dQuote{\code{angle.count.plot}} element in the \code{simulations} argument. Each
  row will correspond to a BAF value, and the following
  columns will be included:
    \itemize{
      \item \code{BAF}: BAF (\eqn{{m}^{2}/ha}{m^2/ha}) of the simulated plots used for computing
      the estimated relative bias.
      \item Column(s) \sQuote{\code{<x>.<y>}}: numeric column(s) containing
      estimated relative bias between \sQuote{\code{<x>}}, a field estimation,
      and \sQuote{\code{<y>}}, a TLS counterpart.
    }
  }
}


\section{Output Files}{
During the execution, if the \code{save.result} argument is \code{TRUE}, the
function will print the matrices described in the \sQuote{Value} section to files. These
are written without row names in \code{dir.result} directory using
\code{\link{write.csv}} function from the \pkg{utils} package. The pattern used
for naming these files is \file{RB.<plot design>.csv}, where
\file{<plot design>} is equal to \dQuote{\code{fixed.area.plot}},
\dQuote{\code{k.tree.plot}} or \dQuote{\code{angle.count.plot}} is according to the
plot design.

Furthermore, if the \code{save.result} argument is \code{TRUE}, interactive line
charts graphically representing relative bias values will also be created and saved
in the \code{dir.result} directory by means of the \link[htmlwidgets]{saveWidget} function
in the \CRANpkg{htmlwidgets} package. Generated widgets allow users to
consult relative bias data directly on the plots, select/deselect different
sets of traces, to zoom and scroll, and so on. The pattern used for naming
these files is \file{RB.<x>.<plot design>.html}, where \file{<plot design>} is
indicated for the previously described files, and \file{<x>} equals \code{N},
\code{G}, \code{V}, \code{d} and/or \code{h} according to the \code{variables}
argument. All relative biases related to diameters are
plotted in the same chart (files named as \file{RB.d.<plot design>.html}), and
the same applies to those related to heights (files named as
\file{RB.h.<plot design>.html}).
}



\note{
The results obtained using this function are
merely descriptive, and they do not guarantee any type of statistical accuracy
in using TLS metrics instead of field estimations in order to estimate forest
attributes of interest.
}


\author{
Juan Alberto Molina-Valero and
Adela Martínez-Calvo.}


\seealso{
\code{\link{simulations}}, \code{\link{correlations}}.
}


\examples{
\donttest{

# Load variables estimated from field data, and TLS metrics
# corresponding to Rioja data set

data("Rioja.simulations")


# Establish directory where relative bias results corresponding to Rioja example
# will be saved. For instance, current working directory

dir.result <- tempdir()


# Compute relative bias between field-based estimates of TLS metrics
# corresponding to Rioja example
# Relative bias for variables by default

rb <- relative.bias(simulations = Rioja.simulations, dir.result = dir.result)


# Relative bias for variable 'N'

rb <- relative.bias(simulations = Rioja.simulations, variables = "N",
                    dir.result = dir.result)


# Relative bias corresponding to angle-count design for all available variables

rb <- relative.bias(simulations = Rioja.simulations["angle.count"],
                    variables <- c("N", "G", "V", "d", "dg", "dgeom", "dharm",
                                   "d.0", "dg.0", "dgeom.0", "dharm.0", "h",
                                   "hg", "hgeom", "hharm", "h.0", "hg.0",
                                   "hgeom.0", "hharm.0"),
                    dir.result = dir.result)
  }
}

\encoding{UTF-8}

\name{k_tree_cpp}

\alias{k_tree_cpp}

\title{Calculate dominant diameters and heights for simulations for angle-count plots.}

\description{Calculate dominant diameters and heights for simulations for angle-count plots.}

\usage{
  k_tree_cpp(k_seq, radius_seq, k, d, h, num)}

\arguments{

  \item{k_seq}{BAF sequence for simulations}

  \item{radius_seq}{BAF sequence for simulations}

  \item{k}{BAF masimum for simulations}

  \item{d}{vector with diameters opf trees}

  \item{h}{vector with heights of trees}

  \item{num}{number of trees considered for calculating diameter and dominant height}}

\value{Data frame.}


\author{
  Juan Alberto Molina-Valero,
  María José Ginzo Villamayor,
  Manuel Antonio Novo Pérez,
  Adela Martínez-Calvo,
  Juan Gabriel Álvarez-González,
  Fernando Montes and
  César Pérez-Cruzado.}

\keyword{internal}

\encoding{UTF-8}


\name{correlations}


\alias{correlations}


\title{
Correlation Between Field Estimations and TLS Metrics
}


\description{
Computes correlations between variables estimates from field data and
metrics derived from TLS data. Field estimates and TLS metrics for a common
set of plots are required in order to compute correlations. These data must
be obtained from any of the three different plot designs currently available (fixed
area, k-tree and angle-count), and correspond to plots with incremental values
for the plot design parameter (radius, k and BAF, respectively). Two
correlation measures are implemented: Pearson's correlation coefficient and
Spearman's \emph{rho}. In addition to estimating these measures, tests for
association are also executed, and interactive line charts graphically
representing correlations are generated.
}


\usage{
correlations(simulations,
             variables = c("N", "G", "d", "dg", "d.0", "h", "h.0"),
             method = c("pearson", "spearman"), save.result = TRUE,
             dir.result = NULL)
}


\arguments{

  \item{simulations}{
  List including estimated variables based on field data and metrics
  derived from TLS data. The structure and format must be analogous to output
  returned by the \code{\link{simulations}} function. Specifically, it must have
  at least one of the following named elements:
    \itemize{
      \item \code{fixed.area.plot}: data frame with field estimates and TLS
      metrics under a circular fixed area plot design. Each row corresponds to
      a (plot, radius) pair, and all or any of the following
      columns are included:

      Plot identification and radius:
        \itemize{
          \item \code{id}, \code{radius}: same description and format as
          indicated for same named columns of \code{fixed.area.plot} in
          \code{\link{simulations}} \sQuote{Value}.
        }
      Variables estimated on the basis of simulated field plots:
        \itemize{
          \item \code{N}, \code{G}, \code{V}, \code{d}, \code{dg},
          \code{dgeom}, \code{dharm}, \code{h}, \code{hg}, \code{hgeom},
          \code{hharm}, \code{d.0}, \code{dg.0}, \code{dgeom.0},
          \code{dharm.0}, \code{h.0}, \code{hg.0}, \code{hgeom.0},
          \code{hharm.0}: same description and format as indicated for same
          named columns of \code{fixed.area.plot} in \code{\link{simulations}} \sQuote{Value}.
        }
      TLS metrics derived from simulated TLS plots:
        \itemize{
          \item \code{N.tls}, \code{N.hn}, \code{N.hr}, \code{N.hn.cov},
          \code{N.hr.cov}, \code{N.sh}, \code{num.points}, \cr
          \code{num.points.est}, \code{num.points.hom},
          \code{num.points.hom.est}, \code{G.tls}, \cr \code{G.hn},
          \code{G.hr}, \code{G.hn.cov}, \code{G.hr.cov}, \code{G.sh},
          \code{V.tls}, \code{V.hn}, \code{V.hr}, \code{V.hn.cov}, \cr
          \code{V.hr.cov}, \code{V.sh}, \code{d.tls}, \code{dg.tls},
          \code{dgeom.tls}, \code{dharm.tls}, \code{h.tls}, \cr \code{hg.tls},
          \code{hgeom.tls}, \code{hharm.tls}, \code{d.0.tls}, \code{dg.0.tls},
          \code{dgeom.0.tls}, \code{dharm.0.tls}, \code{h.0.tls},
          \code{hg.0.tls}, \code{hgeom.0.tls}, \code{hharm.0.tls}, \code{P01},
          \code{P05}, \code{P10}, \code{P20}, \code{P25}, \code{P30},
          \code{P40}, \code{P50}, \code{P60}, \code{P70}, \code{P75},
          \code{P80}, \code{P90}, \code{P95}, \code{P99}: same description and
          format as indicated for same named columns of \code{fixed.area.plot}
          in \code{\link{simulations}} \sQuote{Value}.
        }
      If the \code{fixed.area.plot} element is included in \code{simulations}
      argument, it must contain at least \code{id} and \code{radius} columns,
      one of the field estimates columns and one of the TLS metrics
      columns.

      \item \code{k.tree.plot}: data frame with field estimates and TLS
      metrics under the k-tree plot design. Each row corresponds to a
      (plot, k) pair, and all or any of the following columns are included:

      Plot identification and k:
        \itemize{
          \item \code{id}, \code{k}: same description and format as indicated
          for same named columns of \code{k.tree.plot} in \code{\link{simulations}} \sQuote{Value}.
        }
      Variables estimates on the basis of simulated field plots:
        \itemize{
          \item \code{N}, \code{G}, \code{V}, \code{d}, \code{dg},
          \code{dgeom}, \code{dharm}, \code{h}, \code{hg}, \code{hgeom},
          \code{hharm}, \code{d.0}, \code{dg.0}, \code{dgeom.0},
          \code{dharm.0}, \code{h.0}, \code{hg.0}, \code{hgeom.0},
          \code{hharm.0}: same description and format as indicated for same
          named columns of \code{k.tree.plot} in \code{\link{simulations}} \sQuote{Value}.
        }
      TLS metrics derived from simulated TLS plots:
        \itemize{
          \item \code{N.tls}, \code{N.hn}, \code{N.hr}, \code{N.hn.cov},
          \code{N.hr.cov}, \code{N.sh}, \code{num.points}, \cr
          \code{num.points.est}, \code{num.points.hom},
          \code{num.points.hom.est}, \code{G.tls}, \cr \code{G.hn},
          \code{G.hr}, \code{G.hn.cov}, \code{G.hr.cov}, \code{G.sh},
          \code{V.tls}, \code{V.hn}, \code{V.hr}, \cr \code{V.hn.cov},
          \code{V.hr.cov}, \code{V.sh}, \code{d.tls}, \code{dg.tls},
          \code{dgeom.tls}, \code{dharm.tls}, \code{h.tls}, \code{hg.tls},
          \code{hgeom.tls}, \code{hharm.tls}, \code{d.0.tls}, \code{dg.0.tls},
          \cr \code{dgeom.0.tls}, \code{dharm.0.tls}, \code{h.0.tls},
          \code{hg.0.tls}, \code{hgeom.0.tls}, \cr \code{hharm.0.tls},
          \code{P01}, \code{P05}, \code{P10}, \code{P20}, \code{P25},
          \code{P30}, \code{P40}, \code{P50}, \code{P60}, \code{P70},
          \code{P75}, \code{P80}, \code{P90}, \code{P95}, \code{P99}: same
          description and format as indicated for same named columns of
          \code{k.tree.plot} in \code{\link{simulations}} \sQuote{Value}.
        }
      If a \code{k.tree.plot} element is included in the \code{simulations} argument,
      it must include at least \code{id} and \code{k} columns, one of the field
      estimate columns, and one of the TLS metrics columns.

      \item \code{angle.count.plot}: data frame with field estimates and TLS
      metrics under the angle-count plot design. Each row corresponds
      to a (plot, BAF) pair, and all or one of the following
      columns are included:

      Plot identification and BAF:
        \itemize{
          \item \code{id}, \code{BAF}: same description and format as indicated
          for same named columns of \code{angle.count.plot} in \code{\link{simulations}} \sQuote{Value}.
        }
      Variables estimated on the basis of simulated field plots:
        \itemize{
          \item \code{N}, \code{G}, \code{V}, \code{d}, \code{dg},
          \code{dgeom}, \code{dharm}, \code{h}, \code{hg}, \code{hgeom},
          \code{hharm}, \code{d.0}, \code{dg.0}, \code{dgeom.0},
          \code{dharm.0}, \code{h.0}, \code{hg.0}, \code{hgeom.0},
          \code{hharm.0}: same description and format as indicated for same
          named columns of \code{angle.count.plot} in \code{\link{simulations}} \sQuote{Value}.
        }
      TLS metrics derived from simulated TLS plots:
        \itemize{
          \item \code{N.tls}, \code{N.pam}, \code{num.points},
          \code{num.points.est}, \code{num.points.hom}, \cr
          \code{num.points.hom.est}, \code{G.tls}, \code{G.pam}, \code{V.tls},
          \code{V.pam}, \code{d.tls}, \code{dg.tls}, \code{dgeom.tls},
          \code{dharm.tls}, \code{h.tls}, \code{hg.tls}, \code{hgeom.tls},
          \code{hharm.tls}, \cr \code{d.0.tls}, \code{dg.0.tls},
          \code{dgeom.0.tls}, \code{dharm.0.tls}, \code{h.0.tls}, \cr
          \code{hg.0.tls}, \code{hgeom.0.tls}, \code{hharm.0.tls}, \code{P01},
          \code{P05}, \code{P10}, \code{P20}, \code{P25}, \code{P30},
          \code{P40}, \code{P50}, \code{P60}, \code{P70}, \code{P75},
          \code{P80}, \code{P90}, \code{P95}, \code{P99}: same description and
          format as indicated for same named columns of \code{angle.count.plot}
          in \code{\link{simulations}} \sQuote{Value}.
        }
      If the \code{angle.count.plot} element is included in the \code{simulations}
      argument, it must contain at least \code{id} and \code{BAF} columns, one
      of the field estimates columns and one of the TLS metrics columns.
    }
  }

  \item{variables}{
  Optional character vector naming field estimates for which correlations
  between these and all the available TLS metrics will be computed. If
  this argument is specified by the user, it must include at least one of the
  following character strings: \dQuote{\code{N}}, \dQuote{\code{G}},
  \dQuote{\code{V}}, \dQuote{\code{d}}, \dQuote{\code{dg}},
  \dQuote{\code{dgeom}}, \dQuote{\code{dharm}}, \dQuote{\code{d.0}},
  \dQuote{\code{dg.0}}, \dQuote{\code{dgeom.0}}, \dQuote{\code{dharm.0}},
  \dQuote{\code{h}}, \dQuote{\code{hg}}, \dQuote{\code{hgeom}},
  \dQuote{\code{hharm}}, \dQuote{\code{h.0}}, \dQuote{\code{hg.0}},
  \dQuote{\code{hgeom.0}}, or \dQuote{\code{hharm.0}}. If this argument is not
  specified by the user, it will be set to

  \code{c("N", "G", "V", "d", "dg", "d.0", "h", "h.0")} by default. In both
  cases all the elements in the \code{simulations} argument
  must include at least the columns corresponding to the field estimates specified in the
  \code{variables} argument.
  }

  \item{method}{
  Optional character vector naming which correlation measurements will be used.
  If this argument is specified by the user, it must include at least one of
  the following character strings: \dQuote{\code{pearson}} or
  \dQuote{\code{spearman}}. If this argument is not specified by the user, it
  will be set to \code{c("pearson", "spearman")} by default.
  }

  \item{save.result}{
  Optional logical indicating wheter or not the output files described in
  \sQuote{Output Files} section must be saved in \code{dir.result} or not. If
  this argument is not specified by the user, it will be set to \code{TRUE} by
  default and, as a consequence, the output files will be saved.
  }

  \item{dir.result}{
  Optional character string naming the absolute path of an existing directory
  where files described in the \sQuote{Output Files} section will be saved.
  \code{.Platform$file.sep} must be used as the path separator in
  \code{dir.result}. If this argument is not specified by the user, and
  \code{save.result} is \code{TRUE}, it will be set to \code{NULL} by default and,
  as consequence, the current working directory of the \R process will be
  assigned to \code{dir.result} during execution.
  }
}


\details{
For each radius, k or BAF value (according to the currently available plot
designs: circular fixed area, k-tree and angle-count), this function computes
correlations between each variable estimated from field data
specified in the \code{variables} argument and all the metrics derived from TLS
data existing in the data frames included in the \code{simulations} argument.

Two correlation measures are implemented at present: Pearson's correlation
coefficient and Spearman's \emph{rho}. For each method, in addition to the
estimated measure, the p-value of a test for association is also returned.
The \code{\link{cor.test}} function from the \pkg{utils} package is used to compute
both the estimated correlations and the p-values of the associated tests; more
details about these measures and their tests for association can be found in
the corresponding documentation. There cannot be
missing data for three or more plots, and there cannot be zero standard deviation,
in order to prevent missing correlation values for each (field estimation, TLS
metric) pair and plot design parameter (radius, k or BAF).

Apart from estimated correlations and their corresponding p-values, for ecah method, the
function also returns the plot design parameter and field
estimates, the value of the optimal correlation (i.e. the maximum of the
absolute value of available correlations) and the TLS metric to which it
corresponds.
}


\value{

  \item{correlations}{
  A list including the estimated correlations for each measure specified in the
  \code{method} argument will be generated. This will include all or any of the following named
  elements:
    \itemize{
      \item \code{pearson}: if \dQuote{\code{pearson}} is not included in
      \code{method} parameter, missing; otherwise, the list will include the
      estimated Pearson's correlations for each plot design specified in the
      \code{simulations} argument. In the latter case, the list will include all or any
      of the following named elements:
        \itemize{
          \item \code{fixed.area.plot}: if the \code{simulations} argument does not
          have an element named \dQuote{\code{fixed.area.plot}}, missing;
          otherwise, the matrix will include the estimated Pearson's correlations
          for a circular fixed area plot design. Each row will correspond to a
          radius value, and the following columns will be included:
            \itemize{
              \item \code{radius}: radius (m) of the simulated plots used for
              computing the estimated correlations.
              \item Column(s) \sQuote{\code{<x>.<y>}}: numeric column(s)
              containing estimated Pearson's correlations between
              \sQuote{\code{<x>}}, a field estimate, and \sQuote{\code{<y>}},
              a TLS metric.
            }
          \item \code{k.tree.plot}: if the \code{simulations} argument does not
          include an element named \dQuote{\code{k.tree.plot}}, missing;
          otherwise, the matrix will include the estimated Pearson's correlations
          for a k-tree plot design. Each row will correspond to a k
          value, and the following columns will be included:
            \itemize{
              \item \code{k}: number of trees (trees) of the simulated plots
              used for computing the estimated correlations.
              \item Column(s) \sQuote{\code{<x>.<y>}}: same description and
              format as indicated in
              \code{correlations$pearson$fixed.area.plot} element.
            }
          \item \code{angle.count.plot}: if the \code{simulations} argument does
          not have any element named \dQuote{\code{angle.count}},  missing;
          otherwise, the matrix will include the estimated Pearson's correlations
          for the angle-count plot design. Each row will correspond to a
          BAF value, and the following columns will be included:
            \itemize{
              \item \code{BAF}: BAF (\eqn{{m}^{2}/ha}{m^2/ha}) of the simulated plots used for
              computing the estimated correlations.
              \item Column(s) \sQuote{\code{<x>.<y>}}: same description
              and format as indicated in
              \code{correlations$pearson$fixed.area.plot} element.
            }
        }
      \item \code{spearman}: if \dQuote{\code{spearman}} is not included in
      \code{method} parameter, missing; otherwise, the list will include the
      estimated Spearman's correlations for each plot design specified in
      \code{simulations} argument. In the latter case, the structure and format
      will be analogous to that indicated for the previous element but
      estimated Pearson's correlations will be replaced by Spearman's correlations.
    }
  }

  \item{correlations.pval}{
  List containing the p-value of the test for association corresponding to each
  measure specified in \code{method} argument. The structure and format will be
  the same as indicated for the previous element but estimated
  correlations will be replaced by p-values for their corresponding tests for association.
  }

  \item{opt.correlations}{
  List containing the optimal correlations, and the names of the TLS metrics to
  which they correspond, for each measure specified in \code{method} argument.
  The list will will include all or any of the following named elements:
    \itemize{
      \item \code{pearson}: if \dQuote{\code{pearson}} is not included in
      \code{method} parameter, missing; otherwise, the list will include the optimal
      Pearson's correlations, and the names of the TLS metrics to which they
      correspond, for each plot design specified in \code{simulations}
      argument. In the latter case, it will include all or any of the following
      named elements:
        \itemize{
          \item \code{fixed.area.plot}: if \code{simulations} argument does not
          have any element named \dQuote{\code{fixed.area.plot}}, missing;
          otherwise, the data frame will include the optimal Pearson's correlations,
          and the names of the TLS metrics to which they correspond, for a
          circular fixed area plot design. Each row will correspond to a radius
          value, and the following columns will be included:
            \itemize{
              \item \code{radius}: radius (m) of the simulated plots used for
              computing the estimated correlations.
              \item Columns \sQuote{\code{<x>.cor}} and
              \sQuote{\code{<x>.metric}}: the former, numeric column(s)
              including optimal Pearson's correlations between
              \sQuote{\code{<x>}}, a field estimate, and all the available
              TLS metrics; and the latter, character column(s) will include names
              of the TLS metrics to which they correspond.
            }
          \item \code{k.tree.plot}: if the \code{simulations} argument does not
          have any element named \dQuote{\code{k.tree.plot}}, missing;
          otherwise, the data frame will include the optimal Pearson's correlations
          and the names of the TLS metrics to which they correspond for the
          k-tree plot design. Each row will correspond to a k value,
          and the following columns will be included:
            \itemize{
              \item \code{k}: number of trees (trees) of the simulated plots
              used for computing the estimated correlations.
              \item Columns \sQuote{\code{<x>.cor}} and
              \sQuote{\code{<x>.metric}}: same description and format as
              indicated in \code{opt.correlations$pearson$fixed.area.plot}
              element.
            }
          \item \code{angle.count.plot}: if the \code{simulations} argument does
          not have any element named \dQuote{\code{angle.count}}, missing;
          otherwise, the data frame will include the optimal Pearson's correlations,
          and the names of the TLS metrics to which they correspond for the
          angle-count plot design. Each row will correspond to a BAF
          value, and the following columns will be included:
            \itemize{
              \item \code{BAF}: BAF (\eqn{{m}^{2}/ha}{m^2/ha}) of the simulated plots used for
              computing the estimated correlations.
              \item Columns \sQuote{\code{<x>.cor}} and
              \sQuote{\code{<x>.metric}}: same description and format as
              indicated in \code{opt.correlations$pearson$fixed.area.plot}
              element.
            }
        }
      \item \code{spearman}: if \dQuote{\code{spearman}} is not included in
      \code{method} parameter, missing; otherwise, the list will include the
      optimal Spearman's correlations, and the names of the TLS metrics to
      which they correspond, for each plot design specified in
      \code{simulations} argument. In the latter case, the structure and
      format will be analogous to that indicated for the previous element, but
      optimal Pearson's correlations will be replaced by Spearman's correlations.
    }
  }
}


\section{Output Files}{
During the execution, if the \code{save.result} argument is \code{TRUE}, the
function prints to files the matrices and data frames included in
\code{correlations} and \code{opt.correlations} elements described in
\sQuote{Value}. Both are written without row names in
\code{dir.result} directory by using the \code{\link{write.csv}} function in the
\pkg{utils} package. The patterns used for naming these files are
\file{correlations.<plot design>.<method>.csv} and
\file{opt.correlations.<plot design>.plot.<method>.csv} for correlation
matrices and optimal correlation data frames, respectively, where
\file{<plot design>} is equal to \dQuote{\code{fixed.area.plot}},
\dQuote{\code{k.tree.plot}} or \dQuote{\code{angle.count.plot}} according to
plot design, and \file{<method>} equals \dQuote{\code{pearson}} or
\dQuote{\code{spearman}} according to the correlation measure.

Furthermore, if the \code{save.result} argument is \code{TRUE}, interactive line
charts graphically representing correlations will also be created and saved
in the \code{dir.result} directory by means of \link[htmlwidgets]{saveWidget} function
in the \CRANpkg{htmlwidgets} package. Generated widgets enable users to
consult correlation data directly on the plots, select/deselect different
sets of traces, to zoom and scroll, etc. The pattern used for naming
these files is \file{correlations.<x>.<plot design>.<method>.html}, where both
\file{<plot design>} and \file{<method>} are as indicated for the previous
described files, and \file{<x>} is equal to any of elements specified in the
\code{variables} argument.
}


\note{
This function is particularly useful for further steps related to model-based
and model-assisted approaches, as correlations measure the strength of a relationship between two variables (linear for
Pearson's correlation, monotonic for Spearman's correlation).
}


\author{
Juan Alberto Molina-Valero and
Adela Martínez-Calvo.}


\seealso{
\code{\link{simulations}}, \code{\link{optimize.plot.design}}.

\code{\link{cor.test}} in \pkg{utils} package.
}


\examples{
\donttest{

# Load field estimates and TLS metrics corresponding to Rioja data set

data("Rioja.simulations")


# Establish directory where correlation results corresponding to the Rioja example
# will be saved. For instance, current working directory

# dir.result <- tempdir()


# Compute correlations between field estimates and TLS metrics corresponding
# to Rioja example
# Pearson's and Spearman's correlations for variables by default

# corr <- correlations(simulations = Rioja.simulations, dir.result = dir.result)


# Pearson's and Spearman's correlations for variable 'N'

# corr <- correlations(simulations = Rioja.simulations, variables = "N",
#                      dir.result = dir.result)


# Only Pearson's correlations for variables by default

# corr <- correlations(simulations = Rioja.simulations, method = "pearson",
#                      dir.result = dir.result)


# Pearson's and Spearman's correlations corresponding to angle-count design for
# all available variables

# corr <- correlations(simulations = Rioja.simulations["angle.count"],
#                      variables <- c("N", "G", "V", "d", "dg", "dgeom", "dharm",
#                                     "d.0", "dg.0", "dgeom.0", "dharm.0", "h",
#                                     "hg", "hgeom", "hharm", "h.0", "hg.0",
#                                     "hgeom.0", "hharm.0"),
#                      dir.result = dir.result)

  }

}

\encoding{UTF-8}

\name{metrics.variables}

\alias{metrics.variables}

\title{Compute Metrics and Variables for Terrestrial-Based Technologies Point Clouds}

\description{This function computes a set of metrics and variables from the Terrestrial-Based Technologies point cloud data, which have a high potential to be related or used as direct estimates (in the case of variables) of forest attributes at plot level. These can be obtained for different plot designs (circular fixed area, k-tree and angle-count plots). This function also includes methodologies for correcting occlusions generated in TLS single-scan point clouds.}

\usage{
metrics.variables(tree.tls, tree.ds = NULL, tree.field = NULL,
                  plot.design = c("fixed.area", "k.tree", "angle.count"),
                  plot.parameters = data.frame(radius = 25, k = 10, BAF = 2),
                  scan.approach = "multi",
                  var.metr = list(tls = NULL, field = NULL),
                  v.calc = "parab", dbh.min = 4, h.min = 1.3,
                  max.dist = Inf, dir.data = NULL,
                  save.result = TRUE, dir.result = NULL)}

\arguments{

  \item{tree.tls}{Data frame with information about trees detected from terrestrial-based technologies point clouds data in the same format as \code{\link{tree.detection.single.scan}} or \code{\link{tree.detection.multi.scan}} \sQuote{Values}.}

  \item{tree.ds}{Optional list containing detection probabilities of trees with distance sampling methods. The format must be the same as the \sQuote{Value} obtained with \code{\link{distance.sampling}}. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, TLS metrics using distance sampling based correction will not be calculated for circular fixed area and k-tree plot designs.}

    \item{tree.field}{
  Data frame with information about trees measured in the field plots. Each
  row must correspond to a (plot, tree) pair, and it must include at least the
  following columns:
    \itemize{
      \item \code{id}: plot identification encoded as character string or
      numeric. Plot identifications must coincide with those
      included in \code{id} column of the \code{tree.tls} argument.
      \item \code{tree}: trees numbering.
      \item \code{h.dist}: horizontal distance (m) from plot
      centre to tree centre. Centres of the field plots must coincide with
      centres of their corresponding Terrestrial-based technologies plots.
      \item \code{dbh}: tree diameter (cm) at breast height (1.3 m).
      \item \code{h}: tree total height (m).
      \item \code{h.blc}: height based live crown (m) (optional).
      \item \code{h.bdc}: height based dead crown (m) (optional).
      \item \code{v.user}: tree volume (m^3) (optional).
      \item \code{w.user}: tree biomass (Mg) (optional).
      \item \code{dead}: integer value indicating for each tree if it is dead
      (1) or not (NA).
    }
  }

  \item{plot.design}{Vector containing the plot designs considered. By default, all plot designs will be considered (circular fixed area, k-tree and angle-count plots).}

  \item{plot.parameters}{Data frame containing parameters for circular fixed area, k-tree and angle-count plot designs. If there is a \code{stratum} column in the \code{tree.list.tls} argument, it must have the same number of rows as strata values and they must be named using strata encoding. If plot parameters are not specified, the corresponding plot designs will not be considered in the function. If no parameter is specified, the function will stop giving an error message! The parameters are as follows:}

  \itemize{

  \item{\code{radius}}: plot radius (m) considered for circular fixed area plots. Absence of this argument rules out this plot design.

\item{\code{k.tree}}: number of trees (trees) considered for k-tree plots. Absence of this argument rules out this plot design.

\item{\code{BAF}}: basal area factor (\eqn{{m}^{2}/ha}{m^2/ha}) considered for angle-count plots. Absence of this argument rules out this plot design.

\item{\code{num.trees}}: number of dominant trees per ha (tree/ha), i.e. those with largest \emph{dbh}, considered for calculating dominant diameters and heights. In the absence of this argument, the number will be assumed to be 100 trees/ha.}

  \item{scan.approach}{Character parameter indicating TLS single-scan (\sQuote{single}) or TLS multi-scan approach or SLAM point clouds (\sQuote{multi}) approaches. If this argument is not specified by the user, it will be set to \sQuote{multi} approach.}

  \item{var.metr}{Optional vector containing all the metrics and variables of interest. By default it will be set as NULL and thus, all the metrics and variables available will be generated.}

  \item{v.calc}{Optional parameter to calculate volume when is not included in tree.tls input data.}

  \item{dbh.min}{Optional minimum \emph{dbh} (cm) considered for detecting trees. By default it will be set at 4 cm.}

  \item{h.min}{Optional minimum \emph{h} (m) considered for detecting trees. By default it will be set at 1.3 m.}

  \item{max.dist}{Optional argument to specify the maximum horizontal distance considered in which trees will be included.}

  \item{dir.data}{Optional character string naming the absolute path of the directory where TXT
  files containing TLS point clouds are located. \code{.Platform$file.sep} must
  be used as the path separator in \code{dir.data}, and TXT files in the
  directory must have the same description and format as indicated for TXT
  files in \code{\link{normalize}} \sQuote{Output Files}. If this
  argument is not specified by the user, it will be set to \code{NULL} by default
  and, as consequence, the current working directory of the \R process will be
  assigned to \code{dir.dat} during the execution.}

  \item{save.result}{Optional logical which indicates whether or not the output files described in \sQuote{Output Files} section must be saved in \code{dir.result}. If this argument is not specified by the user, it will be set to \code{TRUE} by default and, as consequence, the output files will be saved.}

  \item{dir.result}{Optional character string naming the absolute path of an existing directory where files described in \sQuote{Output Files} section will be saved. \code{.Platform$file.sep} must be used as the path separator in \code{dir.result}. If this argument is not specified by the user, and \code{save.result} is \code{TRUE}, it will be set to \code{NULL} by default and, as a consequence, the current working directory of the \R process will be assigned to \code{dir.result} during the execution.}}

\details{This function also works for several plots. In this case, a column named "id" to identify plots (string character or numeric) must be included in the \code{tree.list.tls} database argument. This must coincide with the corresponding "id" assigned in \code{\link{normalize}} to TXT files saved in \code{dir.data} (for more details see \code{\link{normalize}}). In addition, if there are several strata, they can be processed separately according to \code{plot.parameters} values (where each row will represent one stratum). If \code{tree.list.tls} does not include a specific "stratum" column, it will be assumed to have only one stratum, which will be encoded according to \code{rownames(plot.parameters)[1]}.

Using TLS data, this function computes metrics and estimations of variables (see \sQuote{Value}) for plot design specified in the \code{plot.parameters} argument. The notation used for variables is based on IUFRO (1959).

At this stage, three plot designs are available:

\itemize{
    \item Circular fixed area plots, simulated only if a \code{radius} value is
    specified in the \code{plot.parameters} argument.
    \item k-tree plots, simulated only if a \code{k.tree} value is
    specified in the \code{plot.parameters} argument.
    \item Angle-count plots, simulated only if a \code{BAF} value is
    specified in the \code{plot.parameters} argument.
  }

Volume is estimated modelling stem profile as a paraboloid and calculating the volumes of revolution; where trees \emph{dbh} are estimated in \code{\link{tree.detection.single.scan}} and \code{\link{tree.detection.multi.scan}}, and total heights are estimated as percentile 99 of z coordinate of points delimited by Voronoi polygons.

Regarding occlusion corrections for estimating variables, apart from distance sampling methods considered in \code{\link{distance.sampling}}, another occlusion corection based on correcting the shadowing effect (Seidel & Ammer, 2014) has been included to estimate some variables in circular fixed area and k-tree plots. In the case of angle-count plots, occlusion corrections are based on gap probability attenuation with distance to TLS depending on Poisson model (Strahler et al., 2008; Lovell et al., 2011).}


\value{List including TLS-based metrics and variables computed for plot designs considered. The list will contain one element per plot design considered (fixed.area.plot, k.tree.plot and angle.count.plot):

\item{fixed.area.plot}{If no value for \code{plot.radius} is specified in the \code{plot.parameters} argument, \code{NULL}; otherwise, the data frame will include TLS metrics and variables estimated in a circular fixed area plot design (rows represent simulations). The data frame will have the following columns:

Stratum, plot identification and radius:
    \itemize{
    \item \code{stratum}: stratum identification encoded as a character string or
      numeric. It must coincide with those included in the \code{stratum} column of \code{tree.list.tls}.
      \item \code{id}: plot identification encoded as character string or
      numeric. It will coincide with those included in the \code{id}
      column of \code{tree.list.tls} and, if applicable, the
      \code{distance.sampling} argument.
      \item \code{radius}: radius (m) of the simulated plot.}

      Terrestrial-Based Technologies variables:

      \itemize{

      \item \code{N.tls}: stand density (trees/ha) without occlusion
      corrections.
      \item \code{N.hn}, \code{N.hr}, \code{N.hn.cov}, \code{N.hr.cov}: stand
      density (trees/ha) with occlusion corrections based on distance sampling
      methodologies (see \code{\link{distance.sampling}}). These columns will not appear if the \code{distance.sampling}
      argument is \code{NULL}.
      \item \code{N.sh}: stand density (trees/ha) with correction of the
      shadowing effect.
      respectively).
      \item \code{G.tls}: stand basal area (\eqn{{m}^{2}/ha}{m^2/ha}) without occlusion
      corrections.
      \item \code{G.hn}, \code{G.hr}, \code{G.hn.cov}, \code{G.hr.cov}: stand
      basal area (\eqn{{m}^{2}/ha}{m^2/ha}) with occlusion corrections based on distance sampling
      methodologies (see \code{\link{distance.sampling}}). These columns are missing if \code{distance.sampling}
      argument is \code{NULL}.
      \item \code{G.sh}: stand basal area (\eqn{{m}^{2}/ha}{m^2/ha}) with correction of the
      shadowing effect.
      \item \code{V.tls}: stand volume (\eqn{{m}^{3}/ha}{m^3/ha}) without occlusion corrections.
      \item \code{V.hn}, \code{V.hr}, \code{V.hn.cov}, \code{V.hr.cov}: stand
      volume (\eqn{{m}^{3}/ha}{m^3/ha}) with occlusion corrections based on distance sampling
      methodologies (see \code{\link{distance.sampling}}). These  columns will be missing if the \code{distance.sampling}
      argument is \code{NULL}.
      \item \code{V.sh}: stand volume (\eqn{{m}^{3}/ha}{m^3/ha}) with correction of the
      shadowing effect.
      \item \code{d.tls}, \code{dg.tls}, \code{dgeom.tls}, \code{dharm.tls}:
      mean tree diameters (cm) at breast height (1.3 m), calculated from the arithmetic mean,
      quadratic mean, geometric mean and harmonic mean, respectively.
      \item \code{h.tls}, \code{hg.tls}, \code{hgeom.tls}, \code{hharm.tls}:
      mean tree heights (m), calculated from the arithmetic mean, quadratic mean, geometric
      mean and harmonic mean, respectively.
      \item \code{d.0.tls}, \code{dg.0.tls}, \code{dgeom.0.tls},
      \code{dharm.0.tls}: dominant mean tree diameters (cm) at breast height
      (1.3 m), calculated from the arithmetic mean, quadratic mean, geometric mean and
      harmonic mean, respectively.
      \item \code{h.0.tls}, \code{hg.0.tls}, \code{hgeom.0.tls},
      \code{hharm.0.tls}: dominant mean tree heights (m), calculated from the arithmetic
      mean, quadratic mean, geometric mean and harmonic mean, respectively.}

      Terrestrial-Based Technologies metrics:

      \itemize{

      \item \code{n.pts}, \code{n.pts.est}, \code{n.pts.red}, \code{n.pts.red.est}: number of points and estimated number of
      points (points) belonging to trees normal sections (+/- 5 cm) in the
      original point cloud (\code{n.pts} and \code{n.pts.est},
      respectively); and number of points and estimated number of points
      (points) belonging to trees normal sections (+/- 5 cm) in the reduced
      point cloud (\code{n.pts.red} and \code{n.pts.red.est},
      respectively).
      \item \code{P01}, \code{P05}, \code{P10}, \code{P20}, \code{P25},
      \code{P30}, \code{P40}, \code{P50}, \code{P60}, \code{P70}, \code{P75},
      \code{P80}, \code{P90}, \code{P95}, \code{P99}: height percentiles
      derived from z coordinates of TLS point clouds relative to ground level (m).
      \item \code{mean.arit.z/rho/phi}, \code{mean.qua.z/rho/phi}, \code{mean.geom.z/rho/phi}, \code{mean.harm.z/rho/phi}, \code{median.z/rho/phi}, \code{mode.z/rho/phi}: central tendency statistics for z, rho (horizontal distance) and r (euclidean distance) coordinates (arithmetic, quadratic, geometrical and harmonic means, median and mode, respectively).
      \item \code{var.z/rho/phi}, \code{sd.z/rho/phi}, \code{CV.z/rho/phi}, \code{D.z/rho/phi}, \code{ID.z/rho/phi}, \code{max.z/rho/phi}, \code{min.z/rho/phi}: dispersion tendency statistics for z, rho and r coordinates (variance, standar deviation, coefficient of variation, range, interquartile range, maximum and minimum, respectively).
      \item \code{kurt.z/rho/phi}, \code{skw.z/rho/phi}: curtosis and skewness, respectively.
      \item \code{L1.z/rho/phi}, \code{L2.z/rho/phi}, \code{L3.z/rho/phi}, \code{L4.z/rho/phi}, \code{L-CV.z/rho/phi}, \code{MAD-median.z/rho/phi}, \code{MAD-mode.z/rho/phi}: L-moments of order 2, 3 and 4, median of the absolute deviations from the overall median and mode of the absolute deviations from the overall mode, respectively.
      \item \code{L3.mu.z/rho/phi}, \code{L4.mu.z/rho/phi}: third and fourth central moments, respectively.
      \item \code{PA.2m}, \code{PA.mean.z}, \code{PA.mode.z}: percentage of laser returns above 2 m, mean and mode, respectively.
      \item \code{PB.2m}, \code{PB.mean.z}, \code{PB.median.z}, \code{PB.mode.z}: percentage of laser returns below 2 m, mean and mode, respectively.
      \item \code{weibull.b.z/rho/phi}, \code{weibull.c.z/rho/phi}: scale and shape parameters, respectively, for Weibull distribution fitted for z coordinates of TLS point clouds relative to ground level.}}


\item{k.tree.plot}{If no value for \code{k.tree} is specified in the \code{plot.parameters} argument, \code{NULL}; otherwise, the data frame will include TLS metrics and variables estimated in the k-tree plot design (rows represent simulations). The data frame will include the following columns (same description and format as indicated in \code{fixed.area.plot} element):

Stratum, plot identification and k:
    \itemize{
    \item \code{stratum}: stratum identification encoded as character string or
      numeric. It must coincide with those included in the \code{stratum} column of \code{tree.list.tls}.
      \item \code{id}: plot identification encoded as character string or numeric value. It will coincide with those included in \code{id}
      column of \code{tree.list.tls} or, if applicable, the
      \code{distance.sampling} argument.
      \item \code{k}: number of trees in the simulated plot.}

      Terrestrial-Based Technologies variables:
    \itemize{
      \item \code{N.tls}, \code{N.hn}, \code{N.hr}, \code{N.hn.cov},
      \code{N.hr.cov}, \code{N.sh}, \cr
      \code{G.tls}, \code{G.hn}, \code{G.hr}, \code{G.hn.cov}, \code{G.hr.cov},
      \code{G.sh}, \cr \code{V.tls}, \code{V.hn}, \code{V.hr}, \code{V.hn.cov},
      \code{V.hr.cov}, \code{V.sh}, \cr \code{d.tls}, \code{dg.tls},
      \code{dgeom.tls}, \code{dharm.tls}, \cr \code{h.tls}, \code{hg.tls},
      \code{hgeom.tls}, \code{hharm.tls}, \cr \code{d.0.tls}, \code{dg.0.tls},
      \code{dgeom.0.tls}, \code{dharm.0.tls}, \cr \code{h.0.tls}, \code{hg.0.tls},
      \code{hgeom.0.tls}, \code{hharm.0.tls}}

    Terrestrial-Based Technologies metrics:
    \itemize{
      \item \code{num.points},
      \code{num.points.est}, \code{num.points.hom}, \code{num.points.hom.est}, \cr          \code{P01},\code{P05},
      \code{P10}, \code{P20}, \code{P25}, \code{P30}, \code{P40}, \code{P50},
      \code{P60}, \code{P70}, \code{P75}, \code{P80}, \code{P90}, \code{P95},
      \code{P99}: same description and format
as indicated in fixed.area.plot element.}}


\item{angle.count.plot}{If no value for \code{BAF} is specified in the \code{plot.parameters} argument, \code{NULL}; otherwise, the data frame will include TLS metrics and variables estimated in the angle-count plot design (rows represent simulations). The data frame will include the following columns:

Stratum, plot identification and BAF:
    \itemize{
    \item \code{stratum}: stratum identification encoded as character string or
      numeric. It must coincide with those included in \code{stratum} column of \code{tree.list.tls}.
      \item \code{id}: plot identification encoded as character string or numeric. It will coincide with those included in the \code{id} column of \code{tree.list.tls}.
      \item \code{BAF}: BAF (\eqn{{m}^{2}/ha}{m^2/ha}) of the simulated plot.}


Terrestrial-Based Technologies variables:
    \itemize{
      \item \code{N.tls}: same description and format as indicated in the
      \code{fixed.area.plot} element.
      \item \code{N.pam}: stand density (trees/ha) with occlusion correction
      based on gap probability attenuation with distance to TLS.
      \code{fixed.area.plot} element.
      \item \code{G.tls}: same description and format as indicated for the
      \code{fixed.area.plot} element.
      \item \code{G.pam}: stand basal area (\eqn{{m}^{2}/ha}{m^2/ha}) with occlusion correction
      based on gap probability attenuation with distance from TLS.
      \item \code{V.tls}: same description and format as indicated for the
      \code{fixed.area.plot} element.
      \item \code{V.pam}: stand volume (\eqn{{m}^{3}/ha}{m^3/ha}) with occlusion correction based
      on gap probability attenuation with distance from TLS.
      \item \code{d.tls}, \code{dg.tls}, \code{dgeom.tls}, \code{dharm.tls}, \cr
      \code{h.tls}, \code{hg.tls}, \code{hgeom.tls}, \code{hharm.tls}, \cr
      \code{d.0.tls}, \code{dg.0.tls}, \code{dgeom.0.tls}, \code{dharm.0.tls}, \cr
      \code{h.0.tls}, \code{hg.0.tls}, \code{hgeom.0.tls}, \code{hharm.0.tls}}

      Terrestrial-Based Technologies metrics:
    \itemize{
      \item \code{num.points}, \code{num.points.est}, \code{num.points.hom},
      \code{num.points.hom.est}, \cr
      \code{P01}, \code{P05}, \code{P10}, \code{P20}, \code{P25}, \code{P30},
      \code{P40}, \code{P50}, \code{P60}, \code{P70}, \code{P75}, \code{P80},
      \code{P90}, \code{P95}, \code{P99}: same description and format as
      indicated for the \code{fixed.area.plot} element.}}}



\section{Output Files}{

After computing metrics and variables, if the \code{save.result} argument is \code{TRUE}, the function will save the elements in the list described in \sQuote{Value} (\code{fixed.area.plot}, \code{k.tree.plot} and \code{angle.count.plot}), which are different from \code{NULL} as CSV files.  Data frames are written without row names in the \code{dir.result} directory by using \code{\link{write.csv}} function from the \pkg{utils} package. The pattern used for naming these files is ‘metrics.variables.<plot design>.csv’,
being ‘<plot design>’ equal to “fixed.area.plot”, “k.tree.plot” or “angle.count.plot”
according to the plot design.}



\note{

In order to optimize plot designs and, therefore, for better use of \code{\link{metrics.variables}}, other functions such as \code{\link{correlations}}, \code{\link{relative.bias}} and \code{\link{estimation.plot.size}} should be used.

This function will be updated as new metrics are developed.}


\references{IUFRO (1959). Standarization of symbols in forest mensuration. Vienna, Austria, IUFRO.

Lovell, J. L., Jupp, D. L. B., Newnham, G. J., & Culvenor, D. S. (2011). Measuring tree stem diameters using intensity profiles from ground-based scanning lidar from a fixed viewpoint. \emph{ISPRS Journal of Photogrammetry and Remote Sensing}, \bold{66(1)}, 46-55. \doi{10.1016/j.isprsjprs.2010.08.006}

Seidel, D., & Ammer, C. (2014). Efficient measurements of basal area in short rotation forests based on terrestrial laser scanning under special consideration of shadowing. \emph{iForest-Biogeosciences and Forestry}, \bold{7(4)}, 227. \doi{10.3832/ifor1084-007}

Strahler, A. H., Jupp, D. L. B., Woodcock, C. E., Schaaf, C. B., Yao, T., Zhao,
F., Yang, X., Lovell, J., Culvenor, D., Newnham, G., Ni-Miester, W., &
Boykin-Morris, W. (2008). Retrieval of forest structural parameters using a
ground-based lidar instrument (Echidna®). \emph{Canadian Journal of Remote
Sensing}, \bold{34(sup2)}, S426-S440. \doi{10.5589/m08-046}}

\author{
Juan Alberto Molina-Valero,
Adela Martínez-Calvo.}

\seealso{
\code{\link{tree.detection.single.scan}}, \code{\link{tree.detection.multi.scan}}, \code{\link{tree.detection.several.plots}}, \code{\link{distance.sampling}},
\code{\link{normalize}}.
}

\examples{

\donttest{

# Establishment of working directories (optional)
# By default here we propose the current working directory of the R process

dir.data <- tempdir()
dir.result <- tempdir()


# Loading example data included in FORTLS

data("Rioja.data")
tree.tls <- Rioja.data$tree.tls
tree.tls <- tree.tls[tree.tls$id == "1", ]

# Download example of TXT file corresponding to plot 1 from Rioja data set

download.file(url = "https://www.dropbox.com/s/w4fgcyezr2olj9m/Rioja_1.txt?dl=1",
              destfile = file.path(dir.data, "1.txt"), mode = "wb")


# Considering distance sampling methods (only for single-scan approaches)

# ds <- distance.sampling(tree.tls)

met.var.TLS <- metrics.variables(tree.tls = tree.tls,
                                 # tree.ds = ds,
                                 plot.parameters = data.frame(radius = 10, k = 10, BAF = 2),
                                 dir.data = dir.data, dir.result = dir.result)


  }

}

\encoding{UTF-8}

\name{normalize}

\alias{normalize}

\title{Relative Coordinates and Density Reduction for Terrestrial-Based Technologies Point Clouds}

\description{This function obtains coordinates relative to the plot centre speciefied for Terrestrial Laser Scanner (TLS) and Mobile Laser Scanner (MLS) point clouds (supplied as LAS or LAZ files). Point clouds obtained from other devices/approaches (e.g. photogrammetry) can be also used, but the guarantee of good performance is likely to be lower. In addition, the point cropping process developed by Molina-Valero et al., (2019) is applied as a criterion for reducing point density homogeneously in space and proportionally to object size when TLS single-scans are provided.}

\usage{
normalize(las, normalized = NULL,
          x.center = NULL, y.center = NULL,
          x.side = NULL, y.side = NULL,
          xpoly = NULL, ypoly = NULL,
          max.dist = NULL, min.height = NULL, max.height = 50,
          algorithm.dtm = "knnidw", res.dtm = 0.2,
          csf = list(cloth_resolution = 0.5),
          intensity = NULL, RGB = NULL,
          scan.approach = "multi",
          voxel_size = NULL,
          threads = 1,
          id = NULL, file = NULL, plot = TRUE,
          dir.data = NULL, save.result = TRUE, dir.result = NULL,
          save.las = NULL)}

\arguments{

  \item{las}{Character string containing the name of LAS/LAZ file belonging to point cloud, including .las/.laz extensions (see \sQuote{Examples}). Planimetric coordinates of point cloud data must be in local, representing TLS scan point the origin with Cartesian coordinates x and y as (0, 0).}

  \item{normalized}{Optional argument to establish as \code{TRUE} when point cloud is already normalized.}

  \item{x.center}{Planimetric x center coordinate of point cloud data.}

  \item{y.center}{Planimetric y center coordinate of point cloud data.}

  \item{x.side}{x-side (m) of the plot when the plot is square or rectangular.}

  \item{y.side}{y-side (m) of the plot when the plot is square or rectangular.}

  \item{xpoly}{Vector containing the x-coordinates of the vertices of the clipping polygon.}

  \item{ypoly}{Vector containing the y-coordinates of the vertices of the clipping polygon.}

  \item{max.dist}{Optional maximum horizontal distance (m) considered from the plot centre. All points farther than \code{max.dist} will be discarded after the normalization process. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, all points will be used in processing, with \code{max.dist} representing the farthest point.}

  \item{min.height}{Optional minimum height (m) considered from ground level. All points below \code{min.height} will be discarded after the normalization process. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, all points will be used in processing, with \code{min.height} representing the lowest point.}

  \item{max.height}{Optional maximum height (m) considered from ground level. All points above \code{max.height} will be discarded after the normalization process. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, all points will be used in processing, with \code{max.height} representing the highest point.}

    \item{algorithm.dtm}{Algorithm used to generate the digital terrain model (DTM) from the TLS point cloud. There are two posible options based on spatial interpolation: \sQuote{tin} and \sQuote{knnidw} (see \sQuote{Details}). If this argument is not specified by the user, it will be set to \sQuote{tin} algorithm.}

      \item{res.dtm}{Numeric parameter. Resolution of the DTM generated to normalize point cloud (see \sQuote{Details}). If this argument is not specified by the user, it will be set to 0.2 m.}

      \item{csf}{List containing parameters of CSF algorithm:}

\itemize{

  \item {\code{cloth_resolution}}: by default 0.5.}

  \item{intensity}{Logical parameter useful when point clouds have LiDAR intesinty values.}

  \item{RGB}{Logical parameter useful when point clouds are colorized, thus including values of RGB colors. It is based on the Green Leaf Algorithm (GLA) (see \sQuote{Details}).}

  \item{scan.approach}{Character parameter indicating TLS single-scan (\sQuote{single}) or TLS multi-scan approach or SLAM point clouds (\sQuote{multi}) approaches. If this argument is not specified by the user, it will be set to \sQuote{multi} approach.}

  \item{voxel_size}{Defines the size of the 3D grid cells used for downsampling.}

  \item{threads}{Number of threads.}

  \item{id}{Optional plot identification encoded as character string or numeric. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, the plot will be encoded as 1.}

  \item{file}{Optional file name identification encoded as character string or numeric value. If it is null, file will be encoded as \code{id} by default.}

  \item{plot}{Optional logical which indicates whether or not the normalized point cloud will be plot. If this argument is not specified by the user, it will be set to \code{TRUE} by default and, as consequence, the normalized point cloud will be plot.}

  \item{dir.data}{Optional character string naming the absolute path of the directory where LAS files containing TLS point clouds are located. \code{.Platform$file.sep} must be used as the path separator in \code{dir.data}. If this argument is not specified by the user, it will be set to \code{NULL} by default and, as a consequence, the current working directory of the \R process will be assigned to \code{dir.data} during the execution.}

  \item{save.result}{Optional logical which indicates whether or not the output files described in \sQuote{Output Files} section must be saved in \code{dir.result}. If this argument is not specified by the user, it will be set to \code{TRUE} by default and, as consequence, the output files will be saved.}

  \item{dir.result}{Optional character string naming the absolute path of an existing directory where files described in \sQuote{Output Files} section will be saved. \code{.Platform$file.sep} must be used as the path separator in \code{dir.result}. If this argument is not specified by the user, and \code{save.result} is \code{TRUE}, it will be set to \code{NULL} by default and, as a consequence, the current working directory of the \R process will be assigned to \code{dir.result} during the execution.}

  \item{save.las}{Optional logical which indicates whether or not the imput point cloud must be saved in \code{dir.result} as LAZ file.}}

\details{

Relative coordinates are obtained by means of a normalization process, generating a digital terrain model (DTM) from the TLS point cloud, with the ground height set at 0 m. The DTM is generated by spatial interpolation of ground points classified with the CSF algorithm (Zhang et al., (2016)). Two algorithms are available for that purpose: (i) spatial interpolation based on a Delaunay triangulation, which performs a linear interpolation within each triangle (\sQuote{tin}); (ii) spatial interpolation using a k-nearest neighbour (KNN) approach with inverse-distance weighting (IDW) (\sQuote{knnidw}). Note that normalization process is based on \CRANpkg{lidR} package functions: \link[lidR]{classify_ground}, \link[lidR]{grid_terrain} and \link[lidR]{normalize_height}.

The point cropping process reduces the point cloud density proportionally to the likelihood that objects will receive points according to their distance from TLS and their size, which is determined by angle aperture (the farther they are, the lower the density is). The result is an approximately homogeneous point cloud in three-dimensional space (for more details see Molina-Valero et al., (2019)).

The Green Leaf Algorithm (GLA) is calculated according to Louhaichi et al., (2001)as follows:

Those points with values below 0 are clasified as woody parts, thus retained for tree detection in further functions.}

\value{Data frame of normalized point cloud including the following columns (each row corresponds to one point):

\item{id}{Plot identification encoded as a character string or numeric in the argument \code{id}.}

\item{file}{File name identification encoded as character string or numeric, corresponding to the normalized and reduced point clouds saved. This coincides with the TXT file in the absolute path specified in \code{dir.result} (if \code{save.result} is set to \code{TRUE}).}

\item{Coordinates}{

  Cartesian (according to \url{https://en.wikipedia.org/wiki/Cartesian_coordinate_system} notation):
    \itemize{
      \item \code{x}: x axis distance (m).
      \item \code{y}: y axis distance (m).
      \item \code{z}: height (m).
    }

  Cylindrical (according to \url{https://en.wikipedia.org/wiki/Cylindrical_coordinate_system} notation):
    \itemize{
      \item \code{rho}: horizontal distance (m).
      \item \code{phi}: angular coordinate (rad).
      \item \code{z}: height (m).
    }

  Spherical (according to \url{https://en.wikipedia.org/wiki/Spherical_coordinate_system} notation):
    \itemize{
      \item \code{r}: radial distance (m).
      \item \code{theta}: polar angle (rad).
      \item \code{phi}: azimuthal angle (rad)
    }}

\item{slope}{Slope of the terrain (rad).}

\item{intensity}{Intensity (only if point cloud has intensity values and specified in arguments).}

\item{R}{Red (only if point cloud is colorized and specified in arguments).}

\item{G}{Green (only if point cloud is colorized and specified in arguments).}

\item{B}{Blue (only if point cloud is colorized and specified in arguments).}

\item{GLA}{Green Leaf Algorithm (only if point cloud is colorized and specified in arguments).}

\item{prob}{selection probability assigned in point cropping process (0-1]. Only the farthest will have probability of 1.}

\item{prob.select}{final selection probability assigned in point cropping process. Selected (1) and discarded point (0).}}

\section{Output Files}{

At the end of the normalization process, if the \code{save.result} argument is \code{TRUE}, the function will save the reduced point cloud as TXT file and encoded according to \code{file} argument. The format is the same as data frame described in \sQuote{Value}, except for a \code{prob} column, which is removed because all points selected after the point cropping process have a final selection probability of 1. The data frame is written without row names in \code{dir.result} directory using the \code{\link[vroom]{vroom_write}} function in the \CRANpkg{vroom} package.}

\note{
Note that \code{max.dist}, \code{min.height} and \code{max.height} arguments may be useful for optimizing computing time as well as for removing unnecessary and/or outlier points. These values may be selected more appropriately when inventory data are already available, or the user has some knowledge about autoecology of scanned tree species.

Note also that the linear interpolation algorithm (\sQuote{tin} in this package) showed the highest accuracy in Liang et al., (2018) in DTM generation with single-scans. In this work a DTM resolution of 0.2 m was also considered adequately for square plots of 32 x 32 m.}

\references{
Liang, X., Hyyppä, J., Kaartinen, H., Lehtomäki, M., Pyörälä, J., Pfeifer, N., ... & Wang, Y. (2018). International benchmarking of terrestrial laser scanning approaches for forest inventories. \emph{ISPRS journal of photogrammetry and remote sensing}, \bold{144}, 137-179. \doi{https://doi.org/10.1016/j.isprsjprs.2018.06.021}

Louhaichi, M., Borman, M. M., & Johnson, D. E. (2001). Spatially located platform and aerial photography for documentation of grazing impacts on wheat. \emph{Geocarto International}, \bold{16(1)}, 65-70. \doi{https://doi.org/10.1080/10106040108542184}

Molina-Valero J. A., Ginzo-Villamayor M. J., Novo Pérez M. A., Álvarez-González J. G., & Pérez-Cruzado C. (2019). Estimación del área basimétrica en masas maduras de Pinus sylvestris en base a una única medición del escáner laser terrestre (TLS). \emph{Cuadernos de la Sociedad Espanola de Ciencias Forestales}, \bold{45(3)}, 97-116. \doi{https://doi.org/10.31167/csecfv0i45.19887}.

Zhang, W., Qi, J., Wan, P., Wang, H., Xie, D., Wang, X., & Yan, G. (2016). An easy-to-use airborne LiDAR data filtering method based on cloth simulation. \emph{Remote Sensing}, \bold{8(6)}, 501. \doi{https://doi.org/10.3390/rs8060501}.}

\seealso{
\code{\link{tree.detection.single.scan}}, \code{\link{tree.detection.multi.scan}}, \code{\link{tree.detection.several.plots}}.
}

\author{
Juan Alberto Molina-Valero and
Adela Martínez-Calvo.}

\examples{

\donttest{

# Establishment of working directories (optional)
# Establishment of working directories using tempdir() to avoid leaving files in
# user filespace

dir.data <- tempdir()
dir.result <- tempdir()


# TLS SINGLE-SCAN APPROACH

# Loading example TLS data (LAZ file) to dir.data

download.file("https://www.dropbox.com/s/17yl25pbrapat52/PinusRadiata.laz?dl=1",
              destfile = file.path(dir.data, "PinusRadiata.laz"),
              mode = "wb")


# Normalizing the whole point cloud data without considering arguments

# Note that in the case of TLS single-scans, the coordinates of the plot center
# must be specified by the x.center and y.center arguments, which are
# x.center = 0 and y.center = 0 in this case

pcd <- normalize(las = "PinusRadiata.laz",

                 id = "PinusRadiata",

                 scan.approach = "single",

                 x.center = 0, y.center = 0,

                 dir.data = dir.data, dir.result = dir.result)


# MLS OR TLS MULTI-SCAN APPROACHES

# Loading example MLS data (LAZ file) to dir.data

download.file(
"www.dropbox.com/scl/fi/es5pfj87wj0g6y8414dpo/PiceaAbies.laz?rlkey=ayt21mbndc6i6fyiz2e7z6oap&dl=1",
              destfile = file.path(dir.data, "PiceaAbies.laz"),
              mode = "wb")

# Normalizing the whole point cloud data without considering arguments

pcd <- normalize(las = "PiceaAbies.laz",

                 id = "PiceaAbies",

                 scan.approach = "multi",

                 threads = 2,

                 dir.data = dir.data, dir.result = dir.result)

  }

}

\encoding{UTF-8}

\name{angle_count_cpp}

\alias{angle_count_cpp}

\title{Calculate dominant diameters and heights for simulations for angle-count plots.}

\description{Calculate dominant diameters and heights for simulations for angle-count plots.}

\usage{
angle_count_cpp(baf_seq, baf, d, h, num)}

\arguments{

  \item{baf_seq}{BAF sequence for simulations}

  \item{baf}{BAF masimum for simulations}

  \item{d}{vector with diameters opf trees}

  \item{h}{vector with heights of trees}

  \item{num}{number of trees considered for calculating diameter and dominant height}}

\value{Data frame.}


\author{
Juan Alberto Molina-Valero,
María José Ginzo Villamayor,
Manuel Antonio Novo Pérez,
Adela Martínez-Calvo,
Juan Gabriel Álvarez-González,
Fernando Montes and
César Pérez-Cruzado.}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_fortls_python_deps.R
\name{install_fortls_python_deps}
\alias{install_fortls_python_deps}
\title{Install Python dependencies required by FORTLS}
\usage{
install_fortls_python_deps(envname = "fortls_env", method = "auto")
}
\arguments{
\item{envname}{Name of the virtual environment or conda environment.}

\item{method}{Either "virtualenv", "conda", or "auto".}
}
\description{
Install Python dependencies required by FORTLS
}

\encoding{UTF-8}

\name{tree.detection.single.scan}

\alias{tree.detection.single.scan}

\title{Tree-Level Variables Estimation for TLS Single-Scan Approach}

\description{Detects trees from TLS point clouds corresponding to a single scan. For each tree detected, the function calculates the central coordinates and estimates the diameter at 1.3 m above ground level (which is known as \emph{dbh}, diameter at breast height) and classifies it as fully visible or partially occluded. Finally, the function obtains the number of points belonging to normal sections of trees (those corresponding to \emph{dbh} +/- 5 cm) and estimates them for both original and reduced (with point cropping process) point clouds.}

\usage{
tree.detection.single.scan(data, single.tree = NULL,
                           dbh.min = 4, dbh.max = 200, h.min = 1.3,
                           geo.dist = 0.1,
                           tls.resolution = list(),
                           tls.precision = NULL,
                           stem.section = c(0.7, 3.5), stem.range = NULL, breaks = NULL,
                           slice = 0.1, understory = NULL, bark.roughness = 1,
                           den.type = 1, d.mer = NULL, segmentation = NULL,
                           plot.attributes = NULL, plot = TRUE,
                           threads = 1,
                           dir.data = NULL, save.result = TRUE, dir.result = NULL)}

\arguments{

  \item{data}{Data frame with same description and format as indicated for \code{\link{normalize}} \sQuote{Value}.}

  \item{single.tree}{Optional argument to indicate if there is only one tree.}

  \item{dbh.min}{Optional minimum \emph{dbh} (cm) considered for detecting trees. By default it will be set at 4 cm.}

  \item{dbh.max}{Optional maximum \emph{dbh} (cm) considered for detecting trees. By default it will be set at 200 cm.}

  \item{h.min}{Optional minimum \emph{h} (m) considered for detecting trees. By default it will be set at 1.3 m.}

  \item{geo.dist}{Local surface variation (also known as normal change rate, NCR). By default it will be set as 0.1. For better understanding of this argument see \sQuote{Details}.}

  \item{tls.resolution}{List containing parameters of TLS resolution. This can be defined by the angle aperture:}

\itemize{

  \item {\code{horizontal.angle}}: horizontal angle aperture (degrees).

  \item {\code{vertical.angle}}: vertical angle aperture (degrees).

  \item {\code{point.dist}}: distance (mm) between two consecutive points.

  \item {\code{tls.dist}}: distance (m) from TLS at which two consecutive points are separated by \code{point.dist}.

  If this argument is not specified by the user, it will be set to NULL by default and, as a consequence the function will stop giving an error message.}

  \item{tls.precision}{Average point cloud precision in cm.}

  \item{stem.section}{Section free of noise (shurb, branches, etc.) considered to detect trees. If not specified, an automatic internal algorithm will be applied (see \sQuote{Details}).}

  \item{stem.range}{Section considered to estimate straightness tree attributes.}

  \item{breaks}{Height above ground level (m) of slices considered for detecting trees. By default it will be considered all possible sections from 0.1 m to maximum height by 0.3 m intervals (+/- 5 cm).}

  \item{slice}{Slice width considered for detecting trees. By default it will be considered as 0.1 m.}

  \item{understory}{Optional argument to indicate if there is dense understory vegetation.}

  \item{bark.roughness}{Bark roughness established in 3 degrees (1 < 2 < 3). By default it will be considered as 1.}

  \item{den.type}{Numeric argument indicating the dendrometic type used to estimate volumen when there are not sections enough to fit a taper equation. Dendrometrics types available are the following: cylinder = 0, paraboloid = 1 (by default), cone = 2 and neiloid = 3.}

  \item{d.mer}{Top stem diameter (cm) considered to estimate commercial timber volume.}

  \item{segmentation}{Tree segmentation.}

  \item{plot.attributes}{Data frame with attributes at plot level. It must contain a column named \code{id} (character string or numeric value) with encoding coinciding with that used in \code{id} argument of \code{\link{normalize}} for identifying plots. If there are strata, another column named \sQuote{stratum} (numeric) will be required for other functionalities of \pkg{FORTLS} (see, for instance, \code{\link{estimation.plot.size}} or \code{\link{metrics.variables}}). If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, the function will not add these possible plot attributes.}

  \item{plot}{Optional logical which indicates whether or not the normalized point cloud will be plot. If this argument is not specified by the user, it will be set to \code{TRUE} by default and, as consequence, the normalized point cloud will be plot.}

  \item{threads}{Number of threads.}

  \item{dir.data}{Optional character string naming the absolute path of the directory where LAS files containing TLS point clouds are located. \code{.Platform$file.sep} must be used as the path separator in \code{dir.data}. If this argument is not specified by the user, it will be set to \code{NULL} by default and, as a consequence, the current working directory of the \R process will be assigned to \code{dir.data} during the execution.}

  \item{save.result}{Optional logical which indicates whether or not the output files described in \sQuote{Output Files} section should be saved in \code{dir.result}. If this argument is not specified by the user, it will be set to \code{TRUE} by default and, as a consequence, the output files will be saved.}

  \item{dir.result}{Optional character string naming the absolute path of an existing directory where the files described in \sQuote{Output Files} section will be saved. \code{.Platform$file.sep} must be used as the path separator in \code{dir.result}. If this argument is not specified by the user, and the \code{save.result} is \code{TRUE}, it will be set to \code{NULL} by default and, as a consequence, the current working directory of the \R process will be assigned to \code{dir.result} during the execution.}}

\details{

Slices determined by \code{breaks} argument are clustered using the DBSCAN algorithm (Ester et al., 1996) on the horizontal plane according to Cartesian coordinates (x, y). Before and after this process, several algorithms are used to remove noisy points and apply classification criteria to select the clusters of trees.

\emph{dbh} is directly estimated for the section of 1.3 m above ground level, and estimated from other sections using \emph{dbh}~\emph{breaks} linear regression. Finally, the mean value of all estimates is provided in \sQuote{Value} as the \emph{dbh} of the tree section.

The number of points corresponding to a normal section (+/- 5 cm) is estimated in proportion to \emph{dbh}, using the average number of points per radius unit as reference. In this respect, only tree sections fully visible at 1.3 m above ground level will be considered for estimating the average number of points.

Local surface variation (also known as normal change rate ,NCR), is a quantitative measure of curvature feature (Pauly et al., 2002). This is useful for distinguishing points belonging to fine branches and foliage (e.g. leaves, shrubs) and stem points (e.g. Jin et al., 2016; Zhang et al., 2019). Just as we considered 5 cm as suitable for calculating local surface variation for the stem separation in forests, according to other authors (Ma et al., 2015; Xia et al., 2015), we also established the NCR threshold as 0.1, according to Zhang et al. (2019). However, this argument (\code{ncr.threshold}) may be modified in order to use more appropriate values.}

\value{Data frame with the following columns for every tree detected (each row corresponds to one tree detected):

  \item{id}{Optional plot identification encoded as a character string or numeric. If this argument is not specified by the user, it will be set to NULL by default and, as a consequence, the plot will be encoded as 1.}

  \item{file}{Optional file name identification encoded as character string or numeric. If it is null, the file will be encoded as \code{id} by default.}

  \item{tree}{tree numbering}

\item{Coordinates}{

  Cartesian (according to \url{https://en.wikipedia.org/wiki/Cartesian_coordinate_system} notation):}
    \itemize{
      \item \code{x}: distance on x axis (m) of tree centre.
      \item \code{y}: distance on y axis (m) of tree centre.}

  Azimuthal angles:
    \itemize{
      \item \code{phi}: angular coordinate (rad) of tree centre.
      \item \code{phi.left}: angular coordinate (rad) of left border of tree section.
      \item \code{phi.right}: angular coordinate (rad) of right border of tree section}

\item{h.dist}{horizontal distance (m) from plot centre to tree centre.}

\item{\emph{dbh}}{estimated tree diameter (cm) at breast height (1.3 m).}

\item{\emph{h}}{estimated tree total height (m).}

\item{\emph{h.com}}{estimated commercial tree height (m) according to the top diameter defined in the argument \code{d.mer}.}

\item{\emph{v}}{estimated tree stem volume (m^3).}

\item{\emph{v.com}}{estimated commercial tree stem volume (m^3) according to the top diameter defined in the argument \code{d.mer}.}

\item{\emph{SS.max}}{Maximum sagitta (see Prendes et al. (2022)).}

\item{\emph{sinuosity}}{Sinuosity (see Prendes et al. (2022)).}

\item{\emph{lean}}{Lean (see Prendes et al. (2022)).}

\item{n.pts}{number of points corresponding to a normal section (+/- 5 cm) in the original point cloud.}

\item{n.pts.red}{number of points corresponding to a normal section (+/- 5 cm) in the point cloud reduced by the point cropping process.}

\item{n.pts.est}{number of points estimated for a normal section (+/- 5 cm) in the original point cloud.}

\item{n.pts.red.est}{number of points estimated for a normal section (+/- 5 cm) in the point cloud reduced by the point cropping process.}

\item{partial.occlusion}{yes (1) or no (0)}}

\section{Output Files}{

At the end of the tree detection process, if the \code{save.result} argument is \code{TRUE}, the function will save the data frame described in \sQuote{Value} as a CSV file named \sQuote{tree.tls.csv}. The data frame will be written without row names in the \code{dir.result} directory by using \code{\link[utils]{write.csv}} function from the \pkg{utils} package.}


\note{
Although \code{tree.detection} also works with reduced point clouds, thus reducing the computing time, use of the original point cloud is recommended in order to detect more trees. This will also depend on forest conditions, especially those related to visibility. The more distant the trees are, the lower the density of points will be, and using reduced point clouds will therefore complicate detection of the most distant trees.

Note that \code{dbh.min} and \code{dbh.max} are important for avoiding outlier values when inventory data are used for reference purposes. Otherwise, knowledge about the autoecology of species could be used for filtering anomalous values of \emph{dbh}.

The argument \code{breaks = 1.3} could be sufficient for detecting trees visible at \emph{dbh}, involving lower computational cost. However, those trees not detected at \emph{dbh}, may be estimated from lower and/or higher sections. Considering the three default sections in the argument \code{breaks = c(1.0, 1.3, 1.6)} maintains a good balance in the case study of this package.}


\references{
Ester, M., Kriegel, H. P., Sander, J., & Xu, X. (1996). A density-based algorithm for discovering clusters in large spatial databases with noise. In Kdd (Vol. 96, No. 34, pp. 226-231).

Jin, S., Tamura, M., & Susaki, J. (2016). A new approach to retrieve leaf normal distribution using terrestrial laser scanners. J. \emph{Journal of Forestry Research}, \bold{27(3)}, 631-638. \doi{https://doi.org/10.1007/s11676-015-0204-z}

Ma, L., Zheng, G., Eitel, J. U., Moskal, L. M., He, W., & Huang, H. (2015). Improved salient feature-based approach for automatically separating photosynthetic and nonphotosynthetic components within terrestrial lidar point cloud data of forest canopies. \emph{IEEE Transactions Geoscience Remote Sensing}, \bold{54(2)}, 679-696. \doi{https://doi.org/10.1109/TGRS.2015.2459716}

Pauly, M., Gross, M., & Kobbelt, L. P., (2002). Efficient simplification of point-sampled surfaces. In IEEE Conference on Visualization. (pp. 163-170). Boston, USA. \doi{https://doi.org/10.1109/VISUAL.2002.1183771}

Prendes, C., Canga, E., Ordoñez, C., Majada, J., Acuna, M., & Cabo, C. (2022). Automatic assessment of individual stem shape parameters in forest stands from tls point clouds: Application in pinus pinaster. Forests, 13(3), 431. \doi{https://doi.org/10.3390/f13030431}

Xia, S., Wang, C., Pan, F., Xi, X., Zeng, H., & Liu, H. (2015). Detecting stems in dense and homogeneous forest using single-scan TLS. \emph{Forests}. \bold{6(11)}, 3923-3945. \doi{https://doi.org/10.3390/f6113923}

Zhang, W., Wan, P., Wang, T., Cai, S., Chen, Y., Jin, X., & Yan, G. (2019). A novel approach for the detection of standing tree stems from plot-level terrestrial laser scanning data. \emph{Remote Sens}. \bold{11(2)}, 211. \doi{https://doi.org/10.3390/rs11020211}}

\author{
Juan Alberto Molina-Valero
and Adela Martínez-Calvo.}

\seealso{
\code{\link{normalize}}, \code{\link{tree.detection.multi.scan}}, \code{\link{tree.detection.several.plots}}, \code{\link{distance.sampling}}, \code{\link{estimation.plot.size}}, \code{\link{simulations}}, \code{\link{metrics.variables}}
}

\examples{

\donttest{

# Establishment of working directories (optional)
# Establishment of working directories using tempdir() to avoid leaving files in
# user filespace

dir.data <- tempdir()
dir.result <- tempdir()


# Loading example data (LAZ file) to dir.data

download.file("https://www.dropbox.com/s/17yl25pbrapat52/PinusRadiata.laz?dl=1",
              destfile = file.path(dir.data, "PinusRadiata.laz"),
              mode = "wb")


# Normalizing the whole point cloud data without considering arguments

# Note that in the case of TLS single-scans, the coordinates of the plot center
# must be specified by the x.center and y.center arguments, which are
# x.center = 0 and y.center = 0 in this case

pcd <- normalize(las = "PinusRadiata.laz",

                 id = "PinusRadiata",

                 scan.approach = "single",

                 x.center = 0, y.center = 0,

                 max.dist = 7.5,

                 dir.data = dir.data, dir.result = dir.result)


# Tree detection without considering arguments
# For this case study, TLS resolution was established as:
# point.dist = 6.34 mm and tls.dist = 10 m

tree.tls <- tree.detection.single.scan(data = pcd,

                                       tls.resolution = list(point.dist = 6.34, tls.dist = 10),

                                       breaks = 1.3,

                                       threads = 2,

                                       dir.data = dir.data, dir.result = dir.result)

  }

}

\encoding{UTF-8}
\name{FORTLS-package}
\alias{FORTLS}
\alias{FORTLS-package}
\docType{package}
\title{FORTLS: Automatic Processing of Terrestrial-Based Technologies Point Cloud Data for Forestry Purposes}
\description{
\packageDescription{FORTLS}
}
\details{

Usage of \pkg{FORTLS} includes the following functionalities:

\itemize{

\item{Tree detection: this is the first and necessary step for the other functionalities of \pkg{FORTLS}. This can be achieved using the following functions:}

\enumerate{

\item{\code{\link{normalize}}: mandatory first step for obtaining the relative coordinates of a TLS point cloud.}
\item{\code{\link{tree.detection.single.scan}}: detects as many trees as possible from a normalized TLS single-scan point clouds.}
\item{\code{\link{tree.detection.multi.scan}}: detects as many trees as possible from a normalized TLS multi-scan, SLAM, or similar terrestrial-based technologies point clouds.}
\item{\code{\link{tree.detection.several.plots}}: includes the two previous functions for a better workflow when there are several plots to be sequentially analyzed.}}

\item{Estimation of variables when no field data are available: this is the main functionality of \pkg{FORTLS} and can be achieved using the following functions:}

\enumerate{

\item{\code{\link{distance.sampling}}: optional function which can be used for considering methodologies for correcting occlusion effects in estimating variables.}

\item{\code{\link{estimation.plot.size}}: enables the best plot design to be determined on the basis of TLS data only.}

\item{\code{\link{metrics.variables}}: is used for estimating metrics and variables potentially related to forest attributes at stand level.}}

\item{Estimation of variables when field data are available: this is the main and most desirable functionality of \pkg{FORTLS} and can be achieved using the following functions:}

\enumerate{

\item{\code{\link{distance.sampling}}: as before.}

\item{\code{\link{simulations}}: computes simulations of TLS and field data for different plot designs. This is a prior step to the next functions.}

\item{\code{\link{relative.bias}}: uses \code{\link{simulations}} output to assess the accuracy of direct estimations of variables according to homologous TLS and field data.}

\item{\code{\link{correlations}}: uses \code{\link{simulations}} output to assess correlations among metrics and variables obtained from TLS data, and variables of interest estimated from field data.}

\item{\code{\link{optimize.plot.design}}: using \code{\link{correlations}} output, represents the best correlations for variables of interest according to the plot design. It is thus possible to select the best plot design for estimating forest attributes from TLS data. }

\item{\code{\link{metrics.variables}}: as before, but in this case plot parameters will be choosen on the basis of field data and better estimates will therefore be obtained.}}}}


\author{
\strong{Maintainer}: Juan Alberto Molina-Valero \email{juanalberto.molina.valero@usc.es} [copyright holder]

Authors:
\itemize{
  \item María José Ginzo Villamayor [contributor]
  \item Manuel Antonio Novo Pérez [contributor]
  \item Adela Martínez-Calvo [contributor]
  \item Juan Gabriel Álvarez-González [contributor]
  \item Fernando Montes [contributor]
  \item César Pérez-Cruzado [contributor]
}}

\references{

Molina-Valero J. A., Ginzo-Villamayor M. J., Novo Pérez M. A., Martínez-Calvo A., Álvarez-González J. G., Montes F., & Pérez-Cruzado C. (2019). FORTLS: an R package for processing TLS data and estimating stand variables in forest inventories. \emph{The 1st International Electronic Conference on Forests — Forests for a Better Future: Sustainability, Innovation, Interdisciplinarity}. \doi{https://doi.org/10.3390/IECF2020-08066}}

\encoding{UTF-8}

\name{fit_circle_cpp_modified}

\alias{fit_circle_cpp_modified}

\title{This function fit a circle based on 3 points}

\description{This function fit a circle based on 3 points}


\usage{
fit_circle_cpp_modified(points)}

\arguments{

  \item{points}{points}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}

\name{distance.sampling}

\alias{distance.sampling}

\title{Distance Sampling Methods for Correcting Occlusions Effects}

\description{Calculation of the probability of detection of every tree by using distance sampling methodologies (more specifically point transects methods), by means of fitting detection functions to the histogram of tree distribution according to their distance to TLS. Use both half normal and hazard rate functions, without and with \emph{dbh} as covariate. These probabilities are used for correcting estimation bias caused by lack of detection of trees due to occlusion.}

\usage{
distance.sampling(tree.tls,
                  id.plots = NULL,
                  strata.attributes = NULL)}

\arguments{

  \item{tree.tls}{Data frame with a list of trees detected and their \emph{dbh} and horizontal distances from TLS with the same structure and format as \code{\link{tree.detection.single.scan}} and \code{\link{tree.detection.several.plots}} \sQuote{Value}.}

  \item{id.plots}{Optional vector with plot identification encoded as character string or numeric for the plots considered. In this case, \code{tree.tls} argument must include a common column named \sQuote{id}. If this argument is not specified by the user, it will be set to NULL by default, and as a consequence, all plots will be considered.}

  \item{strata.attributes}{Optional data frame inluding plot radius considered at strata level. It must contain a column named \sQuote{stratum} (numeric) with encoding coinciding with that used in previous functions (\code{\link{normalize}}, \code{\link{tree.detection.single.scan}} and \code{\link{tree.detection.several.plots}}) for identifying strata. Therefore, strata must heve been included previously in \sQuote{tree.tls}. Another column named \sQuote{plot.radius} (numeric) will be required to set maximum horizontal distance (m) considered for fitting detection probability functions. If this argument is not specified by the user, it will be set to NULL by default, and as a consequence, all trees will be included.}}

\details{All internal functions related to distance sampling methodologies are fitted with the \link[Distance]{ds} function included in the \CRANpkg{Distance} package.

Detection functions are left-truncated at 1 m, according to Astrup et al., (2014).

Same warning messages as \link[Distance]{ds} function are provided when fits do not converge or another warnings occur.

For further details on these point transects methods  and similar sampling methodologies, as well as their application with R, see Buckland et al., (2001); Marques & Buckland, (2003); Miller & Thomas, (2015) and Clark (2016). Examples of distance sampling analyses, as well as lectures, are available at \url{https://distancesampling.org/resources/vignettes.html}.}

\value{List containing the following elements:

\item{tree}{Data frame with detection probabilities for every tree and method.}

\itemize{

  \item {\code{stratum}}: stratum identification (coincident with strata of \code{tree.tls}). If there are not strata, it will be set as a single stratum encoded as 1 (numeric).

  \item {\code{id}}: plot identification (coincident with id of \code{tree.tls}).

  \item {\code{tree}}: tree numbering (coincident with \code{tree} of \code{tree.tls}).

  \item {\code{P.hn}}: tree detection probability according to half normal function.

  \item {\code{P.hn.cov}}: tree detection probability according to half normal function with \emph{dbh} as covariate.

  \item {\code{P.hr}}: tree detection probability according to half rate function.

  \item {\code{P.hr.cov}}: tree detection probability according to half rate function with \emph{dbh} as covariate.}

\item{parameters}{Data frame with parameters estimated for detection functions (see references for understanding their meaning).}

\itemize{

  \item {\code{P.hn.scale}}: scale parameter for half normal function (sigma).

  \item {\code{P.hn.cov.scale.intercept}}: alpha.0 parameter of scale parameter for half normal function with \emph{dbh} as covariate.

  \item {\code{P.hn.cov.dbh}}: alpha.1 parameter of scale parameter for half normal function with \emph{dbh} as covariate.

  \item {\code{P.hr.scale}}: scale parameter for half rate function (sigma).

  \item {\code{P.hr.shape}}: shape parameter for half rate function (b).

  \item {\code{P.hr.cov.scale.intercept}}: alpha.0 parameter of scale parameter for half normal function with \emph{dbh} as covariate.

  \item {\code{P.hr.cov.dbh}}: alpha.1 parameter of scale parameter for half normal function with \emph{dbh} as covariate.

  \item {\code{P.hr.cov.shape}}: shape parameter for half rate function with \emph{dbh} as covariate (b).}


\item{AIC}{Data frame with Akaike information criterions (AIC) of every detection function fit.}

\itemize{

  \item {\code{P.hn}}: AIC of half normal function fit.

  \item {\code{P.hn.cov}}:  AIC of half normal function with \emph{dbh} as covariate fit.

  \item {\code{P.hr}}:  AIC of half rate function fit.

  \item {\code{P.hr.cov}}:  AIC of half rate function with \emph{dbh} as covariate fit.}}

  \note{Although this step is optional for other functionalities of \pkg{FORTLS}, such as obtaining metrics and assessing the best plot designs (implemented in \code{\link{metrics.variables}}, \code{\link{correlations}}, \code{\link{relative.bias}} and \code{\link{optimize.plot.design}}), its inclusion is highly recommended, especially with high rates of occlusions.

  Note that this function could be more useful after assessing the best possible plot design with \code{\link{estimation.plot.size}}, \code{\link{correlations}}, \code{\link{relative.bias}} or \code{\link{optimize.plot.design}} functions.}

\references{
Astrup, R., Ducey, M. J., Granhus, A., Ritter, T., & von Lüpke, N. (2014). Approaches for estimating stand-level volume using terrestrial laser scanning in a single-scan mode. \emph{Canadian Journal of Forest Research}, \bold{44(6)}, 666-676. \doi{https://doi.org/10.1139/cjfr-2013-0535}.

Buckland, S. T., Anderson, D. R., Burnham, K. P., Laake, J. L., Borchers, D. L., & Thomas, L. (2001). \emph{Introduction to distance sampling: estimating abundance of biological populations}, Oxford, United Kindown, Oxford University Press.

Clark, R. G. (2016). Statistical efficiency in distance sampling. \emph{PloS one}, \bold{11(3)}, e0149298. \doi{https://doi.org/10.1371/journal.pone.0149298}.

Marques, F. F., & Buckland, S. T. (2003). Incorporating covariates into standard line transect analyses. \emph{Biometrics}, \bold{59(4)}, 924-935. \doi{https://doi.org/10.1111/j.0006-341X.2003.00107.x}.

Miller, D. L., & Thomas, L. (2015). Mixture models for distance sampling detection functions. \emph{PloS one}, \bold{10(3)}, e0118726. \doi{https://doi.org/10.1371/journal.pone.0118726}.}

\author{
Juan Alberto Molina-Valero
and Adela Martínez-Calvo.}

\seealso{
\code{\link{tree.detection.single.scan}}, \code{\link{tree.detection.several.plots}}, \code{\link{metrics.variables}}, \code{\link{simulations}}.

\link[Distance]{ds} in \CRANpkg{Distance} package.
}


\examples{

\donttest{

# Loading example data

data(Rioja.data)

tree.tls <- Rioja.data$tree.tls

# Whithout considering maximum distance

ds <- distance.sampling(tree.tls)


  }

}

\encoding{UTF-8}

\name{geometric_features_dist}

\alias{geometric_features_dist}

\title{This function obtains geometric features at point level}

\description{This function obtains geometric features at point level}


\usage{
geometric_features_dist(points,
                        dist,
                        Anisotropy = FALSE,
                        Eigenentropy = FALSE,
                        Eigenvalue_sum = FALSE,
                        First_eigenvalue = FALSE,
                        Linearity = FALSE,
                        Normal_x = FALSE,
                        Normal_y = FALSE,
                        Normal_z = FALSE,
                        Number_of_points = FALSE,
                        Omnivariance = FALSE,
                        PCA_1 = FALSE,
                        PCA_2 = FALSE,
                        Planarity = FALSE,
                        Second_eigenvalue = FALSE,
                        Sphericity = FALSE,
                        Surface_density = FALSE,
                        Surface_variation = FALSE,
                        Third_eigenvalue = FALSE,
                        Verticality = FALSE,
                        Volume_density = FALSE,
                        num_threads = 0,
                        use_spatial_index = TRUE,
                        grid_cell_size = -1.0)}

\arguments{

  \item{points}{points}

  \item{dist}{dist}

  \item{Anisotropy}{Anisotropy}

  \item{Eigenentropy}{Eigenentropy}

  \item{Eigenvalue_sum}{Eigenvalue_sum}

  \item{First_eigenvalue}{First_eigenvalue}

  \item{Linearity}{Linearity}

  \item{Normal_x}{Normal_x}

  \item{Normal_y}{Normal_y}

  \item{Normal_z}{Normal_z}

  \item{Number_of_points}{Number_of_points}

  \item{Omnivariance}{Omnivariance}

  \item{PCA_1}{PCA_1}

  \item{PCA_2}{PCA_2}

  \item{Planarity}{Planarity}

  \item{Second_eigenvalue}{Second_eigenvalue}

  \item{Sphericity}{Sphericity}

  \item{Surface_density}{Surface_density}

  \item{Surface_variation}{Surface_variation}

  \item{Third_eigenvalue}{Third_eigenvalue}

  \item{Verticality}{Verticality}

  \item{Volume_density}{Volume_density}

  \item{num_threads}{num_threads}

  \item{use_spatial_index}{use_spatial_index}

  \item{grid_cell_size}{grid_cell_size}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}

\name{voxel_grid_downsampling}

\alias{voxel_grid_downsampling}

\title{Voxel down sampling}

\description{Voxel down sampling}


\usage{
voxel_grid_downsampling(points, voxel_size)}

\arguments{

  \item{points}{points}

  \item{voxel_size}{voxel_size}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}
\name{Rioja.simulations}
\docType{data}
\alias{Rioja.simulations}
\title{Simulated Metrics and Variables for a Stand Case Study in La Rioja}
\description{This list contains metrics and variables estimated from field data and TLS data from \code{\link{Rioja.data}}.

The elements on this list correspond to \code{\link{simulations}} \sQuote{Value}, as follows:

\enumerate{
    \item \code{fixed.area}: data frame with TLS metrics and variables estimated on the basis of simulated plots in a fixed area plot design with radius increment of 0.1 m (from smallest possible radius to 20 m). The following variables are provided for each pair (plot, radius) (see \code{\link{simulations}} \sQuote{Value} for more details):
    \tabular{rll}{
      [,1] \tab id   \tab character/numeric\cr
      [,2] \tab radius    \tab numeric\cr
      [,3:5] \tab N, G, V    \tab numeric\cr
      [,6:9] \tab d, dg, dgeom, dharm   \tab numeric\cr
      [,10:13] \tab h, hg, hgeom, hharm   \tab numeric\cr
      [,14:17] \tab d.0, dg.0, dgeom.0,  dharm.0  \tab numeric\cr
      [,18:21] \tab h.0, hg.0, hgeom.0, hharm.0   \tab numeric\cr
      [,22:27] \tab N.tls, N.hn.tls, N.hr.tls, N.hn.cov.tls, N.hr.cov.tls, N.sh.tls  \tab numeric\cr
      [,28:31] \tab num.points, num.points.est, num.points.hom, num.points.hom.est   \tab numeric\cr
      [,32:37] \tab G.tls, G.hn.tls, G.hr.tls, G.hn.cov.tls, G.hr.cov.tls, G.sh.tls  \tab numeric\cr
      [,38:43] \tab V.tls, V.hn.tls, V.hr.tls, V.hn.cov.tls, V.hr.cov.tls, V.sh.tls   \tab numeric\cr
      [,44:47] \tab d.tls, dg.tls, dgeom.tls, dharm.tls   \tab numeric \cr
      [,48:51] \tab h.tls, hg.tls, hgeom.tls, hharm.tls   \tab numeric\cr
      [,52:55] \tab d.0, dg.0, dgeom.0, dharm.0  \tab numeric\cr
      [,56:59] \tab h.0, hg.0, hgeom.0, hharm.0   \tab numeric\cr
      [,60:74] \tab P01, P05, P10, P20, P25, P30, P40, P50, P60, P70, P75, P80, P90, P95, P99   \tab numeric\cr}

 \item \code{k.tree}: data frame with TLS metrics and variables estimated on the basis of simulated plots under k-tree plot design for incremental values of 1 tree (from 1 to largest number of trees in one plot). The following variables are provided for each pair (plot, k) (see \code{\link{simulations}} \sQuote{Value} for more details):
    \tabular{rll}{
      [,1] \tab id   \tab character/numeric\cr
      [,2] \tab k    \tab numeric\cr
      [,3:5] \tab N, G, V    \tab numeric\cr
      [,6:9] \tab d, dg, dgeom, dharm   \tab numeric\cr
      [,10:13] \tab h, hg, hgeom, hharm   \tab numeric\cr
      [,14:17] \tab d.0, dg.0, dgeom.0,  dharm.0  \tab numeric\cr
      [,18:21] \tab h.0, hg.0, hgeom.0, hharm.0   \tab numeric\cr
      [,22:27] \tab N.tls, N.hn.tls, N.hr.tls, N.hn.cov.tls, N.hr.cov.tls, N.sh.tls  \tab numeric\cr
      [,28:31] \tab num.points, num.points.est, num.points.hom, num.points.hom.est   \tab numeric\cr
      [,32:37] \tab G.tls, G.hn.tls, G.hr.tls, G.hn.cov.tls, G.hr.cov.tls, G.sh.tls  \tab numeric\cr
      [,38:43] \tab V.tls, V.hn.tls, V.hr.tls, V.hn.cov.tls, V.hr.cov.tls, V.sh.tls   \tab numeric\cr
      [,44:47] \tab d.tls, dg.tls, dgeom.tls, dharm.tls   \tab numeric \cr
      [,48:51] \tab h.tls, hg.tls, hgeom.tls, hharm.tls   \tab numeric\cr
      [,52:55] \tab d.0, dg.0, dgeom.0, dharm.0  \tab numeric\cr
      [,56:59] \tab h.0, hg.0, hgeom.0, hharm.0   \tab numeric\cr
      [,60:74] \tab P01, P05, P10, P20, P25, P30, P40, P50, P60, P70, P75, P80, P90, P95, P99   \tab numeric\cr}

      \item \code{angle.count}: data frame with TLS metrics and variables estimated on the basis of simulated
plots in an angle-count plot design. They plots are simulated for correlative angle-count plots and incremental values of 0.1 \eqn{{m}^{2}/ha}{m^2/ha} for BAF. The following variables are provided for each pair (plot, BAF) (see \code{\link{simulations}} \sQuote{Value} for more details):

    \tabular{rll}{
      [,1] \tab id   \tab character/numeric\cr
      [,2] \tab BAF    \tab numeric\cr
      [,3:5] \tab N, G, V    \tab numeric\cr
      [,6:9] \tab d, dg, dgeom, dharm   \tab numeric\cr
      [,10:13] \tab h, hg, hgeom, hharm   \tab numeric\cr
      [,14:17] \tab d.0, dg.0, dgeom.0,  dharm.0  \tab numeric\cr
      [,18:21] \tab h.0, hg.0, hgeom.0, hharm.0   \tab numeric\cr
      [,22:23] \tab N.tls, N.pam.tls   \tab numeric\cr
      [,24:27] \tab num.points, num.points.est, num.points.hom, num.points.hom.est   \tab numeric\cr
      [,28:29] \tab G.tls, G.pam.tls   \tab numeric\cr
      [,30:31] \tab V.tls, V.pam.tls   \tab numeric\cr
      [,32:35] \tab d.tls, dg.tls, dgeom.tls, dharm.tls   \tab numeric \cr
      [,48:51] \tab h.tls, hg.tls, hgeom.tls, hharm.tls   \tab numeric\cr
      [,36:39] \tab d.0, dg.0, dgeom.0, dharm.0  \tab numeric\cr
      [,40:43] \tab h.0, hg.0, hgeom.0, hharm.0   \tab numeric\cr
      [,44:62] \tab P01, P05, P10, P20, P25, P30, P40, P50, P60, P70, P75, P80, P90, P95, P99   \tab numeric\cr}}}

\usage{data(Rioja.simulations)}
\format{List with 3 data frames containing 2224 observations and 74 variables (simulations.fixed.area.plot), 272 observations and 74 variables (simulations.k.tree.plot), and 576 observations and 62 variables (simulations.angle.count.plot).}
\keyword{datasets}

\encoding{UTF-8}

\name{estimation.plot.size}

\alias{estimation.plot.size}

\title{Assess Consistency of Metrics for Simulated TLS Plots}

\description{Plots empirical linear charts of density (\emph{N}, trees/ha) and basal area (\emph{G}, \eqn{{m}^{2}/ha}{m^2/ha}) estimates (derived from simulated TLS plots) as a function of plot size (estimation-size charts) for different plot designs (circular fixed area, k-tree and angle-count), through continuous size increments (radius, k and BAF respectively). Size increments are set at 0.1 m, 1 tree and 0.1 \eqn{{m}^{2}/ha}{m^2/ha} for fixed area, k-tree and angle-count plot designs, respectively. These size-estimation line charts represent the consistency in predicting the stand variables across different values of radius, k and BAF. Size-estimation charts can be drawn for individual sample plots (including all plots together in the same charts) or for mean values (global mean computed for all the sample plots, or for group means if different strata are considered). Finally,
different plot designs can be compared if specified in the arguments, producing one size-estimation chart per variable (\emph{N} and \emph{G}).}

\usage{
estimation.plot.size(tree.tls,
                     plot.parameters = data.frame(radius.max = 25,
                                                  k.max = 50,
                                                  BAF.max = 4),
                     dbh.min = 4,
                     average = FALSE, all.plot.designs = FALSE)}

\arguments{

  \item{tree.tls}{Data frame with information of trees detected from TLS point cloud data in the same format as \code{\link{tree.detection.single.scan}} and \code{\link{tree.detection.multi.scan}} \sQuote{Value}.}

  \item{plot.parameters}{Optional data frame containing parameters for circular fixed area, k-tree and angle-count plot designs. The parameters are as follows:}

  \itemize{

  \item{\code{radius.max}}: maximum plot radius (m) considered for circular fixed area plots. If the \code{radius.max} specified is larger than the farthest tree from the plot centre, the horizontal distance from the farthest tree will be considered the maximum \code{radius}. By default, the \code{radius.max} will be 25 m.

\item{\code{k.tree.max}}: maximum number of trees considered for k-tree plots. If \code{k.tree.max} specified is larger than the maximum number of trees of the densest plot, this number of trees will be considered the maximum \code{k.tree.max}. By default, \code{k.tree.max} is 50.

\item{\code{BAF.max}}: maximum basal area factor (\eqn{{m}^{2}/ha}{m^2/ha}) considered for angle-count plots. By default, \code{BAF.max} is 4.}

  \item{dbh.min}{Optional minimum \emph{dbh} (cm) considered for detecting trees. By default it will be set at 4 cm.}

  \item{average}{Logical; if \code{TRUE}, plot means values and standard deviation of estimations will be represented. By default, it will be set as \code{FALSE}.}

  \item{all.plot.designs}{Logical; if \code{TRUE}, charts for each plot design are drawn together. By default, it will be set as \code{FALSE}.}}

\details{If there are strata in the \code{tree.tls} argument, they will be differentiated in charts with different colours. Strata must be specified in a numeric column named \code{stratum}.

The \code{all.plot.designs} argument only works for single strata, and therefore if there are additional strata in the \code{tree.tls} argument, they will be considered equal.

The outputs of this function are inspired by Fig. 3 of Brunner and Gizachew (2014).}

\value{\code{Invisible NULL}}

\references{
Brunner, A., & Gizachew, B. (2014). Rapid detection of stand density, tree positions, and tree diameter with a 2D terrestrial laser scanner. \emph{European Journal of Forest Research}, \bold{133(5)}, 819-831.}

\author{
Juan Alberto Molina-Valero and
Adela Martínez-Calvo.}

\note{Mean values are relevant when plots are representing homogenous strata.

Note that this is an option for choosing the best plot design when  field data are not available. Otherwise, using \code{\link{correlations}}, \code{\link{relative.bias}} and \code{\link{optimize.plot.design}} will be more desirable for obtaining the best possible plot design.}

\seealso{
\code{\link{tree.detection.single.scan}}, \code{\link{tree.detection.multi.scan}}, \code{\link{tree.detection.several.plots}}
}

\examples{
\donttest{

# Loading dataset with trees detected from TLS single-scans

data("Rioja.data")

tree.tls <- Rioja.data$tree.tls


# Without strata and plot parameters by default

estimation.plot.size(tree.tls)

estimation.plot.size(tree.tls, average = TRUE)

estimation.plot.size(tree.tls, all.plot.designs = TRUE)


  }

}

\encoding{UTF-8}

\name{internal_ransac}

\alias{internal_ransac}

\title{Apply RANSAC algorithm to estimate diameters.}

\description{Apply RANSAC algorithm to estimate diameters.}

\usage{
internal_ransac(dat,
                dist = 0.05)}

\arguments{

  \item{dat}{dat}

  \item{dist}{dist}}

\value{Data frame}


\author{
Juan Alberto Molina-Valero}

\keyword{internal}

\encoding{UTF-8}

\name{RANSAC_cpp}

\alias{RANSAC_cpp}

\title{Apply RANSAC algorithm to estimate diameters.}

\description{Apply RANSAC algorithm to estimate diameters.}

\usage{
RANSAC_cpp(data, dist)}

\arguments{

  \item{data}{Points}

  \item{dist}{Distance}}

\value{Data frame.}


\author{
Juan Alberto Molina-Valero,
María José Ginzo Villamayor,
Manuel Antonio Novo Pérez,
Adela Martínez-Calvo,
Juan Gabriel Álvarez-González,
Fernando Montes and
César Pérez-Cruzado.}

\keyword{internal}

\encoding{UTF-8}

\name{weighted_mean_sqrt}

\alias{weighted_mean_sqrt}

\title{Calculate weighted quadratic mean.}

\description{Calculate weighted quadratic mean.}

\usage{

  weighted_mean_sqrt(x, w)}

\arguments{

  \item{x}{attribute for which calculate mean.}

  \item{w}{vector with weights associated to x.}}

\value{Data frame.}


\author{
  Juan Alberto Molina-Valero,
  María José Ginzo Villamayor,
  Manuel Antonio Novo Pérez,
  Adela Martínez-Calvo,
  Juan Gabriel Álvarez-González,
  Fernando Montes and
  César Pérez-Cruzado.}

\keyword{internal}

\encoding{UTF-8}


\name{simulations}


\alias{simulations}


\title{
Compute Metrics and Variables for Simulated TLS and Field Plots
}


\description{
Computes TLS metrics derived from simulated TLS plots and variables estimated on the basis of simulated field plots. Real TLS and field data from the same
set of plots are required in order to build simulated plots. Three different
plot designs are currently available: circular fixed area, k-tree and angle-count.
During the simulation process, plots with incremental values for radius, k and
BAF are simulated for circular fixed area, k-tree and angle-count designs, respectively,
according to the parameters specified in the \code{plot.parameters} argument. For
TLS metrics, different method are included for correcting occlusions generated in
TLS point clouds.
}


\usage{
simulations(tree.tls, tree.ds = NULL, tree.field,
            plot.design = c("fixed.area", "k.tree", "angle.count"),
            plot.parameters = data.frame(radius.max = 25, k.max = 50, BAF.max = 4),
            scan.approach = "single", var.metr = list(tls = NULL, field = NULL),
            v.calc = "parab", dbh.min = 4, h.min = 1.3, max.dist = Inf,
            dir.data = NULL, save.result = TRUE, dir.result = NULL)
}


\arguments{

  \item{tree.tls}{
  Data frame with information about trees detected from TLS point cloud data. The
  structure and format must be analogous to output  returned by
  \code{\link{tree.detection.single.scan}} and \code{\link{tree.detection.multi.scan}} (or \code{\link{tree.detection.several.plots}}) functions. In particular, each row must
  correspond to a (plot, tree) pair, and it must include at least the following columns:
    \itemize{
      \item \code{id}, \code{file}, \code{tree}, \code{x}, \code{y},
      \code{phi.left}, \code{phi.right}, \code{horizontal.distance},
      \code{dbh}, \code{num.points}, \code{num.points.hom},
      \code{num.points.est}, \code{num.points.hom.est},
      \code{partial.occlusion}: same description and format as indicated for the
      same named columns in \code{\link{tree.detection.single.scan}} \sQuote{Value}.
    }
  }

  \item{tree.ds}{
  An optional list containing results arises from the application of distance
  sampling methodologies. The structure and format must be analogous to output
  returned by \code{\link{distance.sampling}} function. In particular, it must
  include at least the following elements:
    \itemize{
      \item \code{tree}: data frame with detection probabilities for each tree
      using distance sampling methodologies. Each row must correspond
      to a (plot, tree) pair, and it must include at least the following columns:
        \itemize{
          \item \code{id}, \code{tree}, \code{P.hn}, \code{P.hn.cov},
          \code{P.hr}, \code{P.hr.cov}: same description and format as
          indicated for same named columns of \code{tree} in \code{\link{distance.sampling}} \sQuote{Value}. In addition, plot
          identification and tree numbering included in \code{id} and
          \code{tree} columns must coincide with those included in the same
          named columns of \code{tree.tls} argument.
        }

  If this argument is not specified by the user, it will be set to \code{NULL} by
  default and, as a consequence, the TLS metrics using distance sampling based
  correction will not be calculated for a circular fixed area or k-tree plot designs.
}}

  \item{tree.field}{
  Data frame with information about trees measured in the field plots. Each
  row must correspond to a (plot, tree) pair, and it must include at least the
  following columns:
    \itemize{
      \item \code{id}: plot identification encoded as character string or
      numeric. Plot identifications must coincide with those
      included in \code{id} column of the \code{tree.tls} argument.
      \item \code{tree}: trees numbering.
      \item \code{horizontal.distance}: horizontal distance (m) from plot
      centre to tree centre. Centres of the field plots must coincide with
      centres of their corresponding TLS plots.
      \item \code{dbh}: tree diameter (cm) at breast height (1.3 m).
      \item \code{total.height}: tree total height (m).
      \item \code{dead}: integer value indicating for each tree if it is dead
      (1) or not (NA).
    }
  }

  \item{plot.design}{Vector containing the plot designs considered. By default, all plot designs will be considered (circular fixed area, k-tree and angle-count plots).}

  \item{plot.parameters}{
  Optional list containing parameters for circular fixed area, k-tree and angle-count
  plot designs. User can set all or any of the following parameters specifying
  them as named elements of the list:
    \itemize{
      \item \code{radius.max}: maximum radius (m) allowed for the increasing
      sequence of radius values to be used in the simulation process of TLS and
      field plots under circular fixed area plot design. If this element is not included
      in the argument, circular fixed area plots will not be simulated.
      \item \code{radius.increment}: positive increment (m) for the increasing
      sequence of radius values to be used in the simulation process of TLS and
      field plots in a circular fixed area plot design. If this element is not included
      in the argument, it is set to 0.1 m by default.
      \item \code{k.tree.max}: maximum number of trees (trees) allowed for the
      increasing sequence of k values to be used in the simulation process of
      TLS and field plots under k-tree plot design. If this element is not
      included in the argument, k-tree plots will not be simulated.
      \item \code{BAF.max}: maximum BAF (\eqn{{m}^{2}/ha}{m^2/ha}) allowed for the increasing
      sequence of BAF values to be used in the simulation process of TLS and
      field plots in an angle-count plot design. If this element is not
      included in the argument, angle-count plots will not be simulated.
      \item \code{BAF.increment}: positive increment (\eqn{{m}^{2}/ha}{m^2/ha}) for the increasing
      sequence of BAF values to be used in the simulation process of TLS and
      field plots under angle-count plot design. If this element is not
      included in the argument, it is set to 0.1 \eqn{{m}^{2}/ha}{m^2/ha} by default.
      \item \code{num.trees}: number of dominant trees per ha (tree/ha) to be
      used for calculating dominant diameters and heights during the simulation
      process of TLS and field plots under all the available plot designs.
      Dominant trees are those with the largest diameter at breast height. If
      this element is not included in \code{plot.parameters} argument, it is
      set to 100 trees/ha by default.
    }
  If this argument is specified by the user, it must include at least one of
  the following elements: \code{radius.max}, \code{k.tree.max} or
  \code{BAF.max}. If this argument is not specified by the user, it is set to
  \code{list(radius.max = 25, k.tree.max = 50, BAF.max = 4)} by default and, as a
  consequence, the three available plot designs will be simulated.
  }

  \item{scan.approach}{Character parameter indicating TLS single-scan (\sQuote{single}) or TLS multi-scan approach or SLAM point clouds (\sQuote{multi}) approaches. If this argument is not specified by the user, it will be set to \sQuote{multi} approach.}

  \item{var.metr}{Optional vector containing all the metrics and variables of interest. By default it will be set as NULL and thus, all the metrics and variables available will be generated.}

  \item{v.calc}{Optional parameter to calculate volume when is not included in tree.tls input data.}

  \item{dbh.min}{Optional minimum \emph{dbh} (cm) considered for detecting trees. By default it will be set at 4 cm.}

  \item{h.min}{Optional minimum \emph{h} (m) considered for detecting trees. By default it will be set at 1.3 m.}

  \item{max.dist}{Optional argument to specify the maximum horizontal distance considered in which trees will be included.}

  \item{dir.data}{
  Optional character string naming the absolute path of the directory where TXT
  files containing TLS point clouds are located. \code{.Platform$file.sep} must
  be used as the path separator in \code{dir.dat}, and TXT files in the
  directory must have the same description and format as indicated for TXT
  files in \code{\link{normalize}} \sQuote{Output Files}. If this
  argument is not specified by the user, it will be set to \code{NULL} by default
  and, as a consequence, the current working directory of the \R process will be
  assigned to \code{dir.dat} during the execution.
  }

  \item{save.result}{
  Optional logical which indicates wheter or not the output files described in
  \sQuote{Output Files} section must be saved in \code{dir.result}. If
  this argument is not specified by the user, it will be set to \code{TRUE} by
  default and, as a consequence, the output files are saved.
  }

  \item{dir.result}{
  Optional character string naming the absolute path of an existing directory
  where files described in \sQuote{Output Files} section will be saved.
  \code{.Platform$file.sep} must be used as the path separator in
  \code{dir.result}. If this argument is not specified by the user, and
  \code{save.result} is \code{TRUE}, it will be set to \code{NULL} by default and,
  as a consequence, the current working directory of the \R process will be
  assigned to \code{dir.result} during the execution.
  }
}


\details{
Using real TLS and field data from the same set of plots, this function enables construction
of simulated plots under different plot designs and computation of the
corresponding TLS metrics and estimated variables. The notation used for
variables is based on IUFRO (1959).

At this stage, three plot designs are available:
  \itemize{
    \item Circular fixed area plots, simulated only if a \code{radius.max} value is
    specified in the \code{plot.parameters} argument.
    \item k-tree plots, simulated only if a \code{k.tree.max} value is
    specified in the \code{plot.parameters} argument.
    \item Angle-count plots, simulated only if a \code{BAF.max} value is
    specified in the \code{plot.parameters} argument.
  }
For each real plot, a simulation process is run under each of the plot designs
specified by means of elements of the \code{plot.parameters} argument. Although there
are some minor differences depending on the plot design, the rough outline of
the simulation process is similar for all, and it consists of the
following main steps:
  \enumerate{
    \item Define an increasing sequence of the plot design parameter (radius,
    k or BAF) according to the maximum value and, if applicable, the positive
    increment set in \code{plot.parameters} argument.
    \item Build simulated plots for each parameter value in the previous
    sequence based on either TLS or field data.
    \item Compute either TLS metrics or variables estimated on the basis of
    simulated plots for each parameter value (see \sQuote{Value} section for
    details). For the simulated TLS plots, note that in
    addition to the counterparts of variables computed for the simulated field
    plots, the function also computes the following:
      \itemize{
        \item Metrics related to the number of points belonging to normal tree
        sections.
        \item Metrics with occlusion corrections based on the following:
        \itemize{
          \item Distance sampling methodologies (Astrup et al., 2014) for
          circular fixed area and k-tree plot designs, if the \code{distance.sampling}
          argument is not \code{NULL}.
          \item Correction of the shadowing effect (Seidel & Ammer, 2014) for
          circular fixed area and k-tree plot designs.
          \item Gap probability attenuation with distance to TLS (Strahler et
          al., 2008; Lovell et al., 2011) for angle-count plot design.
        }
        \item Height percentiles derived from z coordinates of TLS point clouds
         relative to ground level.
      }
  }
}


\value{List with field estimates and TLS metrics for plot designs considered. It will contain one element per plot design considered (fixed.area.plot, k.tree.plot and angle.count.plot)

  \item{fixed.area.plot}{
  If no value for \code{radius.max} is specified in the \code{plot.parameters}
  argument, \code{NULL}; otherwise, data frame with TLS metrics and variables will be estiamted on the basis of simulated plots in a circular fixed area plot design. Each row will correspond to a (plot, radius) pair, and the
  following columns will be included:

  Plot identification and radius:
    \itemize{
      \item \code{id}: plot identification encoded as a character string or
      numeric. It will coincide with those included in the \code{id}
      column of \code{tree.tls}, \code{tree.field} or, if applicable,
      \code{distance.sampling} arguments.
      \item \code{radius}: radius (m) of the simulated plot.
    }
  Variables estimated on the basis of simulated field plots:
    \itemize{
      \item \code{N}: stand density (trees/ha).
      \item \code{G}: stand basal area (\eqn{{m}^{2}/ha}{m^2/ha}).
      \item \code{V}: stand volume (\eqn{{m}^{3}/ha}{m^3/ha}).
      \item \code{d}, \code{dg}, \code{dgeom}, \code{dharm}: mean tree
      diameters (cm) at breast height (1.3 m), calculated from the arithmetic mean, quadratic
      mean, geometric mean and harmonic mean, respectively.
      \item \code{h}, \code{hg}, \code{hgeom}, \code{hharm}: mean tree
      heights (m), calculated from the arithmetic mean, quadratic mean, geometric mean and
      harmonic mean, respectively.
      \item \code{d.0}, \code{dg.0}, \code{dgeom.0}, \code{dharm.0}: dominant
      mean tree diameters (cm) at breast height (1.3 m), calcualted from the arithmetic mean,
      quadratic mean, geometric mean and harmonic mean, respectively.
      \item \code{h.0}, \code{hg.0}, \code{hgeom.0}, \code{hharm.0}: dominant
      mean tree heights (m), calculated from the arithmetic mean, quadratic mean, geometric
      mean and harmonic mean, respectively.
      }
  TLS variables derived from simulated TLS plots:
    \itemize{
      \item \code{N.tls}: stand density (trees/ha) without occlusion
      corrections.
      \item \code{N.hn}, \code{N.hr}, \code{N.hn.cov}, \code{N.hr.cov}: stand
      density (trees/ha) with occlusion corrections based on distance sampling
      methodologies. These columns will be missing if the \code{distance.sampling}
      argument is \code{NULL}.
      \item \code{N.sh}: stand density (trees/ha) with correction of the
      shadowing effect.
      \item \code{G.tls}: stand basal area (\eqn{{m}^{2}/ha}{m^2/ha}) without occlusion
      corrections.
      \item \code{G.hn}, \code{G.hr}, \code{G.hn.cov}, \code{G.hr.cov}: stand
      basal area (\eqn{{m}^{2}/ha}{m^2/ha}) with occlusion corrections based on distance sampling
      methodologies. These columns will be missing if the \code{distance.sampling}
      argument is \code{NULL}.
      \item \code{G.sh}: stand basal area (\eqn{{m}^{2}/ha}{m^2/ha}) with correction of the
      shadowing effect.
      \item \code{V.tls}: stand volume (\eqn{{m}^{3}/ha}{m^3/ha}) without occlusion corrections.
      \item \code{V.hn}, \code{V.hr}, \code{V.hn.cov}, \code{V.hr.cov}: stand
      volume (\eqn{{m}^{3}/ha}{m^3/ha}) with occlusion corrections based on distance sampling
      methodologies. These  columns will be missing if the \code{distance.sampling}
      argument is \code{NULL}.
      \item \code{V.sh}: stand volume (\eqn{{m}^{3}/ha}{m^3/ha}) with correction of the
      shadowing effect.
      \item \code{d.tls}, \code{dg.tls}, \code{dgeom.tls}, \code{dharm.tls}:
      mean tree diameters (cm) at breast height (1.3 m), calculated from the arithmetic mean,
      quadratic mean geometric mean, and harmonic mean, respectively.
      \item \code{h.tls}, \code{hg.tls}, \code{hgeom.tls}, \code{hharm.tls}:
      mean tree heights (m), calculated from the arithmetic mean, quadratic mean, geometric
      mean and harmonic mean, respectively.
      \item \code{d.0.tls}, \code{dg.0.tls}, \code{dgeom.0.tls},
      \code{dharm.0.tls}: dominant mean tree diameters (cm) at breast height
      (1.3 m), calculated from the arithmetic mean, quadratic mean, geometric mean and
      harmonic mean, respectively.
      \item \code{h.0.tls}, \code{hg.0.tls}, \code{hgeom.0.tls},
      \code{hharm.0.tls}: dominant mean tree heights (m), calculated from the arithmetic
      mean, quadratic mean, geometric mean and harmonic mean, respectively.
    }
    TLS metrics derived from simulated TLS plots:
    \itemize{
      \item \code{num.points}, \code{num.points.est}, \code{num.points.hom},
      \code{num.points.hom.est}: number of points and estimated number of
      points (points) belonging to trees with normal sections (+/- 5 cm) in the
      original point cloud (\code{num.points} and \code{num.points.est},
      respectively); and number of points and estimated number of points
      (points) belonging to trees normal sections (+/- 5 cm) in the reduced
      point cloud (\code{num.points.hom} and \code{num.points.hom.est},
      respectively).
      \item \code{P01}, \code{P05}, \code{P10}, \code{P20}, \code{P25},
      \code{P30}, \code{P40}, \code{P50}, \code{P60}, \code{P70}, \code{P75},
      \code{P80}, \code{P90}, \code{P95}, \code{P99}: height percentiles
      derived from z coordinates of TLS point clouds relative to ground level.
    }
  }

  \item{k.tree.plot}{
  If no value for \code{k.tree.max} is specified in the \code{plot.parameters}
  argument, \code{NULL}; otherwise the data frame with TLS metrics and estimations
  of variables will be based on simulated plots in the k-tree plot design. Each of row will correspond to a (plot, k) pair, and the following
  columns will be included:

  Plot identification and k:
    \itemize{
      \item \code{id}: plot identification encoded as character string or
      numeric. The \code{id} will coincide with those included in the \code{id}
      column of \code{tree.tls}, \code{tree.field} or, if applicable,
      \code{distance.sampling} arguments.
      \item \code{k}: number of trees (trees) in the simulated plot.
    }
  Estimated variables based on simulated field plots:
    \itemize{
      \item \code{N}, \code{G}, \code{V}, \code{d}, \code{dg}, \code{dgeom},
      \code{dharm}, \code{h}, \code{hg}, \code{hgeom}, \code{hharm},
      \code{d.0}, \code{dg.0}, \code{dgeom.0}, \cr \code{dharm.0}, \code{h.0},
      \code{hg.0}, \code{hgeom.0}, \code{hharm.0}: same description and format
      as indicated in the \code{fixed.area.plot} element.
    }
  TLS variables derived from simulated TLS plots:
    \itemize{
      \item \code{N.tls}, \code{N.hn}, \code{N.hr}, \code{N.hn.cov}, \code{N.hr.cov}, \code{N.sh}, \cr
      \code{G.tls}, \code{G.hn}, \code{G.hr}, \code{G.hn.cov}, \code{G.hr.cov}, \code{G.sh}, \cr
      \code{V.tls}, \code{V.hn}, \code{V.hr}, \code{V.hn.cov}, \code{V.hr.cov}, \code{V.sh}, \cr
      \code{d.tls}, \code{dg.tls}, \code{dgeom.tls}, \code{dharm.tls}, \cr
      \code{h.tls}, \code{hg.tls}, \code{hgeom.tls}, \code{hharm.tls}, \cr
      \code{d.0.tls}, \code{dg.0.tls}, \code{dgeom.0.tls}, \code{dharm.0.tls}, \cr
      \code{h.0.tls}, \code{hg.0.tls}, \code{hgeom.0.tls}, \code{hharm.0.tls}}

    TLS metrics derived from simulated TLS plots:
    \itemize{
      \item \code{num.points}, \code{num.points.est}, \code{num.points.hom}, \code{num.points.hom.est}, \cr
      \code{P01}, \code{P05}, \code{P10}, \code{P20}, \code{P25}, \code{P30}, \code{P40}, \code{P50}, \code{P60}, \code{P70}, \code{P75}, \code{P80}, \code{P90}, \code{P95}, \code{P99}: same description and format as indicated in
      \code{fixed.area.plot} element.}}


  \item{angle.count.plot}{
  If no value for \code{BAF.max} is specified in the \code{plot.parameters}
  argument, \code{NULL}; otherwise the data frame will include TLS metrics and estimated
  variables based on simulated plots in the angle-count plot design. Each
  row will correspond to a (plot, BAF) pair, and the
  following columns will be included:

  Plot identification and BAF:
    \itemize{
      \item \code{id}: plot identification encoded as character string or
      numeric. The \code{id} will coincide with those included in the \code{id}
      column of \code{tree.tls} and \code{tree.field}.
      \item \code{BAF}: BAF (\eqn{{m}^{2}/ha}{m^2/ha}) of the simulated plot.
    }
  Estimated variables based on simulated field plots:
    \itemize{
      \item \code{N}, \code{G}, \code{V}, \code{d}, \code{dg}, \code{dgeom},
      \code{dharm}, \code{h}, \code{hg}, \code{hgeom}, \code{hharm},
      \code{d.0}, \code{dg.0}, \code{dgeom.0}, \cr \code{dharm.0}, \code{h.0},
      \code{hg.0}, \code{hgeom.0}, \code{hharm.0}: same description and format
      as indicated in the \code{fixed.area.plot} element.
    }
  TLS variables derived from simulated TLS plots:
    \itemize{
      \item \code{N.tls}: same description and format as indicated in the
      \code{fixed.area.plot} element.
      \item \code{N.pam}: stand density (trees/ha) with occlusion correction
      based on gap probability attenuation with distance to TLS.
      \item \code{G.tls}: same description and format as indicated in
      \code{fixed.area.plot} element.
      \item \code{G.pam}: stand basal area (\eqn{{m}^{2}/ha}{m^2/ha}) with occlusion correction
      based on gap probability attenuation with distance to TLS.
      \item \code{V.tls}: same description and format as indicated in
      \code{fixed.area.plot} element.
      \item \code{V.pam}: stand volume (\eqn{{m}^{3}/ha}{m^3/ha})with occlusion correction based
      on gap probability attenuation with distance to TLS.
      \item \code{d.tls}, \code{dg.tls}, \code{dgeom.tls}, \code{dharm.tls}, \cr
      \code{h.tls}, \code{hg.tls}, \code{hgeom.tls}, \code{hharm.tls}, \cr
      \code{d.0.tls}, \code{dg.0.tls}, \code{dgeom.0.tls}, \code{dharm.0.tls}, \cr
      \code{h.0.tls}, \code{hg.0.tls}, \code{hgeom.0.tls}, \code{hharm.0.tls}
    }
    TLS metrics derived from simulated TLS plots:
    \itemize{
      \item \code{num.points}, \code{num.points.est}, \code{num.points.hom},
      \code{num.points.hom.est}, \cr
      \code{P01}, \code{P05}, \code{P10}, \code{P20}, \code{P25}, \code{P30},
      \code{P40}, \code{P50}, \code{P60}, \code{P70}, \code{P75}, \code{P80},
      \code{P90}, \code{P95}, \code{P99}: same description and format as
      indicated in \code{fixed.area.plot} element.
    }
  }
}


\section{Output Files}{
At the end of the simulation process, if the \code{save.result} argument is
\code{TRUE}, the function will print all the elements described in
\sQuote{Value} section and which are different from \code{NULL} to files. Data frames are
written without row names in \code{dir.result} directory using
the \code{\link{write.csv}} function from the \pkg{utils} package. The pattern used
for naming these files is \file{simulations.<plot design>.csv}, where
\file{<plot design>} is equal to \dQuote{\code{fixed.area.plot}},
\dQuote{\code{k.tree.plot}} or \dQuote{\code{angle.count.plot}} according to
plot design.
}


\references{
Astrup, R., Ducey, M. J., Granhus, A., Ritter, T., & von Lüpke, N. (2014).
Approaches for estimating stand level volume using terrestrial laser scanning
in a single-scan mode. \emph{Canadian Journal of Forest Research},
\bold{44(6)}, 666-676. \doi{10.1139/cjfr-2013-0535}

IUFRO (1959). \emph{Standarization of symbols in forest mensuration}. IUFRO,
Wien, 32 pp.

Lovell, J. L., Jupp, D. L. B., Newnham, G. J., & Culvenor, D. S. (2011).
Measuring tree stem diameters using intensity profiles from ground-based
scanning lidar from a fixed viewpoint. \emph{ISPRS Journal of Photogrammetry
and Remote Sensing}, \bold{66(1)}, 46-55. \doi{10.1016/j.isprsjprs.2010.08.006}

Seidel, D., & Ammer, C. (2014). Efficient measurements of basal area in short
rotation forests based on terrestrial laser scanning under special
consideration of shadowing. \emph{iForest-Biogeosciences and Forestry},
\bold{7(4)}, 227. \doi{10.3832/ifor1084-007}

Strahler, A. H., Jupp, D. L. B., Woodcock, C. E., Schaaf, C. B., Yao, T., Zhao,
F., Yang, X., Lovell, J., Culvenor, D., Newnham, G., Ni-Miester, W., &
Boykin-Morris, W. (2008). Retrieval of forest structural parameters using a
ground-based lidar instrument (Echidna®). \emph{Canadian Journal of Remote
Sensing}, \bold{34(sup2)}, S426-S440. \doi{10.5589/m08-046}
}


\note{
The simulation process implemented in this function is computationally intensive.
Although the function currently uses the \link[vroom]{vroom} function from the \CRANpkg{vroom} package for reading large files and contains fast implementations of
several critical calculations (C++ via \CRANpkg{Rcpp} package), long
computation times may be required when a large number of plots are considered,
number of points in TLS point clouds are very high, or the radius, k
or BAF sequences used in the simulation process are very long.

Using reduced point clouds (according to point cropping process implemented in the
\code{\link{normalize}} function), rather than original ones, may be
recommended in order to cut down on computing time. Another possibility would
be to specify large increments for radius
and BAF, and/or low maximum values for radius, number of trees and BAF in the \code{plot.parameters} argument. This
would make the function more efficient, though there may be a notable
loss of detail in the results generated.
}


\author{
Juan Alberto Molina-Valero and
Adela Martínez-Calvo.}


\seealso{
\code{\link{tree.detection.single.scan}}, \code{\link{tree.detection.multi.scan}}, \code{\link{tree.detection.several.plots}}, \code{\link{distance.sampling}},
\code{\link{normalize}}.
}


\examples{
\donttest{

# Load information of trees detected from TLS point clouds data corresponding to
# plots 1 and 2 from Rioja data set

data("Rioja.data")
example.tls <- subset(Rioja.data$tree.tls, id < 3)

# Compute detection probabilities using distance sampling methods

example.ds <- distance.sampling(example.tls)

# Load information of trees measured in field plots corresponding to plot 1 and 2
# from Rioja data set

example.field <- subset(Rioja.data$tree.field, id < 3)

# Establish directory where TXT file containing TLS point cloud corresponding to
# plot 1 from Rioja data set is located. For instance, current working directory

dir.data <- tempdir()

# Download example of TXT file corresponding to plots 1 and 2 from Rioja data set

download.file(url = "https://www.dropbox.com/s/w4fgcyezr2olj9m/Rioja_1.txt?dl=1",
              destfile = file.path(dir.data, "1.txt"), mode = "wb")

download.file(url = "https://www.dropbox.com/s/sghmw3zud424s11/Rioja_2.txt?dl=1",
              destfile = file.path(dir.data, "2.txt"), mode = "wb")

# Establish directory where simulation results corresponding to plots 1 and 2
# from Rioja data set will be saved. For instance, current working directory

dir.result <- tempdir()

# Compute metrics and variables for simulated TLS and field plots corresponding
# to plots 1 and 2 from Rioja data set
# Without occlusion correction based on distance sampling methods

sim <- simulations(tree.tls = example.tls, tree.field = example.field,
                   plot.parameters = data.frame(radius.max = 10, k.max = 20,
                                                BAF.max = 2),
                   dir.data = dir.data, dir.result = dir.result)


}
}

\encoding{UTF-8}
\name{Rioja.data}
\docType{data}
\alias{Rioja.data}
\title{Inventoried Plots Data for a Stand Case Study in La Rioja}
\description{This list includes trees detected with TLS for 16 single scans corresponding to plots located in La Rioja, a region of Spain, in the north of the Iberian Peninsula (first element), as well as those inventoried in the field for these 16 plots (second element). Plot attributes related to stand stratum are also included (third element).

The elements of the list are as follows:
\enumerate{
    \item \code{tree.tls}: data frame that includes the list of trees detected with \code{\link{tree.detection.single.scan}} for 16 TLS single-scan sampling points. The following
variables are provided for each tree (see \code{\link{tree.detection.single.scan}} \sQuote{Value} for more details):

\tabular{rll}{
[,1] \tab id   \tab character/numeric\cr
[,2] \tab file   \tab character\cr
[,3] \tab tree    \tab numeric\cr
[,4] \tab x    \tab numeric\cr
[,5] \tab y   \tab numeric\cr
[,6:8] \tab phi, phi.left, phi.right     \tab numeric\cr
[,9] \tab h.dist   \tab numeric\cr
[,10] \tab dbh   \tab numeric\cr
[,11] \tab h   \tab numeric\cr
[,12] \tab v   \tab numeric\cr
[,13:16] \tab n.pts, n.pts.red, n.pts.est, n.pts.red.est   \tab numeric\cr
[,15] \tab partial.occlusion   \tab numeric\cr}

\item \code{tree.list.field}: data frame that includes the list of trees measured in 16 circular fixed area plots of radius 20 m, whose centres coincide with TLS single-scans points. The following variables are provided for each tree:

\tabular{rlll}{
[,1] \tab id   \tab numeric \tab plot identification (coincident to TLS scans)\cr
[,2] \tab tree    \tab numeric \tab trees numbering\cr
[,3] \tab Sp    \tab numeric \tab specie code according to NFI ()\cr
[,4] \tab x    \tab numeric \tab x cartesian coordinate\cr
[,5] \tab y   \tab numeric \tab 7 cartesian coordinate\cr
[,6] \tab h.dist   \tab numeric \tab horizontal distance (m) from plot center to tree center\cr
[,7] \tab dbh   \tab numeric \tab tree diameter (cm) at breast height (1.3 m)\cr
[,8] \tab h   \tab numeric \tab tree total height (m)\cr
[,9] \tab dead   \tab numeric \tab dead (1) or not (NA)\cr
[,10] \tab v.user    \tab numeric \tab stem volume (m^3) estimated with allometric equations\cr
[,11] \tab w.user   \tab numeric \tab stem biomass (Mg) estimated with allometric equations\cr}}}
\usage{data(Rioja.data)}
\format{List with 2 data frames containing 604 observations and 17 variables (tree.tls) and 659 observations and 11 variables (tree.field).}
\keyword{datasets}
